﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Net;
    using System.Net.Http;
    using System.Security.Claims;
    using System.Web.Http.Controllers;
    using System.Web.Http.Filters;

    internal class ClaimsPrincipalAuthorizationFilter : AuthorizationFilterAttribute
    {
        public override void OnAuthorization(HttpActionContext actionContext)
        {
            if (actionContext.ControllerContext.ControllerDescriptor.ControllerName == "Heartbeat")
            {
                return; // No authentication/authorization for heartbeat
            }

            try
            {
                base.OnAuthorization(actionContext);

                var claimsPrincipal = actionContext.RequestContext.Principal as ClaimsPrincipal;
                if (claimsPrincipal == null || claimsPrincipal.Identity == null || !claimsPrincipal.Identity.IsAuthenticated)
                {
                    actionContext.Response = actionContext.Request.CreateResponse(HttpStatusCode.Forbidden);
                }
            }
            catch (Exception ex)
            {
                Instrumentation.Agent.Instance.LogException(ex, "proxy", "authorization");

                //todo eat exception or rethrow?
                throw;
            }
        }
    }
}