﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Linq;

    public static class ValidationHelper
    {
        /// <summary>
        /// Validates that a required parameter exists in query params and has a value in the range [minValue, maxValue] inclusive.
        /// </summary>
        public static void ValidateParameterWithRange(
            string paramName,
            bool isRequiredParam,
            Dictionary<string, List<string>> queryParams,
            int minValue, 
            int maxValue)
        {
            if (queryParams.TryGetValue(paramName, out var paramList))
            {
                string paramValueString = paramList.FirstOrDefault();

                if (int.TryParse(paramValueString, out int paramValue))
                {
                    if (paramValue < minValue || paramValue > maxValue)
                    {
                        // param value is out of range.
                        throw new ProxyException(ProxyException.ExceptionType.ClientParams, $"Input parameter {paramName} should be in the range ({minValue}-{maxValue}).");
                    }
                }
                else
                {
                    // Couldn't parse the param string - param might not be a valid integer. 
                    throw new ProxyException(ProxyException.ExceptionType.ClientParams, $"Input parameter {paramName} does not contain a valid integer value.");
                }
            }
            else
            {
                if (isRequiredParam)
                {
                    // 'paramName' is a required param. It should be present.
                    throw new ProxyException(ProxyException.ExceptionType.ClientParams, $"Input parameter {paramName} is missing.");
                }
            }
        }
    }
}