﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.Controllers.GeoLocation
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;

    using Microsoft.Azure.LocationServices.GeoLocation;
    using Microsoft.Azure.LocationServices.Models.GeoLocation;
    using Microsoft.Azure.LocationServices.Utility;

    /// <summary>
    /// Implements controller actions for /GeoLocation/* path
    /// </summary>
    public class GeoLocationIPController : BaseProxyController
    {
        // to be injected in constructor by depenedency resolver
        private readonly IIPLookup ipLookupService = Globals.Instance.IPLookupService;
        
        public async Task<HttpResponseMessage> GetIpLocation(string formatstring)
        {
            try
            {
                var versionobject = new Versioning();
                var proxyObj = (BaseGeoLocationProxy)versionobject.GetProxyObj(this.Version, BaseProxy.ProxyType.GeoLocation, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response = await proxyObj.ExecuteRequest(
                                        this.ipLookupService,
                                        BaseGeoLocationProxy.GeoLocationType.ByIP,
                                        this.querystrings,
                                        this.Request.GetClientIpAddress(),
                                        formatstring);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "Failed to initialize workitem instance."));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }

        internal override void ClientValidateParameters()
        {
            if (this.Format != "json")
            {
                throw new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            }
        }
    }
}