﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;

    public class MapImageryController : BaseProxyController
    {
        public async Task<HttpResponseMessage> GetMapImagery(string formatstring)
        {
            try
            {
                List<string> style = new List<string>();
                List<string> zoom = new List<string>();
                List<string> x = new List<string>();
                List<string> y = new List<string>();

                querystrings.TryGetValue("style", out style);
                querystrings.TryGetValue("zoom", out zoom);
                querystrings.TryGetValue("x", out x);
                querystrings.TryGetValue("y", out y);

                querystrings.Remove("style");
                querystrings.Remove("zoom");
                querystrings.Remove("x");
                querystrings.Remove("y");

                if (string.Equals(style.First(), "satellite", StringComparison.OrdinalIgnoreCase))
                {
                    BaseProxyMapImagery proxyObj = (BaseProxyMapImagery)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Imagery, this.ProxyOperation);
                    if (proxyObj != null)
                    {
                        this.response =
                            await
                                proxyObj.SatelliteImagery(
                                    x.First(),
                                    y.First(),
                                    zoom.First());
                    }
                    else
                    {
                        throw (new ProxyException(ProxyException.ExceptionType.ServerError, "Failed to initialize proxy object"));
                    }
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "Invalid 'style' value. Currently only 'satellite' is supported."));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (this.Format != "png")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "Invalid 'format'. Currently only 'png' is supported."));
            }

            if (!querystrings.ContainsKey("style") || !querystrings.ContainsKey("zoom") || !querystrings.ContainsKey("x") || !querystrings.ContainsKey("y"))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "style||zoom||x||y invalid"));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}