﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http;
    using GeoJSON.Net;
    using GeoJSON.Net.Feature;
    using GeoJSON.Net.Geometry;
    using Microsoft.Azure.LocationServices.Geography;
    using Microsoft.Azure.LocationServices.Utility;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Linq;

    public class RouteReachableRangeController : BaseProxyController
    {
        [HttpGet]
        public async Task<HttpResponseMessage> GetRouteReachableRange(string formatstring)
        {
            try
            {
                BaseProxyRoute proxyObj = (BaseProxyRoute)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Route, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response =
                        await
                            proxyObj.RouteProxy(
                                BaseProxyRoute.RouteType.ReachableRange,
                                WebUtility.UrlDecode(this.Query),
                                formatstring,
                                this.querystrings);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (string.IsNullOrEmpty(this.Query))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty"));
            }
            else
            {
                try
                {
                    Coordinates coordinates = new Coordinates(this.Query);
                    if (coordinates.IsNull)
                    {
                        throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string"));
                    }
                }
                catch
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string"));
                }
            }

            if (this.Format != "json" && this.Format != "xml")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string"));
            }

            if ((!this.querystrings.ContainsKey("timeBudgetInSec")) && (!this.querystrings.ContainsKey("energyBudgetInkWh")) && (!this.querystrings.ContainsKey("fuelBudgetInLiters")))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "missing required restriction param"));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}