﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Geography;
    using Microsoft.Azure.LocationServices.Utility;

    public class SearchNearbyController : BaseProxyController
    {
        private static readonly HashSet<string> ValidFormatSet = new HashSet<string>
        {
            "json",
            "xml"
        };

        public async Task<HttpResponseMessage> GetSearchNearby(string formatstring)
        {
            try
            {
                BaseProxySearch proxyObj = (BaseProxySearch)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Search, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response =
                        await
                            proxyObj.SearchProxy(
                                BaseProxySearch.SearchType.Nearby,
                                string.Empty,
                                formatstring,
                                this.querystrings);
                }
                else
                {
                    throw new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (!SearchNearbyController.ValidFormatSet.Contains(this.Format))
            {
                throw new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            }

            var latValue = string.Empty;
            var lonValue = string.Empty;

            if (this.querystrings.TryGetValue("lat", out var valueList))
            {
                latValue = valueList.FirstOrDefault();
            }

            if (this.querystrings.TryGetValue("lon", out valueList))
            {
                lonValue = valueList.FirstOrDefault();
            }

            if (!string.IsNullOrEmpty(latValue) || !string.IsNullOrEmpty(lonValue))
            {
                var coordinates = new Coordinates(latValue, lonValue);
                if (coordinates.IsNull)
                {
                    throw new ProxyException(ProxyException.ExceptionType.ClientParams, "lat and/or lon parameters are invalid.");
                }
            }

            // Limit
            ValidationHelper.ValidateParameterWithRange(Constants.LimitParamKey, false, this.querystrings, 1, 100);
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}
