﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Utility;

    public class SearchPOIController : BaseProxyController
    {
        public async Task<HttpResponseMessage> GetSearchPOI(string formatstring)
        {
            try
            {
                BaseProxySearch proxyObj = (BaseProxySearch)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Search, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response =
                        await
                            proxyObj.SearchProxy(
                                BaseProxySearch.SearchType.POI,
                                WebUtility.UrlDecode(this.Query),
                                formatstring,
                                this.querystrings);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (string.IsNullOrEmpty(this.Query))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty"));
            }

            if (this.Format != "json" && this.Format != "xml")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string"));
            }

            // Limit
            ValidationHelper.ValidateParameterWithRange(Constants.LimitParamKey, false, this.querystrings, 1, 100);
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}