﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Utility;

    /// <summary>
    /// The Get Polygon service allows you to request the geometry data such as a city or country outline for a set of entities, previously retrieved from an Online Search request.
    /// </summary>
    public class SearchPolygonController : BaseProxyController
    {
        public async Task<HttpResponseMessage> GetSearchPolygon(string formatstring)
        {
            try
            {
                BaseProxySearch proxyObj = (BaseProxySearch)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Search, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response =
                        await
                            proxyObj.SearchProxy(
                                BaseProxySearch.SearchType.PolygonData,
                                WebUtility.UrlDecode(this.Query),
                                formatstring,
                                this.querystrings);
                }
                else
                {
                    this.response = this.BuildErrorResponseAndLog(new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (this.Format != "json")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "JSON is the only format allowed"));
            }

            if (!this.querystrings.TryGetValue(Constants.GeometriesQueryParamKey, out var geometryIds) || geometryIds.Count == 0 || string.IsNullOrEmpty(geometryIds[0]))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, $"{Constants.GeometriesQueryParamKey} parameter is expected with atleast one geometry id"));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}
