﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Geography;

    public class TimezoneByCoordinatesController : BaseProxyController
    {
        public async Task<HttpResponseMessage> GetTimezoneCoordinates(string formatstring)
        {
            try
            {
                var proxyObj = (BaseProxyTimezone)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Timezone, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response = 
                        await
                            proxyObj.TimezoneProxy(
                                BaseProxyTimezone.TimezoneType.ByCoordinates,
                                WebUtility.UrlDecode(this.Query),
                                formatstring,
                                this.headers,
                                this.querystrings);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (string.IsNullOrEmpty(this.Query))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty"));
            }
            else
            {
                try
                {
                    Coordinates coordinates = new Coordinates(this.Query);
                    if (coordinates.IsNull)
                    {
                        throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string"));
                    }
                }
                catch
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string"));
                }
            }

            if (this.Format != "json")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string"));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}