﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;

    public class TimezoneEnumIANAController : BaseProxyController
    {
        public async Task<HttpResponseMessage> GetTimezoneIANA(string formatstring)
        {
            try
            {
                var proxyObj = (BaseProxyTimezone)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Timezone, this.ProxyOperation);
                if (proxyObj != null)
                {
                    this.response =
                        await
                            proxyObj.TimezoneProxy(
                                BaseProxyTimezone.TimezoneType.EnumIANA,
                                string.Empty,
                                formatstring,
                                this.headers,
                                this.querystrings);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (this.Format != "json")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string"));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}