﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Linq;
    using System.Net;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Geography;
    using Microsoft.Azure.LocationServices.Utility;

    public class TrafficFlowSegmentController : BaseProxyController
    {
        private const int INCLUSIVE_MIN_ZOOM_INDEX = 0;
        private const int INCLUSIVE_MAX_ZOOM_INDEX = 18;

        public async Task<HttpResponseMessage> GetTrafficFlowSegment(string formatstring)
        {
            try
            {
                BaseProxyTraffic proxyObj = (BaseProxyTraffic)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Traffic, this.ProxyOperation);
                if (proxyObj != null)
                {
                    List<string> style = new List<string>();
                    List<string> zoom = new List<string>();

                    querystrings.TryGetValue("style", out style);
                    querystrings.TryGetValue("zoom", out zoom);

                    querystrings.Remove("style");
                    querystrings.Remove("zoom");

                    this.response =
                        await
                            proxyObj.TrafficFlow(
                                     BaseProxyTraffic.TrafficType.FlowSegment, 
                                     style.First(),
                                     zoom.First(),
                                     WebUtility.UrlDecode(this.Query),
                                     string.Empty,
                                     formatstring,
                                     this.querystrings);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (string.IsNullOrEmpty(this.Query))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty"));
            }
            else
            {
                try
                {
                    Coordinates coordinates = new Coordinates(this.Query);
                    if (coordinates.IsNull)
                    {
                        throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string"));
                    }
                }
                catch
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string"));
                }
            }

            if (this.Format != "json" && this.Format != "xml")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string"));
            }

            if (!querystrings.ContainsKey("style") || !querystrings.ContainsKey("zoom"))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "style||zoom invalid"));
            }

            string zoomParamVal = querystrings["zoom"]?.FirstOrDefault();
            var flowSegmentTileValidator = new TileValidator(INCLUSIVE_MIN_ZOOM_INDEX, INCLUSIVE_MAX_ZOOM_INDEX);
            if (!flowSegmentTileValidator.IsZoomValid(zoomParamVal))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, $"Zoom ({zoomParamVal}) must be between 0 and 18 inclusive."));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}