﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Controllers
{
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Utility;

    public class TrafficFlowTileController : BaseProxyController
    {
        private const int INCLUSIVE_MIN_ZOOM_INDEX = 0;
        private const int INCLUSIVE_MAX_ZOOM_INDEX = 18;

        public async Task<HttpResponseMessage> GetTrafficFlowTile(string formatstring)
        {
            try
            {
                BaseProxyTraffic proxyObj = (BaseProxyTraffic)versionObject.GetProxyObj(this.Version, BaseProxy.ProxyType.Traffic, this.ProxyOperation);
                if (proxyObj != null)
                {
                    List<string> style = new List<string>();
                    List<string> zoom = new List<string>();
                    List<string> x = new List<string>();
                    List<string> y = new List<string>();
             
                    querystrings.TryGetValue("style", out style);
                    querystrings.TryGetValue("zoom", out zoom);
                    querystrings.TryGetValue("x", out x);
                    querystrings.TryGetValue("y", out y);

                    querystrings.Remove("style");
                    querystrings.Remove("zoom");
                    querystrings.Remove("x");
                    querystrings.Remove("y");

                    this.response =
                        await
                            proxyObj.TrafficFlow(
                                     BaseProxyTraffic.TrafficType.FlowTile, 
                                     style.First(),
                                     zoom.First(),
                                     x.First(),
                                     y.First(),
                                     formatstring,
                                     this.querystrings);
                }
                else
                {
                    throw (new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed"));
                }
            }
            catch (ProxyException ex)
            {
                this.response = this.BuildErrorResponseAndLog(ex);
            }

            return this.response;
        }

        internal override void ClientValidateParameters()
        {
            if (this.Format != "png")
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string"));
            }

            if (!querystrings.ContainsKey("style") || !querystrings.ContainsKey("zoom") || !querystrings.ContainsKey("x") || !querystrings.ContainsKey("y"))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, "style||zoom||x|y invalid"));
            }

            string xParamVal = querystrings["x"]?.FirstOrDefault();
            string yParamVal = querystrings["y"]?.FirstOrDefault();
            string zoomParamVal = querystrings["zoom"]?.FirstOrDefault();
            var flowTileValidator = new TileValidator(INCLUSIVE_MIN_ZOOM_INDEX, INCLUSIVE_MAX_ZOOM_INDEX);
            if (this.Format == "png" && !flowTileValidator.IsValidTile(xParamVal, yParamVal, zoomParamVal))
            {
                throw (new ProxyException(ProxyException.ExceptionType.ClientParams, $"(x:{xParamVal}, y:{yParamVal}, z:{zoomParamVal}) is not a valid tile."));
            }
        }

        internal override void ClientOverrideRequest(string query, ref Dictionary<string, List<string>> querystrings)
        {
        }
    }
}