﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation.Loggers
{
    using Microsoft.Azure.LocationServices.Instrumentation.Models;
    using Microsoft.Cloud.InstrumentationFramework;

    /// <summary>
    /// Logger for Failure metric in Geneva
    /// </summary>
    internal class FailureMetricLogger : AbstractMetricLogger
    {
        private static readonly string[] Dimension =
            {
                "ResourceId",
                "OperationName",
                "Service Category",
                "Instance Location",
                "OperationSourceName",
                "ResultDetails",
                "ExternalResultCode",
                "OperationResultCode"
            };

        public FailureMetricLogger(MetricHeader header, string instanceLocation)
            : base(header, Dimension, instanceLocation)
        {
        }

        /// <inheritdoc />
        protected override string[] ExtractDimensionValues(OperationDetails operationDetails)
        {
            return new string[]
            {
                operationDetails.ResourceId,
                operationDetails.OperationName,
                OperationDetailsHelper.GetServiceCategory(operationDetails),
                this.InstanceLocation,
                operationDetails.OperationSourceName,
                operationDetails.OperationResultDetails,
                operationDetails.ExternalResultCode,
                operationDetails.OperationResultCode
            };
        }

        /// <inheritdoc />
        protected override bool ShouldLogMetric(OperationDetails operationDetails)
        {
            return operationDetails.OperationResult == OperationResult.Failure;
        }
    }
}
