﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation.Loggers
{
    using Microsoft.Azure.LocationServices.Instrumentation.Models;

    /// <summary>
    /// Logger for Latency metric in Geneva
    /// </summary>
    internal class LatencyMetricLogger : AbstractMetricLogger
    {
        private static readonly string[] Dimension =
            {
                "ResourceId",
                "OperationName",
                "OperationResult",
                "Service Category",
                "Instance Location",
                "OperationSourceName",
                "ResultDetails",
                "ExternalResultCode",
                "OperationResultCode"
            };

        public LatencyMetricLogger(MetricHeader header, string instanceLocation)
            : base(header, Dimension, instanceLocation)
        {
        }

        /// <inheritdoc />
        protected override string[] ExtractDimensionValues(OperationDetails operationDetails)
        {
            return new string[]
            {
                operationDetails.ResourceId,
                operationDetails.OperationName,
                operationDetails.OperationResult.ToString(),
                OperationDetailsHelper.GetServiceCategory(operationDetails),
                this.InstanceLocation,
                operationDetails.OperationSourceName,
                operationDetails.OperationResultDetails,
                operationDetails.ExternalResultCode,
                operationDetails.OperationResultCode
            };
        }
    }
}
