﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.Instrumentation
{
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.Azure.LocationServices.Instrumentation.Loggers;
    using Microsoft.Azure.LocationServices.Instrumentation.Models;

    public class Metrics
    {
        /// <summary>
        /// Logs end-to-end customer facing latency metrics to Shoebox
        /// </summary>
        public readonly IMetricLogger ShoeboxLatencyLogger;

        /// <summary>
        /// Logs end-to-end customer facing failure metrics to Shoebox
        /// </summary>
        public readonly IMetricLogger ShoeboxFailureLogger;

        /// <summary>
        /// Logs latency metric for external dependency
        /// </summary>
        public readonly IMetricLogger ExternalLatencyLogger;

        public Metrics(IBootstrapConfiguration bootstrapConfiguration)
        {
            var shoeboxLatencyHeaders = new MetricHeader(
                bootstrapConfiguration.GetValue("shoeboxMonitoringAccount"),
                "ServiceOperations",
                "Latency");

            var shoeboxFailureyHeaders = new MetricHeader(
                bootstrapConfiguration.GetValue("shoeboxMonitoringAccount"),
                "ServiceOperations",
                "Failure");

            var externalLatencyHeaders = new MetricHeader(
                bootstrapConfiguration.GetValue("externalMonitoringAccount"),
                "Internal QoS",
                "External call latency");

            var instanceRegion = bootstrapConfiguration.GetValue("instanceRegion");

            this.ShoeboxLatencyLogger = new LatencyMetricLogger(shoeboxLatencyHeaders, instanceRegion);
            this.ShoeboxFailureLogger = new FailureMetricLogger(shoeboxFailureyHeaders, instanceRegion);
            this.ExternalLatencyLogger = new LatencyMetricLogger(externalLatencyHeaders, instanceRegion);
        }
    }
}