﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation.Models
{
    using Microsoft.Cloud.InstrumentationFramework;

    public abstract class AbstractMetricLogger : IMetricLogger
    {
        private readonly MeasureMetric metric;

        protected AbstractMetricLogger(MetricHeader header, string[] dimension, string instanceLocation)
        {
            var mdmErrorContext = default(ErrorContext);
            this.metric = header.CreateMeasureMetric(mdmErrorContext, dimension);
            this.InstanceLocation = instanceLocation;

            if (this.metric == null)
            {
                Agent.Instance.LogEvent(
                    Instrumentation.EventType.ProgramError,
                    $"Failed to create metric: {header.MetricName}, error code: {mdmErrorContext.ErrorCode}, message: {mdmErrorContext.ErrorMessage} using account: {header.MonitoringAccount} namespace: {header.MetricNamespace}");
            }
        }

        /// <summary>
        /// Instance Location/Region corresponding to this.metric
        /// </summary>
        public string InstanceLocation { get; }

        /// <inheritdoc />
        public void LogDimensionValues(OperationDetails operationDetails, int duration)
        {
            if (!this.ShouldLogMetric(operationDetails) || this.metric == null)
            {
                return;
            }

            this.metric.LogValue(duration, this.ExtractDimensionValues(operationDetails));
        }

        /// <summary>
        /// Parse OperationDetails and return a string[] of dimension values
        /// </summary>
        /// <param name="operationDetails">Operation details containing information on dimension values</param>
        /// <returns>Dimension values for logging to Geneva</returns>
        protected abstract string[] ExtractDimensionValues(OperationDetails operationDetails);

        /// <summary>
        /// Whether to log to Geneva or not based on OperationDetails
        /// </summary>
        /// <param name="operationDetails">>Operation details containing information on dimension values</param>
        /// <returns>Whether to log to Geneva or not</returns>
        protected virtual bool ShouldLogMetric(OperationDetails operationDetails)
        {
            return true;
        }
    }
}