﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation.Models
{
    using Microsoft.Cloud.InstrumentationFramework;

    /// <summary>
    /// Holds information for constructing a MeasureMetric
    /// </summary>
    public class MetricHeader
    {
        /// <summary>
        /// Constructor for MetricHeader
        /// </summary>
        /// <param name="monitoringAccount">Monitoring account name for Geneva</param>
        /// <param name="metricNamespace">Namespace associated with the monitoring account</param>
        /// <param name="metricName">Metric name associated with the namespace</param>
        public MetricHeader(string monitoringAccount, string metricNamespace, string metricName)
        {
            this.MonitoringAccount = monitoringAccount;
            this.MetricNamespace = metricNamespace;
            this.MetricName = metricName;
        }

        /// <summary>
        /// The name of the MDM account
        /// </summary>
        public string MonitoringAccount { get; }

        /// <summary>
        /// The name of the MDM namespace
        /// </summary>
        public string MetricNamespace { get; }

        /// <summary>
        /// The name of the MDM metric name
        /// </summary>
        public string MetricName { get; }

        /// <summary>
        /// Construct a Microsoft.Cloud.InstrumentationFramework.MeasureMetric 
        /// </summary>
        /// <param name="mdmErrorContext">ErrorContext - Has ErrorCode and ErrorMessage properties</param>
        /// <param name="listDimensionNames">Dimension names for a metric</param>
        /// <returns></returns>
        public MeasureMetric CreateMeasureMetric(ErrorContext mdmErrorContext, params string[] listDimensionNames)
        {
            return MeasureMetric.Create(this.MonitoringAccount, this.MetricNamespace, this.MetricName, ref mdmErrorContext, listDimensionNames);
        }
    }
}