﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation.Models
{
    using Microsoft.Cloud.InstrumentationFramework;

    /// <summary>
    /// Operation details for user API calls
    /// </summary>
    public struct OperationDetails
    {
        /// <summary>
        /// Resource ID
        /// Ex. /subscriptions/{subscription id}/resourceGroups/{resource group}/providers/Microsoft.Maps/accounts/{account name}
        /// </summary>
        public string ResourceId;

        /// <summary>
        /// Operation Name or API Name
        /// Ex. MapTile.GetMapTile
        /// </summary>
        public string OperationName;

        /// <summary>
        /// Operation Result
        /// Ex. Success
        /// </summary>
        public OperationResult OperationResult;

        /// <summary>
        /// Proxy Type or Service/API Category
        /// Ex. Route
        /// </summary>
        public BaseProxy.ProxyType ProxyType;

        /// <summary>
        /// Operation Source Name
        /// </summary>
        public string OperationSourceName;

        /// <summary>
        /// Operation Result Details
        /// </summary>
        public string OperationResultDetails;

        /// <summary>
        /// External Result Code
        /// HTTP Code from TomTom, etc. to Proxy
        /// Ex. 200
        /// </summary>
        public string ExternalResultCode;

        /// <summary>
        /// Operation Result Code
        /// HTTP Code for API Management from Proxy
        /// Ex. 200
        /// </summary>
        public string OperationResultCode;
    }
}