﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation
{
    using Microsoft.Azure.LocationServices.Instrumentation.Models;

    public static class OperationDetailsHelper
    {
        /// <summary>
        /// Gets the operation result type from operation details
        /// </summary>
        public static string GetResultType(OperationDetails operationDetails)
        {
            int.TryParse(operationDetails.OperationResultCode, out var statusCode);

            if (statusCode >= 200 && statusCode < 400)
            {
                return "Success";
            }
            else if (statusCode >= 400 && statusCode < 500)
            {
                return "Client Error";
            }

             return "Server Error";
        }

        /// <summary>
        /// Gets the service category (matching the API docs) from operation details
        /// </summary>
        public static string GetServiceCategory(OperationDetails operationDetails)
        {
            switch (operationDetails.ProxyType)
            {
                case BaseProxy.ProxyType.Map:
                    return "Render";
                default:
                    return operationDetails.ProxyType.ToString();
            }
        }
    }
}
