﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Models
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models.Common.Batch;
    using Microsoft.Azure.LocationServices.Shared;
    using Newtonsoft.Json;

    public class MicrosoftV1BatchProxy : BaseProxyBatch, IBatchCustomSerializationProvider
    {
        // Generate a unique id for the Batch once lazily.
        private readonly Lazy<string> batchIdLazy = new Lazy<string>(() => Guid.NewGuid().ToString());

        public MicrosoftV1BatchProxy(IProxyOperation proxyOperation)
            : base(proxyOperation)
        {
        }

        // Batch Id for the current batch.
        private string BatchId => this.batchIdLazy.Value;

        public override async Task<HttpResponseMessage> BatchProxyPost(Uri requestUri, string requestBody, string accountId)
        {
            return
                await BatchApiProxyHelper.ProcessPostBatchApiRequest(
                    Globals.Instance.BlobService,
                    Globals.Instance.QueueService,
                    requestUri,
                    requestBody,
                    BatchId,
                    SharedUtilities.BatchApiContainerName, 
                    accountId);
        }

        public override async Task<HttpResponseMessage> BatchProxyGet(Uri requestUri, string batchId)
        {
            return
                await BatchApiProxyHelper.ProcessGetBatchApiRequest(
                    Globals.Instance.BlobService,
                    requestUri,
                    batchId,
                    SharedUtilities.BatchApiContainerName);
        }

        /// <summary>
        /// Serializes the input query list to a custom string object.
        /// </summary>
        /// <param name="requestUri"> The request Uri. </param>
        /// <param name="queryList"></param>
        /// <returns>A custom serialized string</returns>
        public string SerializeBatchQueryList(Uri requestUri, IList<string> queryList)
        {
            try
            {
                return JsonConvert.SerializeObject(BatchApiProxyHelper.ConvertRelativeQueriesInBatchToAbsoluteUris(requestUri, queryList, BatchId), new JsonConverter[0]);
            }
            catch (Exception ex) when (ex is JsonSerializationException || ex is ArgumentException)
            {
                throw new ProxyException(ProxyException.ExceptionType.ClientParams, "Serialization to Json failed", ex);
            }
        }
    }
}