﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.Azure.LocationServices.Models.GeoLocation;
    using Microsoft.Azure.LocationServices.Models.Map;

    public class Versioning : IVersioning
    {
        public BaseProxy GetProxyObj(double version, BaseProxy.ProxyType proxyType, IProxyOperation proxyOperation)
        {
            proxyOperation.ProxyType = proxyType;

            switch (proxyType)
            {
                case BaseProxy.ProxyType.Search:
                    {
                        if (1.0 == version)
                        {
                            return new TomTomV2SearchProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Map:
                    {
                        if (1.0 == version)
                        {
                            return new TomTomV1MapProxy(proxyOperation);
                        }

                        if (2.0 == version && Globals.Instance.ProxySettings.MdpApiEnabled)
                        {
                            return new MicrosoftV2MapProxy(proxyOperation, Globals.Instance.ProxySettings);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Imagery:
                    {
                        if (1.0 == version)
                        {
                            return new BingV1MapProxy(proxyOperation);
                        }
                       
                        break;
                    }

                case BaseProxy.ProxyType.Timezone:
                    {
                        if (1.0 == version)
                        {
                            return new MicrosoftV1TimezoneProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Route:
                    {
                        if (1.0 == version)
                        {
                            return new TomTomV1RouteProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Traffic:
                    {
                        if (1.0 == version)
                        {
                            return new TomTomV4TrafficProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Heartbeat:
                    {
                        if (1.0 == version)
                        {
                            return new HeartbeatProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.GeoLocation:
                    {
                        if (version == 1.0)
                        {
                            return new MicrosoftV1GeoLocationProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Geofence:
                    {
                        if (1.0 == version)
                        {
                            return new MicrosoftV1GeofenceProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.UserData:
                    {
                        if (1.0 == version)
                        {
                            return new MicrosoftV1UploadProxy(proxyOperation);
                        }

                        break;
                    }

                case BaseProxy.ProxyType.Batch:
                    {
                        if (1.0 == version)
                        {
                            return new MicrosoftV1BatchProxy(proxyOperation);
                        }

                        break;
                    }
            }

            ProxyException ex = new ProxyException(ProxyException.ExceptionType.ClientParams, "client passed in unsupported version number");
            throw (ex);
        }
    }
}