﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.Models.GeoLocation
{
    using System;
    using System.Collections.Generic;
    using System.Diagnostics;
    using System.Net;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Threading.Tasks;

    using Microsoft.Azure.LocationServices.GeoLocation;
    using Microsoft.Azure.LocationServices.Instrumentation;

    using Newtonsoft.Json;

    /// <summary>
    /// Implements base workitem for geo location actions.
    /// </summary>
    public abstract class BaseGeoLocationProxy : BaseProxy
    {
        protected static readonly HttpResponseMessage BadRequest = new HttpResponseMessage(HttpStatusCode.BadRequest);
        private static readonly JsonSerializerSettings JsonSerializeSettings = new JsonSerializerSettings() { NullValueHandling = NullValueHandling.Ignore };

        /// <summary>
        /// Creates new instance of geo location workitem.
        /// </summary>
        protected BaseGeoLocationProxy(IProxyOperation proxyOperation) : base(proxyOperation)
        {
        }

        /// <summary>
        /// Defines geo location request types.
        /// </summary>
        public enum GeoLocationType
        {
            ByIP
        }

        /// <summary>
        /// Creates an action specific request handler.
        /// </summary>
        public abstract Task<HttpResponseMessage> ExecuteRequest(
            IIPLookup ipLookupService,
            GeoLocationType locationType,
            IDictionary<string, List<string>> querystrings,
            string clientIpAddress,
            string formatstring);

        internal override void ClientValidateReponse(ref HttpResponseMessage rsp)
        {
        }

        protected static HttpResponseMessage CreateResponse<T>(T result)
        {
            return CreateResponse(
                HttpStatusCode.OK,
                JsonConvert.SerializeObject(result, Formatting.Indented, JsonSerializeSettings),
                new MediaTypeHeaderValue("application/json"));
        }

        protected static HttpResponseMessage CreateResponse(HttpStatusCode statusCode, string message, MediaTypeHeaderValue mediaType = null)
        {
            var response = new HttpResponseMessage(statusCode) { Content = new StringContent(message) };
            response.Content.Headers.ContentType = mediaType;
            return response;
        }

        protected Task<HttpResponseMessage> InstrumentOperation(Func<HttpResponseMessage> action)
        {
            return this.InstrumentOperation(() => Task.FromResult(action()));
        }

        protected Task<HttpResponseMessage> InstrumentOperation(Func<Task<HttpResponseMessage>> action)
        {
            var sw = Stopwatch.StartNew();
            try
            {
                return action();
            }
            finally
            {
                this.Log_EXT_ms((int)sw.ElapsedMilliseconds);
            }
        }
    }
}
