﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.Models.GeoLocation
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net;
    using System.Net.Http;
    using System.Threading.Tasks;

    using Microsoft.Azure.LocationServices.GeoLocation;
    using Microsoft.Azure.LocationServices.Instrumentation;

    /// <summary>
    /// Implements V1 api for geo location
    /// </summary>
    public class MicrosoftV1GeoLocationProxy : BaseGeoLocationProxy
    {
        public MicrosoftV1GeoLocationProxy(IProxyOperation proxyOperation) : base(proxyOperation)
        {
        }

        public override Task<HttpResponseMessage> ExecuteRequest(
            IIPLookup ipLookupService,
            GeoLocationType locationType,
            IDictionary<string, List<string>> queryParams,
            string clientIpAddress,
            string outputFormat)
        {
            return base.InstrumentOperation(
                       delegate
                           {
                               switch (locationType)
                               {
                                   case (GeoLocationType.ByIP):
                                       return this.FindLocationByIP(queryParams, clientIpAddress, ipLookupService);

                                   default:
                                       return BadRequest;
                               }
                           });
        }

        private static bool TryCreateFromRequestParams(
            IDictionary<string, List<string>> querystrings,
            string clientIpAddress,
            out IPAddress address,
            out string errorDetails)
        {
            const string IPAddressParameterName = "ip";
            const string UseClientAddressParameterName = "useClientAddress";

            if (querystrings.ContainsKey(IPAddressParameterName))
            {
                if (!IPAddress.TryParse(querystrings[IPAddressParameterName].First(), out address))
                {
                    errorDetails = "Incorrect IP address specified.";
                    return false;
                }
            }
            else
            {
                if (querystrings.ContainsKey(UseClientAddressParameterName))
                {
                    if (!IPAddress.TryParse(clientIpAddress, out address))
                    {
                        errorDetails = "Client IP address is not available.";
                        return false;
                    }
                }
                else
                {
                    errorDetails = "IP address is not specified.";
                    address = null;
                    return false;
                }
            }

            errorDetails = string.Empty;
            return true;
        }

        private HttpResponseMessage FindLocationByIP(
            IDictionary<string, List<string>> queryParams,
            string clientIpAddress,
            IIPLookup ipLookupService)
        {
            try
            {
                if (!TryCreateFromRequestParams(queryParams, clientIpAddress, out var address, out var errorDetails))
                {
                    throw new ProxyException(ProxyException.ExceptionType.ClientParams, errorDetails);
                }

                var result = ipLookupService.GetNetworkAssignment(address);
                return CreateResponse(result);
            }
            catch (ProxyException)
            {
                throw;
            }
            catch (Exception ex)
            {
                throw new ProxyException(ProxyException.ExceptionType.ServerError, "An error occurred while processing the request. Please try again later.", ex);
            }
        }
    }
}