﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;

    public class BingV1MapProxy : BaseProxyMapImagery
    {
        public BingV1MapProxy(IProxyOperation proxyOperation) : base(proxyOperation)
        {
        }

        public override async Task<HttpResponseMessage> SatelliteImagery(string x, string y, string zoom)
        {
            string quadkey = TileXYToQuadKey(Convert.ToInt32(x), Convert.ToInt32(y), Convert.ToInt32(zoom));
            var imageUrl = Globals.Instance.BingMapImgUrl;
            imageUrl = imageUrl.Replace("{subdomain}", "t0").Replace("{quadkey}", quadkey);
            return await this.ExecuteRequest(new HttpRequestMessage(HttpMethod.Get, imageUrl));
        }

        // Bing Maps Tile system converts tile coordinates to quadkey
        // see https://msdn.microsoft.com/en-us/library/bb259689.aspx
        private static string TileXYToQuadKey(int tileX, int tileY, int zoomLevel)
        {
            System.Text.StringBuilder quadKey = new System.Text.StringBuilder();
            for (int i = zoomLevel; i > 0; i--)
            {
                char digit = '0';
                int mask = 1 << (i - 1);
                if ((tileX & mask) != 0)
                {
                    digit++;
                }

                if ((tileY & mask) != 0)
                {
                    digit++;
                    digit++;
                }

                quadKey.Append(digit);
            }

            return quadKey.ToString();
        }
    }
}