﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;

    public abstract class BaseProxyMap : BaseProxy
    {
        public BaseProxyMap(IProxyOperation proxyOperation)
            : base(proxyOperation)
        {
        }

        public enum MapType
        {
            RasterTile,
            VectorTile,
            StaticTile,
            WorldCopyright,
            TileCopyright,
            BoundingCopyright,
            CaptionCopyright
        }

        public abstract Task<HttpResponseMessage> MapTileVector(
            string layer,
            string style,
            string zoom,
            string x,
            string y,
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> MapTileRaster( 
            string layer, 
            string style, 
            string zoom, 
            string x, 
            string y, 
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> MapStatic(string formatstring, Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> MapCopyright(
            MapType type,
            string formatstring,
            Dictionary<string, List<string>> querystrings,
            string zoom = "",
            string x = "",
            string y = "");

        internal override void ClientValidateReponse(ref HttpResponseMessage response)
        {
        }
    }
}