﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Text;
    using System.Threading.Tasks;
    using System.Xml.Linq;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Newtonsoft.Json.Linq;
    using Utility;

    public abstract class BaseProxyRoute : BaseProxy
    {
        protected ResponseFormat Format;
        private static readonly HashSet<string> BlackListValues = new HashSet<string>(StringComparer.OrdinalIgnoreCase) { "countryCode", "stateCode" };

        public BaseProxyRoute(IProxyOperation proxyOperation)
            : base(proxyOperation)
        {
        }

        public enum RouteType
        {
            Normal,
            Batch,
            Reconstruct,
            Matrix,
            ReachableRange
        }

        public abstract Task<HttpResponseMessage> RouteProxy(
            RouteType searchType,
            string querystring,
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> RouteProxy(
            RouteType searchType,
            string querystring,
            string formatstring,
            Dictionary<string, List<string>> querystrings,
            string requestBody);

        public abstract Task<HttpResponseMessage> RouteMatrixProxyPost(
            Uri requestUri,
            string formatstring,
            Dictionary<string, List<string>> querystrings,
            string requestBody,
            string accountId,
            string trackingId,
            bool forceAsync);

        public abstract Task<HttpResponseMessage> RouteMatrixProxyGet(Uri requestUri, string matrixRequestId);

        internal override void ClientValidateReponse(ref HttpResponseMessage response)
        {
            var responseString = response.Content.ReadAsStringAsync().Result;
            base.ResponseOverride.Override(ref response);
            switch (this.Format)
            {
                case ResponseFormat.JSON:
                    var node = JToken.Parse(responseString);
                    JsonUtility.FindNodes(node, BlackListValues).ForEach(x => x.Remove());
                    response.Content = new StringContent(node.ToString(Newtonsoft.Json.Formatting.None, null), Encoding.UTF8, "application/json");

                    break;
                case ResponseFormat.XML:
                    var doc = XDocument.Parse(responseString);
                    doc.Descendants().Where(x => BlackListValues.Contains(x.Name.LocalName)).ToList().ForEach(x => x.Remove());
                    response.Content = new StringContent(doc.ToString(SaveOptions.DisableFormatting), Encoding.UTF8, "application/xml");
                    break;
                default:
                    break;
            }
        }
    }
}