﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Collections.Generic;
    using GeoJSON.Net;

    /// <summary>
    /// This Interface defines the serialization methods required to convert a valid GeoJSON format to a custom format.
    /// Any service that accepts a custom format should implement this interface and provide the custom serialization logic 
    /// as part of the implementation. 
    /// </summary>
    public interface ISearchCustomSerializationProvider
    {
        /// <summary>
        /// Serializes the input GeoJson geometry object to a custom string object.
        /// </summary>
        /// <param name="searchType">Type of search</param>
        /// <param name="geoJsonGeometry">GeoJSON geometry object</param>
        /// <returns>A custom serialized string</returns>
        string SerializeGeoJsonGeometry(BaseProxySearch.SearchType searchType, IGeoJSONObject geoJsonGeometry);
    }
}
