﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;

    public abstract class BaseProxyTraffic : BaseProxy
    {
        public BaseProxyTraffic(IProxyOperation proxyOperation) : base(proxyOperation)
        {
        }

        public enum TrafficType
        {
            FlowTile,
            FlowSegment,
            IncidentTile,
            IncidentDetails,
            IncidentViewport
        }

        public abstract Task<HttpResponseMessage> TrafficFlow( 
            TrafficType type,
            string style, 
            string zoom, 
            string x, 
            string y, 
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> IncidentTile(
            TrafficType type,
            string style,
            string zoom,
            string x,
            string y,
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> IncidentViewport(
            TrafficType type,
            string boundingbox,
            string boundingzoom,
            string overviewbox,
            string overviewzoom,
            string copyright,
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        public abstract Task<HttpResponseMessage> IncidentDetails(
            TrafficType type,
            string style,
            string boundingbox,
            string zoom,
            string trafficmodelid,
            string formatstring,
            Dictionary<string, List<string>> querystrings);

        internal override void ClientValidateReponse(ref HttpResponseMessage response)
        {
        }
    }
}