﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Models
{
    using System.IO;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;

    /// <summary>
    /// Base class for upload proxy
    /// </summary>
    public abstract class BaseProxyUpload : BaseProxy
    {
        /// <summary>
        /// Creates a new instance of upload proxy.
        /// </summary>
        protected BaseProxyUpload(IProxyOperation proxyOperation) : base(proxyOperation)
        {
        }

        /// <summary>
        /// Defines all the valid upload data formats.
        /// </summary>
        public enum UploadDataFormat
        {
            // Default
            Undefined = 0,

            // Shapefile GIS data
            Shapefile,

            // Keyhole Markup data
            Kml,

            // GeoJSON data
            Geojson
        }

        /// <summary>
        /// Defines all the valid upload types.
        /// </summary>
        public enum UploadType
        {
            // Upload to Blob storage
            Blob = 0
        }

        /// <summary>
        /// Uploads 'data' of type 'format' to destination of type 'uploadType'.
        /// </summary>
        public abstract Task<HttpResponseMessage> UploadData(string accountId, Stream data, UploadDataFormat format, UploadType uploadType);

        /// <summary>
        /// Updates 'data' with id equal to 'udid' and of type 'format' using the passed in data. Old file will be overwritten with new data.
        /// </summary>
        public abstract Task<HttpResponseMessage> UpdateData(string accountId, string udid, Stream data, UploadType uploadType);

        /// <summary>
        /// Deletes the file with the passed-in 'udid' from the user's storage.
        /// </summary>
        public abstract Task<HttpResponseMessage> DeleteData(string accountId, string udid, UploadType uploadType);

        /// <summary>
        /// Returns a list of all the 'data' (files) uploaded by the user.
        /// </summary>
        public abstract Task<HttpResponseMessage> ListData(string accountId, UploadType uploadType);

        internal override void ClientValidateReponse(ref HttpResponseMessage rsp)
        {
        }
    }
}
