﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Diagnostics;
    using System.Threading;

    using Microsoft.Azure.LocationServices.Utility;
    using Microsoft.ServiceFabric.Services.Runtime;

    internal static class Program
    {
        /// <summary>
        ///     This is the entry point of the service host process.
        /// </summary>
        private static void Main()
        {
            ProxyHostSettings proxyHost = ProxyHostSettings.CreateFromEnvironment();
            Instrumentation.Agent.CreateInstance(proxyHost.MdmParameters);

            try
            {
                // The ServiceManifest.XML file defines one or more service type names.
                // Registering a service maps a service type name to a .NET type.
                // When Service Fabric creates an instance of this service type,
                // an instance of the class is created in this host process.
                ServiceRuntime.RegisterServiceAsync("AzureLBSType", context => new AzureLBS(context))
                               .GetAwaiter()
                               .GetResult();

                Instrumentation.Agent.Instance.LogServiceTypeRegistered(Process.GetCurrentProcess().Id, typeof(AzureLBS).Name);

                // Prevents this host process from terminating so services keeps running. 
                Thread.Sleep(Timeout.Infinite);
            }
            catch (Exception ex)
            {
                Instrumentation.Agent.Instance.LogException(ex, "proxy", "initialization");
                throw;
            }
        }
    }
}