﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System;

    [Serializable]
    public sealed class ProxyException : Exception
    {
        public ProxyException(ExceptionType type, string userMessage)
            : base(userMessage)
        {
            this.Type = type;
        }

        /// <summary>
        /// ctor if wrapping an exception.   Note that innerException.Message will be returned to the user
        /// </summary>
        /// <param name="type"></param>
        /// <param name="userMessage"></param>
        /// <param name="innerException"></param>
        public ProxyException(ExceptionType type, string userMessage, Exception innerException)
            : base(userMessage, innerException)
        {
            this.Type = type;
        }

        public enum ExceptionType
        {
            Unauthorized,
            ClientParams,
            ServerError,
            ExtError,
            Throttled,
            Other
        }

        public ExceptionType Type { get; set; }

        public override void GetObjectData(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context)
        {
            if (info == null)
            {
                throw new ArgumentNullException("info");
            }

            info.AddValue("Type", Type);
            base.GetObjectData(info, context);
        }
    }
}