﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Utility
{
    using System;
    using System.Collections.Generic;
    using Newtonsoft.Json.Linq;

    public static class JsonUtility
    {
        /// <summary>
        /// Resursively goes through and returns all node that match node name.
        /// </summary>
        /// <param name="node">Root node.</param>
        /// <param name="findValue">Name of node to search for.</param>
        /// <returns>List of all nodes that matched.</returns>
        public static List<JToken> FindNodes(JToken node, string findValue)
        {
            return FindNodes(node, new[] { findValue }); 
        }

        /// <summary>
        /// Resursively goes through and returns all node that match node name.
        /// </summary>
        /// <param name="node">Root node.</param>
        /// <param name="findValues">Collection of name of nodes to search for.</param>
        /// <returns>List of all nodes that matched.</returns>
        public static List<JToken> FindNodes(JToken node, IEnumerable<string> findValues)
        {
            List<JToken> result = new List<JToken>();
            var findValuesDict = new HashSet<string>(findValues, StringComparer.OrdinalIgnoreCase);
            FindNodes(node, result, findValuesDict);
            return result; 
        }
        
        private static void FindNodes(JToken node, List<JToken> findResults, HashSet<string> findValues)
        {
            if (node.Type == JTokenType.Object)
            {
                foreach (JProperty child in node.Children<JProperty>())
                {
                    if (findValues.Contains(child.Name))
                    {
                        findResults.Add(child);
                    }
                    else
                    {
                        FindNodes(child.Value, findResults, findValues);
                    }
                }
            }
            else if (node.Type == JTokenType.Array)
            {
                foreach (JToken child in node.Children())
                {
                    FindNodes(child, findResults, findValues);
                }
            }
        }
    }
}
