﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Utility
{
    using System;

    class TileValidator
    {
        private readonly int inclusiveMinZoom;
        private readonly int inclusiveMaxZoom;

        public TileValidator(int inclusiveMinZoom, int inclusiveMaxZoom)
        {
            this.inclusiveMinZoom = inclusiveMinZoom;
            this.inclusiveMaxZoom = inclusiveMaxZoom;
        }

        public bool IsZoomValid(int zoom) => (zoom >= this.inclusiveMinZoom) && (zoom <= this.inclusiveMaxZoom);

        public bool IsZoomValid(string zoomStr) => Int32.TryParse(zoomStr, out int zoom) && this.IsZoomValid(zoom);

        public bool IsValidTile(int x, int y, int zoom)
        {
            if (!this.IsZoomValid(zoom))
            {
                return false;
            }

            double inclusiveMaxTileIndex = Math.Pow(2, zoom) - 1;

            if (x < 0 || x > inclusiveMaxTileIndex)
            {
                return false;
            }

            if (y < 0 || y > inclusiveMaxTileIndex)
            {
                return false;
            }

            return true;
        }

        public bool IsValidTile(string xStr, string yStr, string zoomStr)
        {
            if (!Int32.TryParse(xStr, out int x))
            {
                return false;
            }

            if (!Int32.TryParse(yStr, out int y))
            {
                return false;
            }

            if (!Int32.TryParse(zoomStr, out int zoom))
            {
                return false;
            }

            return this.IsValidTile(x, y, zoom);
        }
    }
}
