﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Batch
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class RouteDirectionsBatchIntegrationTests : ApimTestBase
    {
        [Ignore]
        [TestMethod, TestCategory("L3")]
        public async Task PostRouteDirectionsBatch_ValidRequest_Accepted()
        {
            var batchRequestBody =
                new BatchRequestBody(
                    new List<string>
                    {
                        "?query=47.639987,-122.128384:47.621252,-122.184408:47.596437,-122.332000&routeType=fastest&travelMode=car"
                    });

            var httpContent = new StringContent(JObject.FromObject(batchRequestBody).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            HttpResponseMessage routeDirectionsBatchResult = await this.MapsClient.PostRouteDirectionsBatch(Constants.Json, httpContent).ConfigureAwait(false);

            // Validate that the request was accepted.
            Assert.AreEqual(routeDirectionsBatchResult.StatusCode, HttpStatusCode.Accepted);

            // Validate that the location header was present.
            Assert.IsNotNull(routeDirectionsBatchResult.Headers.Location);
        }

        [Ignore]
        [TestMethod, TestCategory("L3")]
        public async Task PostRouteDirectionsBatch_InvalidQueryCount_BadRequest()
        {
            // Batch request must have at least 1 query.
            var batchRequestBody = new BatchRequestBody(new List<string>());

            var httpContent = new StringContent(JObject.FromObject(batchRequestBody).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            HttpResponseMessage routeDirectionsBatchResult = await this.MapsClient.PostRouteDirectionsBatch(Constants.Json, httpContent).ConfigureAwait(false);

            // Validate the request failed.
            await EnsureErrorStatusCode(routeDirectionsBatchResult, HttpStatusCode.BadRequest);
        }

        [Ignore]
        [TestMethod, TestCategory("L3")]
        public async Task PostRouteDirectionsBatch_InvalidFormat_BadRequest()
        {
            // Json is the only valid format for batch.
            var batchRequestBody =
                new BatchRequestBody(
                    new List<string>
                    {
                        "?query=47.639987,-122.128384:47.621252,-122.184408:47.596437,-122.332000&routeType=fastest&travelMode=car"
                    });

            var httpContent = new StringContent(JObject.FromObject(batchRequestBody).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            HttpResponseMessage routeDirectionsBatchResult = await this.MapsClient.PostRouteDirectionsBatch(Constants.Xml, httpContent).ConfigureAwait(false);

            // Validate the request failed.
            await EnsureErrorStatusCode(routeDirectionsBatchResult, HttpStatusCode.BadRequest);
        }
    }
}