﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Batch
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class SearchAddressBatchIntegrationTests : ApimTestBase
    {
        [Ignore]
        [TestMethod, TestCategory("L3")]
        public async Task PostSearchAddressBatch_ValidRequest_Accepted()
        {
            var batchRequestBody = 
                new BatchRequestBody(
                    new List<string>
                    {
                        "?query=One, Microsoft Way, Redmond, WA 98052&limit=1"
                    });

            var httpContent = new StringContent(JObject.FromObject(batchRequestBody).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            HttpResponseMessage searchAddressBatchResult = await this.MapsClient.PostSearchAddressBatch(Constants.Json, httpContent).ConfigureAwait(false);

            // Validate that the request was accepted.
            Assert.AreEqual(searchAddressBatchResult.StatusCode, HttpStatusCode.Accepted);

            // Validate that the location header was present.
            Assert.IsNotNull(searchAddressBatchResult.Headers.Location);
        }

        [Ignore]
        [TestMethod, TestCategory("L3")]
        public async Task PostSearchAddressBatch_InvalidQueryCount_BadRequest()
        {
            // Batch request must have at least 1 query.
            var batchRequestBody = new BatchRequestBody(new List<string>());

            var httpContent = new StringContent(JObject.FromObject(batchRequestBody).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            HttpResponseMessage searchAddressBatchResult = await this.MapsClient.PostSearchAddressBatch(Constants.Json, httpContent).ConfigureAwait(false);

            // Validate the request failed.
            await EnsureErrorStatusCode(searchAddressBatchResult, HttpStatusCode.BadRequest);
        }

        [Ignore]
        [TestMethod, TestCategory("L3")]
        public async Task PostSearchAddressBatch_InvalidFormat_BadRequest()
        {
            // Json is the only valid format for batch.
            var batchRequestBody =
                new BatchRequestBody(
                    new List<string>
                    {
                        "?query=One, Microsoft Way, Redmond, WA 98052&limit=1"
                    });

            var httpContent = new StringContent(JObject.FromObject(batchRequestBody).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            HttpResponseMessage searchAddressBatchResult = await this.MapsClient.PostSearchAddressBatch(Constants.Xml, httpContent).ConfigureAwait(false);

            // Validate the request failed.
            await EnsureErrorStatusCode(searchAddressBatchResult, HttpStatusCode.BadRequest);
        }
    }
}