﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web;

    public partial class MapsClient : IDisposable
    {
        private const string ApiVersionKey = "api-version";
        private const string SubscriptionKeyName = "subscription-key";

        private readonly HttpClient _client;
        private readonly string _subscriptionKey;
        private bool _disposed;

        public MapsClient(string baseAddress, string subscriptionKey)
        {
            this._client = new HttpClient { BaseAddress = new Uri(baseAddress) };
            this._subscriptionKey = subscriptionKey;
        }

        #region Utility Methods

        public string AddCommonParameters(string path)
        {
            var qBuilder = new UriBuilder("http://localhost" + path);

            var defaultCollection = HttpUtility.ParseQueryString(qBuilder.Query);
            defaultCollection.Add(ApiVersionKey, Constants.ApiVersion10);
            defaultCollection.Add(SubscriptionKeyName, _subscriptionKey);

            qBuilder.Query = defaultCollection.ToString();
            return qBuilder.Uri.PathAndQuery;
        }

        #endregion

        #region Wrapper Methods for HttpClient

        public void AddHeaders(string name, string value)
        {
            this._client.DefaultRequestHeaders.Add(name, value);
        }

        public async Task<HttpResponseMessage> GetAsync(string requestUrl)
        {
            return await this._client.GetAsync(this.AddCommonParameters(requestUrl)).ConfigureAwait(false);
        }

        public async Task<HttpResponseMessage> PostAsync(string requestUrl, HttpContent requestBody)
        {
            return await this._client.PostAsync(this.AddCommonParameters(requestUrl), requestBody).ConfigureAwait(false);
        }

        #endregion

        #region Dispose

        public void Dispose()
        {
            this.Dispose(true);
        }

        public void Dispose(bool disposing)
        {
            if (_disposed)
            {
                return;
            }

            if (disposing)
            {
                try
                {
                    this._client?.Dispose();
                }
                catch (Exception)
                {
                    // ignored
                }
            }

            _disposed = true;
        }

        #endregion
    }
}
