﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System;
    using System.Collections.Generic;
    using System.Globalization;

    public static class MapsUriBuilder
    {
        public static string GetTimezoneByCoordinatesUrl(
            string query,
            string options = null,
            string timeStamp = null,
            string transitionsFrom = null,
            int? transitionsYears = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("options", options);
            queryParams.AddOptional("timeStamp", timeStamp);
            queryParams.AddOptional("transitionsFrom", transitionsFrom);
            queryParams.AddOptional("transitionsYears", transitionsYears);

            return "timezone/bycoordinates".BuildApimUrl(format, queryParams);
        }

        public static string GetTimezoneByIdUrl(
            string query,
            string options = null,
            string timeStamp = null,
            string transitionsFrom = null,
            int? transitionsYears = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("options", options);
            queryParams.AddOptional("timeStamp", timeStamp);
            queryParams.AddOptional("transitionsFrom", transitionsFrom);
            queryParams.AddOptional("transitionsYears", transitionsYears);

            return "timezone/byid".BuildApimUrl(format, queryParams);
        }

        public static string GetTimezoneIanaUrl(string format = Constants.Json)
        {
            return "timezone/enumiana".BuildApimUrl(format);
        }

        public static string GetTimezoneWindowsUrl(string format = Constants.Json)
        {
            return "timezone/enumwindows".BuildApimUrl(format);
        }

        public static string GetTimezoneIanaVersionUrl(string format = Constants.Json)
        {
            return "timezone/ianaversion".BuildApimUrl(format);
        }

        public static string GetTimezoneWindowsToIanaUrl(
            string query,
            string territory = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("territory", territory);

            return "timezone/windowstoiana".BuildApimUrl(format, queryParams);
        }

        public static string GetTrafficFlowSegmentUrl(
            string style,
            int zoom,
            string query,
            string unit = null,
            int? thickness = null,
            bool? openLr = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "style", style },
                                  { "zoom", zoom.ToString() },
                                  { "query", query },
                              };

            queryParams.AddOptional("unit", unit);
            queryParams.AddOptional("thickness", thickness);
            queryParams.AddOptional("openLr", openLr);

            return "traffic/flow/segment".BuildApimUrl(format, queryParams);
        }

        public static string GetTrafficFlowTileUrl(
            string style,
            int zoom,
            int x,
            int y,
            int? thickness = null,
            string format = Constants.Png)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "style", style },
                                  { "zoom", zoom.ToString() },
                                  { "x", x.ToString() },
                                  { "y", y.ToString() },
                              };

            queryParams.AddOptional("thickness", thickness);

            return "traffic/flow/tile".BuildApimUrl(format, queryParams);
        }

        public static string GetTrafficIncidentTileUrl(
            string style,
            int zoom,
            int x,
            int y,
            string trafficState = null,
            string format = Constants.Png)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "style", style },
                                  { "zoom", zoom.ToString() },
                                  { "x", x.ToString() },
                                  { "y", y.ToString() },
                              };

            queryParams.AddOptional("t", trafficState);

            return "traffic/incident/tile".BuildApimUrl(format, queryParams);
        }

        public static string GetTrafficIncidentViewportUrl(
            string boundingbox,
            int boundingzoom,
            string overviewbox,
            int overviewzoom,
            bool copyright,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "boundingbox", boundingbox },
                                  { "boundingzoom", boundingzoom.ToString() },
                                  { "overviewbox", overviewbox },
                                  { "overviewzoom", overviewzoom.ToString() },
                                  { "copyright", copyright.ToString().ToLower() },
                              };

            return "traffic/incident/viewport".BuildApimUrl(format, queryParams);
        }

        public static string GetTrafficIncidentDetailsUrl(
            string style,
            string boundingbox,
            int boundingZoom,
            string trafficmodelid,
            string language = null,
            string projection = null,
            string geometries = null,
            bool? expandCluster = null,
            bool? originalPosition = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "style", style },
                                  { "boundingbox", boundingbox },
                                  { "boundingZoom", boundingZoom.ToString() },
                                  { "trafficmodelid", trafficmodelid },
                              };

            queryParams.AddOptional("language", language);
            queryParams.AddOptional("projection", projection);
            queryParams.AddOptional("geometries", geometries);
            queryParams.AddOptional("expandCluster", expandCluster);
            queryParams.AddOptional("originalPosition", originalPosition);

            return "traffic/incident/detail".BuildApimUrl(format, queryParams);
        }

        public static string GetCopyRightCaptionUrl(string format = Constants.Json)
        {
            return "map/copyright/caption".BuildApimUrl(format);
        }

        public static string GetCopyRightBoundingUrl(
            string mincoordinates,
            string maxcoordinates,
            string text = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "mincoordinates", mincoordinates },
                                  { "maxcoordinates", maxcoordinates },
                              };

            queryParams.AddOptional("text", text);

            return "map/copyright/bounding".BuildApimUrl(format, queryParams);
        }

        public static string GetCopyrightTileUrl(
            int zoom,
            int x,
            int y,
            string text = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "zoom", zoom.ToString() },
                                  { "x", x.ToString() },
                                  { "y", y.ToString() },
                              };

            queryParams.AddOptional("text", text);

            return "map/copyright/tile".BuildApimUrl(format, queryParams);
        }

        public static string GetCopyrightWorldUrl(string format = Constants.Json)
        {
            return "map/copyright/world".BuildApimUrl(format);
        }

        public static string GetMapTileUrl(
            string layer,
            string style,
            int zoom,
            int x,
            int y,
            int? tileSize = null,
            string language = null,
            string format = Constants.Pbf)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "layer", layer },
                                  { "style", style },
                                  { "zoom", zoom.ToString() },
                                  { "x", x.ToString() },
                                  { "y", y.ToString() },
                              };

            queryParams.AddOptional("tileSize", tileSize);
            queryParams.AddOptional("language", language);

            return "map/tile".BuildApimUrl(format, queryParams);
        }

        public static string GetMapTileStaticUrl(
            string layer = null,
            string style = null,
            int? zoom = null,
            string bbox = null,
            string center = null,
            int? height = null,
            int? width = null,
            string language = null,
            string format = Constants.Png)
        {
            var queryParams = new Dictionary<string, string>();

            queryParams.AddOptional("layer", layer);
            queryParams.AddOptional("style", style);
            queryParams.AddOptional("zoom", zoom);
            queryParams.AddOptional("bbox", bbox);
            queryParams.AddOptional("center", center);
            queryParams.AddOptional("height", height);
            queryParams.AddOptional("width", width);
            queryParams.AddOptional("language", language);

            return "map/static".BuildApimUrl(format, queryParams);
        }

        public static string GetIpLocationUrl(string ip, string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "ip", ip }, };
            return "geolocation/ip".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchAddressUrl(
            string query,
            bool? typeahead = null,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("typeahead", typeahead);
            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("ofs", ofs);
            queryParams.AddOptional("countrySet", countrySet);
            queryParams.AddOptional("lat", lat);
            queryParams.AddOptional("lon", lon);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("topLeft", topLeft);
            queryParams.AddOptional("btmRight", btmRight);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);

            return "search/address".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchAddressStructuredUrl(
            string countryCode,
            int? limit = null,
            int? ofs = null,
            string streetNumber = null,
            string streetName = null,
            string crossStreet = null,
            string municipality = null,
            string municipalitySubdivision = null,
            string countryTertiarySubdivision = null,
            string countrySecondarySubdivision = null,
            string countrySubdivision = null,
            string postalCode = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "countryCode", countryCode }, };

            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("ofs", ofs);
            queryParams.AddOptional("streetNumber", streetNumber);
            queryParams.AddOptional("streetName", streetName);
            queryParams.AddOptional("crossStreet", crossStreet);
            queryParams.AddOptional("municipality", municipality);
            queryParams.AddOptional("municipalitySubdivision", municipalitySubdivision);
            queryParams.AddOptional("countryTertiarySubdivision", countryTertiarySubdivision);
            queryParams.AddOptional("countrySecondarySubdivision", countrySecondarySubdivision);
            queryParams.AddOptional("countrySubdivision", countrySubdivision);
            queryParams.AddOptional("postalCode", postalCode);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);

            return "search/address/structured".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchAddressReverseUrl(
            string query,
            bool? returnSpeedLimit = null,
            double? heading = null,
            double? radius = null,
            string number = null,
            bool? returnRoadUse = null,
            string roadUse = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("returnSpeedLimit", returnSpeedLimit);
            queryParams.AddOptional("heading", heading);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("number", number);
            queryParams.AddOptional("returnRoadUse", returnRoadUse);
            queryParams.AddOptional("roadUse", roadUse);

            return "search/address/reverse".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchAddressReverseCrossStreetUrl(
            string query,
            bool? returnSpeedLimit = null,
            double? heading = null,
            double? radius = null,
            string language = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("returnSpeedLimit", returnSpeedLimit);
            queryParams.AddOptional("heading", heading);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("language", language);

            return "search/address/reverse/crossStreet".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchPoiUrl(
            string query,
            bool? typeahead = null,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("typeahead", typeahead);
            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("ofs", ofs);
            queryParams.AddOptional("countrySet", countrySet);
            queryParams.AddOptional("lat", lat);
            queryParams.AddOptional("lon", lon);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("topLeft", topLeft);
            queryParams.AddOptional("btmRight", btmRight);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);

            return "search/poi".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchPoiCategoryUrl(
            string query,
            bool? typeahead = null,
            int? limit = null,
            int? multiplier = null,
            int? routingTimeout = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("typeahead", typeahead);
            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("multiplier", multiplier);
            queryParams.AddOptional("routingTimeout", routingTimeout);
            queryParams.AddOptional("countrySet", countrySet);
            queryParams.AddOptional("lat", lat);
            queryParams.AddOptional("lon", lon);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("topLeft", topLeft);
            queryParams.AddOptional("btmRight", btmRight);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);

            return "search/poi/category".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchNearbyUrl(
            double lat,
            double lon,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? radius = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "lat", lat.ToString(CultureInfo.InvariantCulture) },
                                  { "lon", lon.ToString(CultureInfo.InvariantCulture) },
                              };

            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("ofs", ofs);
            queryParams.AddOptional("countrySet", countrySet);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);

            return "search/nearby".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchFuzzyUrl(
            string query,
            bool? typehead = null,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            int? minFuzzyLevel = null,
            int? maxFuzzyLevel = null,
            List<string> idxSet = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("typehead", typehead);
            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("ofs", ofs);
            queryParams.AddOptional("countrySet", countrySet);
            queryParams.AddOptional("lat", lat);
            queryParams.AddOptional("lon", lon);
            queryParams.AddOptional("radius", radius);
            queryParams.AddOptional("topLeft", topLeft);
            queryParams.AddOptional("btmRight", btmRight);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);
            queryParams.AddOptional("minFuzzyLevel", minFuzzyLevel);
            queryParams.AddOptional("maxFuzzyLevel", maxFuzzyLevel);
            queryParams.AddOptional("idxSet", idxSet);

            return "search/fuzzy".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchAlongRouteUrl(
            string query,
            string route,
            int maxDetourTime,
            int? limit = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "query", query },
                                  { "route", route },
                                  { "maxDetourTime", maxDetourTime.ToString() },
                              };

            queryParams.AddOptional("limit", limit);

            return "search/alongRoute".BuildApimUrl(format, queryParams);
        }

        public static string PostSearchAlongRouteUrl(
            string query,
            int maxDetourTime,
            int? limit = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "query", query },
                                  { "maxDetourTime", maxDetourTime.ToString() },
                              };

            queryParams.AddOptional("limit", limit);

            return "search/alongRoute".BuildApimUrl(format, queryParams);
        }

        public static string GetSearchGeometryUrl(
            string query,
            string geometry,
            int? limit = null,
            string language = null,
            string extendedPostalCodesFor = null,
            List<string> idxSet = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>()
                              {
                                  { "query", query },
                                  { "geometry", geometry },
                              };

            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);
            queryParams.AddOptional("idxSet", idxSet);

            return "search/geometry".BuildApimUrl(format, queryParams);
        }

        public static string PostSearchGeometryUrl(
            string query,
            int? limit = null,
            string language = null,
            string extendedPostalCodesFor = null,
            List<string> idxSet = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("limit", limit);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("extendedPostalCodesFor", extendedPostalCodesFor);
            queryParams.AddOptional("idxSet", idxSet);

            return "search/geometry".BuildApimUrl(format, queryParams);
        }

        public static string RouteCalculateUrl(
            string query,
            int? maxAlternatives = null,
            string alternativeType = null,
            int? minDeviationDistance = null,
            DateTime? arriveAt = null,
            DateTime? departAt = null,
            int? minDeviationTime = null,
            string instructionsType = null,
            string language = null,
            bool? computeBestOrder = null,
            string routeRepresentation = null,
            string computeTravelTimeFor = null,
            int? vehicleHeading = null,
            string report = null,
            string sectionType = null,
            int? vehicleAxleWeight = null,
            double? vehicleWidth = null,
            double? vehicleHeight = null,
            double? vehicleLength = null,
            int? vehicleMaxSpeed = null,
            int? vehicleWeight = null,
            string windingness = null,
            string hilliness = null,
            string travelMode = null,
            string avoid = null,
            bool? traffic = null,
            string routeType = null,
            string vehicleLoadType = null,
            string vehicleEngineType = null,
            double? constantSpeedConsumptionInLitersPerHundredkm = null,
            double? currentFuelInLiters = null,
            double? auxiliaryPowerInLitersPerHour = null,
            double? fuelEnergyDensityInMJoulesPerLiter = null,
            double? accelerationEfficiency = null,
            double? decelerationEfficiency = null,
            double? uphillEfficiency = null,
            double? downhillEfficiency = null,
            double? constantSpeedConsumptionInkWhPerHundredkm = null,
            string currentChargeInkWh = null,
            string maxChargeInkWh = null,
            string auxiliaryPowerInkW = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string> { { "query", query }, };

            queryParams.AddOptional("maxAlternatives", maxAlternatives);
            queryParams.AddOptional("alternativeType", alternativeType);
            queryParams.AddOptional("minDeviationDistance", minDeviationDistance);
            queryParams.AddOptional("arriveAt", arriveAt);
            queryParams.AddOptional("departAt", departAt);
            queryParams.AddOptional("minDeviationTime", minDeviationTime);
            queryParams.AddOptional("instructionsType", instructionsType);
            queryParams.AddOptional("language", language);
            queryParams.AddOptional("computeBestOrder", computeBestOrder);
            queryParams.AddOptional("routeRepresentation", routeRepresentation);
            queryParams.AddOptional("computeTravelTimeFor", computeTravelTimeFor);
            queryParams.AddOptional("vehicleHeading", vehicleHeading);
            queryParams.AddOptional("report", report);
            queryParams.AddOptional("sectionType", sectionType);
            queryParams.AddOptional("vehicleAxleWeight", vehicleAxleWeight);
            queryParams.AddOptional("vehicleWidth", vehicleWidth);
            queryParams.AddOptional("vehicleHeight", vehicleHeight);
            queryParams.AddOptional("vehicleLength", vehicleLength);
            queryParams.AddOptional("vehicleMaxSpeed", vehicleMaxSpeed);
            queryParams.AddOptional("vehicleWeight", vehicleWeight);
            queryParams.AddOptional("windingness", windingness);
            queryParams.AddOptional("hilliness", hilliness);
            queryParams.AddOptional("travelMode", travelMode);
            queryParams.AddOptional("avoid", avoid);
            queryParams.AddOptional("traffic", traffic);
            queryParams.AddOptional("routeType", routeType);
            queryParams.AddOptional("vehicleLoadType", vehicleLoadType);
            queryParams.AddOptional("vehicleEngineType", vehicleEngineType);
            queryParams.AddOptional("constantSpeedConsumptionInLitersPerHundredkm", constantSpeedConsumptionInLitersPerHundredkm);
            queryParams.AddOptional("currentFuelInLiters", currentFuelInLiters);
            queryParams.AddOptional("auxiliaryPowerInLitersPerHour", auxiliaryPowerInLitersPerHour);
            queryParams.AddOptional("fuelEnergyDensityInMJoulesPerLiter", fuelEnergyDensityInMJoulesPerLiter);
            queryParams.AddOptional("accelerationEfficiency", accelerationEfficiency);
            queryParams.AddOptional("decelerationEfficiency", decelerationEfficiency);
            queryParams.AddOptional("uphillEfficiency", uphillEfficiency);
            queryParams.AddOptional("downhillEfficiency", downhillEfficiency);
            queryParams.AddOptional("constantSpeedConsumptionInkWhPerHundredkm", constantSpeedConsumptionInkWhPerHundredkm);
            queryParams.AddOptional("currentChargeInkWh", currentChargeInkWh);
            queryParams.AddOptional("maxChargeInkWh", maxChargeInkWh);
            queryParams.AddOptional("auxiliaryPowerInkW", auxiliaryPowerInkW);

            return "route/directions".BuildApimUrl(format, queryParams);
        }

        public static string GetRouteReachableRangeUrl(
            string query,
            string fuelBudgetInLiters = null,
            string energyBudgetInkWh = null,
            string timeBudgetInSec = null,
            DateTime? departAt = null,
            string routeType = null,
            bool? traffic = null,
            string avoid = null,
            string travelMode = null,
            string hilliness = null,
            string windingness = null,
            int? vehicleAxleWeight = null,
            double? vehicleWidth = null,
            double? vehicleHeight = null,
            double? vehicleLength = null,
            int? vehicleMaxSpeed = null,
            int? vehicleWeight = null,
            string vehicleLoadType = null,
            string vehicleEngineType = null,
            double? constantSpeedConsumptionInLitersPerHundredkm = null,
            double? currentFuelInLiters = null,
            double? auxiliaryPowerInLitersPerHour = null,
            double? fuelEnergyDensityInMJoulesPerLiter = null,
            double? accelerationEfficiency = null,
            double? decelerationEfficiency = null,
            double? uphillEfficiency = null,
            double? downhillEfficiency = null,
            double? constantSpeedConsumptionInkWhPerHundredkm = null,
            string currentChargeInkWh = null,
            string maxChargeInkWh = null,
            string auxiliaryPowerInkW = null,
            string format = Constants.Json)
        {
            var queryParams = new Dictionary<string, string>() { { "query", query }, };

            queryParams.AddOptional("fuelBudgetInLiters", fuelBudgetInLiters);
            queryParams.AddOptional("energyBudgetInkWh", energyBudgetInkWh);
            queryParams.AddOptional("timeBudgetInSec", timeBudgetInSec);
            queryParams.AddOptional("departAt", departAt);
            queryParams.AddOptional("routeType", routeType);
            queryParams.AddOptional("traffic", traffic);
            queryParams.AddOptional("avoid", avoid);
            queryParams.AddOptional("travelMode", travelMode);
            queryParams.AddOptional("hilliness", hilliness);
            queryParams.AddOptional("windingness", windingness);
            queryParams.AddOptional("vehicleAxleWeight", vehicleAxleWeight);
            queryParams.AddOptional("vehicleWidth", vehicleWidth);
            queryParams.AddOptional("vehicleHeight", vehicleHeight);
            queryParams.AddOptional("vehicleLength", vehicleLength);
            queryParams.AddOptional("vehicleMaxSpeed", vehicleLength);
            queryParams.AddOptional("vehicleWeight", vehicleWeight);
            queryParams.AddOptional("vehicleLoadType", vehicleLoadType);
            queryParams.AddOptional("vehicleEngineType", vehicleEngineType);
            queryParams.AddOptional("constantSpeedConsumptionInLitersPerHundredkm", constantSpeedConsumptionInLitersPerHundredkm);
            queryParams.AddOptional("currentFuelInLiters", currentFuelInLiters);
            queryParams.AddOptional("auxiliaryPowerInLitersPerHour", auxiliaryPowerInLitersPerHour);
            queryParams.AddOptional("fuelEnergyDensityInMJoulesPerLiter", fuelEnergyDensityInMJoulesPerLiter);
            queryParams.AddOptional("accelerationEfficiency", accelerationEfficiency);
            queryParams.AddOptional("decelerationEfficiency", decelerationEfficiency);
            queryParams.AddOptional("uphillEfficiency", uphillEfficiency);
            queryParams.AddOptional("downhillEfficiency", downhillEfficiency);
            queryParams.AddOptional("constantSpeedConsumptionInkWhPerHundredkm", constantSpeedConsumptionInkWhPerHundredkm);
            queryParams.AddOptional("currentChargeInkWh", currentChargeInkWh);
            queryParams.AddOptional("maxChargeInkWh", maxChargeInkWh);
            queryParams.AddOptional("auxiliaryPowerInkW", auxiliaryPowerInkW);

            return "route/range".BuildApimUrl(format, queryParams);
        }

        public static string PostSearchAddressBatchUrl(string format)
        {
            return "search/address/batch".BuildApimUrl(format, new Dictionary<string, string>());
        }

        public static string PostSearchAddressReverseBatchUrl(string format)
        {
            return "search/address/reverse/batch".BuildApimUrl(format, new Dictionary<string, string>());
        }

        public static string PostSearchFuzzyBatchUrl(string format)
        {
            return "search/fuzzy/batch".BuildApimUrl(format, new Dictionary<string, string>());
        }

        public static string PostRouteDirectionsBatchUrl(string format)
        {
            return "route/directions/batch".BuildApimUrl(format, new Dictionary<string, string>());
        }
    }
}
