﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common.Models
{
    using System.Collections.Generic;
    using Newtonsoft.Json;

    [JsonObject]
    public class FeatureCollection
    {
        public FeatureCollection(IEnumerable<List<double>> coordinateList)
        {
            this.FeatureList = new List<Coordinate>();
            foreach (var coordinate in coordinateList)
            {
                this.FeatureList.Add(
                    new Coordinate()
                    {
                        Feature = new Feature() { Geometry = new Geometry("Point", new List<double>() { coordinate[0], coordinate[1] }), }
                    });
            }
        }

        [JsonProperty("coordinateList")]
        public List<Coordinate> FeatureList { get; set; }
    }

    [JsonObject]
    public class Coordinate
    {
        [JsonProperty("position")]
        public Feature Feature { get; set; }
    }

    [JsonObject]
    public class Feature
    {
        public Feature()
        {
            this.Type = "Feature";
            this.Properties = new Properties();
        }

        [JsonProperty("type")]
        public string Type { get; set; }

        [JsonProperty("geometry")]
        public Geometry Geometry { get; set; }

        [JsonProperty("properties")]
        public Properties Properties { get; set; }
    }

    [JsonObject]
    public class Properties
    {
    }
}
