﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System;
    using System.Net.Http;
    using System.Threading.Tasks;

    public partial class MapsClient
    {
        #region RouteCalculate Controller

        public Task<HttpResponseMessage> RouteCalculate(
            string query,
            bool postContent = false,
            StringContent httpContent = null,
            int? maxAlternatives = null,
            string alternativeType = null,
            int? minDeviationDistance = null,
            DateTime? arriveAt = null,
            DateTime? departAt = null,
            int? minDeviationTime = null,
            string instructionsType = null,
            string language = null,
            bool? computeBestOrder = null,
            string routeRepresentation = null,
            string computeTravelTimeFor = null,
            int? vehicleHeading = null,
            string report = null,
            string sectionType = null,
            int? vehicleAxleWeight = null,
            double? vehicleWidth = null,
            double? vehicleHeight = null,
            double? vehicleLength = null,
            int? vehicleMaxSpeed = null,
            int? vehicleWeight = null,
            string windingness = null,
            string hilliness = null,
            string travelMode = null,
            string avoid = null,
            bool? traffic = null,
            string routeType = null,
            string vehicleLoadType = null,
            string vehicleEngineType = null,
            double? constantSpeedConsumptionInLitersPerHundredkm = null,
            double? currentFuelInLiters = null,
            double? auxiliaryPowerInLitersPerHour = null,
            double? fuelEnergyDensityInMJoulesPerLiter = null,
            double? accelerationEfficiency = null,
            double? decelerationEfficiency = null,
            double? uphillEfficiency = null,
            double? downhillEfficiency = null,
            double? constantSpeedConsumptionInkWhPerHundredkm = null,
            string currentChargeInkWh = null,
            string maxChargeInkWh = null,
            string auxiliaryPowerInkW = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.RouteCalculateUrl(
                query,
                maxAlternatives,
                alternativeType,
                minDeviationDistance,
                arriveAt,
                departAt,
                minDeviationTime,
                instructionsType,
                language,
                computeBestOrder,
                routeRepresentation,
                computeTravelTimeFor,
                vehicleHeading,
                report,
                sectionType,
                vehicleAxleWeight,
                vehicleWidth,
                vehicleHeight,
                vehicleLength,
                vehicleMaxSpeed,
                vehicleWeight,
                windingness,
                hilliness,
                travelMode,
                avoid,
                traffic,
                routeType,
                vehicleLoadType,
                vehicleEngineType,
                constantSpeedConsumptionInLitersPerHundredkm,
                currentFuelInLiters,
                auxiliaryPowerInLitersPerHour,
                fuelEnergyDensityInMJoulesPerLiter,
                accelerationEfficiency,
                decelerationEfficiency,
                uphillEfficiency,
                downhillEfficiency,
                constantSpeedConsumptionInkWhPerHundredkm,
                currentChargeInkWh,
                maxChargeInkWh,
                auxiliaryPowerInkW,
                format);

            return postContent ? this.PostAsync(url, httpContent) : this.GetAsync(url);
        }

        #endregion

        #region RouteReachableRange Controller

        public Task<HttpResponseMessage> GetRouteReachableRange(
            string query,
            string fuelBudgetInLiters = null,
            string energyBudgetInkWh = null,
            string timeBudgetInSec = null,
            DateTime? departAt = null,
            string routeType = null,
            bool? traffic = null,
            string avoid = null,
            string travelMode = null,
            string hilliness = null,
            string windingness = null,
            int? vehicleAxleWeight = null,
            double? vehicleWidth = null,
            double? vehicleHeight = null,
            double? vehicleLength = null,
            int? vehicleMaxSpeed = null,
            int? vehicleWeight = null,
            string vehicleLoadType = null,
            string vehicleEngineType = null,
            double? constantSpeedConsumptionInLitersPerHundredkm = null,
            double? currentFuelInLiters = null,
            double? auxiliaryPowerInLitersPerHour = null,
            double? fuelEnergyDensityInMJoulesPerLiter = null,
            double? accelerationEfficiency = null,
            double? decelerationEfficiency = null,
            double? uphillEfficiency = null,
            double? downhillEfficiency = null,
            double? constantSpeedConsumptionInkWhPerHundredkm = null,
            string currentChargeInkWh = null,
            string maxChargeInkWh = null,
            string auxiliaryPowerInkW = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetRouteReachableRangeUrl(
                query,
                fuelBudgetInLiters,
                energyBudgetInkWh,
                timeBudgetInSec,
                departAt,
                routeType,
                traffic,
                avoid,
                travelMode,
                hilliness,
                windingness,
                vehicleAxleWeight,
                vehicleWidth,
                vehicleHeight,
                vehicleLength,
                vehicleMaxSpeed,
                vehicleWeight,
                vehicleLoadType,
                vehicleEngineType,
                constantSpeedConsumptionInLitersPerHundredkm,
                currentFuelInLiters,
                auxiliaryPowerInLitersPerHour,
                fuelEnergyDensityInMJoulesPerLiter,
                accelerationEfficiency,
                decelerationEfficiency,
                uphillEfficiency,
                downhillEfficiency,
                constantSpeedConsumptionInkWhPerHundredkm,
                currentChargeInkWh,
                maxChargeInkWh,
                auxiliaryPowerInkW,
                format);

            return this.GetAsync(url);
        }

        #endregion
    }
}
