﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;

    public partial class MapsClient
    {
        #region SearchAddress Controller

        public Task<HttpResponseMessage> GetSearchAddress(
            string query,
            bool? typeahead = null,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchAddressUrl(
                query, typeahead, limit, ofs, countrySet, lat, lon, radius, topLeft, btmRight, language, extendedPostalCodesFor, format);
            return this.GetAsync(url);
        }

        public Task<HttpResponseMessage> GetSearchAddressStructured(
            string countryCode,
            int? limit = null,
            int? ofs = null,
            string streetNumber = null,
            string streetName = null,
            string crossStreet = null,
            string municipality = null,
            string municipalitySubdivision = null,
            string countryTertiarySubdivision = null,
            string countrySecondarySubdivision = null,
            string countrySubdivision = null,
            string postalCode = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchAddressStructuredUrl(
                countryCode, limit, ofs, streetNumber, streetName, crossStreet, municipality, municipalitySubdivision, countryTertiarySubdivision, countrySecondarySubdivision, countrySubdivision, postalCode, extendedPostalCodesFor, format);
            return this.GetAsync(url);
        }

        #endregion

        #region SearchReverseAddress Controller

        public Task<HttpResponseMessage> GetSearchAddressReverse(
            string query,
            bool? returnSpeedLimit = null,
            double? heading = null,
            double? radius = null,
            string number = null,
            bool? returnRoadUse = null,
            string roadUse = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchAddressReverseUrl(query, returnSpeedLimit, heading, radius, number, returnRoadUse, roadUse, format);
            return this.GetAsync(url);
        }

        public Task<HttpResponseMessage> GetSearchAddressReverseCrossStreet(
            string query,
            bool? returnSpeedLimit = null,
            double? heading = null,
            double? radius = null,
            string language = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchAddressReverseCrossStreetUrl(query, returnSpeedLimit, heading, radius, language, format);
            return this.GetAsync(url);
        }

        #endregion

        #region SearchPoi Controller

        public Task<HttpResponseMessage> GetSearchPoi(
            string query,
            bool? typeahead = null,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchPoiUrl(query, typeahead, limit, ofs, countrySet, lat, lon, radius, topLeft, btmRight, language, extendedPostalCodesFor, format);
            return this.GetAsync(url);
        }

        #endregion

        #region SearchPoiCategory Controller

        public Task<HttpResponseMessage> GetSearchPoiCategory(
            string query,
            bool? typeahead = null,
            int? limit = null,
            int? multiplier = null,
            int? routingTimeout = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchPoiCategoryUrl(query, typeahead, limit, multiplier, routingTimeout, countrySet, lat, lon, radius, topLeft, btmRight, language, extendedPostalCodesFor, format);
            return this.GetAsync(url);
        }

        #endregion

        #region SearchNearby Controller

        public Task<HttpResponseMessage> GetSearchNearby(
            double lat,
            double lon,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? radius = null,
            string language = null,
            string extendedPostalCodesFor = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchNearbyUrl(lat, lon, limit, ofs, countrySet, radius, language, extendedPostalCodesFor, format);
            return this.GetAsync(url);
        }

        #endregion

        #region SearchFuzzy Controller

        public Task<HttpResponseMessage> GetSearchFuzzy(
            string query,
            bool? typehead = null,
            int? limit = null,
            int? ofs = null,
            List<string> countrySet = null,
            double? lat = null,
            double? lon = null,
            double? radius = null,
            string topLeft = null,
            string btmRight = null,
            string language = null,
            string extendedPostalCodesFor = null,
            int? minFuzzyLevel = null,
            int? maxFuzzyLevel = null,
            List<string> idxSet = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchFuzzyUrl(
                query, typehead, limit, ofs, countrySet, lat, lon, radius, topLeft, btmRight, language, extendedPostalCodesFor, minFuzzyLevel, maxFuzzyLevel, idxSet, format);
            return this.GetAsync(url);
        }

        #endregion

        #region SearchAlongRoute Controller

        public Task<HttpResponseMessage> GetSearchAlongRoute(
            string query,
            string route,
            int maxDetourTime,
            int? limit = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchAlongRouteUrl(query, route, maxDetourTime, limit, format);
            return this.GetAsync(url);
        }

        public Task<HttpResponseMessage> PostSearchAlongRoute(
            StringContent httpContent,
            string query,
            int maxDetourTime,
            int? limit = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.PostSearchAlongRouteUrl(query, maxDetourTime, limit, format);
            return this.PostAsync(url, httpContent);
        }

        #endregion

        #region SearchByGeometry Controller

        public Task<HttpResponseMessage> GetSearchGeometry(
            string query,
            string geometry,
            int? limit = null,
            string language = null,
            string extendedPostalCodesFor = null,
            List<string> idxSet = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetSearchGeometryUrl(query, geometry, limit, language, extendedPostalCodesFor, idxSet, format);
            return this.GetAsync(url);
        }

        public Task<HttpResponseMessage> PostSearchGeometry(
            StringContent httpContent,
            string query,
            int? limit = null,
            string language = null,
            string extendedPostalCodesFor = null,
            List<string> idxSet = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.PostSearchGeometryUrl(query, limit, language, extendedPostalCodesFor, idxSet, format);
            return this.PostAsync(url, httpContent);
        }

        #endregion
    }
}
