﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System.Net.Http;
    using System.Threading.Tasks;

    public partial class MapsClient
    {
        #region TimezoneByCoordinates Controller

        public Task<HttpResponseMessage> GetTimezoneByCoordinates(
            string query,
            string acceptLanguage = null,
            string options = null,
            string timeStamp = null,
            string transitionsFrom = null,
            int? transitionsYears = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTimezoneByCoordinatesUrl(query, options, timeStamp, transitionsFrom, transitionsYears, format);

            if (acceptLanguage != null)
            {
                AddHeaders("Accept-Language", acceptLanguage);
            }

            return this.GetAsync(url);
        }

        #endregion

        #region TimezoneById Controller

        public Task<HttpResponseMessage> GetTimezoneById(
            string query,
            string acceptLanguage = null,
            string options = null,
            string timeStamp = null,
            string transitionsFrom = null,
            int? transitionsYears = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTimezoneByIdUrl(query, options, timeStamp, transitionsFrom, transitionsYears, format);

            if (acceptLanguage != null)
            {
                AddHeaders("Accept-Language", acceptLanguage);
            }

            return this.GetAsync(url);
        }

        #endregion

        #region TimezoneEnumIANA Controller

        public Task<HttpResponseMessage> GetTimezoneIana(string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTimezoneIanaUrl(format);
            return this.GetAsync(url);
        }

        #endregion

        #region TimezoneEnumWindows Controller

        public Task<HttpResponseMessage> GetTimezoneWindows(string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTimezoneWindowsUrl(format);
            return this.GetAsync(url);
        }

        #endregion

        #region TimezoneVersion Controller

        public Task<HttpResponseMessage> GetTimezoneIanaVersion(string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTimezoneIanaVersionUrl(format);
            return this.GetAsync(url);
        }

        #endregion

        #region TimezoneWindowsToIana Controller

        public Task<HttpResponseMessage> GetTimezoneWindowsToIana(
            string query,
            string territory = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTimezoneWindowsToIanaUrl(query, territory, format);
            return this.GetAsync(url);
        }

        #endregion
    }
}
