﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System.Net.Http;
    using System.Threading.Tasks;

    public partial class MapsClient
    {
        #region TrafficFlowSegment Controller

        public Task<HttpResponseMessage> GetTrafficFlowSegment(
            string style,
            int zoom,
            string query,
            string unit = null,
            int? thickness = null,
            bool? openLr = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTrafficFlowSegmentUrl(style, zoom, query, unit, thickness, openLr, format);
            return this.GetAsync(url);
        }

        #endregion

        #region TrafficFlowTile Controller

        public Task<HttpResponseMessage> GetTrafficFlowTile(
            string style,
            int zoom,
            int x,
            int y,
            int? thickness = null,
            string format = Constants.Png)
        {
            var url = MapsUriBuilder.GetTrafficFlowTileUrl(style, zoom, x, y, thickness, format);
            return this.GetAsync(url);
        }

        #endregion

        #region TrafficIncident Controller

        public Task<HttpResponseMessage> GetTrafficIncidentTile(
            string style,
            int zoom,
            int x,
            int y,
            string trafficState = null,
            string format = Constants.Png)
        {
            var url = MapsUriBuilder.GetTrafficIncidentTileUrl(style, zoom, x, y, trafficState, format);
            return this.GetAsync(url);
        }

        public Task<HttpResponseMessage> GetTrafficIncidentViewport(
            string boundingbox,
            int boundingzoom,
            string overviewbox,
            int overviewzoom,
            bool copyright,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTrafficIncidentViewportUrl(boundingbox, boundingzoom, overviewbox, overviewzoom, copyright, format);
            return this.GetAsync(url);
        }

        public Task<HttpResponseMessage> GetTrafficIncidentDetails(
            string style,
            string boundingbox,
            int boundingZoom,
            string trafficmodelid,
            string language = null,
            string projection = null,
            string geometries = null,
            bool? expandCluster = null,
            bool? originalPosition = null,
            string format = Constants.Json)
        {
            var url = MapsUriBuilder.GetTrafficIncidentDetailsUrl(
                style, boundingbox, boundingZoom, trafficmodelid, language, projection, geometries, expandCluster, originalPosition, format);
            return this.GetAsync(url);
        }

        #endregion
    }
}
