﻿// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common
{
    using System;
    using System.Collections.Generic;
    using System.Web;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    public static class Utilities
    {
        public static string GetTestSetting(this TestContext context, string key)
        {
            if (context.Properties.Contains(key))
            {
                return context.Properties[key].ToString();
            }

            throw new ArgumentException($"{key} was not found in the test settings.");
        }

        public static void AddOptional(this Dictionary<string, string> q, string key, string value)
        {
            if (value != null)
            {
                q.Add(key, value);
            }
        }

        public static void AddOptional(this Dictionary<string, string> q, string key, int? value)
        {
            if (value != null)
            {
                q.Add(key, value.ToString());
            }
        }

        public static void AddOptional(this Dictionary<string, string> q, string key, double? value)
        {
            if (value != null)
            {
                q.Add(key, value.ToString());
            }
        }

        public static void AddOptional(this Dictionary<string, string> q, string key, bool? value)
        {
            if (value != null)
            {
                q.Add(key, value.ToString().ToLower());
            }
        }

        public static void AddOptional(this Dictionary<string, string> q, string key, DateTime? value)
        {
            if (value != null)
            {
                q.Add(key, value.ToString());
            }
        }

        public static void AddOptional(this Dictionary<string, string> q, string key, List<string> value)
        {
            if (value != null)
            {
                q.Add(key, string.Join(",", value));
            }
        }

        public static string BuildApimUrl(this string path, string format = Constants.Json, Dictionary<string, string> qParams = null)
        {
            var qBuilder = new UriBuilder("localhost") { Path = path + $"/{format}" };

            var defaultCollection = HttpUtility.ParseQueryString(qBuilder.Query);

            if (qParams != null)
            {
                foreach (var param in qParams)
                {
                    defaultCollection.Add(param.Key, param.Value);
                }
            }

            qBuilder.Query = defaultCollection.ToString();
            return qBuilder.Uri.PathAndQuery;
        }
    }
}
