﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Map
{
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class MapCopyrightIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetCopyrightWorld_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetCopyrightWorld().ConfigureAwait(false);
            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "generalCopyrights");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetCopyrightTile_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetCopyrightTile(zoom: 6, x: 9, y: 22, text: "yes").ConfigureAwait(false);
            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "generalCopyrights");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetCopyrightBounding_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetCopyRightBounding(
                mincoordinates: "52.41064,4.84228",
                maxcoordinates: "52.41072,4.84239",
                text: "yes").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "generalCopyrights");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetCopyrightCaption_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetCopyRightCaption().ConfigureAwait(false);
            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "copyrightsCaption");
        }
    }
}
