﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Route
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class RouteCalculateIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetRouteCalculate_ValidRequest_OK()
        {
            var result = await this.MapsClient.RouteCalculate(
                             query: "52.50931,13.42936:52.50274,13.43872",
                             avoid: "tollRoads").ConfigureAwait(false);
            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "routes");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetRouteCalculate_LongDistance_Fail()
        {
            var result = await this.MapsClient.RouteCalculate(query: "19.191866,72.976685:47.604848,-122.167253").ConfigureAwait(false);
            await EnsureErrorStatusCode(result, HttpStatusCode.BadRequest).ConfigureAwait(false);
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetRouteCalculate_BadDestination_Fail()
        {
            var result = await this.MapsClient.RouteCalculate(query: "28.405224,119.948824:-36.240759,-145.057867").ConfigureAwait(false);
            await EnsureErrorStatusCode(result, HttpStatusCode.BadRequest).ConfigureAwait(false);
        }

        [TestMethod, TestCategory("L3")]
        public async Task PostRouteCalculate_ValidRequest_OK()
        {
            var input = new FeatureCollection(
                new List<List<double>>()
                {
                    new List<double>() { 13.42936, 52.50931 },
                    new List<double>() { 13.43872, 52.50274 }
                });

            var httpContent = new StringContent(JObject.FromObject(input).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            var result = await this.MapsClient.RouteCalculate(
                             postContent: true,
                             httpContent: httpContent,
                             query: "52.50931,13.42936:52.50274,13.43872",
                             travelMode: "motorcycle").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "routes");
        }
    }
}
