﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Search
{
    using System.Net;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class SearchAddressIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetAddress_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetSearchAddress(query: "15127 NE 24th Street, Redmond, WA 98052").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            var query = json.SelectToken("summary.query")?.Value<string>();
            Assert.IsNotNull(query, "Cannot find 'summary.query' value in the response");
            Assert.IsTrue(query.Contains("15127"), "Cannot find street number '15127' in the response");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetAddress_MissingQuery_BadRequest()
        {
            var url = $"/search/address/json?api-version={ApiVersion}&subscription-key={SubscriptionKey}";
            var httpResponse = await GetAsyncWithHttpResponse(url, ensureSuccess: false).ConfigureAwait(false);
            await EnsureErrorStatusCode(httpResponse, HttpStatusCode.NotFound).ConfigureAwait(false);
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetAddressStructured_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetSearchAddressStructured(
                countryCode: "US",
                streetNumber: "15127",
                streetName: "NE&2024th&20Street",
                municipality: "Redmond",
                countrySubdivision: "WA",
                postalCode: "98052").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            var query = json.SelectToken("summary.query")?.Value<string>();
            Assert.IsNotNull(query, "Cannot find 'summary.query' value in the response");
            Assert.IsTrue(query.Contains("15127"), "Cannot find street number '15127' in the response");
        }
    }
}
