﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Search
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class SearchAlongRouteIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetSearchAlongRoute_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetSearchAlongRoute(
                query: "burger",
                route: "{\"type\":\"LineString\",\"coordinates\":[[-122.143035,47.653536],[-122.187164,47.617556],[-122.114981,47.570599],[-122.132756,47.654009]]}",
                maxDetourTime: 1000,
                limit: 2).ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "summary");
        }

        [TestMethod, TestCategory("L3")]
        public async Task PostSearchAlongRoute_ValidRequest_OK()
        {
            var input = new SearchAlongRouteRequestBody(
                new List<List<double>>()
                {
                    new List<double>() { -122.143035, 47.653536 },
                    new List<double>() { -122.187164, 47.617556 },
                    new List<double>() { -122.114981, 47.570599 },
                    new List<double>() { -122.132756, 47.654009 },
                });

            var httpContent = new StringContent(JObject.FromObject(input).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            var result = await this.MapsClient.PostSearchAlongRoute(
                             httpContent,
                             query: "burger",
                             maxDetourTime: 1000, limit: 2).ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "summary");
        }
    }
}
