﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Search
{
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Net.Http.Headers;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class SearchByGeometryIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetSearchGeometry_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetSearchGeometry(
                query: "burger",
                limit: 2,
                geometry:
                "{\"type\":\"GeometryCollection\",\"geometries\":[{\"type\":\"Polygon\",\"coordinates\":[[[-122.43576049804686,37.7524152343544]," +
                "[-122.43301391601562,37.70660472542312],[-122.36434936523438,37.712059855877314],[-122.43576049804686,37.7524152343544]]]}]}").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "summary");
        }

        [TestMethod, TestCategory("L3")]
        public async Task PostSearchGeometry_ValidRequest_OK()
        {
            var input = new SearchInsideGeometryRequestBody(
                new List<List<double>>()
                {
                    new List<double> { -122.43576049804686, 37.7524152343544 },
                    new List<double> { -122.43301391601562, 37.70660472542312 },
                    new List<double> { -122.36434936523438, 37.712059855877314 },
                    new List<double> { -122.43576049804686, 37.7524152343544 },
                });

            var httpContent = new StringContent(JObject.FromObject(input).ToString());
            httpContent.Headers.ContentType = new MediaTypeHeaderValue("application/json");

            var result = await this.MapsClient.PostSearchGeometry(
                             httpContent,
                             query: "burger",
                             limit: 2).ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "summary");
        }
    }
}
