﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Timezone
{
    using System.Net;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class TimezoneByCoordinatesIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetTimezoneCoordinates_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetTimezoneByCoordinates(query: "47.0,-122").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            var query = json.SelectToken("TimeZones[0].Id")?.Value<string>();
            Assert.IsNotNull(query);
            Assert.AreEqual("America/Los_Angeles".ToLower(), query.ToLower());
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetTimezoneCoordinates_InvalidCoordinates_Badrequest()
        {
            var result = await this.MapsClient.GetTimezoneByCoordinates(query: "-91,-122").ConfigureAwait(false);

            Assert.IsNotNull(result);
            Assert.AreEqual(HttpStatusCode.BadRequest, result.StatusCode);
        }
    }
}
