﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Timezone
{
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class TimezoneByIdIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetTimezoneByID_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetTimezoneById(query: "Asia/Riyadh").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            var query = json.SelectToken("TimeZones[0].Id")?.Value<string>();
            Assert.IsNotNull(query);
            Assert.AreEqual("Asia/Riyadh".ToLower(), query.ToLower());
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetTimezoneByID_ValidRequestWithLocalization_OK()
        {
            var result = await this.MapsClient.GetTimezoneById(query: "Asia/Riyadh", acceptLanguage: "ar").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            var query = json.SelectToken("TimeZones[0].Names.Generic")?.Value<string>();
            Assert.IsNotNull(query);
            Assert.AreEqual("التوقيت العربي", query);
        }
    }
}
