﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Timezone
{
    using System.Linq;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class TimezoneWindowsToIanaIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetTimezoneWindowsToIana_ValidRequest_OK()
        {
            var httpResponse = await this.MapsClient.GetTimezoneWindowsToIana(query: "Pacific Standard Time").ConfigureAwait(false);

            var content = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
            Assert.IsNotNull(content);

            var jsonArray = JArray.Parse(content);
            Assert.IsTrue(jsonArray.Count > 0);

            var ianaZone = jsonArray.FirstOrDefault(x => x.Value<string>("Id") == "America/Los_Angeles");
            Assert.IsNotNull(ianaZone);
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetTimezoneWindowsToIana_WithTerritoryValidRequest_OK()
        {
            var httpResponse = await this.MapsClient.GetTimezoneWindowsToIana(query: "Pacific Standard Time", territory: "CA").ConfigureAwait(false);

            var content = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
            Assert.IsNotNull(content);

            var jsonArray = JArray.Parse(content);
            Assert.IsTrue(jsonArray.Count > 0);

            var ianaZone = jsonArray.FirstOrDefault(x => x.Value<string>("Id") == "America/Vancouver");
            Assert.IsNotNull(ianaZone);
        }
    }
}
