﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Traffic
{
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ProxyServerIntegrationTests.Common;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class TrafficIncidentIntegrationTests : ApimTestBase
    {
        [TestMethod, TestCategory("L3")]
        public async Task GetTrafficIncidentTile_ValidRequest_OK()
        {
            var httpResponse = await this.MapsClient.GetTrafficIncidentTile(style: "s3", zoom: 12, x: 2044, y: 1360).ConfigureAwait(false);

            var content = await httpResponse.Content.ReadAsByteArrayAsync().ConfigureAwait(false);
            Assert.IsNotNull(content, "Http response content cannot be null");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetTrafficIncidentViewport_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetTrafficIncidentViewport(
                boundingbox: "-939584.4813015489,-23954526.723651607,14675583.153020501,25043442.895825107",
                boundingzoom: 2,
                overviewbox: "-939584.4813018347,-23954526.723651607,14675583.153020501,25043442.8958229083",
                overviewzoom: 2,
                copyright: true).ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "viewpResp");
        }

        [TestMethod, TestCategory("L3")]
        public async Task GetTrafficIncidentDetails_ValidRequest_OK()
        {
            var result = await this.MapsClient.GetTrafficIncidentDetails(
                style: "s3",
                boundingbox: "6841263.950712,511972.674418,6886056.049288,582676.925582",
                boundingZoom: 11,
                trafficmodelid: "1335294634919").ConfigureAwait(false);

            var json = await ParseJson(result).ConfigureAwait(false);
            EnsureTokenExists(json, "tm");
        }
    }
}
