﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace ProxyServerUnitTests
{
    using System.Net.Http;
    using System.Web.Http;
    using System.Web.Http.Hosting;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class BaseProxyControllerTests
    {
        private Mock<SearchFuzzyController> searchFuzzyController;

        [TestInitialize]
        public void TestInitialize()
        {
            // Create Sample request
            HttpRequestMessage request = new HttpRequestMessage(HttpMethod.Get, "https://localhost/search/fuzzy/json?api-version=1.0&query=seattle&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");

            // Create Sample configuration
            var configuration = new HttpConfiguration();
            request.Properties[HttpPropertyKeys.HttpConfigurationKey] = configuration;

            searchFuzzyController = new Mock<SearchFuzzyController>() { CallBase = true };
            searchFuzzyController.Object.Request = request;
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            searchFuzzyController.Object.SetProxyOperation(proxyOperation);
            if (Agent.Instance == null)
            {
                var p = new InitParams("DevTestIfx", "AzLocTest");
                p.SetLocalTracing(@"c:\ifxTrace");
                p.SetEtwProvider("DevTest", "DB25DB83-F4C7-4D76-9DBA-5D08A7DEC472");
                Agent.CreateInstance(p);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void BuildErrorResponseAndLogTest_ServerError()
        {
            ProxyException exception = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");
            HttpResponseMessage buildErrorResponseAndLog = searchFuzzyController.Object.BuildErrorResponseAndLog(exception);
            Assert.AreEqual(System.Net.HttpStatusCode.InternalServerError, buildErrorResponseAndLog.StatusCode);
        }

        [TestMethod, TestCategory("L0")]
        public void BuildErrorResponseAndLogTest_Default()
        {
            ProxyException exception = new ProxyException(ProxyException.ExceptionType.Other, "other exception");
            HttpResponseMessage buildErrorResponseAndLog = searchFuzzyController.Object.BuildErrorResponseAndLog(exception);
            Assert.AreEqual(System.Net.HttpStatusCode.ServiceUnavailable, buildErrorResponseAndLog.StatusCode);
        }
    }
}
