﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class HeartbeatControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<HeartbeatController> HeartbeatController;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            HeartbeatController = new Mock<HeartbeatController>() { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void GetHeartbeatTest_ProxyObjNull()
        {
            //Inject dependency
            HeartbeatController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Heartbeat, It.IsAny<IProxyOperation>())).Returns((BaseProxySearch)null);
            HeartbeatController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = HeartbeatController.Object.GetHeartbeat();
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Heartbeat, null), Times.Once);
            HeartbeatController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetHeartbeatTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://localhost/search/fuzzy/json?api-version=1.0&query=seattle&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Heartbeat");
            Mock<BaseProxyHeartbeat> baseProxySearchMock = new Mock<BaseProxyHeartbeat>(proxyOperation);
            HeartbeatController.Object.Request = Request;

            //Inject dependency
            HeartbeatController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Heartbeat, It.IsAny<IProxyOperation>())).Returns(baseProxySearchMock.Object);

            //Call and verify
            var output = HeartbeatController.Object.GetHeartbeat();
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Heartbeat, It.IsAny<IProxyOperation>()), Times.Once);
            Assert.AreEqual(System.Net.HttpStatusCode.OK, output.StatusCode);
        }
    }
}
