﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class MapCopyrightControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<MapCopyrightController> MapCopyrightController;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            MapCopyrightController = new Mock<MapCopyrightController>() { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void Initialize_MapCopyrightController()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapCopyrightController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/copyright/world/json?api-version=1&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightWorld");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapCopyrightController.Object.Request = Request;

            //Call and verify
            MapCopyrightController.Object.Initialize();
            Assert.AreEqual(json, MapCopyrightController.Object.Format);
            Assert.AreEqual(1, MapCopyrightController.Object.Version);
            Assert.AreEqual(null, MapCopyrightController.Object.Query);
        }

        [TestMethod, TestCategory("L0")]
        public void Initialize_GetCopyrightTile()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapCopyrightController.Object.SetProxyOperation(proxyOperation);
            List<string> zoom = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/copyright/tile/json?api-version=1.0&zoom=6&x=9&y=22&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapCopyrightController.Object.Request = Request;

            //Call and verify
            MapCopyrightController.Object.Initialize();
            MapCopyrightController.Object.GetQueryStrings().TryGetValue("zoom", out zoom);
            Assert.AreEqual(json, MapCopyrightController.Object.Format);
            Assert.AreEqual(1, MapCopyrightController.Object.Version);
            Assert.AreEqual("6", zoom.First());
        }

        [TestMethod, TestCategory("L0")]
        public void Initialize_GetCopyrightTile_Nozoom()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapCopyrightController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/copyright/tile/json?api-version=1.0&x=9&y=22&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapCopyrightController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "zoom||x||y missing");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                MapCopyrightController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void Initialize_GetCopyrightBounding()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapCopyrightController.Object.SetProxyOperation(proxyOperation);
            List<string> mincoordinates = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/copyright/bounding/json?api-version=1.0&mincoordinates=52.41064,4.84228&maxcoordinates=52.41072,4.84239&text=true&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightBounding");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapCopyrightController.Object.Request = Request;

            //Call and verify
            MapCopyrightController.Object.Initialize();
            MapCopyrightController.Object.GetQueryStrings().TryGetValue("mincoordinates", out mincoordinates);
            Assert.AreEqual(json, MapCopyrightController.Object.Format);
            Assert.AreEqual(1, MapCopyrightController.Object.Version);
            Assert.AreEqual("52.41064,4.84228", mincoordinates.First());
        }

        [TestMethod, TestCategory("L0")]
        public void Initialize_GetCopyrightBounding_Nomincoordinates()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapCopyrightController.Object.SetProxyOperation(proxyOperation);
            List<string> mincoordinates = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/copyright/bounding/json?api-version=1.0&maxcoordinates=52.41072,4.84239&text=true&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightBounding");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapCopyrightController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "mincoordinates||maxcoordinates missing");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                MapCopyrightController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void GetCopyrightWorldTest_ProxyObjNull()
        {
            //Inject dependency
            MapCopyrightController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns((BaseProxyMap)null);
            MapCopyrightController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = MapCopyrightController.Object.GetCopyrightWorld("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, null), Times.Once);
            MapCopyrightController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetCopyrightWorldTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxyMap> BaseProxyMapMock = new Mock<BaseProxyMap>(proxyOperation);

            //Inject dependency
            MapCopyrightController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns(BaseProxyMapMock.Object);
            BaseProxyMapMock.Setup(x => x.MapCopyright(BaseProxyMap.MapType.WorldCopyright, It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>())).Returns(task.Task);

            //Call and verify
            var output = MapCopyrightController.Object.GetCopyrightWorld("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxyMapMock.Verify(x => x.MapCopyright(BaseProxyMap.MapType.WorldCopyright, It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>()));
            Assert.AreEqual(task.Task.Result, output.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetCopyrightTileTest_ProxyObjNull()
        {
            //Inject dependency
            MapCopyrightController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns((BaseProxyMap)null);
            MapCopyrightController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = MapCopyrightController.Object.GetCopyrightTile("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, null), Times.Once);
            MapCopyrightController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetCopyrightTileTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxyMap> BaseProxyMapMock = new Mock<BaseProxyMap>(proxyOperation);
            string zoom = "";
            List<string> zoomList = new List<string>();
            zoomList.Add(zoom);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("zoom", zoomList);
            querystrings.Add("x", xList);
            querystrings.Add("y", yList);
            MapCopyrightController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            MapCopyrightController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns(BaseProxyMapMock.Object);
            BaseProxyMapMock.Setup(p => p.MapCopyright(BaseProxyMap.MapType.TileCopyright, It.IsAny<string>(), querystrings, It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>())).Returns(task.Task);

            //Call and verify
            var output = MapCopyrightController.Object.GetCopyrightTile("");
            this.versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxyMapMock.Verify(p => p.MapCopyright(BaseProxyMap.MapType.TileCopyright, It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>()));
            Assert.AreEqual(task.Task.Result, output.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetCopyrightBoundingTest_ProxyObjNull()
        {
            //Inject dependency
            MapCopyrightController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns((BaseProxyMap)null);
            MapCopyrightController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = MapCopyrightController.Object.GetCopyrightBounding("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, null), Times.Once);
            MapCopyrightController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetCopyrightBoundingTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxyMap> BaseProxyMapMock = new Mock<BaseProxyMap>(proxyOperation);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("mincoordinates", xList);
            querystrings.Add("maxcoordinates", yList);
            MapCopyrightController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            MapCopyrightController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns(BaseProxyMapMock.Object);
            BaseProxyMapMock.Setup(p => p.MapCopyright(BaseProxyMap.MapType.BoundingCopyright, It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>())).Returns(task.Task);

            //Call and verify
            var output = MapCopyrightController.Object.GetCopyrightBounding("");
            this.versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxyMapMock.Verify(p => p.MapCopyright(BaseProxyMap.MapType.BoundingCopyright, It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>()));
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
