﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class MapTileControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<MapTileController> MapTileController;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            MapTileController = new Mock<MapTileController>() { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetMapStatic()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapTileController.Object.SetProxyOperation(proxyOperation);
            List<string> request = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/static/png?api-version=1.0&service=WMS&version=1.1.1&request=GetMap&srs=EPSG:4326&height=768&width=1024&layers=basic&styles=1&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string png = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", png);
            routeValueDictionary.Add("action", "GetMapStatic");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapTileController.Object.Request = Request;

            //Call and verify
            MapTileController.Object.Initialize();
            MapTileController.Object.GetQueryStrings().TryGetValue("request", out request);
            Assert.AreEqual(png, MapTileController.Object.Format);
            Assert.AreEqual(1, MapTileController.Object.Version);
            Assert.AreEqual("GetMap", request.First());
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetMapTile()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapTileController.Object.SetProxyOperation(proxyOperation);
            List<string> layer = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/tile/png?api-version=1.0&layer=basic&style=main&zoom=6&x=10&y=22&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string png = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", png);
            routeValueDictionary.Add("action", "GetMapTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapTileController.Object.Request = Request;

            //Call and verify
            MapTileController.Object.Initialize();
            MapTileController.Object.GetQueryStrings().TryGetValue("layer", out layer);
            Assert.AreEqual(png, MapTileController.Object.Format);
            Assert.AreEqual(1, MapTileController.Object.Version);
            Assert.AreEqual("basic", layer.First());
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetMapTile_Nolayer()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            MapTileController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/map/tile/png?api-version=1.0&style=main&zoom=6&x=10&y=22&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string png = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", png);
            routeValueDictionary.Add("action", "GetMapTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            MapTileController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "layer||style||zoom||x||y invalid");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                MapTileController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void GetMapTileTest_ProxyObjNull()
        {
            //Inject dependency
            MapTileController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns((BaseProxyMap)null);
            MapTileController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = MapTileController.Object.GetMapTile("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, null), Times.Once);
            MapTileController.Verify(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>()), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetMapTileTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxyMap> BaseProxyMapMock = new Mock<BaseProxyMap>(proxyOperation);
            string layer = "";
            List<string> layerList = new List<string>();
            layerList.Add(layer);
            string style = "";
            List<string> styleList = new List<string>();
            styleList.Add(style);
            string zoom = "";
            List<string> zoomList = new List<string>();
            zoomList.Add(zoom);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            string formatstring = "";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("layer", layerList);
            querystrings.Add("style", styleList);
            querystrings.Add("zoom", zoomList);
            querystrings.Add("x", xList);
            querystrings.Add("y", yList);
            MapTileController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            MapTileController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns(BaseProxyMapMock.Object);
            BaseProxyMapMock.Setup(p => p.MapTileRaster(layer, style, zoom, x, y, formatstring, querystrings)).Returns(task.Task);

            //Call and verify
            var output = MapTileController.Object.GetMapTile("");
            this.versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxyMapMock.Verify(p => p.MapTileRaster(It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>()));
            Assert.AreEqual(task.Task.Result, output.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetMapStaticTest_ProxyObjNull()
        {
            //Inject dependency
            MapTileController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns((BaseProxyMap)null);
            MapTileController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = MapTileController.Object.GetMapTileStatic("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, null), Times.Once);
            MapTileController.Verify(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>()), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetMapStaticTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxyMap> BaseProxyMapMock = new Mock<BaseProxyMap>(proxyOperation);
            string formatstring = "";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            MapTileController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            MapTileController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>())).Returns(BaseProxyMapMock.Object);
            BaseProxyMapMock.Setup(p => p.MapStatic(formatstring, querystrings)).Returns(task.Task);

            //Call and verify
            var output = MapTileController.Object.GetMapTileStatic("");
            this.versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Map, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxyMapMock.Verify(p => p.MapStatic(It.IsAny<string>(), It.IsAny<Dictionary<string, List<string>>>()));
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
