﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class RouteCalculateControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<RouteCalculateController> RouteCalculateController;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            RouteCalculateController = new Mock<RouteCalculateController>() { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetRouteCalculate_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            RouteCalculateController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/route/directions/json?api-version=1.0&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            RouteCalculateController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                RouteCalculateController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetRouteCalculate_invalidQuery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            RouteCalculateController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/route/directions/json?api-version=1.0&query=test&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            RouteCalculateController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                RouteCalculateController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetRouteCalculate_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            RouteCalculateController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/route/directions/png?api-version=1.0&query=52.50931,13.42936:52.50274,13.43872&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            RouteCalculateController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                RouteCalculateController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetRouteCalculate()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            RouteCalculateController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/route/directions/json?api-version=1.0&query=52.50931,13.42936:52.50274,13.43872&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetCopyrightTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            RouteCalculateController.Object.Request = Request;

            //Call and verify
            RouteCalculateController.Object.Initialize();
            Assert.AreEqual(json, RouteCalculateController.Object.Format);
            Assert.AreEqual(1, RouteCalculateController.Object.Version);
            Assert.AreEqual("52.50931,13.42936:52.50274,13.43872", RouteCalculateController.Object.Query);
        }

        [TestMethod, TestCategory("L0")]
        public void GetRouteCalculateTest_ProxyObjNull()
        {
            //Inject dependency
            RouteCalculateController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Route, It.IsAny<IProxyOperation>())).Returns((BaseProxyRoute)null);
            RouteCalculateController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = RouteCalculateController.Object.GetRouteCalculate("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Route, null), Times.Once);
            RouteCalculateController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetRouteCalculateTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxyRoute> BaseProxyRouteMock = new Mock<BaseProxyRoute>(proxyOperation);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            RouteCalculateController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            RouteCalculateController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Route, It.IsAny<IProxyOperation>())).Returns(BaseProxyRouteMock.Object);
            BaseProxyRouteMock.Setup(x => x.RouteProxy(BaseProxyRoute.RouteType.Normal, It.IsAny<string>(), It.IsAny<string>(), querystrings)).Returns(task.Task);

            //Call and verify
            var output = RouteCalculateController.Object.GetRouteCalculate("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Route, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxyRouteMock.Verify(x => x.RouteProxy(BaseProxyRoute.RouteType.Normal, It.IsAny<string>(), It.IsAny<string>(), querystrings));
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
