﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class SearchAddressControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<SearchAddressController> SearchAddressController;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            SearchAddressController = new Mock<SearchAddressController>() { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddress()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            SearchAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/json?api-version=1.0&query=15127 NE 24th Street, Redmond, WA 98052&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddress");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchAddressController.Object.Request = Request;

            //Call and verify
            SearchAddressController.Object.Initialize();
            Assert.AreEqual(json, SearchAddressController.Object.Format);
            Assert.AreEqual(1, SearchAddressController.Object.Version);
            Assert.AreEqual("15127 NE 24th Street, Redmond, WA 98052", SearchAddressController.Object.Query);
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddress_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            SearchAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/json?api-version=1.0&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddress");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddress_InvalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            SearchAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/png?api-version=1.0&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddress");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressTest_ProxyObjNull()
        {
            //Inject dependency
            SearchAddressController.Object.versionObject = versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns((BaseProxySearch)null);
            SearchAddressController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = SearchAddressController.Object.GetAddress("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, null), Times.Once);
            SearchAddressController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxySearch> BaseProxySearchMock = new Mock<BaseProxySearch>(proxyOperation);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            SearchAddressController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            SearchAddressController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns(BaseProxySearchMock.Object);
            BaseProxySearchMock.Setup(x => x.SearchProxy(BaseProxySearch.SearchType.Address, It.IsAny<string>(), It.IsAny<string>(), querystrings, null)).Returns(task.Task);

            //Call and verify
            var output = SearchAddressController.Object.GetAddress("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxySearchMock.Verify(x => x.SearchProxy(BaseProxySearch.SearchType.Address, It.IsAny<string>(), It.IsAny<string>(), querystrings, null));
            Assert.AreEqual(output.Result, task.Task.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressStructuredTest_ProxyObjNull()
        {
            //Inject dependency
            SearchAddressController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns((BaseProxySearch)null);
            SearchAddressController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = SearchAddressController.Object.GetAddressStructured("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, null), Times.Once);
            SearchAddressController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressStructuredTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            Mock<BaseProxySearch> BaseProxySearchMock = new Mock<BaseProxySearch>(proxyOperation);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            SearchAddressController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            SearchAddressController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns(BaseProxySearchMock.Object);
            BaseProxySearchMock.Setup(x => x.SearchProxy(BaseProxySearch.SearchType.AddressStructured, string.Empty, It.IsAny<string>(), querystrings, null)).Returns(task.Task);

            //Call and verify
            var output = SearchAddressController.Object.GetAddressStructured("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxySearchMock.Verify(x => x.SearchProxy(BaseProxySearch.SearchType.AddressStructured, string.Empty, It.IsAny<string>(), querystrings, null));
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
