﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class SearchReverseAddressControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<SearchReverseAddressController> SearchReverseAddressController;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            SearchReverseAddressController = new Mock<SearchReverseAddressController>() { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressReverse_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/json?api-version=1.0&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressReverse");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchReverseAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressReverse_invalidQuery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/json?api-version=1.0&query=test&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressReverse");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchReverseAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressReverse_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/png?api-version=1.0&query=37.337,-121.89&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressReverse");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchReverseAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressReverse()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/json?api-version=1.0&query=37.337,-121.89&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressReverse");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;

            //Call and verify
            SearchReverseAddressController.Object.Initialize();
            Assert.AreEqual(json, SearchReverseAddressController.Object.Format);
            Assert.AreEqual(1, SearchReverseAddressController.Object.Version);
            Assert.AreEqual("37.337,-121.89", SearchReverseAddressController.Object.Query);
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressCrossStreet_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/json?api-version=1.0&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressCrossStreet");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchReverseAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressCrossStreet_invalidQuery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/json?api-version=1.0&query=test&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressCrossStreet");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchReverseAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressCrossStreet_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/png?api-version=1.0&query=37.337,-121.89&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressCrossStreet");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                SearchReverseAddressController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetAddressCrossStreet()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            SearchReverseAddressController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/search/address/reverse/crossStreet/json?api-version=1.0&query=37.337,-121.89&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetAddressCrossStreet");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            SearchReverseAddressController.Object.Request = Request;

            //Call and verify
            SearchReverseAddressController.Object.Initialize();
            Assert.AreEqual(json, SearchReverseAddressController.Object.Format);
            Assert.AreEqual(1, SearchReverseAddressController.Object.Version);
            Assert.AreEqual("37.337,-121.89", SearchReverseAddressController.Object.Query);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressReverseTest_ProxyObjNull()
        {
            //Inject dependency
            SearchReverseAddressController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns((BaseProxySearch)null);
            SearchReverseAddressController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = SearchReverseAddressController.Object.GetAddressReverse("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, null), Times.Once);
            SearchReverseAddressController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressReverseTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            Mock<BaseProxySearch> BaseProxySearchMock = new Mock<BaseProxySearch>(proxyOperation);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            SearchReverseAddressController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            SearchReverseAddressController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns(BaseProxySearchMock.Object);
            BaseProxySearchMock.Setup(x => x.SearchProxy(BaseProxySearch.SearchType.ReverseAddress, It.IsAny<string>(), It.IsAny<string>(), querystrings, null)).Returns(task.Task);

            //Call and verify
            var output = SearchReverseAddressController.Object.GetAddressReverse("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxySearchMock.Verify(x => x.SearchProxy(BaseProxySearch.SearchType.ReverseAddress, It.IsAny<string>(), It.IsAny<string>(), querystrings, null));
            Assert.AreEqual(task.Task.Result, output.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressCrossStreetTest_ProxyObjNull()
        {
            //Inject dependency
            SearchReverseAddressController.Object.versionObject = this.versionObj.Object;
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns((BaseProxySearch)null);
            SearchReverseAddressController.Setup(x => x.BuildErrorResponseAndLog(It.IsAny<ProxyException>())).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = SearchReverseAddressController.Object.GetAddressCrossStreet("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, null), Times.Once);
            SearchReverseAddressController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetAddressCrossStreetTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            Mock<BaseProxySearch> BaseProxySearchMock = new Mock<BaseProxySearch>(proxyOperation);
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            SearchReverseAddressController.Object.SetQueryStrings(querystrings);

            //Inject dependency
            SearchReverseAddressController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>())).Returns(BaseProxySearchMock.Object);
            BaseProxySearchMock.Setup(x => x.SearchProxy(BaseProxySearch.SearchType.CrossStreet, It.IsAny<string>(), It.IsAny<string>(), querystrings, null)).Returns(task.Task);

            //Call and verify
            var output = SearchReverseAddressController.Object.GetAddressCrossStreet("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Search, It.IsAny<IProxyOperation>()), Times.Once);
            BaseProxySearchMock.Verify(x => x.SearchProxy(BaseProxySearch.SearchType.CrossStreet, It.IsAny<string>(), It.IsAny<string>(), querystrings, null));
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
