﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TimezoneByCoordinatesControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<TimezoneByCoordinatesController> timezoneByCoordinatesControllerMock;

        private Mock<BaseProxyTimezone> baseProxyTimezone;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            timezoneByCoordinatesControllerMock = new Mock<TimezoneByCoordinatesController>() { CallBase = true };
            IProxyOperation proxyOperation = new ProxyOperation("Timezone");
            baseProxyTimezone = mockRepository.Create<BaseProxyTimezone>(proxyOperation);
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTimezoneCoordinatesTest_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            timezoneByCoordinatesControllerMock.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/timezone/byCoordinates/json?api-version=1.0&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTimezoneCoordinatesTest");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            timezoneByCoordinatesControllerMock.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                timezoneByCoordinatesControllerMock.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTimezoneCoordinatesTest_invalidQuery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            timezoneByCoordinatesControllerMock.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/timezone/byCoordinates/json?api-version=1.0&query=test&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTimezoneCoordinatesTest");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            timezoneByCoordinatesControllerMock.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid query string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                timezoneByCoordinatesControllerMock.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTimezoneCoordinatesTest_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            timezoneByCoordinatesControllerMock.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/timezone/byCoordinates/png?api-version=1.0&query=47.0,-122&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTimezoneCoordinatesTest");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            timezoneByCoordinatesControllerMock.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                timezoneByCoordinatesControllerMock.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTimezoneCoordinatesTest()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            timezoneByCoordinatesControllerMock.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/timezone/byCoordinates/json?api-version=1.0&query=47.0,-122&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTimezoneCoordinatesTest");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            timezoneByCoordinatesControllerMock.Object.Request = Request;

            //Call and verify
            timezoneByCoordinatesControllerMock.Object.Initialize();
            Assert.AreEqual(json, timezoneByCoordinatesControllerMock.Object.Format);
            Assert.AreEqual(1, timezoneByCoordinatesControllerMock.Object.Version);
            Assert.AreEqual("47.0,-122", timezoneByCoordinatesControllerMock.Object.Query);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTimezoneCoordinatesTest()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            Dictionary<string, List<string>> headers = new Dictionary<string, List<string>>();

            //Inject dependency
            timezoneByCoordinatesControllerMock.Object.versionObject = versionObj.Object;

            //Setup dependency
            versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Timezone, It.IsAny<IProxyOperation>())).Returns(baseProxyTimezone.Object);
            baseProxyTimezone.Setup(
                x => x.TimezoneProxy(
                    BaseProxyTimezone.TimezoneType.ByCoordinates,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    headers,
                    It.IsAny<Dictionary<string, List<string>>>())).Returns(task.Task);

            //Call and verify
            var output = timezoneByCoordinatesControllerMock.Object.GetTimezoneCoordinates("");
            versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Timezone, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTimezone.Verify(x => x.TimezoneProxy(BaseProxyTimezone.TimezoneType.ByCoordinates,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    headers,
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Once);
            Assert.AreEqual(output.Result, task.Task.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTimezoneCoordinatesTest_ProxyObjNull()
        {
            //Setup vars
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");
            Dictionary<string, List<string>> headers = new Dictionary<string, List<string>>();

            //Inject dependency
            timezoneByCoordinatesControllerMock.Object.versionObject = versionObj.Object;

            //Setup dependency
            versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Timezone, It.IsAny<IProxyOperation>())).Returns((BaseProxyTimezone)null);
            timezoneByCoordinatesControllerMock.Setup(x => x.BuildErrorResponseAndLog(expectedException)).Returns((HttpResponseMessage)null);

            //Call and verify
            var output = timezoneByCoordinatesControllerMock.Object.GetTimezoneCoordinates("");
            versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Timezone, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTimezone.Verify(x => x.TimezoneProxy(BaseProxyTimezone.TimezoneType.ByCoordinates,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    headers,
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Never);
            timezoneByCoordinatesControllerMock.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }
    }
}
