﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TrafficFlowSegmentControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<TrafficFlowSegmentController> trafficFlowSegmentController;

        private Mock<BaseProxyTraffic> baseProxyTraffic;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            trafficFlowSegmentController = new Mock<TrafficFlowSegmentController>() { CallBase = true };
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            baseProxyTraffic = this.mockRepository.Create<BaseProxyTraffic>(proxyOperation);
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowSegment_Nozoom()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            trafficFlowSegmentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/segment/json?api-version=1.0&style=absolute&query=52.41072,4.84239&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficFlowSegment");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            trafficFlowSegmentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "zoom||x||y missing");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                trafficFlowSegmentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowSegment_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            trafficFlowSegmentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/segment/json?api-version=1.0&style=absolute&zoom=10&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficFlowSegment");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            trafficFlowSegmentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                trafficFlowSegmentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowSegment_Invalidquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            trafficFlowSegmentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/segment/json?api-version=1.0&style=absolute&zoom=10&query=test&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficFlowSegment");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            trafficFlowSegmentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                trafficFlowSegmentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowSegment()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            trafficFlowSegmentController.Object.SetProxyOperation(proxyOperation);
            List<string> zoom = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/segment/json?api-version=1.0&style=absolute&zoom=10&query=52.41072,4.84239&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficFlowSegment");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            trafficFlowSegmentController.Object.Request = Request;

            //Call and verify
            trafficFlowSegmentController.Object.Initialize();
            trafficFlowSegmentController.Object.GetQueryStrings().TryGetValue("zoom", out zoom);
            Assert.AreEqual(json, trafficFlowSegmentController.Object.Format);
            Assert.AreEqual(1, trafficFlowSegmentController.Object.Version);
            Assert.AreEqual("10", zoom.First());
        }

        [TestMethod, TestCategory("L0")]
        public void Get_Traffic_Flow_Segment_Test_Null_VerObj()
        {
            //Setup vars
            ProxyException exception = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Inject dependency
            trafficFlowSegmentController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null)).Returns((BaseProxySearch)null);
            trafficFlowSegmentController.Setup(x => x.BuildErrorResponseAndLog(exception)).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = trafficFlowSegmentController.Object.GetTrafficFlowSegment("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null), Times.Once);
            trafficFlowSegmentController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == exception.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void Get_Traffic_Flow_Segment_Test()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            List<string> styleList = new List<string>();
            styleList.Add("s3");
            List<string> zoomList = new List<string>();
            zoomList.Add("12");
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>(StringComparer.OrdinalIgnoreCase);
            querystrings.Add("style", styleList);
            querystrings.Add("zoom", zoomList);

            //Inject dependency
            trafficFlowSegmentController.Object.versionObject = versionObj.Object;
            trafficFlowSegmentController.Object.SetQueryStrings(querystrings);

            //Setup dependency
            versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>())).Returns(baseProxyTraffic.Object);
            baseProxyTraffic.Setup(
                x => x.TrafficFlow(
                    BaseProxyTraffic.TrafficType.FlowSegment,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<Dictionary<string, List<string>>>())).Returns(task.Task);

            //Call and verify
            var output = trafficFlowSegmentController.Object.GetTrafficFlowSegment("");
            versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTraffic.Verify(x => x.TrafficFlow(BaseProxyTraffic.TrafficType.FlowSegment,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Once);
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
