﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TrafficFlowTileControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<TrafficFlowTileController> TrafficFlowTileController;

        private Mock<BaseProxyTraffic> baseProxyTraffic;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            TrafficFlowTileController = new Mock<TrafficFlowTileController>() { CallBase = true };
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            baseProxyTraffic = this.mockRepository.Create<BaseProxyTraffic>(proxyOperation);
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowTile_Nozoom()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            TrafficFlowTileController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/tile/png?api-version=1.0&style=absolute&x=52&y=4&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string format = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", format);
            routeValueDictionary.Add("action", "GetTrafficFlowTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficFlowTileController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "zoom||x||y missing");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficFlowTileController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowTile_Invalidformat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            TrafficFlowTileController.Object.SetProxyOperation(proxyOperation);
            List<string> zoom = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/tile/json?api-version=1.0&style=absolute&zoom=10&x=52&y=4&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string format = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", format);
            routeValueDictionary.Add("action", "GetTrafficFlowTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficFlowTileController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficFlowTileController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficFlowTile()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            TrafficFlowTileController.Object.SetProxyOperation(proxyOperation);
            List<string> zoom = new List<string>();
            List<string> x = new List<string>();
            List<string> y = new List<string>();

            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/flow/tile/png?api-version=1.0&style=absolute&zoom=10&x=52&y=4&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string format = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", format);
            routeValueDictionary.Add("action", "GetTrafficFlowTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficFlowTileController.Object.Request = Request;

            //Call and verify
            TrafficFlowTileController.Object.Initialize();
            TrafficFlowTileController.Object.GetQueryStrings().TryGetValue("zoom", out zoom);
            TrafficFlowTileController.Object.GetQueryStrings().TryGetValue("x", out x);
            TrafficFlowTileController.Object.GetQueryStrings().TryGetValue("y", out y);
            Assert.AreEqual(format, TrafficFlowTileController.Object.Format);
            Assert.AreEqual(1, TrafficFlowTileController.Object.Version);
            Assert.AreEqual("10", zoom.First());
            Assert.AreEqual("52", x.First());
            Assert.AreEqual("4", y.First());
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficFlowTileTest_Null_VerObj()
        {
            //Setup vars
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Inject dependency
            TrafficFlowTileController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null)).Returns((BaseProxySearch)null);
            TrafficFlowTileController.Setup(x => x.BuildErrorResponseAndLog(expectedException)).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = TrafficFlowTileController.Object.GetTrafficFlowTile("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null), Times.Once);
            TrafficFlowTileController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficFlowTile_Test()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            string layer = "";
            List<string> layerList = new List<string>();
            layerList.Add(layer);
            string style = "";
            List<string> styleList = new List<string>();
            styleList.Add(style);
            string zoom = "";
            List<string> zoomList = new List<string>();
            zoomList.Add(zoom);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            string formatstring = "";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("layer", layerList);
            querystrings.Add("style", styleList);
            querystrings.Add("zoom", zoomList);
            querystrings.Add("x", xList);
            querystrings.Add("y", yList);

            //Inject dependency
            TrafficFlowTileController.Object.versionObject = versionObj.Object;
            TrafficFlowTileController.Object.SetQueryStrings(querystrings);

            //Setup dependency
            versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>())).Returns(baseProxyTraffic.Object);
            baseProxyTraffic.Setup(
                p => p.TrafficFlow(
                    BaseProxyTraffic.TrafficType.FlowTile, style, zoom, x, y, formatstring, querystrings)).Returns(task.Task);

            //Call and verify
            var output = TrafficFlowTileController.Object.GetTrafficFlowTile("");
            versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTraffic.Verify(p => p.TrafficFlow(BaseProxyTraffic.TrafficType.FlowTile,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Once);
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
