﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Net.Http;
    using System.Threading.Tasks;
    using System.Web.Http.Routing;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Controllers;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TrafficIncidentControllerTests
    {
        private MockRepository mockRepository;

        private Mock<IVersioning> versionObj;

        private Mock<TrafficIncidentController> TrafficIncidentController;

        private Mock<BaseProxyTraffic> baseProxyTraffic;

        [TestInitialize]
        public void TestInitialize()
        {
            this.mockRepository = new MockRepository(MockBehavior.Strict) { CallBase = true };
            this.versionObj = this.mockRepository.Create<IVersioning>();
            TrafficIncidentController = new Mock<TrafficIncidentController>() { CallBase = true };
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            baseProxyTraffic = this.mockRepository.Create<BaseProxyTraffic>(proxyOperation);
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentDetails_Noboundingbox()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/detail/json?api-version=1.0&style=s3&trafficmodelid=1335294634919&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentDetails");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "boundingbox||boundingzoom||style||trafficmodelid invalid");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficIncidentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentDetails_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/detail/png?api-version=1.0&style=s3&boundingbox=6841263.950712,511972.674418,6886056.049288,582676.925582&trafficmodelid=1335294634919&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentDetails");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficIncidentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentDetails()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            List<string> boundingbox = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/detail/json?api-version=1.0&style=s3&boundingzoom=2&boundingbox=6841263.950712,511972.674418,6886056.049288,582676.925582&trafficmodelid=1335294634919&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentDetails");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;

            //Call and verify
            TrafficIncidentController.Object.Initialize();
            this.TrafficIncidentController.Object.GetQueryStrings().TryGetValue("boundingbox", out boundingbox);
            Assert.AreEqual(json, TrafficIncidentController.Object.Format);
            Assert.AreEqual(1, TrafficIncidentController.Object.Version);
            Assert.AreEqual("6841263.950712,511972.674418,6886056.049288,582676.925582", boundingbox.First());
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentViewport_Noboundingbox()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/viewport/json?api-version=1.0&boundingzoom=2&overviewbox=-939584.4813018347,-23954526.723651607,14675583.153020501,25043442.8958229083&overviewzoom=2©right=true&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentViewport");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "boundingbox||boundingzoom||style||trafficmodelid invalid");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficIncidentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentViewport_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/viewport/png?api-version=1.0&boundingbox=-939584.4813015489,-23954526.723651607,14675583.153020501,25043442.895825107&boundingzoom=2&overviewbox=-939584.4813018347,-23954526.723651607,14675583.153020501,25043442.8958229083&overviewzoom=2©right=true&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentViewport");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficIncidentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentViewport()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            List<string> boundingbox = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/viewport/json?api-version=1.0&boundingbox=-939584.4813015489,-23954526.723651607,14675583.153020501,25043442.895825107&boundingzoom=2&overviewbox=-939584.4813018347,-23954526.723651607,14675583.153020501,25043442.8958229083&overviewzoom=2&copyright=true&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentViewport");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;

            //Call and verify
            TrafficIncidentController.Object.Initialize();
            this.TrafficIncidentController.Object.GetQueryStrings().TryGetValue("boundingbox", out boundingbox);
            Assert.AreEqual(json, TrafficIncidentController.Object.Format);
            Assert.AreEqual(1, TrafficIncidentController.Object.Version);
            Assert.AreEqual("-939584.4813015489,-23954526.723651607,14675583.153020501,25043442.895825107", boundingbox.First());
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentTile_Noquery()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/tile/json?api-version=1.0&style=s3&zoom=12&x=2044&y=1360&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "query param is empty");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficIncidentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentTile_invalidFormat()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/tile/json?api-version=1.0&style=s3&zoom=12&x=2044&y=1360?api-version=1.0&query=pacific standard time&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string json = "json";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", json);
            routeValueDictionary.Add("action", "GetTrafficIncidentTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;
            ProxyException ClientParams = new ProxyException(ProxyException.ExceptionType.ClientParams, "invalid format string");
            ProxyException caughtException = null;

            //Call and verify
            try
            {
                TrafficIncidentController.Object.Initialize();
            }
            catch (ProxyException e)
            {
                caughtException = e;
            }

            if (caughtException != null)
            {
                Assert.AreEqual(ClientParams.InnerException, caughtException.InnerException);
            }
            else
            {
                Assert.IsTrue(false);
            }
        }

        [TestMethod, TestCategory("L0")]
        public void InitializeTest_GetTrafficIncidentTile()
        {
            //Setup vars
            IProxyOperation proxyOperation = new ProxyOperation("Search");
            TrafficIncidentController.Object.SetProxyOperation(proxyOperation);
            List<string> style = new List<string>();
            HttpRequestMessage Request = new HttpRequestMessage(HttpMethod.Get, "https://atlas.microsoft.com/traffic/incident/tile/png?api-version=1.0&style=s3&zoom=12&x=2044&y=1360&query=pacific standard time&subscription-id=GS61xORRtw-KVNYqocNmZp4iZizSFXr2FpebcwyaXDQ&tracking-id=1");
            string png = "png";
            IHttpRoute HttpRoute = new HttpRoute();
            HttpRouteValueDictionary routeValueDictionary = new HttpRouteValueDictionary();
            routeValueDictionary.Add("formatstring", png);
            routeValueDictionary.Add("action", "GetTrafficIncidentTile");
            HttpRouteData routeData = new HttpRouteData(HttpRoute, routeValueDictionary);
            Request.SetRouteData(routeData);
            TrafficIncidentController.Object.Request = Request;

            //Call and verify
            TrafficIncidentController.Object.Initialize();
            this.TrafficIncidentController.Object.GetQueryStrings().TryGetValue("style", out style);
            Assert.AreEqual(png, TrafficIncidentController.Object.Format);
            Assert.AreEqual(1, TrafficIncidentController.Object.Version);
            Assert.AreEqual("s3", style.First());
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficIncidentTileTest_Null_VerObj()
        {
            //Setup vars
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Inject dependency
            TrafficIncidentController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null)).Returns((BaseProxySearch)null);
            TrafficIncidentController.Setup(x => x.BuildErrorResponseAndLog(expectedException)).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = TrafficIncidentController.Object.GetTrafficIncidentTile("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null), Times.Once);
            TrafficIncidentController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficIncidentTile_Test()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            string layer = "";
            List<string> layerList = new List<string>();
            layerList.Add(layer);
            string style = "";
            List<string> styleList = new List<string>();
            styleList.Add(style);
            string zoom = "";
            List<string> zoomList = new List<string>();
            zoomList.Add(zoom);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            string formatstring = "";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("layer", layerList);
            querystrings.Add("style", styleList);
            querystrings.Add("zoom", zoomList);
            querystrings.Add("x", xList);
            querystrings.Add("y", yList);

            //Inject dependency
            TrafficIncidentController.Object.versionObject = versionObj.Object;
            TrafficIncidentController.Object.SetQueryStrings(querystrings);

            //Setup dependency
            versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>())).Returns(baseProxyTraffic.Object);
            baseProxyTraffic.Setup(
                p => p.IncidentTile(
                    BaseProxyTraffic.TrafficType.IncidentTile, style, zoom, x, y, formatstring, querystrings)).Returns(task.Task);

            //Call and verify
            var output = TrafficIncidentController.Object.GetTrafficIncidentTile("");
            versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTraffic.Verify(p => p.IncidentTile(BaseProxyTraffic.TrafficType.IncidentTile,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Once);
            Assert.AreEqual(task.Task.Result, output.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficIncidentViewportTest_Null_VerObj()
        {
            //Setup vars
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Inject dependency
            TrafficIncidentController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null)).Returns((BaseProxySearch)null);
            TrafficIncidentController.Setup(x => x.BuildErrorResponseAndLog(expectedException)).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = TrafficIncidentController.Object.GetTrafficIncidentViewport("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null), Times.Once);
            TrafficIncidentController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficIncidentViewport_Test()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            string layer = "";
            List<string> layerList = new List<string>();
            layerList.Add(layer);
            string style = "";
            List<string> styleList = new List<string>();
            styleList.Add(style);
            string zoom = "";
            List<string> zoomList = new List<string>();
            zoomList.Add(zoom);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            string formatstring = "";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("boundingbox", layerList);
            querystrings.Add("boundingzoom", styleList);
            querystrings.Add("overviewbox", zoomList);
            querystrings.Add("overviewzoom", xList);
            querystrings.Add("copyright", yList);

            //Inject dependency
            TrafficIncidentController.Object.versionObject = versionObj.Object;
            TrafficIncidentController.Object.SetQueryStrings(querystrings);

            //Setup dependency
            versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>())).Returns(baseProxyTraffic.Object);
            baseProxyTraffic.Setup(
                p => p.IncidentViewport(
                    BaseProxyTraffic.TrafficType.IncidentViewport, style, zoom, x, y, layer, formatstring, querystrings)).Returns(task.Task);

            //Call and verify
            var output = TrafficIncidentController.Object.GetTrafficIncidentViewport("");
            versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTraffic.Verify(p => p.IncidentViewport(BaseProxyTraffic.TrafficType.IncidentViewport,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Once);
            Assert.AreEqual(task.Task.Result, output.Result);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficIncidentDetailsTest_Null_VerObj()
        {
            //Setup vars
            ProxyException expectedException = new ProxyException(ProxyException.ExceptionType.ServerError, "failed to initialize proxy object failed");

            //Inject dependency
            TrafficIncidentController.Object.versionObject = this.versionObj.Object;

            //Setup dependency
            this.versionObj.Setup(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null)).Returns((BaseProxySearch)null);
            TrafficIncidentController.Setup(x => x.BuildErrorResponseAndLog(expectedException)).Returns((HttpResponseMessage)null);

            //Call and verify
            var response = TrafficIncidentController.Object.GetTrafficIncidentDetails("");
            this.versionObj.Verify(x => x.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, null), Times.Once);
            TrafficIncidentController.Verify(x => x.BuildErrorResponseAndLog(It.Is<ProxyException>(e => e.Message == expectedException.Message)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void GetTrafficIncidentDetails_Test()
        {
            //Setup vars
            HttpResponseMessage response = new HttpResponseMessage();
            response.Headers.Add("WWW-Authenticate", "Basic");
            TaskCompletionSource<HttpResponseMessage> task = new TaskCompletionSource<HttpResponseMessage>();
            task.SetResult(response);
            string layer = "";
            List<string> layerList = new List<string>();
            layerList.Add(layer);
            string style = "";
            List<string> styleList = new List<string>();
            styleList.Add(style);
            string zoom = "";
            List<string> zoomList = new List<string>();
            zoomList.Add(zoom);
            string x = "";
            List<string> xList = new List<string>();
            xList.Add(x);
            string y = "";
            List<string> yList = new List<string>();
            yList.Add(y);
            string formatstring = "";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            querystrings.Add("style", layerList);
            querystrings.Add("boundingbox", styleList);
            querystrings.Add("boundingzoom", zoomList);
            querystrings.Add("trafficmodelid", xList);
            querystrings.Add("y", yList);

            //Inject dependency
            TrafficIncidentController.Object.versionObject = versionObj.Object;
            TrafficIncidentController.Object.SetQueryStrings(querystrings);

            //Setup dependency
            versionObj.Setup(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>())).Returns(baseProxyTraffic.Object);
            baseProxyTraffic.Setup(
                p => p.IncidentDetails(
                    BaseProxyTraffic.TrafficType.IncidentDetails, style, zoom, x, y, formatstring, querystrings)).Returns(task.Task);

            //Call and verify
            var output = TrafficIncidentController.Object.GetTrafficIncidentDetails("");
            versionObj.Verify(p => p.GetProxyObj(It.IsAny<double>(), BaseProxy.ProxyType.Traffic, It.IsAny<IProxyOperation>()), Times.Once);
            baseProxyTraffic.Verify(p => p.IncidentDetails(BaseProxyTraffic.TrafficType.IncidentDetails,
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<string>(),
                    It.IsAny<Dictionary<string, List<string>>>()), Times.Once);
            Assert.AreEqual(task.Task.Result, output.Result);
        }
    }
}
