﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System.Linq;
    using Microsoft.Azure.LocationServices.Utility;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class JsonUtilityTest
    {
        [TestMethod]
        public void TestFindEmpty()
        {
            var node = JToken.Parse("{}");
            var result = JsonUtility.FindNodes(node, "");
            Assert.IsFalse(result.Any()); 
        }

        [TestMethod]
        public void TestFindNoResult()
        {
            var input = new
            {
                First = "Joe",
                Last = "White"
            };
            var node = JToken.Parse(JsonConvert.SerializeObject(input));
            var result = JsonUtility.FindNodes(node, "");
            Assert.IsFalse(result.Any());
        }

        [TestMethod]
        public void TestFindOneResult()
        {
            var input = new
            {
                First = "Joe",
                Last = "White"
            };
            var node = JToken.Parse(JsonConvert.SerializeObject(input));
            var result = JsonUtility.FindNodes(node, "first");
            Assert.AreEqual(1, result.Count()); 
        }

        [TestMethod]
        public void TestFindMultiResult()
        {
            var input = new
            {
                First = "Joe",
                Last = "White"
            };
            var node = JToken.Parse(JsonConvert.SerializeObject(input));
            var result = JsonUtility.FindNodes(node, new[] { "first", "last" });
            Assert.AreEqual(2, result.Count());
        }

        [TestMethod]
        public void TestFindNestedResult()
        {
            var input = new
            {
                First = "Joe",
                Last = "White",
                Child = new { First = "Jack", Last = "White" }
            };
            var node = JToken.Parse(JsonConvert.SerializeObject(input));
            var result = JsonUtility.FindNodes(node, new[] { "first" });
            Assert.AreEqual(2, result.Count());
        }
    }
}
