﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;
    using Newtonsoft.Json.Linq;

    [TestClass]
    public class MicrosoftV1GeofenceProxyTests
    {
        private Mock<MicrosoftV1GeofenceProxy> baseProxyObj;

        [TestInitialize]
        public void TestInitialize()
        {
            IProxyOperation proxyOperation = new ProxyOperation("Geofence");
            baseProxyObj = new Mock<MicrosoftV1GeofenceProxy>(proxyOperation) { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void CalculateResponseDistance_InsideGeofenceWithoutSearchRadius_ShouldSetDefaultDistanceToResponse()
        {
            // Arrange
            baseProxyObj.Object.responseBody = new GeofenceResponseBody();
            var geometry = new Geometry();

            // Act
            baseProxyObj.Object.CalculateResponseDistance(geometry, 1, 1, 100);

            // Assert
            Assert.AreEqual(1, baseProxyObj.Object.responseBody.geometries.Count);
            Assert.AreEqual(MicrosoftV1GeofenceProxy.DefaultInsideDistanceValue, baseProxyObj.Object.responseBody.geometries[0].distance);
        }

        [TestMethod, TestCategory("L0")]
        public void CalculateResponseDistance_InsideGeofenceWithinSearchRadius_ShouldSetNegativeDistanceToResponse()
        {
            // Arrange
            baseProxyObj.Object.responseBody = new GeofenceResponseBody();
            var distance = 1;
            var searchRaiuds = 5;
            var radius = 3;
            var geometry = new Geometry();

            // Act
            baseProxyObj.Object.CalculateResponseDistance(geometry, distance, searchRaiuds, radius);

            // Assert
            Assert.AreEqual(1, baseProxyObj.Object.responseBody.geometries.Count);
            Assert.AreEqual(-Math.Abs(radius - distance), baseProxyObj.Object.responseBody.geometries[0].distance);
        }

        [TestMethod, TestCategory("L0")]
        public void CalculateResponseDistance_OnGeofenceBorder_ShouldSetZeroDistanceToResponse()
        {
            // Arrange
            baseProxyObj.Object.responseBody = new GeofenceResponseBody();
            var distance = 3;
            var searchRaiuds = 2;
            var radius = 3;
            var geometry = new Geometry();

            // Act
            baseProxyObj.Object.CalculateResponseDistance(geometry, distance, searchRaiuds, radius);

            // Assert
            Assert.AreEqual(1, baseProxyObj.Object.responseBody.geometries.Count);
            Assert.AreEqual(0, baseProxyObj.Object.responseBody.geometries[0].distance);
        }

        [TestMethod, TestCategory("L0")]
        public void CalculateResponseDistance_OutsideGeofenceWithinSearchRadius_ShouldSetZeroDistanceToResponse()
        {
            // Arrange
            baseProxyObj.Object.responseBody = new GeofenceResponseBody();
            var distance = 5;
            var searchRaiuds = 3;
            var radius = 3;
            var geometry = new Geometry();

            // Act
            baseProxyObj.Object.CalculateResponseDistance(geometry, distance, searchRaiuds, radius);

            // Assert
            Assert.AreEqual(1, baseProxyObj.Object.responseBody.geometries.Count);
            Assert.AreEqual(Math.Abs(radius - distance), baseProxyObj.Object.responseBody.geometries[0].distance);
        }

        [TestMethod, TestCategory("L0")]
        public void CalculateResponseDistance_OutsideGeofenceWithoutSearchRadius_ShouldNotAddGeometryToResponse()
        {
            // Arrange
            baseProxyObj.Object.responseBody = new GeofenceResponseBody();
            var distance = 100;
            var searchRaiuds = 3;
            var radius = 3;
            var geometry = new Geometry();

            // Act
            baseProxyObj.Object.CalculateResponseDistance(geometry, distance, searchRaiuds, radius);

            // Assert
            Assert.AreEqual(0, baseProxyObj.Object.responseBody.geometries.Count);
        }
    }
}