﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.Models.Map;
    using Microsoft.Azure.LocationServices.Utility;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class MicrosoftV2MapProxyTests
    {
        private Mock<MicrosoftV2MapProxy> baseProxyObj;
        ProxyHostSettings settings = new ProxyHostDevSettings();

        [TestInitialize]
        public void TestInitialize()
        {
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            baseProxyObj = new Mock<MicrosoftV2MapProxy>(proxyOperation, settings) { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void VectorTileTest()
        {
            //Setup vars            
            string layer = "3";
            string style = "s3";
            string zoom = "12";
            string x = "2044";
            string y = "1360";
            string formatString = "pbf";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();
            HttpRequestMessage outputUri = new HttpRequestMessage(HttpMethod.Get, $"http://{settings.MdpApiFqdn}/tile/{layer}/{style}/{zoom}/{x}/{y}.{formatString}?api-version={settings.MdpApiVersion}");

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)));

            //Call and verify
            Task<HttpResponseMessage> response = baseProxyObj.Object.MapTileVector(layer, style, zoom, x, y, formatString, querystrings);
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void RasterTileTest()
        {
            //Setup vars            
            string layer = "3";
            string style = "s3";
            string zoom = "12";
            string x = "2044";
            string y = "1360";
            string formatString = "png";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();

            HttpRequestMessage outputUri = new HttpRequestMessage(HttpMethod.Get, $"http://{settings.MdpApiFqdn}/tile/{layer}/{style}/{zoom}/{x}/{y}.{formatString}?api-version={settings.MdpApiVersion}");

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)));

            //Call and verify
            Task<HttpResponseMessage> response = baseProxyObj.Object.MapTileRaster(layer, style, zoom, x, y, formatString, querystrings);
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)), Times.Once);
        }
    }
}
