﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TomTomV1MapProxyTests
    {
        private Mock<TomTomV1MapProxy> baseProxyObj;

        [TestInitialize]
        public void TestInitialize()
        {
            IProxyOperation proxyOperation = new ProxyOperation("Map");
            baseProxyObj = new Mock<TomTomV1MapProxy>(proxyOperation) { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void MapTileTest()
        {
            //Setup vars
            HttpRequestMessage outputUri = new HttpRequestMessage(HttpMethod.Get, "https://api.azure.tomtom.com/map/1/tile/3/main-azure/12/2044/1360.png?key=");
            string layer = "3";
            string style = "s3";
            string zoom = "12";
            string x = "2044";
            string y = "1360";
            string formatString = "png";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)));

            //Call and verify
            Task<HttpResponseMessage> response = baseProxyObj.Object.MapTileRaster(layer, style, zoom, x, y, formatString, querystrings);
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void MapStaticTest()
        {
            //Setup vars
            HttpRequestMessage outputUri = new HttpRequestMessage(HttpMethod.Get, "https://api.azure.tomtom.com/map/1/staticimage?key=&format=json&watermark=true&style=main-azure");
            string formatString = "json";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)));

            //Call and verify
            Task<HttpResponseMessage> response = baseProxyObj.Object.MapStatic(formatString, querystrings);
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)), Times.Once);
        }
    }
}
