﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TomTomV1RouteProxyTests
    {
        private Mock<TomTomV1RouteProxy> baseProxyObj;

        [TestInitialize]
        public void TestInitialize()
        {
            IProxyOperation proxyOperation = new ProxyOperation("Route");
            baseProxyObj = new Mock<TomTomV1RouteProxy>(proxyOperation) { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void CalculateRouteTest()
        {
            //Setup vars
            HttpRequestMessage outputRequest = new HttpRequestMessage(HttpMethod.Get, "https://api.azure.tomtom.com/routing/1/calculateRoute/3/xml?key=");
            string query = "3";
            string formatString = "xml";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputRequest.RequestUri)));

            //Call and verify
            baseProxyObj.Object.RouteProxy(BaseProxyRoute.RouteType.Normal, query, formatString, querystrings);;
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputRequest.RequestUri)));
        }
    }
}
