﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace ProxyServerUnitTests
{
    using System;
    using System.Collections.Generic;
    using System.Net.Http;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    using Moq;

    [TestClass]
    public class TomTomV4TrafficProxyTests
    {
        private Mock<TomTomV4TrafficProxy> baseProxyObj;

        [TestInitialize]
        public void TestInitialize()
        {
            IProxyOperation proxyOperation = new ProxyOperation("Traffic");
            baseProxyObj = new Mock<TomTomV4TrafficProxy>(proxyOperation) { CallBase = true };
        }

        [TestMethod, TestCategory("L0")]
        public void IncidentTileTest()
        {
            //Setup vars
            HttpRequestMessage outputUri = new HttpRequestMessage(HttpMethod.Get, "https://api.azure.tomtom.com/traffic/map/4/tile/incidents/s3/12/2044/1360.png?key=");
            string style = "s3";
            string zoom = "12";
            string x = "2044";
            string y = "1360";
            string formatString = "png";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)));

            //Call and verify
            Task<HttpResponseMessage> response = baseProxyObj.Object.IncidentTile(BaseProxyTraffic.TrafficType.IncidentTile, style, zoom, x, y, formatString, querystrings);
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)), Times.Once);
        }

        [TestMethod, TestCategory("L0")]
        public void TrafficFlowTest()
        {
            //Setup vars
            HttpRequestMessage outputUri = new HttpRequestMessage(HttpMethod.Get, "https://api.azure.tomtom.com/traffic/services/4/flowSegmentData/absolute/10/json?key=&point=52.41072,4.84239");
            string style = "absolute";
            string zoom = "10";
            string x = "52.41072,4.84239";
            string y = "";
            string formatString = "json";
            Dictionary<string, List<string>> querystrings = new Dictionary<string, List<string>>();

            //Setup dependency
            baseProxyObj.Setup(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)));

            //Call and verify
            Task<HttpResponseMessage> response = baseProxyObj.Object.TrafficFlow(BaseProxyTraffic.TrafficType.FlowSegment, style, zoom, x, y, formatString, querystrings);
            baseProxyObj.Verify(p => p.ExecuteRequest(It.Is<HttpRequestMessage>(s => s.RequestUri == outputUri.RequestUri)), Times.Once);
        }
    }
}
