﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Shared
{
    using System;
    using System.Threading.Tasks;
    using Microsoft.WindowsAzure.Storage.Blob;

    public interface IBlobHelper
    {
        /// <summary>
        /// Initiates an asynchronous operation to set the expiration time for the blob. 
        /// The blob will be deleted by the <see cref="BlobExpirationWorker"/> when it's expired. 
        /// NOTE : The current blob cleanup service needs to know the container name. 
        /// LBSBatchService.GetBatchServiceBlobExpirationWorker method need to be updated.
        /// It currently works only with the LBS subscription that is used by batch service in production. Containers in other subscriptions can't be cleaned.
        /// </summary>
        /// <param name="blob">Blob to set the expiration for</param>
        /// <param name="expirationDateTimeInUtc">DateTime when the blob expires</param>
        Task SetBlobExpiryDate(ICloudBlob blob, DateTime expirationDateTimeInUtc);
    }
}