﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Shared
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Linq;

    public class SharedUtilities
    {
        public const string ExpirationMapBlobName = "blobexpirationmap";
        public const int ResponseBlobExpiryTimeInDays = 14;
        public const string BatchServiceConnectionStringTemplateKey = "malbsbatchservice";
        public const string BatchServiceSasTokenKey = "malbsbatchservice-crudall";
        public const string BatchServicePrimaryQueueName = "batchapirequests";
        public const string RouteMatrixServicePrimaryQueueName = "routematrixrequests";
        public const string RouteMatrixServiceResultsContainerName = "routematrixresultscontainer";
        public const string BatchServiceResponseStatusMetadataKey = "HasResponseData";
        public const string BatchServiceRequestFailedMetadataKey = "HasRequestFailed";
        public const string BatchApiContainerName = "batchapicontainer";

        /// <summary>
        /// This is used in logging to the billing. 
        /// Both for quick matrix calls and long running as well.
        /// </summary>
        public const string PostMatrixMethodName = "PostMatrixRouteCalculate";

        /// <summary>
        /// Gets the codes needed for billing.
        /// </summary>
        /// <param name="responseString"></param>
        /// <returns></returns>
        public static List<int> GetResponseCodesFromTomTomRouteMatrixResponseContent(string responseString)
        {
            try
            {
                var jsonObj = JsonConvert.DeserializeObject<JObject>(responseString);
                var matrixArray = jsonObj.Properties().Where(p => p.Name == "matrix").SelectMany(p => p.Values());
                return matrixArray.SelectMany(m => m.Select(a => ((JObject)a).Properties().Where(rp => rp.Name == "statusCode").Select(e => (int)e.Value).FirstOrDefault())).ToList();
            }
            catch (Exception ex)
            {
                throw new FormatException($"Couldn't parse RouteMatrix response from TT.", ex);
            }
        }
    }
}
