﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.AzLocApiRunner.Utils
{
    using System;
    using System.Security.Cryptography.X509Certificates;
    using System.Threading.Tasks;
    using Microsoft.Azure.KeyVault;
    using Microsoft.IdentityModel.Clients.ActiveDirectory;

    public class KeyVaultUtils
    {
        private readonly X509Certificate2 certPfx;
        private readonly string clientId;

        public KeyVaultUtils(string clientId, X509Certificate2 pfxCert)
        {
            this.certPfx = pfxCert;
            this.clientId = clientId;
        }

        private static ClientAssertionCertificate AssertionCertificate { get; set; }

        public async Task<string> AcquireKeyVaultSecret(string secretUri)
        {            
            if (string.IsNullOrEmpty(secretUri))
            {
                throw new ArgumentNullException(nameof(secretUri), "Unable to load keyvault secret. Check Runner Instance keytype");
            }

            KeyVaultClient keyVaultClient = null;

            if (this.certPfx != null)
            {
                Console.WriteLine("Attempting to acquire secret via certificate");
                GetAssociationCert(this.clientId, this.certPfx);
                keyVaultClient = new KeyVaultClient(GetTokenviaCert);
            }
            else
            {
                throw new ArgumentNullException(nameof(certPfx), $"Certificate Authentication to {secretUri} was not able to be established because of certificate state.");
            }

            var secret = await keyVaultClient.GetSecretAsync(secretUri);

            return secret.Value;
        }
        
        private static async Task<string> GetTokenviaCert(string authority, string resource, string scope)
        {
            var context = new AuthenticationContext(authority, TokenCache.DefaultShared);
            var result = await context.AcquireTokenAsync(resource, AssertionCertificate);
            return result?.AccessToken;
        }

        private static void GetAssociationCert(string clientId, X509Certificate2 certPfx)
        {
            AssertionCertificate = new ClientAssertionCertificate(clientId, certPfx);
        }
    }
}
