﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
namespace Microsoft.Azure.LocationServices.BillingAPI.Meter
{
    using System.Collections.Generic;

    using Microsoft.Azure.LocationServices.BillingAPI.Utility;

    public class Meter
    {
        private static readonly object _syncLock = new object();
        private static Meter _instance;

        private BillingHostSettings hostSettings;

        private Meter()
        {
        }

        public static Meter Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (_syncLock)
                    {
                        if (_instance == null)
                        {
                            _instance = new Meter();
                        }
                    }
                }

                return _instance;
            }
        }

        public IReadOnlyDictionary<string, string> ServiceApiMethodMap => this.hostSettings.ServiceApiToMethodMap;

        public IReadOnlyDictionary<string, int> MethodTransactionMap => this.hostSettings.MethodTransactionCountMap;

        public IReadOnlyDictionary<string, string> MethodMeterMap => this.hostSettings.MethodMeterMap;

        public IReadOnlyDictionary<string, IReadOnlyDictionary<string, string>> SkuMeterIdMap => this.hostSettings.SkuMeterIdMap;

        public void Initialize()
        {
            this.hostSettings = BillingHostSettings.CreateFromEnvironment();
        }
    }
}
