<#
 .SYNOPSIS
    Script to be used to spawn up new CI clusters.

 .DESCRIPTION
    Script to be used to spawn up new CI clusters.
#>

Param
(
    [Parameter(Mandatory=$True)]
    [string] $clusterName,
    
	[Parameter(Mandatory=$False)]    
    [string] $subscriptionId = 'b465bdfb-8975-44c9-86ec-36b250c9b942' # Azure_LBS_CI subscription
)

$clusterNameRegex = "azuremaps-ci\-[0-9]{2}";

if ($clusterName -notmatch $clusterNameRegex) {
	throw "Invalid cluster name. Cluster names should begin with 'azuremaps-ci-'."
}

### Define constants here
$ResourceGroupLocation = 'East US'
###

$modulePath = $PSScriptRoot + "\cloud-automation.psm1"
Import-Module -Name $modulePath -Force

# Create a new cluster resource group
New-CIResourceGroup -ResourceGroupName $clusterName -Location $ResourceGroupLocation

# Create persistent key vault for storing secrets
New-PersistentKeyVault -ResourceGroupName $clusterName -ResourceGroupLocation $ResourceGroupLocation

# Set up an event hub for APIM logging
New-CIApimEventHub -ResourceGroupName $clusterName -Location $ResourceGroupLocation

# Deploy an APIM instance
New-CIApiManagementInstance -ResourceGroupName $clusterName -Location $ResourceGroupLocation -SubscriptionId $subscriptionId

# Sync APIM Git configuration with that of CI APIM instance c-atlas.
Invoke-SyncClusterApimWithCI -ClusterName $clusterName

# Put the APIM subscription key in the persistent key vault
Set-ApimSubscriptionKeyInVault -ClusterName $clusterName