﻿Param
(
    [Parameter(Mandatory=$True, ParameterSetName='useEnv')]
    [ValidateSet('p', 's', 't', 'c')]
    [string]$env,

    [Parameter(Mandatory=$True, ParameterSetName='useRes')]
    $subscriptionId = '21a9967a-e8a9-4656-a70b-96ff1c4d05a0', #DEV IOT_AzureLBS

    [Parameter(Mandatory=$True, ParameterSetName='useRes')]
    $resourceGroupName = 'Global',

    [Parameter(Mandatory=$True, ParameterSetName='useRes')]
    $parametersFilePath = '..\NEW_CREATE\parameters\APIMParameters.json'
)

<#
 .SYNOPSIS
    One script to modify Azure location based service APIM instance in different slot

 .DESCRIPTION
    1. Use env or resource group and parametersFilePath to point to your APIM resource
    2. Deploy the arm template with parameter to override the target APIM 

 .env
    The deployment environment slot name

 .PARAMETER subscriptionId
    The subscription id where the template will be deployed.

 .PARAMETER resourceGroupName
    The resource group where the template will be deployed. Can be the name of an existing or a new resource group.

 .PARAMETER parametersFilePath
    Optional, path to the parameters file. Defaults to parameters.json. If file is not found, will prompt for parameter values based on template.

 .Example
 .\ModifyApim.ps1 -env s
#>

#******************************************************************************
# Script body
# Execution begins here
#******************************************************************************

$nonEmptyString = "NonEmptyString"
 
$modulePath = $PSScriptRoot + "\..\..\Utils\lbsSetupSupport.psd1"
Import-Module -Name $modulePath -Force

if($env)
{
    $subscriptionId = Get-lbsEnvironmentSubscriptionId -env $env;
    $subscription = Select-lbsSubscriptionWithQuickLogin -subscriptionId $subscriptionId

    switch($env.ToLower())
    {
        'p' { $parametersFilePath = '..\PROD_BN\Parameters\APIMParameters.json' } # PROD
        's' { $parametersFilePath = '..\STG_BY\Parameters\APIMParameters.json' } # STG
        't' { $parametersFilePath = '..\TEST_AM\Parameters\APIMParameters.json' } # TEST
        'c' { $parametersFilePath = '..\CI\Parameters\APIMParameters.json' } # CI
    }

    $pfxBytes = Get-CertBytesByEnv -env $env -pfxPassword $nonEmptyString
}

$deploymentName = 'apimApiDeployment' +  (New-Guid).ToString()

.\DeployAPIM.ps1 `
-subscriptionId $subscriptionId `
-resourceGroupName $resourceGroupName  `
-deploymentName $deploymentName `
-pfxBytes $pfxBytes `
-certificatePassword $nonEmptyString `
-parametersFilePath $parametersFilePath `
-isAPIMExisting $true