﻿# ---------------------------------------------------------------------------------------------------------------------------------------------
# download to *.cer file to D:\Certificates
# ---------------------------------------------------------------------------------------------------------------------------------------------

Param
(
    [Parameter(Mandatory=$True)]
    [ValidateSet('p', 's', 't', 'c')]
    [string]$env,

    [Parameter(Mandatory=$True)]
    [ValidateSet('cluster', 'client', 'secrets', 'ma', 'apim', 'tls')]
    [string]$certType,

    [Parameter(Mandatory=$True)]
    [string]$pfxFilePwd,

    [Parameter(Mandatory=$False)]
    [string]$downloadFolder = "D:\SSLAdminCertificates"
)

function DownloadCert
{
    Param (
        [String] $vaultName,
        [String] $certName
    )

    $kvSecret = Get-AzureKeyVaultSecret -VaultName $vaultName -Name $certName
    $kvSecretBytes = [System.Convert]::FromBase64String($kvSecret.SecretValueText)
    $certCollection = New-Object System.Security.Cryptography.X509Certificates.X509Certificate2Collection
    $certCollection.Import($kvSecretBytes,$null,[System.Security.Cryptography.X509Certificates.X509KeyStorageFlags]::Exportable)
    $protectedCertificateBytes = $certCollection.Export([System.Security.Cryptography.X509Certificates.X509ContentType]::Pkcs12, $pfxFilePwd)
    $outPath = [System.IO.Path]::Combine($downloadFolder, $certName+".pfx")
    [System.IO.File]::WriteAllBytes($outPath, $protectedCertificateBytes)

    Write-Host "Downloaded:" $outPath -foregroundcolor Green
}

$modulePath = $PSScriptRoot + "\..\..\Utils\lbsSetupSupport.psd1"
Import-Module -Name $modulePath -Force

$ErrorActionPreference = "Stop"

if ([System.IO.Directory]::Exists($downloadFolder) -eq $false) {
    Write-Host "creating directory " $downloadFolder;
    [System.IO.Directory]::CreateDirectory($downloadFolder);
}

$subscriptionId = Get-lbsEnvironmentSubscriptionId -env $env;
$subscription = Select-lbsSubscriptionWithQuickLogin -subscriptionId $subscriptionId

if ($certType -eq 'apim') {
    $vaultName = Get-lbsGlobalVaultName -env $env
    $certName = Get-lbsGlobalCertificateName -env $env -sfCluster 'svc' -certType $certType
    DownloadCert -vaultName $vaultName -certName $certName 
} else {
    $sfClusters = Get-lbsServiceFabricClusters
    foreach ($sfCluster in $sfClusters) 
    {
		$regions = Get-lbsEnvironmentRegions -env $env -sfCluster $sfCluster
        foreach ($region in $regions) 
        {
            $vaultName = Get-lbsVaultName -env $env -sfCluster $sfCluster -region $region
            $certName = Get-lbsCertificateName -env $env -sfCluster $sfCluster -region $region -certType $certType
            DownloadCert -vaultName $vaultName -certName $certName 
        }
    }
}