﻿Param
(
    [Parameter(Mandatory=$True)]
    [ValidateSet('p', 's', 't', 'c')]
    [string]$env,

    [Parameter(Mandatory=$False)]
    [ValidateSet('true', 'false')]
    [string]$forParamGen = 'true'
)

function GetJsonKeyValue
{
    Param (
        [String] $k,
        [String] $v
    )

    $s = [System.String]::Format("    ""{0}"": {{ ""value"": ""{1}"" }},",  $k, $v)
    return $s
}

$modulePath = $PSScriptRoot + "\..\..\Utils\lbsSetupSupport.psd1"
Import-Module -Name $modulePath -Force
$ErrorActionPreference = "Stop"
$subscriptionId = Get-lbsEnvironmentSubscriptionId -env $env;
$subscription = Select-lbsSubscriptionWithQuickLogin -subscriptionId $subscriptionId

$paramGen = $False
if ($forParamGen.ToLower() -eq "true")
{
    $paramGen = $True
}

$sfClusters = Get-lbsServiceFabricClusters
$certTypes = Get-lbsCertificateTypes
$doubleQuote = '"'

foreach ($sfCluster in $sfClusters) 
{
	$regions = Get-lbsEnvironmentRegions -env $env -sfCluster $sfCluster

    foreach ($region in $regions) 
    {
        $vaultName = Get-lbsVaultName -env $env -sfCluster $sfCluster -region $region
        Write-Host "    // ------------------------------------------------------------------------------------------" -foregroundcolor Cyan
        Write-Host "    // Don't copy directly, match them to ARM template bootstrap array or key vault extension array -- "$vaultName -foregroundcolor Cyan
        Write-Host "    // ------------------------------------------------------------------------------------------" -foregroundcolor Cyan
        foreach ($certType in $certTypes) 
        {
            $certName = Get-lbsCertificateName -env $env -sfCluster $sfCluster -region $region -certType $certType
            $cert = Get-AzureKeyVaultCertificate -VaultName $vaultName -Name $certName
            if ($cert -eq $null) {
                Write-Error 'Cannot find cert:' $certName 'in' $vaultName
            }
            if ($paramGen)
            {
                $k = $certType + 'CertificateUrl'
                Write-Host (GetJsonKeyValue -k $k -v $cert.SecretId)
				$certTypeLower = $certType.ToLower()
                if ($certTypeLower -eq 'client'){
						Write-Host "    // These values should change and be updated when the Intermediate CA issuer certificate changes." -foregroundcolor Cyan
						Write-Host "    // TODO automate this via dsms.dsms.core.windows.net/dsms/issuercertificates" -foregroundcolor Cyan
						Write-Host "    // For now they can be monitored for SSLAdmin: https://www.microsoft.com/pki/mscorp/cps/default.htm and AME: https://onecert-msazurecloud.msappproxy.net/Home/Help" -foregroundcolor Cyan						
						$v = "97EFF3028677894BDD4F9AC53F789BEE5DF4AD86,417E225037FBFAA4F95761D5AE729E1AEA7E3A42,54D9D20239080C32316ED9FF980A48988F4ADF2D,8A38755D0996823FE8FA3116A277CE446EAC4E99,AD898AC73DF333EB60AC1F5FC6C4B2219DDB79B7"
						Write-Host (GetJsonKeyValue -k 'clusterCertificateIssuerThumbprints' -v $v)
						Write-Host (GetJsonKeyValue -k 'clusterClientCertificateCommonName' -v $cert.Certificate.SubjectName.Name.Substring(3))
					}
					elseif($certTypeLower -eq 'cluster'){
						Write-Host (GetJsonKeyValue -k 'clusterCertificateCommonName' -v $cert.Certificate.SubjectName.Name.Substring(3))
					}
					else{
						$k = $certType + 'CertificateThumbprint'
						Write-Host (GetJsonKeyValue -k $k -v $cert.Certificate.Thumbprint)
					}
            } else {
                $cert;
            }
        }

        $certType = 'apim'
        $globalCertName = Get-lbsGlobalCertificateName -env $env -sfCluster $sfCluster -certType $certType
        $secret = Get-AzureKeyVaultSecret -VaultName $vaultName -Name $globalCertName
        if ($secret -eq $null) {
            Write-Error 'Cannot find secret:' $secret 'in' $vaultName
        }
        if ($paramGen)
        {            
            $k = $certType + 'CertificateUrl'
            Write-Host (GetJsonKeyValue -k $k -v $secret.Id)
        } else {
            $secret;
        }
    }
}
