﻿
param(
    [Parameter(Mandatory=$True)]
    [ValidateSet('p', 's', 't', 'c')]
    [string]$srcEnv,

    [Parameter(Mandatory=$True)][string]$srcApimResGroupName,
    [Parameter(Mandatory=$True)][string]$srcApimName,

    [Parameter(Mandatory=$True)][string]$backupResGroupName,
    [Parameter(Mandatory=$True)][string]$backupStorageAcctName,
    [Parameter(Mandatory=$True)][string]$backupName,
    [Parameter(Mandatory=$True)][ValidateSet('true', 'false')][string]$useExistingBackup,

    [Parameter(Mandatory=$True)]
    [ValidateSet('s', 't', 'c')] #purposely remove PROD as a target
    [string]$destEnv,

    [Parameter(Mandatory=$True)][string]$destApimResGroupName,

    [Parameter(Mandatory=$True)][string]$destApimName
)

try
{
    $modulePath = $PSScriptRoot + "\lbsSetupSupport.psd1"
    Import-Module -Name $modulePath -Force

    $srcSubscriptionId = Get-lbsEnvironmentSubscriptionId -env $srcEnv
    Login-AzureRmAccount -SubscriptionId $srcSubscriptionId
    Select-AzureRmSubscription –SubscriptionId $srcSubscriptionId

    $bakStorageKey1 = (Get-AzureRmStorageAccountKey -ResourceGroupName $backupResGroupName -Name $backupStorageAcctName).Value[0]
    $storageAcctCtxt = New-AzureStorageContext -StorageAccountName $backupStorageAcctName -StorageAccountKey $bakStorageKey1;
    $containerName = 'backups'

    if ($useExistingBackup.ToLower() -eq 'false')
    {
        Write-Host "Backing up APIM" $srcApimResGroupName"."$srcApimName "to" $backupResGroupName"."$backupStorageAcctName"."$containerName"."$backupName;
        Backup-AzureRmApiManagement -ResourceGroupName $srcApimResGroupName -Name $srcApimName -StorageContext $storageAcctCtxt -TargetContainerName $containerName -TargetBlobName $backupName   
    }

    $destSubscriptionId = Get-lbsEnvironmentSubscriptionId -env $destEnv
    Write-Host "Restoring APIM" $destApimResGroupName"."$destApimName "from" $backupResGroupName"."$backupStorageAcctName"."$containerName"."$backupName;
    Debug-lbsPromptToContinue
    Login-AzureRmAccount -SubscriptionId $destSubscriptionId
    Select-AzureRmSubscription –SubscriptionId $destSubscriptionId

    Restore-AzureRmApiManagement -ResourceGroupName $destApimResGroupName -Name $destApimName -StorageContext $storageAcctCtxt -SourceContainerName $containerName -SourceBlobName $backupName -Verbose 
}
catch
{
    Write-Error $_.Exception.ToString()
    Exit
}