﻿# ##########################################################################################################################################################
# The script removes application types from a cluster. By default, it only removes unused/old application types. It's used for two purposes:
#
# 1. After EV2 deployment, we need to unregister and delete old application types. EV2 will not do that for us.
# 2. To clean up the cluster before starting a CI build. 
#
# The script is executed in the context of VSTS Service Fabric PowerShell task so cluster connection is already established.
# ##########################################################################################################################################################
Param
(
	[Parameter(Mandatory=$False)]
	[ValidateSet('true', 'false')]
	[string]$removeAll  = "false" # if true, remove all application types, not only unused ones
)

# get all application types in the cluster
$applicationTypes = Get-ServiceFabricApplicationType

foreach($applicationType in $applicationTypes)
{	
	if (($removeAll.ToLower() -eq "true") -and ($applicationType.ApplicationTypeName.ToLower() -ne "fabricmonitoringserviceapplication")) { # we don't want to remove MA agent
		Write-Host "Removing applications under application type '$($applicationType.ApplicationTypeName)', version '$($applicationType.ApplicationTypeVersion)'"
		
		$applications = Get-ServiceFabricApplication -ApplicationTypeName $applicationType.ApplicationTypeName

		# remove all applications
		foreach($application in $applications)
		{
			Write-Host "Removing application: $($application.ApplicationName)"
			Remove-ServiceFabricApplication -ApplicationName $application.ApplicationName -Force
		}
	}
	
	Write-Host "Trying application type '$($applicationType.ApplicationTypeName)', version '$($applicationType.ApplicationTypeVersion)'"
	try 
	{
		Unregister-ServiceFabricApplicationType -ApplicationTypeName $applicationType.ApplicationTypeName -ApplicationTypeVersion $applicationType.ApplicationTypeVersion -Force
	} 
	catch [System.Fabric.FabricException] 
	{
		Write-Host "Skipping unregistering application type '$($applicationType.ApplicationTypeName)', version '$($applicationType.ApplicationTypeVersion)' because it has app(s) in use"
	}
}