<#
 .SYNOPSIS
    Deploys a custom script to a vmss instance

 .PARAMETER subscriptionName
    The subscription name where the target VMSS will be deployed.

 .PARAMETER resourceName
    The name of the VMSS resource

 .PARAMETER resourceGroupName
    The resource group where the VMSS is.

 .PARAMETER uri
    The storage account + script file + sasToken uri which will be used to run the script. 
	ex: "https://albsdevsetup.blob.core.windows.net/scripts/ToggleWindowsUpdate.ps1?{sasToken}"

  .PARAMETER scriptFileName
    The name of the script which to run as part of this deployment
	ex: filename.ps1

  .PARAMETER scriptArguments
    The arguments which the script should accept
	ex: "-notificationLevel 1"
#>

param(
	[Parameter(Mandatory=$True)] 	
	[ValidateSet("Azure_LBS_CI", "Azure_LBS_Test", "IOT_AzureLBS", "Azure_LBS_Staging", "Azure_LBS_Production")]
	[string]
	$subscriptionName,

	[Parameter(Mandatory=$True)] 	
	[string]
	$resourceName,

	[Parameter(Mandatory=$True)]
	[ValidateSet("svc-eastus", "rp-eastus", "jobs-eastus", "svc-westus2", "rp-westus2", "svc-westeurope", "rp-westeurope")]
	[string]
	$resourceGroupName,

	[Parameter(Mandatory=$True)]
	[string]
	$uri,

	[Parameter(Mandatory=$True)]
	[string]
	$scriptFileName,

	[Parameter(Mandatory=$True)]
	[string]
	$scriptArguments
)

#******************************************************************************
# Script body
# Execution begins here
#******************************************************************************
Login-AzureRmAccount -SubscriptionName $subscriptionName

$vmss = Get-AzureRmVmss -ResourceGroupName $resourceGroupName -VMScaleSetName $resourceName
$publicSettings = @{
    "fileUris" = (,"$uri")
    "commandToExecute" = "powershell -ExecutionPolicy Unrestricted -File $scriptFileName $scriptArguments"
}

Write-Host "  ==> Add-AzureRmVmssExtension"
$extName = "customScript"
Add-AzureRmVmssExtension -VirtualMachineScaleSet $vmss `
    -Name $extName `
    -Publisher "Microsoft.Compute" `
    -Type "CustomScriptExtension" `
    -TypeHandlerVersion 1.8 `
    -Setting $publicSettings

$vmss = Update-AzureRmVmss -ResourceGroupName $resourceGroupName -Name $resourceName -VirtualMachineScaleSet $vmss -Verbose -ErrorAction Stop
Write-Host "  ==> Completed Add"
Write-Host "  ==> Remove-AzureRmVmssExtension"
$vmss = Get-AzureRmVmss -ResourceGroupName $resourceGroupName -VMScaleSetName $resourceName
Remove-AzureRmVmssExtension -VirtualMachineScaleSet $vmss -Name $extName
$vmss = Update-AzureRmVmss -ResourceGroupName $resourceGroupName -Name $resourceName -VirtualMachineScaleSet $vmss -Verbose
Write-Host "  ==> Completed Remove"