# ARM has a maximum deployment history of 800 per-resource group. If that limit is reached, the deployment will fail. This script goes through all resource groups in a subscription and clear the history.

Param
(
    [Parameter(Mandatory=$False)]
    [string]$SubscriptionName, # can be null if the script is executed as part of Azure PowerShell (e.g. in VSTS task)
    
    [Parameter(Mandatory=$False)]
    [ValidateRange(0, 800)]
    [int]$HistoryToKeep = 100 # e.g. 100 means delete all history after the 100th entry
)

if (-not ([string]::IsNullOrEmpty($SubscriptionName)))
{
    Login-AzureRmAccount -SubscriptionName $SubscriptionName
}

$AllGroups = Get-AzureRmResourceGroup

foreach ($group in $AllGroups) {
    $AllDeployments = (Get-AzureRmResourceGroupDeployment -ResourceGroupName $group.ResourceGroupName)
    $DeploymentsToDelete = @()

    if ($AllDeployments.Count -gt $HistoryToKeep) {
        $DeploymentsToDelete = $AllDeployments[$HistoryToKeep..$AllDeployments.Count]
    }
    Write-Host "ResourceGroupName: $($group.ResourceGroupName)     Deployments history size: $($AllDeployments.Count)     Deployments to delete: $($DeploymentsToDelete.Count)"

    foreach ($deployment in $DeploymentsToDelete) { # can be optimized by using Jobs, but because of possible ARM throttling, we rather do it slowly one by one than get throttled and fail
        Write-Host "Deleting $($deployment.DeploymentName) $($deployment.Timestamp)"
        Remove-AzureRmResourceGroupDeployment -ResourceGroupName $group.ResourceGroupName -Name $deployment.DeploymentName 
    }   
}