﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Instrumentation
{
    using System;
    using System.Diagnostics.Tracing;

    // taken from MSDN: https://msdn.microsoft.com/en-us/library/dn904628(v=vs.85).aspx
    [EventData] // [EventData] makes it possible to pass an instance of the class as an argument to EventSource.Write()
    internal class ExceptionEventData
    {
        internal ExceptionEventData(string type, string msg, string stack)
        {
            Type = type;
            Message = msg;
            CallStack = stack;
        }

        internal ExceptionEventData(Exception ex, string messagePrefix)
        {
            Type = ex.GetType().ToString();
            CallStack = (string.IsNullOrEmpty(ex.StackTrace)) ? "no callstack" : ex.StackTrace;
            Message = string.IsNullOrEmpty(messagePrefix)
                ? ex.Message
                : $"{messagePrefix}: {ex.Message}";
        }

        internal string Type { get; set; }

        internal string Message { get; set; }

        internal string CallStack { get; set; }

        internal AzLocIfx.ExceptionSchema ToIfxEvent()
        {
            var ee = new AzLocIfx.ExceptionSchema()
                     {
                         message = this.Message,
                         exception = this.Type,
                         call_stack = this.CallStack
                     };
            return ee;
        }
    }
}
