﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.Instrumentation
{
    using Microsoft.Cloud.InstrumentationFramework;

    using static System.String;

    public class ResourceProviderApimOperation : OperationAgent<AzLocResIfx.RPApiManagementOperation>
    {
        public ResourceProviderApimOperation(string operationName)
            : base(operationName)
        {
            this.Operation.ApiType = OperationApiType.InternalCall;

            // looks like whatever protocol ifx is using dislikes null strings (even if marked as optional).
            // so initialize here, and override only with non-null values
            this.Operation.PartC = new AzLocResIfx.RPApiManagementOperation()
            {
                ApimUser = Empty,
                ApimSubscription = Empty,
                ApimProduct = Empty,
                ApimUserGroup = Empty
            };
        }

        public ResourceProviderApimOperation(string operationName, string apimSubscription)
            : this(operationName)
        {
            this.Operation.PartC.ApimSubscription = apimSubscription;
        }

        public void AddNewUser(string apimUser, string subscription, string product, string group)
        {
            this.Operation.PartC = new AzLocResIfx.RPApiManagementOperation()
            {
                ApimUser = apimUser,
                ApimSubscription = subscription,
                ApimProduct = product,
                ApimUserGroup = group
            };
        }

#if DEBUG
        protected override void WriteEtw()
        {
            var partC = this.Operation.PartC;
            Agent.Instance.WriteEtwEvent(EventType.Information,
                                         new
                                         {
                                             Operation = this._operationName,
                                             ApiMSubscription = partC.ApimSubscription,
                                             ApiMUser = partC.ApimUser,
                                             OperationResult = this._operationResult.ToString(),
                                             ResultSignature = this._resultSignature,
                                             ResultDetails = this._resultDetails
                                         });
        }
#endif
    }
}