﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.Instrumentation
{
    using Microsoft.Cloud.InstrumentationFramework;

    using static System.String;

    public class ResourceProviderOperation : OperationAgent<AzLocResIfx.RPAccountOperation>, IResourceProviderOperation
    {
        public ResourceProviderOperation(string operationName)
            : base(operationName)
        {
            this.Operation.ApiType = OperationApiType.ServiceApi;

            // looks like whatever protocol ifx is using dislikes null strings (even if marked as optional).
            // so initialize here, and override only with non-null values
            this.Operation.PartC = new AzLocResIfx.RPAccountOperation()
            {
                SubscriptionId = Empty,
                ResourceGroupName = Empty,
                ResourceName = Empty
            };
        }

        public void LogCommonParameters(string subscriptionId, string resourceGroupName = null, string resourceName = null)
        {
            // truncate the logged strings since they might be logged prior to validation
            if (subscriptionId != null)
            {
                this.Operation.PartC.SubscriptionId = subscriptionId.Truncate(1024);
            }

            if (resourceGroupName != null)
            {
                this.Operation.PartC.ResourceGroupName = resourceGroupName.Truncate(1024);
            }

            if (resourceName != null)
            {
                this.Operation.PartC.ResourceName = resourceName.Truncate(1024);
            }
        }

#if DEBUG
        protected override void WriteEtw()
        {
            var partC = this.Operation.PartC;
            Agent.Instance.WriteEtwEvent(EventType.Information,
                                         new
                                         {
                                             Operation = this._operationName,
                                             Subscription = partC.SubscriptionId,
                                             ResourceGroupName = partC.ResourceGroupName,
                                             ResourceName = partC.ResourceName,
                                             OperationResult = this._operationResult.ToString(),
                                             ResultSignature = this._resultSignature,
                                             ResultDetails = this._resultDetails
                                         });
        }
#endif
    }
}