﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.Instrumentation
{
    using Microsoft.Cloud.InstrumentationFramework;

    public class ResourceProviderSubscriptionStorageOperation : OperationAgent<AzLocResIfx.RPSubscriptionStorageOperation>
    {
        public ResourceProviderSubscriptionStorageOperation(string operationName)
            : base(operationName)
        {
            this.Operation.ApiType = OperationApiType.InternalCall;

            // looks like whatever protocol ifx is using dislikes null strings (even if marked as optional).
            // so initialize here, and override only with non-null values
            this.Operation.PartC = new AzLocResIfx.RPSubscriptionStorageOperation()
                                       {
                                           SubscriptionId = string.Empty,
                                       };
        }

        public void LogCommonParameters(string subscriptionId)
        {
            if (subscriptionId != null)
            {
                this.Operation.PartC.SubscriptionId = subscriptionId;
            }
        }

#if DEBUG
        protected override void WriteEtw()
        {
            var partC = Operation.PartC;
            Agent.Instance.WriteEtwEvent(EventType.Information,
                                         new
                                             {
                                                 Operation = this._operationName,
                                                 Subscription = partC.SubscriptionId,
                                                 OperationResult = this._operationResult.ToString(),
                                                 ResultSignature = this._resultSignature,
                                                 ResultDetails = this._resultDetails
                                             });
        }
#endif
    }
}