﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Collections;
    using System.Collections.Generic;

    /// <summary>
    /// Implements a comparer for byte array.
    /// </summary>
    public class ByteArrayComparer : IComparer<byte[]>, IEqualityComparer<byte[]>
    {
        public static ByteArrayComparer Default { get; } = new ByteArrayComparer();

        public int Compare(byte[] x, byte[] y)
        {
            if (x == null)
            {
                throw new ArgumentNullException(nameof(x), "Argument can't be null");
            }

            if (y == null)
            {
                throw new ArgumentNullException(nameof(y), "Argument can't be null");
            }

            if (x.Length != y.Length)
            {
                throw new ArgumentException("Arguments must have equal length.");
            }

            for (int i = 0; i < x.Length; i++)
            {
                if (x[i] != y[i])
                {
                    return x[i] - y[i];
                }
            }

            return 0;
        }

        public bool Equals(byte[] x, byte[] y)
        {
            if (x is null)
            {
                return y is null;
            }

            if (x.Length != y?.Length)
            {
                return false;
            }

            for (int i = 0; i < x.Length; i++)
            {
                if (x[i] != y[i])
                {
                    return false;
                }
            }

            return true;
        }

        public int GetHashCode(byte[] obj)
        {
            // explicit IStructuralEquatable implementation in Array uses last 8 bytes to get a hash code
            return ((IStructuralEquatable)obj).GetHashCode(EqualityComparer<byte>.Default);
        }
    }
}