﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.Iana
{
    using System.Diagnostics.CodeAnalysis;

    /// <summary>
    /// Represents a single allocation (or assignment) of a specific range of Internet number resources (IPv4, IPv6 or ASN), made by the RIR identified in the record.
    /// More details available at https://www.arin.net/knowledge/statistics/nro_extended_stats_format.pdf
    /// </summary>
    [SuppressMessage("StyleCop.CSharp.DocumentationRules", "SA1650:ElementDocumentationMustBeSpelledCorrectly", Justification = "Reviewed. Suppression is OK here.")]
    public class RirRecord
    {
        public RirRecord(string registry, string countryCode, RirRecordType type, string startAddress, string value, string date, RirRecordStatus status)
        {
            this.Registry = registry;
            this.CountryCode = countryCode;
            this.Type = type;
            this.StartAddress = startAddress;
            this.Value = value;
            this.Date = date;
            this.Status = status;
        }

        public string Registry { get; }

        public string CountryCode { get; }

        public RirRecordType Type { get; }

        public string StartAddress { get; }

        public string Value { get; }

        public string Date { get; }

        public RirRecordStatus Status { get; }
    }
}