﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices
{
    using System;
    using System.Collections.Generic;
    using System.Runtime.CompilerServices;

    /// <summary>
    /// Implements binary search in a sorted list
    /// </summary>
    public static class SortedListExtensions
    {
        public static int BinarySearch<TKey, TValue>(this SortedList<TKey, TValue> list, TKey key)
        {
            if (list == null)
            {
                throw new ArgumentNullException(nameof(list));
            }

            int lo = 0;
            int hi = list.Count - 1;

            while (lo <= hi)
            {
                // i might overflow if lo and hi are both large positive numbers. 
                int i = GetMedian(lo, hi);

                int c;
                try
                {
                    c = list.Comparer.Compare(list.Keys[i], key);
                }
                catch (Exception e)
                {
                    throw new InvalidOperationException("Comparer failed in key search.", e);
                }

                if (c == 0)
                {
                    return i;
                }

                if (c < 0)
                {
                    lo = i + 1;
                }
                else
                {
                    hi = i - 1;
                }
            }

            return ~lo;
        }

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        static int GetMedian(int low, int hi)
        {
            return low + ((hi - low) / 2);
        }
    }
}