﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.GeoLocation.Storage
{
    using System;
    using System.IO;
    using System.Net;
    using System.Threading;
    using System.Threading.Tasks;

    public class FtpContentSource : IContentSource
    {
        private readonly Uri fileUri;

        public FtpContentSource(Uri fileUri)
        {
            this.fileUri = fileUri;
        }

        public string Name => this.fileUri.AbsoluteUri;

        public async Task<Stream> GetSourceAsync(CancellationToken cancellationToken)
        {
            FtpWebRequest request = this.CreateWebRequest(WebRequestMethods.Ftp.DownloadFile);
            var response = (FtpWebResponse)await request.GetResponseAsync();
            return response.GetResponseStream();
        }

        public async Task<DateTimeOffset> GetLastModifiedDateTime(CancellationToken cancellationToken)
        {
            var request = this.CreateWebRequest(WebRequestMethods.Ftp.GetDateTimestamp);
            var response = (FtpWebResponse)await request.GetResponseAsync();
            return response.LastModified;
        }

        private FtpWebRequest CreateWebRequest(string method)
        {
            var request = (FtpWebRequest)WebRequest.Create(this.fileUri);
            request.Credentials = new NetworkCredential("anonymous", "password");
            request.Method = method;
            return request;
        }
    }
}