﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.GeoLocation.Storage
{
    using System;
    using System.Collections.Generic;
    using System.Threading.Tasks;

    /// <summary>
    /// Defines the interface for network assignments storage.
    /// </summary>
    public interface INetworkAssignmentStorage
    {
        /// <summary>
        /// Loads network assignments.
        /// </summary>
        Task<NetworkAssignment[]> LoadAsync();

        /// <summary>
        /// Retrieves the fingerprint associated with the state of network assignments.
        /// </summary>
        Task<string> FetchFingerprintAsync();

        /// <summary>
        /// Retrieves the fingerprint associated with the state of network assignments.
        /// </summary>
        Task<string> FetchFingerprintAsync(string leaseToken);

        /// <summary>
        /// Replaces network assignments in storage if the new fingerprint is different.
        /// </summary>
        Task<bool> ReplaceIfFingerprintDoesntMatchAsync(IEnumerable<NetworkAssignment> assignments, string fingerprint, TimeSpan leaseTimeout);
    }
}