﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.GeoLocation.Storage
{
    using System;
    using System.Net;

    using Validation;

    /// <summary>
    /// Converts from network assignment from storage model into object model.
    /// </summary>
    public static class NetworkAssignmentConverter
    {
        /// <summary>
        /// Convert to object model.
        /// </summary>
        public static NetworkAssignment ToNetworkAssignment(this NetworkAssignmentRecord record)
        {
            if (record.CidrMaskLength > 0)
            {
                return new NetworkAssignment(
                    new IPNetwork(new IPAddress(record.StartAddress), record.CidrMaskLength), 
                    record.CountryCode, 
                    record.ChangeDate,
                    record.RegistryName);
            }
            else if (record.NodeNumber > 0)
            {
                return new NetworkAssignment(
                    new IPNetwork(new IPAddress(record.StartAddress), record.NodeNumber),
                    record.CountryCode,
                    record.ChangeDate,
                    record.RegistryName);
            }

            throw Requires.Fail($"Both CidrMaskLength and NodeNumber shouldn't be populated for network address '{BitConverter.ToString(record.StartAddress)}'.");
        }

        /// <summary>
        /// Convert to storage model.
        /// </summary>
        public static NetworkAssignmentRecord ToNetworkAssignmentRecord(this NetworkAssignment a)
        {
            return new NetworkAssignmentRecord
                       {
                           CountryCode = a.IsoCountryCode,
                           CidrMaskLength = a.Network.CidrPrefix,
                           NodeNumber = a.Network.NodeNumber,
                           RegistryName = a.Registry,
                           StartAddress = a.Network.StartAddress.GetAddressBytes(),
                           ChangeDate = a.ChangeDate
                       };
        }
    }
}