﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.GeoLocation.Tests
{
    using System;

    using Microsoft.Azure.LocationServices.Common.UnitTests;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class ByteArrayComparerTests
    {
        [TestMethod]
        public void CompareRequiresNonNullParameters()
        {
            TestHelpers.AssertThrows<ArgumentNullException>(() => ByteArrayComparer.Default.Compare(null, new byte[0]));
            TestHelpers.AssertThrows<ArgumentNullException>(() => ByteArrayComparer.Default.Compare(new byte[0], null));
            TestHelpers.AssertThrows<ArgumentNullException>(() => ByteArrayComparer.Default.Compare(null, null));
        }

        [TestMethod]
        public void CompareRequiresSameLength()
        {
            TestHelpers.AssertThrows<ArgumentException>(() => ByteArrayComparer.Default.Compare(new byte[1], new byte[2]));
        }

        [TestMethod]
        public void CompareIsCorrect()
        {
            Assert.AreEqual(-1, ByteArrayComparer.Default.Compare(new byte[] { 1 }, new byte[] { 2 }));
            Assert.AreEqual(1, ByteArrayComparer.Default.Compare(new byte[] { 2 }, new byte[] { 1 }));
            Assert.AreEqual(0, ByteArrayComparer.Default.Compare(new byte[] { 1 }, new byte[] { 1 }));
        }

        [TestMethod]
        public void EqualsRequiresNonNullParameters()
        {
            Assert.IsFalse(ByteArrayComparer.Default.Equals(null, new byte[0]));
            Assert.IsFalse(ByteArrayComparer.Default.Equals(new byte[0], null));
        }

        [TestMethod]
        public void EqualsWhenBothNull()
        {
            Assert.IsTrue(ByteArrayComparer.Default.Equals(null, null));
        }

        [TestMethod]
        public void EqualsRequiresSameLength()
        {
            Assert.IsFalse(ByteArrayComparer.Default.Equals(new byte[1], new byte[2]));
        }

        [TestMethod]
        public void EqualsIsCorrect()
        {
            Assert.IsFalse(ByteArrayComparer.Default.Equals(new byte[] { 1 }, new byte[] { 2 }));
            Assert.IsTrue(ByteArrayComparer.Default.Equals(new byte[] { 1 }, new byte[] { 1 }));
        }

        [TestMethod]
        public void GetHashCodeIsConsistent()
        {
            var a = new byte[] { 1, 2, 3 };

            Assert.AreEqual(ByteArrayComparer.Default.GetHashCode(a), ByteArrayComparer.Default.GetHashCode(a));
            Assert.AreEqual(ByteArrayComparer.Default.GetHashCode(a), ByteArrayComparer.Default.GetHashCode(new byte[] { 1, 2, 3 }));
            Assert.AreNotEqual(ByteArrayComparer.Default.GetHashCode(a), ByteArrayComparer.Default.GetHashCode(new byte[] { 3, 2, 1 }));

            // only last 8 bytes participate in hash code computation
            Assert.AreEqual(
                ByteArrayComparer.Default.GetHashCode(new byte[] { 0xA, 1, 2, 3, 4, 5, 6, 7, 8 }), 
                ByteArrayComparer.Default.GetHashCode(new byte[] { 0xB, 1, 2, 3, 4, 5, 6, 7, 8 }));
        }
    }
}