﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.GeoLocation.Tests
{
    using System.IO;
    using System.Text;
    using System.Threading.Tasks;

    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class CsvFormatReaderTests
    {
        readonly string[][] lines =
            {
                new[] { "# a comment" },
                new[] { "a", "b", "c" },
                new[] { "d" },
                new[] { string.Empty },
                new[] { " # a trailing comment" },
            };

        readonly StringBuilder input = new StringBuilder();

        public CsvFormatReaderTests()
        {
            foreach (var s in this.lines)
            {
                this.input.AppendLine(string.Join("|", s));
            }
        }

        [TestMethod]
        public async Task CanParseDelimitedAsync()
        {
            var asyncReader = new CsvFormatReader(new StringReader(this.input.ToString()), '|');
            CollectionAssert.AreEquivalent(this.lines[1], await asyncReader.ReadNextAsync());
            CollectionAssert.AreEquivalent(this.lines[2], await asyncReader.ReadNextAsync());
            Assert.IsNull(await asyncReader.ReadNextAsync());
        }

        [TestMethod]
        public void CanParseDelimited()
        {
            var reader = new CsvFormatReader(new StringReader(this.input.ToString()), '|');
            CollectionAssert.AreEquivalent(this.lines[1], reader.ReadNextAsync().Result);
            CollectionAssert.AreEquivalent(this.lines[2], reader.ReadNextAsync().Result);
            Assert.IsNull(reader.ReadNext());
        }
    }
}