﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.GeoLocation.Tests.IPLookup
{
    using System.Collections.Generic;
    using System.Net;

    using Microsoft.Azure.LocationServices.GeoLocation.Tests.TestDataBuilders;
    using Microsoft.VisualStudio.TestTools.UnitTesting;

    using IPLookup = Microsoft.Azure.LocationServices.GeoLocation.IPLookup;

    [TestClass]
    public class IPLookupTests
    {
        [TestMethod]
        public void CorrectlyFindsIPAddress()
        {
            var assignments = new List<NetworkAssignment>
                                  {
                                      this.aNetworkAssignment().WithNetwork(new IPNetwork(IPAddress.Parse("192.168.1.0"), 24)),
                                      this.aNetworkAssignment().WithNetwork(new IPNetwork(IPAddress.Parse("192.168.2.0"), 24)),
                                      this.aNetworkAssignment().WithNetwork(new IPNetwork(IPAddress.Parse("192.168.3.0"), 24)),
                                  };

            var lookup = new IPLookup(assignments);

            Assert.IsNull(lookup.GetNetworkAssignment(IPAddress.Parse("192.168.0.1")).Country);
            Assert.AreEqual(assignments[0].IsoCountryCode, lookup.GetNetworkAssignment(IPAddress.Parse("192.168.1.1")).Country.IsoCode);
            Assert.AreEqual(assignments[1].IsoCountryCode, lookup.GetNetworkAssignment(IPAddress.Parse("192.168.2.1")).Country.IsoCode);
            Assert.AreEqual(assignments[2].IsoCountryCode, lookup.GetNetworkAssignment(IPAddress.Parse("192.168.3.1")).Country.IsoCode);
            Assert.IsNull(lookup.GetNetworkAssignment(IPAddress.Parse("192.168.4.1")).Country);
        }

        [TestMethod]
        public void CorrectlyReturnsWhenEmptySet()
        {
            const string address = "192.168.1.1";

            var lookup = new IPLookup(new List<NetworkAssignment>());
            var result = lookup.GetNetworkAssignment(IPAddress.Parse(address));

            Assert.IsNotNull(result);
            Assert.IsNull(result.Country);
            Assert.AreEqual(address, result.IPAddress);
        }
        
        private NetworkAssignmentDataBuilder aNetworkAssignment() => new NetworkAssignmentDataBuilder();
    }
}