﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.GeoLocation.Tests
{
    using System.Collections;
    using System.Collections.Generic;

    using Microsoft.VisualStudio.TestTools.UnitTesting;

    [TestClass]
    public class SortedListExtensionTests
    {
        [TestMethod]
        public void LessThanSmallestValue()
        {
            var sortedList = new SortedList<int, object>
                                 {
                                     { 10, new object() }
                                 };

            Assert.AreEqual(-1, sortedList.BinarySearch(1));

            sortedList.Add(2, new object());
            Assert.AreEqual(-1, sortedList.BinarySearch(1));
        }

        [TestMethod]
        public void GreaterThanHighestValue()
        {
            var sortedList = new SortedList<int, object>
                                 {
                                     { 10, new object() }
                                 };

            Assert.AreEqual(-2, sortedList.BinarySearch(20));

            sortedList.Add(19, new object());
            Assert.AreEqual(-3, sortedList.BinarySearch(20));
        }

        [TestMethod]
        public void BetweenContainedValues()
        {
            var sortedList = new SortedList<int, object>
                                 {
                                     { 10, new object() }
                                 };

            Assert.AreEqual(-1, sortedList.BinarySearch(9));

            sortedList.Add(20, new object());
            Assert.AreEqual(-2, sortedList.BinarySearch(19));

            sortedList.Add(30, new object());
            Assert.AreEqual(-3, sortedList.BinarySearch(29));
        }

        [TestMethod]
        public void FindsExactValues()
        {
            var sortedList = new SortedList<int, object>
                                 {
                                     { 10, new object() }
                                 };

            Assert.AreEqual(0, sortedList.BinarySearch(10));

            sortedList.Add(20, new object());
            Assert.AreEqual(1, sortedList.BinarySearch(20));

            sortedList.Add(30, new object());
            Assert.AreEqual(2, sortedList.BinarySearch(30));
        }

        [TestMethod]
        public void DoesntFailForEmptyList()
        {
            var sortedList = new SortedList<int, object>();
            Assert.AreEqual(-1, sortedList.BinarySearch(1));
        }
    }
}