﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.APIMLogToGeneva
{
    using Microsoft.Cloud.InstrumentationFramework;

    public sealed class ApimOperationMetricLogger
    {
        private static readonly string[] Dimensions =
            {
                "APIMOperationType",
                "TrackingId",
                "OperationName",
                "OperationResult",
                "InstanceLocation"
            };

        private readonly MeasureMetric metric;

        internal ApimOperationMetricLogger(string monitoringAccount)
        {
            var mdmErrorContext = default(ErrorContext);
            this.metric = MeasureMetric.Create(monitoringAccount, "APIMOperations", "APIMLatency", ref mdmErrorContext, Dimensions);
        }

        public void UpdateMetric(int duration, string operationType, string trackingId, string operationName, string operationResult, string instanceLocation)
        {
            if (this.metric == null)
            {
                return;
            }

            this.metric.LogValue(duration, operationType, trackingId, operationName, operationResult, instanceLocation);
        }
    }
}