﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 
namespace Microsoft.Azure.LocationServices.APIMLogToGeneva
{
    using System.Fabric;
    using Microsoft.Azure.LocationServices.Instrumentation;

    /// <summary>
    /// Static helper class for common operations with the <see cref="FabricRuntime"/>.
    /// </summary>
    public static class FabricRuntimeHelper
    {
        private const string ConfigurationPackageObjectName = "Config";

        public static Agent ServiceFabricInstrumentInstance => Agent.Instance;

        /// <summary>
        /// Gets the configuration value from the <see cref="FabricRuntime"/>.
        /// </summary>
        /// <param name="sectionName">The section to retrieve.</param>
        /// <param name="parameterName">The parameter to retrieve.</param>
        public static string GetConfigurationValue(string sectionName, string parameterName)
        {
            ConfigurationPackage configurationPackage = FabricRuntime.GetActivationContext().GetConfigurationPackageObject(ConfigurationPackageObjectName);
            if (configurationPackage != null)
            {
                if (configurationPackage.Settings.Sections.Contains(sectionName) && configurationPackage.Settings.Sections[sectionName].Parameters.Contains(parameterName))
                {
                    return configurationPackage.Settings.Sections[sectionName].Parameters[parameterName].Value;
                }
            }

            return null;
        }
    }
}
