﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.IPDataImport
{
    using System;

    using Microsoft.Azure.LocationServices.Instrumentation;

    public class GeoLocationLogger : IGeoLocationLogger
    {
        public void SetNumberOfRecordsParsed(int number, string source)
        {
            Agent.Instance.LogEvent(EventType.Information, $"Loaded {number} records from {source}");
        }

        public void OnFailedToAddNetworkAssigment(Exception ex, string recordStartAddress, string recordRegistry)
        {
            Agent.Instance.LogEvent(EventType.ProgramError, $"Failed to process the record for {recordStartAddress} from {recordRegistry}. Reason: {ex.Message}.");
        } 

        public void OnFailedToParseRirDataSource(string sourceName, Exception exception)
        {
            Agent.Instance.LogException(exception, "geoLocation", $"Failed to parse the RIR dataset from {sourceName}.");
        }

        public void OnNetworkAssignmentRebuildCompleted(bool replaced, long stopwatchElapsedMilliseconds, string fingerprint)
        {
            Agent.Instance.LogEvent(EventType.Information, $"IP dataset successfully rebuilt in {stopwatchElapsedMilliseconds} ms. Replaced: {replaced}. New fingerprint: {fingerprint}.");
        }

        public void OnNetworkAssignmentsContentUpdated(Uri blobClientUri, long bufferLength, string leaseToken, string fingerprint)
        {
            Agent.Instance.LogEvent(EventType.Information, $"IP dataset replaced at {blobClientUri}. Size: {bufferLength}. New fingerprint: {fingerprint}.");
        }

        public void OnNetworkAssignmentBlobUpdateError(Exception exception, Uri blobClientUri, string fingerprint)
        {
            Agent.Instance.LogException(exception, "geoLocation", $"Failed to update the IP dataset at {blobClientUri}. Fingerprint: {fingerprint}.");
        }

        public void OnNetworkAssignmentRefreshError(Exception exception)
        {
            Agent.Instance.LogException(exception, "geoLocation", "Periodic refresh of IP data failed.");
        }

        public void OnNetworkAssignmentsReloaded(string latestFingerprint)
        {
            Agent.Instance.LogEvent(EventType.Information, $"Reloaded IP data with new fingerprint {latestFingerprint}.");
        }
    }
}