﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.LBSBatchService
{
    using System;
    using Microsoft.WindowsAzure.Storage.Table;

    /// <summary>
    /// Define an active account entity. An ActiveAccountEntity in the realm of the Batch Service
    /// represents an active/in-progress batch request for an account. If this entry is present in the table, 
    /// it means the batch service is currently processing a batch request for this acount.
    /// </summary>
    public class ActiveAccountEntity : TableEntity
    {
        public const string ActiveAccountEntityPartitionKey = "BatchServiceInProgressAccount";

        /// <summary>
        /// Initializes a new instance of the <see cref="ActiveAccountEntity"/> class.
        /// An entity type must expose a parameter-less constructor
        /// </summary>
        public ActiveAccountEntity()
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="ActiveAccountEntity"/> class.
        /// Defines the PK and RK.
        /// </summary>
        /// <param name="accountId">The unique id of the active account for the entity.</param>
        /// <param name="expirationDate">The expiration date for the entity.</param>
        public ActiveAccountEntity(string accountId, DateTime expirationDate)
        {
            PartitionKey = ActiveAccountEntityPartitionKey;
            RowKey = accountId;
            ExpirationDate = expirationDate;
        }

        /// <summary>
        /// Gets or sets the expiration date for the entity. 
        /// </summary>
        public DateTime ExpirationDate { get; set; }
    }
}