﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.LBSBatchService
{
    using System;
    using Microsoft.Azure.LocationServices.Common.Validation;
    using Microsoft.Azure.LocationServices.Instrumentation;

    public class BatchServiceLogger : IBatchServiceLogger
    {
        public void LogVerbose(string message)
        {
            Assert.ParamIsNotNullOrEmpty(message, nameof(message));

            Agent.Instance.LogEvent(EventType.Verbose, message);
        }

        public void LogInfo(string message)
        {
            Assert.ParamIsNotNullOrEmpty(message, nameof(message));

            Agent.Instance.LogEvent(EventType.Information, message);
        }

        public void LogWarning(string message)
        {
            Assert.ParamIsNotNullOrEmpty(message, nameof(message));

            Agent.Instance.LogEvent(EventType.Warning, message);
        }

        public void LogError(Exception exception, string message)
        {
            Assert.ParamIsNotNull(exception, nameof(exception));
            Assert.ParamIsNotNullOrEmpty(message, nameof(message));

            Agent.Instance.LogException(exception, "batchService", message);
        }

        public void LogAggregateError(AggregateException aggregateException)
        {
            Assert.ParamIsNotNull(aggregateException, nameof(aggregateException));

            Agent.Instance.LogException(aggregateException, "batchService");
        }
    }
}
