﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="ResourceProviderDataBackupJob.cs" company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.LBSBatchService
{
    using System;
    using System.Fabric;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.Azure.LocationServices.Common.Validation;
    using Microsoft.Azure.LocationServices.ServiceFabricHosting;
    using Microsoft.Azure.LocationServices.Shared;

    public sealed class Config
    {
        private static readonly Lazy<Config> Configuration = new Lazy<Config>(() => new Config());

        private Config()
        {
        }

        public static Config Instance => Configuration.Value;

        public string ConnectionString { get; private set; }

        public async Task InitializeAsync(StatelessServiceContext context)
        {
            Assert.ParamIsNotNull(context, nameof(context));

            var configReader = new ConfigurationReader(context);
            var keyVaultConfigReader = new KeyVaultConfigurationReader();

            // Fetch and Set the ConnectionString
            string connectionStringTemplate = await this.GetSecretValue(configReader, keyVaultConfigReader, SharedUtilities.BatchServiceConnectionStringTemplateKey);
            string sasToken = await this.GetSecretValue(configReader, keyVaultConfigReader, SharedUtilities.BatchServiceSasTokenKey);
            this.ConnectionString = string.Format(connectionStringTemplate, sasToken);
        }

        private async Task<string> GetSecretValue(
            ConfigurationReader configReader, 
            KeyVaultConfigurationReader keyVaultConfigReader, 
            string keyVaultKeyName)
        {
            return await keyVaultConfigReader.GetConfigurationValueAsync(configReader.GetConfigValue<string>(keyVaultKeyName));
        }
    }
}