﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.OSMMirrorOnAzure
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    /// <summary>
    /// Controller content class
    /// </summary>
    public class ControllerContent
    {
        public ControllerContent()
        {
            this.FileDetailsMap = new Dictionary<string, ControllerFileInfo>();
        }

        /// <summary>
        /// Constructor that sets the DateTime to the DateTime.MinValue and the size to 0.
        /// </summary>
        /// <param name="fileNamesList"></param>
        public ControllerContent(List<string> fileNamesList) => this.FileDetailsMap = fileNamesList.ToDictionary(k => k, k => new ControllerFileInfo(DateTimeOffset.MinValue, 0, 0));

        public ControllerContent(ControllerContent controllerContent)
        {
            this.FileDetailsMap = new Dictionary<string, ControllerFileInfo>(controllerContent.FileDetailsMap);
        }

        public Dictionary<string, ControllerFileInfo> FileDetailsMap { get; private set; }

        public bool IsHTMLUpdated { get; private set; }

        public void AdjustWithMasterCopy(ControllerContent masterControllerContent)
        {
            bool rowsModified = false;
            foreach (var kvp in masterControllerContent.FileDetailsMap)
            {
                if (!this.FileDetailsMap.ContainsKey(kvp.Key))
                {
                    this.FileDetailsMap.Add(kvp.Key, kvp.Value);
                    rowsModified = true;
                }
            }

            var keysToBeDeleted = this.FileDetailsMap.Where(t => !masterControllerContent.FileDetailsMap.ContainsKey(t.Key)).Select(k => k.Key).ToList();
            
            if (keysToBeDeleted.Any())
            {
                rowsModified = true;
                keysToBeDeleted.ForEach(t => this.FileDetailsMap.Remove(t));
            }

            if (rowsModified)
            {
                this.IsHTMLUpdated = false;
            }
        }

        public void AddFileDetails(string fileName, DateTimeOffset timeStamp, long size, int blockSize) => this.FileDetailsMap.Add(fileName, new ControllerFileInfo(timeStamp, size, blockSize));

        public void SetIsHtmlUpdatedStatus(bool newStatus) => this.IsHTMLUpdated = newStatus;
    }
}
