﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Program.cs" company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// <summary>
//   Program cs file
// </summary>
// --------------------------------------------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.OSMMirrorOnAzure
{
    using System;
    using System.Diagnostics;
    using System.Threading;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.OSMMirrorOnAzure.Configuration;
    using Microsoft.ServiceFabric.Services.Runtime;

    internal static class Program
    {
        /// <summary>
        /// This is the entry point of the service host process.
        /// </summary>
        private static void Main()
        {
            var osmMirrorHost = OSMMirrorHostSettings.CreateFromEnvironment();
            Agent.CreateInstance(osmMirrorHost.MdmParameters);

            try
            {
                // The ServiceManifest.XML file defines one or more service type names.
                // Registering a service maps a service type name to a .NET type.
                // When Service Fabric creates an instance of this service type,
                // an instance of the class is created in this host process.
                ServiceRuntime.RegisterServiceAsync("OSMMirrorOnAzureType",
                    context => new OSMMirrorOnAzure(context)).GetAwaiter().GetResult();

                // Prevents this host process from terminating so services keep running.
                Thread.Sleep(Timeout.Infinite);
            }
            catch (Exception e)
            {
                Agent.Instance.LogException(e, "osmMirror", "initialization");
                throw;
            }
        }
    }
}
