﻿// --------------------------------------------------------------------------------------------------------------------
// <copyright file="Program.cs" company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// --------------------------------------------------------------------------------------------------------------------
namespace Microsoft.Azure.LocationServices.ResourceProviderDataBackupJob
{
    using System;
    using System.Diagnostics;
    using System.Threading;

    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.ResourceProviderDataBackupJob.Configuration;
    using Microsoft.ServiceFabric.Services.Runtime;

    internal static class Program
    {
        /// <summary>
        /// This is the entry point of the service host process.
        /// </summary>
        private static void Main()
        {
            ResourceProviderDataBackupHostSettings hostSettings = ResourceProviderDataBackupHostSettings.CreateFromEnvironment();
            Agent.CreateInstance(hostSettings.MdmParameters);

            try
            {
                // The ServiceManifest.XML file defines one or more service type names.
                // Registering a service maps a service type name to a .NET type.
                // When Service Fabric creates an instance of this service type,
                // an instance of the class is created in this host process.
                ServiceRuntime.RegisterServiceAsync("ResourceProviderDataBackupJobType",
                    context => new ResourceProviderDataBackupJob(context)).GetAwaiter().GetResult();

                Agent.Instance.LogServiceTypeRegistered(Process.GetCurrentProcess().Id, typeof(ResourceProviderDataBackupJob).Name);

                // Prevents this host process from terminating so services keep running.
                Thread.Sleep(Timeout.Infinite);
            }
            catch (Exception e)
            {
                Agent.Instance.LogException(e, "ResourceProviderDataBackup: Host initialization failed");
                throw;
            }
        }
    }
}
