﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZonesDataUpdate.Configuration
{
    using System;
    using System.Globalization;
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.Azure.LocationServices.Instrumentation;

    public abstract class TimeZonesHostSettings
    {
        /// <summary>
        /// Gets the Instrumentation Parameters
        /// </summary>
        public abstract InitParams MdmParameters { get; }

        /// <summary>
        /// Gets the current region.
        /// </summary>
        protected static string CurrentRegion
        {
            get
            {
                string currentRegion = Environment.GetEnvironmentVariable("Region");
                return currentRegion ?? "DEFAULT";
            }
        }

        /// <summary>
        /// Gets the current environment.
        /// </summary>
        /// <exception cref="ArgumentException">
        /// Throws if the current environment is not specified
        /// </exception>
        private static string CurrentEnvironment
        {
            get
            {
                string currentEnvironment = Environment.GetEnvironmentVariable("Environment");

                if (string.IsNullOrEmpty(currentEnvironment))
                {
                    throw new ArgumentException("Current Environment could not be determined");
                }

                return currentEnvironment;
            }
        }

        /// <summary>
        /// Create an environment definition from the environment environment variable.
        /// </summary>
        /// <returns>The created environment definition.</returns>
        public static TimeZonesHostSettings CreateFromEnvironment()
        {
            switch (CurrentEnvironment.ToUpperInvariant())
            {
                case LogicalEnvironments.Development:
                    return new TimeZonesHostDevSettings();
                case LogicalEnvironments.Test:
                    return new TimeZonesHostTestSettings();
                case LogicalEnvironments.Staging:
                    return new TimeZonesHostStagingSettings();
                case LogicalEnvironments.Production:
                    return new TimeZonesHostProdSettings();
                default:
                    throw new ArgumentException(
                        string.Format(
                            CultureInfo.InvariantCulture,
                            "Environment {0} does not have a corresponding Environment definition defined",
                            CurrentEnvironment));
            }
        }
    }
}
