﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZonesDataUpdate
{
    using System;
    using System.Diagnostics;
    using System.Threading;
    using Microsoft.Azure.LocationServices.TimeZonesDataUpdate.Configuration;
    using Microsoft.ServiceFabric.Services.Runtime;

    internal static class Program
    {
        /// <summary>
        /// This is the entry point of the service host process.
        /// </summary>
        private static void Main()
        {
            TimeZonesHostSettings proxyHost = TimeZonesHostSettings.CreateFromEnvironment();
            Instrumentation.Agent.CreateInstance(proxyHost.MdmParameters);

            try
            {
                ServiceRuntime.RegisterServiceAsync("TimeZonesDataUpdateType",
                    context => new TimeZonesDataUpdate(context)).GetAwaiter().GetResult();

                Instrumentation.Agent.Instance.LogServiceTypeRegistered(Process.GetCurrentProcess().Id, typeof(TimeZonesDataUpdate).Name);

                // Prevents this host process from terminating so services keep running.
                Thread.Sleep(Timeout.Infinite);
            }
            catch (Exception ex)
            {
                Instrumentation.Agent.Instance.LogException(ex, "timezone", "initialization");
                throw;
            }
        }
    }
}
