﻿import { DataContext } from "./MyResourceArea";
import * as ClientResources from "ClientResources";
import ExtensionDefinition = require("../_generated/ExtensionDefinition");
import AssetManager = FxImpl.AssetManager;

export class DeleteCommandViewModel extends MsPortalFx.ViewModels.DeleteAssetConfirmation.CommandViewModel {
	private _dataContext: DataContext;

	constructor(container: MsPortalFx.ViewModels.CommandContainerContract, initialValue: any, dataContext: DataContext) {
		super(container, ExtensionDefinition.AssetTypeNames.maps);
		this._dataContext = dataContext;
		this.enabled(true);
	}

	public onInputsSet(inputs: any): MsPortalFx.Base.Promise {
		return super.onInputsSet(inputs);
	}

	// Not doing any thing here, as we have implemented custom notifications, and don't need the default notification.
	_onSendErrorNotification(data: any, selectedItems: any[]): MsPortalFx.UI.NotificationManager.Notification {
		return null;
	}

	_onDelete(
		data: MsPortalFx.ViewModels.DeleteAssetConfirmation.InputData,
		selectedItems: any[]): void {
		var assetId = data.assetId;
		const invokeDelete = this._dataContext.deleteResource(assetId);

        const resourceName = MsPortalFx.ViewModels.Services.ResourceTypes.parseResourceDescriptor(assetId).resource;
        const notificationResources = ClientResources.Overview.Delete.Notification;

        const notification = new MsPortalFx.Hubs.Notifications.ClientNotification({
            title: notificationResources.InProgress.title.format(ClientResources.MyResourceBladeAsset.lowerSingularDisplayName, resourceName),
            description: notificationResources.InProgress.description.format(ClientResources.MyResourceBladeAsset.lowerSingularDisplayName, resourceName),
            status: MsPortalFx.Hubs.Notifications.NotificationStatus.InProgress,
            percentComplete: 0,
            asset: {
                extensionName: ExtensionDefinition.definitionName,
                assetType: ExtensionDefinition.AssetTypeNames.maps,
                assetId: assetId
            }
        });

        notification.publish();
        
        invokeDelete
            .progress(() => {
               notification.percentComplete = 50;
               notification.publish();
            })
            .then(() => {
               this._dataContext.accountEntities.forceRemove(assetId);
               notification.title = notificationResources.Success.title.format(ClientResources.MyResourceBladeAsset.lowerSingularDisplayName, resourceName);
               notification.description = notificationResources.Success.description.format(ClientResources.MyResourceBladeAsset.lowerSingularDisplayName, resourceName);
               notification.status = MsPortalFx.Hubs.Notifications.NotificationStatus.Success;
               notification.percentComplete = 100;

               notification.publish();
                AssetManager.notifyAssetDeleted(ExtensionDefinition.AssetTypes.Maps.name, assetId, ExtensionDefinition.definitionName);
            })
            .catch((reason: any) => {
               notification.title = notificationResources.Error.title.format(ClientResources.MyResourceBladeAsset.lowerSingularDisplayName, resourceName);
               notification.description = notificationResources.Error.description.format(ClientResources.MyResourceBladeAsset.lowerSingularDisplayName, resourceName);
               notification.status = MsPortalFx.Hubs.Notifications.NotificationStatus.Error;

               notification.publish();
            });
	}
}
