import ExtensionDefinition = require("../_generated/ExtensionDefinition");
import DataModels = LocSvcExtension.DataModels;
//import SubscriptionsData = require("./SubscriptionsData");
import Constants = require("../Shared/Constants");
import ArmUtils = require("../Shared/ArmUtils");

/*
 * A place for shared data to be used between all Resource part view models
 */
export class DataContext {
    /**
     * Provides a 'load by id' style cache for a given resource by name.
     */
    public accountEntities = new MsPortalFx.Data.EntityCache<DataModels.Account, string>({
        entityTypeName: LocSvcExtension.DataModels.AccountType,
        sourceUri: MsPortalFx.Data.uriFormatter(Constants.ArmServiceEndpoint + "{id}?" + Constants.ArmServiceVersion, false),
        supplyData: (httpMethod: string, uri: string, headers?: StringMap<any>, data?: any, params?: any) => {
            return MsPortalFx.Base.Net2.ajax({
                uri: uri,
                type: httpMethod || "GET",
                dataType: "json",
                traditional: true,
                headers: headers,
                contentType: "application/json",
                setAuthorizationHeader: true,
                invokeApi: "api/invoke", // for more detail see http://aka.ms/portalfx/cors
                data: data
            });
        },
        extendEntryLifetimes: true
    });

    /**
     * Delete resource
     * @param resourceId Id of resource to be deleted.
     */
    public deleteResource(resourceId: string): MsPortalFx.Base.PromiseN<ArmUtils.OperationProgress> {
        var deferred = Q.defer();
        ArmUtils.runOperation("{0}{1}?{2}&force=true".format(Constants.ArmServiceEndpoint, resourceId, Constants.ArmServiceVersion), "DELETE")
            .then(deferred.resolve,
            (reason: any) => {
                if (reason && reason.status === 404) {
                    deferred.resolve();
                } else {
                    deferred.reject(reason);
                }
            });

        return deferred.promise;
    }
}
