import { DataContext } from "./MyResourceArea";
import * as BladeReferences from "../_generated/BladeReferences";
import * as ClientResources from "ClientResources";
import ExtensionDefinition = require("../_generated/ExtensionDefinition");
import FxBase = MsPortalFx.Base;
import FxAssets = MsPortalFx.Assets;
import GridFormat = MsPortalFx.ViewModels.Controls.Lists.Grid.Format;
import ResourceTypes = MsPortalFx.ViewModels.Services.ResourceTypes;
import ResourceColumnIds = MsPortalFx.Assets.ResourceColumnIds;

interface MetricsBladeInputs { ResourceId: any; TimeContext: any;}
class MetricsPreviewBladeReference extends FxImpl.Composition.Selectable.PdlBladeReference<MetricsBladeInputs, void> {
	public constructor(inputs: MetricsBladeInputs) {
		super("MetricsBlade", "AppInsightsExtension", { parameters: inputs}, { });
	}
}

export class MyResourceBladeAsset implements MsPortalFx.Assets.ResourceMenuConfigContract,
    MsPortalFx.Assets.BrowseConfigContract,
    MsPortalFx.Assets.SupplementalDataContract {

    public supplementalDataStream = ko.observableArray<FxAssets.SupplementalData>([]);
    private _container: MsPortalFx.ViewModels.ContainerContract;
    private _dataContext: DataContext;

    constructor(container: MsPortalFx.ViewModels.ContainerContract, initialState: any, dataContext: DataContext) {
        this._container = container;
        this._dataContext = dataContext;
    }

    public getBrowseConfig(): FxBase.PromiseV<FxAssets.BrowseConfig> {
        return Q.resolve({
            columns: [
                //// Column for the member ID using a custom column.
                //{
                //    id: CustomColumnIds.CustomColumn1,
                //    name: ko.observable<string>("COL1_DISPLAY_NAME"), // todo - replace with localizable resource string
                //    description: ko.observable<string>("DESCRIPTION"), // todo - replace with localizable resource string
                //    itemKey: CustomColumnIds.CustomColumn1,
                //    width: ko.observable<string>("120px")
                //},
                //{
                //    id: CustomColumnIds.CustomColumn2,
                //    name: ko.observable<string>("COL2_DISPLAY_NAME"), // todo - replace with localizable resource string
                //    description: ko.observable<string>("DESCRIPTION"), // todo - replace with localizable resource string
                //    itemKey: CustomColumnIds.CustomColumn2,
                //    width: ko.observable<string>("80px"),
                //    cssClass: "fxs-hubs-gridStatusIconColumn",
                //    format: GridFormat.SvgIconLookup,
                //    formatOptions: {
                //        svgIconLookup: {
                //            0: {
                //                svg: FxBase.Images.StatusBadge.Success(),
                //                text: "Running" // todo - replace with localizable resource string
                //            },
                //            1: {
                //                svg: FxBase.Images.StatusBadge.Warning(),
                //                text: "Warning" // todo - replace with localizable resource string
                //            },
                //            2: {
                //                svg: FxBase.Images.StatusBadge.Error(),
                //                text: "Error" // todo - replace with localizable resource string
                //            }
                //        }
                //    }
                //}
            ],
            defaultColumns: [
                ResourceColumnIds.Name,
                ResourceColumnIds.ResourceType,
                ResourceColumnIds.ResourceGroup,
                ResourceColumnIds.Subscription
            ],
			contextMenu: {
				commandGroup: ExtensionDefinition.CommandGroupNames.accountCommands,
				commandGroupOwner: ExtensionDefinition.definitionName
			}
        });
    }

    public getSupplementalData(resourceIds: string[], columns: string[]): FxBase.Promise {

        // todo - use a query cache or make an ajax call to get this data - This will get called each time the columns change so you may want to cache, at least while the user is on this page
        //var supplementalData = resourceIds.map((id) => { return { resourceId: id, CustomColumn1: "Sample Column 1", CustomColumn2: 0  }; });
        //this.supplementalDataStream(supplementalData);

        // todo - when you modify this code to fetch real data, return the fetch promise here - this lets the portal show a loading indicator
        return Q();
    }

    public getMenuConfig(resourceInfo: FxAssets.ResourceInformation): FxBase.PromiseV<FxAssets.ResourceMenuConfig> {
		let resource = <HubsExtension.Azure.Resource>resourceInfo.resource;

        const overviewItem: FxAssets.MenuItem = {
            id: "overview", // menu item IDs must be unique, must not be localized, should not contain spaces and should be lowercase
            displayText: ClientResources.MyResourceBladeAsset.singularDisplayName,
            enabled: ko.observable(true),
            keywords: ClientResources.ResourceBlade.overviewSearchKeywords,
            icon: MsPortalFx.Base.Images.Redo(),
            supplyBladeReference: () => {
                return new BladeReferences.MyResourceBladeOverviewBladeReference({ id: resourceInfo.resourceId });
            }
        };

		const resourceBladeResources = ClientResources.ResourceBlade;

        const settingsItem: FxAssets.MenuItem = {
            id: "keys", // menu item IDs must be unique, must not be localized, should not contain spaces and should be lowercase
            displayText: resourceBladeResources.keysCommandLabel,
            enabled: ko.observable(true),
            keywords: resourceBladeResources.keysSearchKeywords,
            icon: MsPortalFx.Base.Images.Polychromatic.Key(),
            supplyBladeReference: () => {
                // return new BladeReferences.MyResourceBladeOverviewBladeReference({ id: resourceInfo.resourceId });
                return new BladeReferences.KeysBladeReference({ id: resourceInfo.resourceId });
            }
        };

        const gettingStartedItem: FxAssets.MenuItem = {
            id: "gettingstarted", // menu item IDs must be unique, must not be localized, should not contain spaces and should be lowercase
            displayText: resourceBladeResources.gettingStartedCommandLabel,
            enabled: ko.observable(true),
            keywords: resourceBladeResources.gettingStartedSearchKeywords,
            icon: MsPortalFx.Base.Images.Polychromatic.QuickStart(),
            supplyBladeReference: () => {
                return new BladeReferences.QuickStartBladeReference({ id: resourceInfo.resourceId });
            }
        };

        const aboutItem: FxAssets.MenuItem = {
            id: "about", // menu item IDs must be unique, must not be localized, should not contain spaces and should be lowercase
            displayText: ClientResources.ResourceBlade.aboutCommandLabel,
            enabled: ko.observable(true),
            keywords: ClientResources.ResourceBlade.aboutSearchKeywords,
            icon: MsPortalFx.Base.Images.Polychromatic.Info(),
            supplyBladeReference: () => {
                return new BladeReferences.AboutBladeReference({ id: resourceInfo.resourceId });
            }
        };

        const settingsGroup: FxAssets.MenuGroup = {
            id: "settings_group",  // menu group IDs must be unique, must not be localized, should not contain spaces and should be lowercase
            displayText: ClientResources.ResourceBlade.settingsGroupLabel,
            items: [settingsItem],
            elevated: true
        };

        const helpGroup: FxAssets.MenuGroup = {
            id: "helpgroup",  // menu group IDs must be unique, must not be localized, should not contain spaces and should be lowercase
            displayText: ClientResources.ResourceBlade.helpGroupLabel,
            items: [gettingStartedItem, aboutItem]
        };

		const groups: MsPortalFx.Assets.MenuGroupInstance[] = [settingsGroup, helpGroup];

		const keywords = resourceBladeResources.Keywords;
		const monitoringGroupExtension: FxAssets.MenuGroupExtension = <FxAssets.MenuGroupExtension>{
			referenceId: FxAssets.MonitoringGroupId,
			items: [{
				 id: "metricsV2",
				 displayText: resourceBladeResources.MetricsV2.displayText, // Localize
				 keywords: [keywords.metrics, keywords.preview], // Localize
				 icon: MsPortalFx.Base.Images.Polychromatic.Chart(),
				 supplyBladeReference: () => {
					return new MetricsPreviewBladeReference({ ResourceId: resourceInfo.resourceId, TimeContext: null });
				 }
			 }]
		};

		/** IMPORTANT: Please add this under this feature flag which we will turn on when we Public Preview. */
		if (MsPortalFx.isFeatureEnabled("enableappinsightsmetricsblade")) {
			groups.push(monitoringGroupExtension);
		}

        // Build the resource menu config.
        const menuConfig: FxAssets.ResourceMenuConfig = {
            overview: overviewItem,
            groups: groups,
            options: {
                enableRbac: true,
                enableTags: true,
                enableSupportTroubleshootV2: true,
                enableProperties: true,
                enableLocks: true,
                enableExportTemplate: false
            },
        };
        return Q(menuConfig);

    }
}

