/// <reference path="./jquery.d.ts" />

// Compiled with TypeScript Version 2.3.3

// FILE: Q\Q.d.ts
/**
 * If value is a Q promise, returns the promise.
 * If value is a promise from another library it is coerced into a Q promise (where possible).
 */
declare function Q<T>(promise: Q.IPromise<T>): Q.Promise<T>;
/**
 * If value is a Q promise, returns the promise.
 * If value is a promise from another library it is coerced into a Q promise (where possible).
 */
declare function Q<T>(promise: JQueryPromise): Q.Promise<T>;
/**
 * If value is not a promise, returns a promise that is fulfilled with value.
 */
declare function Q<T>(value: T): Q.Promise<T>;
/**
 * Returns a promise that is fulfilled with undefined.
 */
declare function Q(): Q.Promise<void>;
declare module Q {
    interface IPromise<T> {
        then<U>(onFulfill: (value: T) => U | IPromise<U>, onReject?: (reason: any) => U | IPromise<U>): IPromise<U>;
    }
    interface Promise<T> {
        /**
         * Like a finally clause, allows you to observe either the fulfillment or rejection of a promise, but to do so without modifying the final value. This is useful for collecting resources regardless of whether a job succeeded, like closing a database connection, shutting a server down, or deleting an unneeded key from an object.
         * finally returns a promise, which will become resolved with the same fulfillment value or rejection reason as promise. However, if callback returns a promise, the resolution of the returned promise will be delayed until the promise returned from callback is finished.
         */
        fin(finallyCallback: () => any): Promise<T>;
        /**
         * Like a finally clause, allows you to observe either the fulfillment or rejection of a promise, but to do so without modifying the final value. This is useful for collecting resources regardless of whether a job succeeded, like closing a database connection, shutting a server down, or deleting an unneeded key from an object.
         * finally returns a promise, which will become resolved with the same fulfillment value or rejection reason as promise. However, if callback returns a promise, the resolution of the returned promise will be delayed until the promise returned from callback is finished.
         */
        finally(finallyCallback: () => any): Promise<T>;
        /**
         * The then method from the Promises/A+ specification, with an additional progress handler.
         */
        then<U>(onFulfill: (value: T) => U | IPromise<U>, onReject?: (reason: any) => U | IPromise<U>, onProgress?: Function): Promise<U>;
        /**
         * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
         *
         * This is especially useful in conjunction with all
         */
        spread<U>(onFulfilled: Function, onRejected?: Function): Promise<U>;
        fail<U>(onRejected: (reason: any) => U | IPromise<U>): Promise<U>;
        /**
         * A sugar method, equivalent to promise.then(undefined, onRejected).
         */
        catch<U>(onRejected: (reason: any) => U | IPromise<U>): Promise<U>;
        /**
         * A sugar method, equivalent to promise.then(undefined, undefined, onProgress).
         */
        progress(onProgress: (progress: any) => any): Promise<T>;
        /**
         * Much like then, but with different behavior around unhandled rejection. If there is an unhandled rejection, either because promise is rejected and no onRejected callback was provided, or because onFulfilled or onRejected threw an error or returned a rejected promise, the resulting rejection reason is thrown as an exception in a future turn of the event loop.
         *
         * This method should be used to terminate chains of promises that will not be passed elsewhere. Since exceptions thrown in then callbacks are consumed and transformed into rejections, exceptions at the end of the chain are easy to accidentally, silently ignore. By arranging for the exception to be thrown in a future turn of the event loop, so that it won't be caught, it causes an onerror event on the browser window, or an uncaughtException event on Node.js's process object.
         *
         * Exceptions thrown by done will have long stack traces, if Q.longStackSupport is set to true. If Q.onerror is set, exceptions will be delivered there instead of thrown in a future turn.
         *
         * The Golden Rule of done vs. then usage is: either return your promise to someone else, or if the chain ends with you, call done to terminate it.
         */
        done<U>(onFulfilled?: (value: T) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>, onProgress?: (progress: any) => any): void;
        /**
         * Returns a promise to get the named property of an object. Essentially equivalent to
         *
         * promise.then(function (o) {
         *     return o[propertyName];
         * });
         */
        get<U>(propertyName: String): Promise<U>;
        set<U>(propertyName: String, value: any): Promise<U>;
        delete<U>(propertyName: String): Promise<U>;
        /**
         * Returns a promise for the result of calling the named method of an object with the given array of arguments. The object itself is this in the function, just like a synchronous method call. Essentially equivalent to
         *
         * promise.then(function (o) {
         *     return o[methodName].apply(o, args);
         * });
         */
        post<U>(methodName: String, args: any[]): Promise<U>;
        /**
         * Returns a promise for the result of calling the named method of an object with the given variadic arguments. The object itself is this in the function, just like a synchronous method call.
         */
        invoke<U>(methodName: String, ...args: any[]): Promise<U>;
        /**
         * Returns a promise for an array of the property names of an object. Essentially equivalent to
         *
         * promise.then(function (o) {
         *     return Object.keys(o);
         * });
         */
        keys(): Promise<string[]>;
        /**
         * A sugar method, equivalent to promise.then(function () { return value; }).
         */
        thenResolve<U>(value: U): Promise<U>;
        /**
         * A sugar method, equivalent to promise.then(function () { throw reason; }).
         */
        thenReject(reason: any): Promise<T>;
        timeout(ms: number, message?: string | Error): Promise<T>;
        /**
         * Returns a promise that will have the same result as promise, but will only be fulfilled or rejected after at least ms milliseconds have passed.
         */
        delay(ms: number): Promise<T>;
        /**
         * Returns whether a given promise is in the fulfilled state. When the static version is used on non-promises, the result is always true.
         */
        isFulfilled(): boolean;
        /**
         * Returns whether a given promise is in the rejected state. When the static version is used on non-promises, the result is always false.
         */
        isRejected(): boolean;
        /**
         * Returns whether a given promise is in the pending state. When the static version is used on non-promises, the result is always false.
         */
        isPending(): boolean;
        valueOf(): any;
        /**
         * Returns a "state snapshot" object, which will be in one of three forms:
         *
         * - { state: "pending" }
         * - { state: "fulfilled", value: <fulfllment value> }
         * - { state: "rejected", reason: <rejection reason> }
         */
        inspect(): PromiseState<T>;
    }
    interface Deferred<T> {
        promise: Promise<T>;
        resolve(value?: T): void;
        reject(reason?: any): void;
        notify(value?: any): void;
    }
    interface PromiseState<T> {
        /**
         * "fulfilled", "rejected", "pending"
         */
        state: string;
        value?: T;
        reason?: any;
    }
    /**
     * Promisify a value and apply onFulfilled, onRejected and onProgress callbacks
     */
    function when<T>(value: IPromise<T>): Promise<T>;
    function when<T>(value: T): Promise<T>;
    function when<T, U>(value: IPromise<T>, onFulfilled: (val: T) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>, onProgress?: (progress: any) => any): Promise<U>;
    function when<T, U>(value: T, onFulfilled: (val: T) => U | IPromise<U>): Promise<U>;
    /**
     * Function call
     */
    function fcall<T>(method: (...args: any[]) => T, ...args: any[]): Promise<T>;
    /**
     * Returns a promise that is fulfilled with an array containing the fulfillment value of each promise, or is rejected with the same rejection reason as the first promise to be rejected.
     */
    function any<T>(promises: IPromise<T>[]): Promise<T>;
    /**
     * Returns a promise that is fulfilled with an array containing the fulfillment value of each promise, or is rejected with the same rejection reason as the first promise to be rejected.
     */
    function any<T>(promises: any[]): Promise<T>;
    /**
     * A sugar method, equivalent to promise.then(undefined, undefined, onProgress).
     */
    function progress<T>(promise: Promise<T>, onProgress: Function): Promise<{}>;
    /**
     * Calls a method on a promise object
     */
    function post(promise: any, methodName: String, args: any[]): any;
    /**
     * Returns a promise that is fulfilled with an array containing the fulfillment value of each promise, or is rejected with the same rejection reason as the first promise to be rejected.
     */
    function all<T>(promises: IPromise<T>[]): Promise<T[]>;
    /**
     * Returns a promise that is fulfilled with an array containing the fulfillment value of each promise, or is rejected with the same rejection reason as the first promise to be rejected.
     */
    function all<T>(promises: any[]): Promise<T[]>;
    /**
     * Returns a promise that is fulfilled with an array of promise state snapshots, but only after all the original promises have settled, i.e. become either fulfilled or rejected.
     */
    function allSettled<T>(promises: IPromise<T>[]): Promise<PromiseState<T>[]>;
    /**
     * Returns a promise that is fulfilled with an array of promise state snapshots, but only after all the original promises have settled, i.e. become either fulfilled or rejected.
     */
    function allSettled<T>(promises: any[]): Promise<PromiseState<T>[]>;
    /**
     * Returns a promise that is fulfilled with the original array of promises, but only after all the original promises have settled, i.e. become either fulfilled or rejected.
     */
    function allResolved<T>(promises: IPromise<T>[]): Promise<Promise<T>[]>;
    /**
     * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
     * This is especially useful in conjunction with all.
     */
    function spread<T1, T2, U>(promises: [T1 | IPromise<T1>, T2 | IPromise<T2>], onFulfilled: (t1: T1, t2: T2) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>): Promise<U>;
    /**
     * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
     * This is especially useful in conjunction with all.
     */
    function spread<T1, T2, T3, U>(promises: [T1 | IPromise<T1>, T2 | IPromise<T2>, T3 | IPromise<T3>], onFulfilled: (t1: T1, t2: T2, t3: T3) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>): Promise<U>;
    /**
     * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
     * This is especially useful in conjunction with all.
     */
    function spread<T1, T2, T3, T4, U>(promises: [T1 | IPromise<T1>, T2 | IPromise<T2>, T3 | IPromise<T3>, T4 | IPromise<T4>], onFulfilled: (t1: T1, t2: T2, t3: T3, t4: T4) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>): Promise<U>;
    /**
     * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
     * This is especially useful in conjunction with all.
     */
    function spread<T1, T2, T3, T4, T5, U>(promises: [T1 | IPromise<T1>, T2 | IPromise<T2>, T3 | IPromise<T3>, T4 | IPromise<T4>, T5 | IPromise<T5>], onFulfilled: (t1: T1, t2: T2, t3: T3, t4: T4, t5: T5) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>): Promise<U>;
    /**
     * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
     * This is especially useful in conjunction with all.
     */
    function spread<T1, T2, T3, T4, T5, T6, U>(promises: [T1 | IPromise<T1>, T2 | IPromise<T2>, T3 | IPromise<T3>, T4 | IPromise<T4>, T5 | IPromise<T5>, T6 | IPromise<T6>], onFulfilled: (t1: T1, t2: T2, t3: T3, t4: T4, t5: T5, t6: T6) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>): Promise<U>;
    /**
     * Like then, but "spreads" the array into a variadic fulfillment handler. If any of the promises in the array are rejected, instead calls onRejected with the first rejected promise's rejection reason.
     * This is especially useful in conjunction with all.
     */
    function spread<T, U>(promises: (T | IPromise<T>)[], onFulfilled: (...args: T[]) => U | IPromise<U>, onRejected?: (reason: any) => U | IPromise<U>): Promise<U>;
    /**
     * Returns a promise that will have the same result as promise, except that if promise is not fulfilled or rejected before ms milliseconds, the returned promise will be rejected with an Error with the given message. If message is not supplied, the message will be "Timed out after " + ms + " ms".
     */
    function timeout<T>(promise: Promise<T>, ms: number, message?: string | Error): Promise<T>;
    /**
     * Returns a promise that will have the same result as promise, but will only be fulfilled or rejected after at least ms milliseconds have passed.
     */
    function delay<T>(promise: Promise<T> | number, ms?: number): Promise<T>;
    /**
     * Returns a promise that will have the same result as promise, but will only be fulfilled or rejected after at least ms milliseconds have passed.
     */
    function delay<T>(value: T | number, ms?: number): Promise<T>;
    /**
     * Returns a promise that will be fulfilled with undefined after at least ms milliseconds have passed.
     */
    function delay(ms: number): Promise<void>;
    /**
     * Returns whether a given promise is in the fulfilled state. When the static version is used on non-promises, the result is always true.
     */
    function isFulfilled(promise: Promise<any>): boolean;
    /**
     * Returns whether a given promise is in the rejected state. When the static version is used on non-promises, the result is always false.
     */
    function isRejected(promise: Promise<any>): boolean;
    /**
     * Returns whether a given promise is in the pending state. When the static version is used on non-promises, the result is always false.
     */
    function isPending(promise: Promise<any>): boolean;
    /**
     * Returns whether a given promise is in the pending state. When the static version is used on non-promises, the result is always false.
     */
    function isPending(obj: any): boolean;
    /**
     * Returns a "deferred" object with a:
     * promise property
     * resolve(value) method
     * reject(reason) method
     * notify(value) method
     */
    function defer<T>(): Deferred<T>;
    /**
     * Returns a promise that is rejected with reason.
     */
    function reject<T>(reason?: any): Promise<T>;
    /**
     * Apply callbacks to a new Q promise
     */
    function promise<T>(resolver: (resolve: (val: IPromise<T>) => void, reject: (reason: any) => void, notify: (progress: any) => void) => void): Promise<T>;
    function promise<T>(resolver: (resolve: (val: T) => void, reject: (reason: any) => void, notify: (progress: any) => void) => void): Promise<T>;
    /**
     * Apply callbacks to a new Q promise
     */
    function Promise<T>(resolver: (resolve: (val: T | IPromise<T>) => void, reject: (reason: any) => void, notify: (progress: any) => void) => void): Promise<T>;
    /**
     * Returns whether the given value is a Q promise.
     */
    function isPromise(obj: any): boolean;
    /**
     * Returns whether the given value is a promise (i.e. it's an object with a then function).
     */
    function isPromiseAlike(obj: any): boolean;
    /**
     * This is an experimental tool for converting a generator function into a deferred function. This has the potential of reducing nested callbacks in engines that support yield.
     */
    function nextTick(callback: Function): void;
    /**
     * A settable property that will intercept any uncaught errors that would otherwise be thrown in the next tick of the event loop, usually as a result of done. Can be useful for getting the full stack trace of an error in browsers, which is not usually possible with window.onerror.
     */
    let onerror: (reason: any) => void;
    /**
     * A settable property that lets you turn on long stack trace support. If turned on, "stack jumps" will be tracked across asynchronous promise operations, so that if an uncaught error is thrown by done or a rejection reason's stack property is inspected in a rejection callback, a long stack trace is produced.
     */
    let longStackSupport: boolean;
    /**
     * Returns the list of errors that were not handled.
     */
    function getUnhandledReasons(): string[];
    /**
     * Clears the list of errors that were not handled.
     * NOTE: Q.d.ts said it returns something, but the implementation never did.
     */
    function resetUnhandledRejections(): void;
    /**
     * Calling resolve with a pending promise causes promise to wait on the passed promise, becoming fulfilled with its fulfillment value or rejected with its rejection reason (or staying pending forever, if the passed promise does).
     * Calling resolve with a rejected promise causes promise to be rejected with the passed promise's rejection reason.
     * Calling resolve with a fulfilled promise causes promise to be fulfilled with the passed promise's fulfillment value.
     * Calling resolve with a non-promise value causes promise to be fulfilled with that value.
     */
    function resolve<T>(obj: IPromise<T>): Promise<T>;
    /**
     * Calling resolve with a pending promise causes promise to wait on the passed promise, becoming fulfilled with its fulfillment value or rejected with its rejection reason (or staying pending forever, if the passed promise does).
     * Calling resolve with a rejected promise causes promise to be rejected with the passed promise's rejection reason.
     * Calling resolve with a fulfilled promise causes promise to be fulfilled with the passed promise's fulfillment value.
     * Calling resolve with a non-promise value causes promise to be fulfilled with that value.
     */
    function resolve<T>(obj: T): Promise<T>;
}
