﻿# ##########################################################################################################################################################
# The portal version needs to be updated in three places:
# 1. AssemblyInfo: This version will be used to name the zipped package
# 2. config.json: This will be uploaded to the storage account and is used by the portal to determine which version to show
# 3. Extension.pdl: This will version the generated JS files
# ##########################################################################################################################################################

Param
(
    [Parameter(Mandatory=$false, Position=0)]
    [ValidateNotNullOrEmpty()]
    [string]$BuildNumber = $Env:BUILD_BUILDNUMBER # $Env:BUILD_BUILDNUMBER is set by the VSTS agent based on what's defined in the build definition
)

Write-Host "Build number: $BuildNumber"

# Assuming VSTS build number. e.g. 20180403.1
$build, $rev = $BuildNumber.split('.')

if ([String]::IsNullOrWhiteSpace($build) -OR [String]::IsNullOrWhiteSpace($rev)) {
    Write-Error "Invalid build number. The value should be in the form of BUILD.REV (e.g. 20180403.1)"
    throw
}

$newVersion;

# AssemblyInfo.cs
$path = "$PSScriptRoot/AssemblyInfo.cs";
$assemblyVersion = '\[assembly: AssemblyVersion\("(.*)"\)\]'
$assemblyFileVersion = '\[assembly: AssemblyFileVersion\("(.*)"\)\]'

(Get-Content $path) | ForEach-Object{
    if($_ -match $assemblyVersion) {
        $currentVersion = [version]$matches[1]
        $newVersion = "{0}.{1}.{2}.{3}" -f $currentVersion.Major, $currentVersion.Minor, $build, $rev
        '[assembly: AssemblyVersion("{0}")]' -f $newVersion
    } elseif ($_ -match $assemblyFileVersion) {
        $currentVersion = [version]$matches[1]
        $newVersion = "{0}.{1}.{2}.{3}" -f $currentVersion.Major, $currentVersion.Minor, $build, $rev
        '[assembly: AssemblyFileVersion("{0}")]' -f $newVersion
    } else {
        # Output line as is
        $_
    }
} | Set-Content $path

# config.json
'{{ "$version": "3", "stage1": "{0}", "stage2": "{0}", "stage3": "{0}", "stage4": "{0}", "stage5": "{0}", "flight": "{0}" }}' -f $newVersion | Set-Content $PSScriptRoot/config.json

# extension.pdl
$extPath = "$PSScriptRoot/Client/extension.pdl"
$content = [xml](get-content (Resolve-Path $extPath))
$content.Definition.Extension.Version = $newVersion
$content.Save((Resolve-Path $extPath))

Write-Host "Set all versions to: $newVersion"