﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderPermissions.Authorization
{
    using System;
    using System.Security.Cryptography.X509Certificates;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ResourceProviderPermissions.Configuration;
    using Microsoft.IdentityModel.Clients.ActiveDirectory;

    public class CachedTokenClient : ITokenClient
    {
        private static TokenCache tokenCache;
        private readonly PermissionsJobHostSettings settings;
        private readonly ICertificateResolver certificateResolver;

        public CachedTokenClient(PermissionsJobHostSettings settings, ICertificateResolver certificateResolver)
        {
            this.settings = settings;
            this.certificateResolver = certificateResolver;
            tokenCache = new TokenCache();
        }

        public async Task<string> AcquireAuthorizationHeaderAsync(string tenantId)
        {
            if (string.IsNullOrWhiteSpace(tenantId))
            {
                throw new ArgumentNullException(nameof(tenantId));
            }

            AuthenticationContext authenticationContext = new AuthenticationContext(
               authority: string.Format(settings.AAD_ARMS2S_AuthorityTemplate, tenantId),
               validateAuthority: true,
               tokenCache: tokenCache);

            X509Certificate2 clientCertificate = await certificateResolver.ResolveCertificateAsync(settings.AAD_ARMS2S_CertificateSubject);
            
            ClientAssertionCertificate certificateCredential = new ClientAssertionCertificate(
               clientId: settings.AAD_ARMS2S_ClientId,
               certificate: clientCertificate);

            AuthenticationResult result = await authenticationContext.AcquireTokenAsync(
                resource: settings.AAD_ARMS2S_Resource,
                clientCertificate: certificateCredential);

            return result.CreateAuthorizationHeader();
        }
    }
}
