﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderPermissions.Configuration
{    
    using System.Collections.ObjectModel;
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.Azure.LocationServices.Instrumentation;

    /// <summary>
    /// The production environment definition.
    /// </summary>
    public class PermissionsJobHostProdSettings : PermissionsJobHostSettings
    {
        /// <inheritdoc />
        public override InitParams MdmParameters => new InitParams("AzLocResIfx", "AzLocRes");

        /// <inheritdoc />
        public override Collection<string> CosmosDbPrefferedLocations
        {
            get
            {
                Collection<string> locations = null;
                switch (CurrentRegion)
                {
                    case AzureRegions.EastUS:
                        locations = CosmosDbConstants.Prod_PreferredEastUsLocations;
                        break;
                    case AzureRegions.WestUS2:
                        locations = CosmosDbConstants.Prod_PreferredWestUs2Locations;
                        break;
                    case AzureRegions.WestEurope:
                        locations = CosmosDbConstants.Prod_PreferredWestEuropeLocations;
                        break;
                    default:
                        locations = CosmosDbConstants.Prod_PreferredEastUsLocations;
                        break;
                }

                return locations;
            }
        }

        /// <inheritdoc />
        public override bool EnableAccountReplication => false;

        /// <inheritdoc />
        protected override string GlobalKeyVaultName => "p-global";

        /// <inheritdoc />
        protected override string RegionalKeyVaultName
        {
            get
            {
                string vaultName = string.Empty;
                switch (CurrentRegion)
                {
                    case AzureRegions.EastUS:
                        vaultName = "p-rp-eastus";
                        break;
                    case AzureRegions.WestUS2:
                        vaultName = "p-rp-westus2";
                        break;
                    case AzureRegions.WestEurope:
                        vaultName = "p-rp-westeurope";
                        break;
                    default: vaultName = "p-rp-eastus"; break;
                }

                return vaultName;
            }
        }

        /// <inheritdoc />
        protected override string MapsFrontEndCosmosDbAccountName => CosmosDbConstants.Production_CosmosDbAccountName;
    }
}
