﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices
{
    using System.Collections.ObjectModel;
    using Microsoft.Azure.LocationServices.Common;    

    public class CosmosDbConstants
    {
        public const string CI_CosmosDbAccountName = "c-azuremaps";
        public const string Test_CosmosDbAccountName = "t-azuremaps";
        public const string Staging_CosmosDbAccountName = "s-azuremaps";
        public const string Production_CosmosDbAccountName = "p-azuremaps";

        /// <summary>
        /// Constant property which is used as the partition key in cosmos db.
        /// </summary>
        public const string PartitionKeyConst = "pk";

        /// <summary>
        /// Constant property value which is used to determine that a document is a role assignment vs role definition.
        /// </summary>
        public const string RoleAssignmentTypePropertyValueConst = "ra";

        /// <summary>
        /// The collection name which will be used to queue refresh subscription's Role definitions and Role assignments
        /// </summary>
        public const string SubscriptionCollectionName = "subscriptions";

        /// <summary>
        /// The property of the subscription which determines the AAD tenant ID which it is homed to.
        /// </summary>
        public const string SubscriptionTenantIdPropertyConst = "tenantId";

        /// <summary>
        /// The collection name used for account lookup on accounts at runtime in data plane
        /// </summary>        
        public const string AccountDocumentCollectionName = "resources";

        /// <summary>
        /// The name of the database which contains the collections for accounts
        /// </summary>
        public const string AccountsDatabaseName = "accounts";

        /// <summary>
        /// The time which the subscription was last enqueued.
        /// </summary>
        public const string EnqueueTimePropertyConst = "enqueueTimeUTC";

        /// <summary>
        /// The property name of the state of the subscription.
        /// </summary>
        public const string SubscriptionStatePropertyConst = "state";

        public static readonly Collection<string> Prod_PreferredEastUsLocations = new Collection<string>()
                        {
                            AzureRegions.EastUS,
                            AzureRegions.WestUS2,
                            AzureRegions.WestEurope
                        };

        public static readonly Collection<string> Prod_PreferredWestUs2Locations = new Collection<string>()
                        {
                            AzureRegions.WestUS2,
                            AzureRegions.EastUS,
                            AzureRegions.WestEurope
                        };

        public static readonly Collection<string> Prod_PreferredWestEuropeLocations = new Collection<string>()
                        {
                            AzureRegions.WestEurope,
                            AzureRegions.EastUS,
                            AzureRegions.WestUS2,
                        };

        public static readonly Collection<string> Staging_PreferredEastUsLocations = new Collection<string>()
                        {
                            AzureRegions.EastUS,
                            AzureRegions.WestUS2,
                            AzureRegions.WestEurope
                        };

        public static readonly Collection<string> Staging_PreferredWestUs2Locations = new Collection<string>()
                        {
                            AzureRegions.WestUS2,
                            AzureRegions.EastUS,
                            AzureRegions.WestEurope
                        };

        public static readonly Collection<string> Staging_PreferredWestEuropeLocations = new Collection<string>()
                        {
                            AzureRegions.WestEurope,
                            AzureRegions.EastUS,
                            AzureRegions.WestUS2,
                        };

        public static readonly Collection<string> Test_PreferredEastUsLocations = new Collection<string>()
                        {
                            AzureRegions.EastUS
                        };

        public static readonly Collection<string> CI_PreferredEastUsLocations = new Collection<string>()
                        {
                            AzureRegions.EastUS
                        };
    }
}
