﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderUnitTests.Mocks
{
    using System.Collections.Generic;
    using System.Linq;
    using Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Storage;

    internal class MockSubscriptionRecordStorage : ISubscriptionRecordStorage
    {
        private readonly Dictionary<string, Dictionary<string, SubscriptionRecord>> dictionary =
            new Dictionary<string, Dictionary<string, SubscriptionRecord>>();

        public void PutSubscriptionRecord(SubscriptionRecord record)
        {
            Validation.Requires.NotNull(record, nameof(record));

            if (!this.dictionary.TryGetValue(record.PartitionKey, out var partition))
            {
                partition = new Dictionary<string, SubscriptionRecord>();
                this.dictionary[record.PartitionKey] = partition;
            }

            // replace semantics
            partition[record.RowKey] = record;
        }

        public SubscriptionRecord GetSubscriptionRecord(string subscriptionId)
        {
            Validation.Requires.NotNull(subscriptionId, nameof(subscriptionId));

            SubscriptionRecord record = null;

            // first find the dictionary for the Primary key, then select values that match
            // Deleted and ResourceGroupName and ResourceName
            var partition = this.dictionary.SingleOrDefault(p => p.Key == subscriptionId);

            if (partition.Value == null)
            {
                return record;
            }

            var item = partition.Value.SingleOrDefault();   // the partitionKey should make it unique

            if (item.Value != null)
            {
                record = item.Value;
            }

            return record;
        }        

        internal IEnumerable<SubscriptionRecord> GetRecords()
        {
            return from partition in this.dictionary from record in partition.Value select record.Value;
        }

        internal void Reset()
        {
            this.dictionary.Clear();
        }
    }
}