﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderUnitTests
{
    using System.Threading;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.ResourceProviderPermissions;
    using Moq;
    using Xunit;

    public class SubscriptionQueueObserverUnitTest
    {
        [Fact(DisplayName = "SubscriptionQueueObserver_RunAsync_UpdateSubscription")]
        [Trait("Category", "L0")]
        public async Task SubscriptionQueueObserver_RunAsync_UpdateSubscription()
        {
            Mock<ISubscriptions> mockSubscriptions = new Mock<ISubscriptions>();
            SubscriptionQueueObserver observer = new SubscriptionQueueObserver(mockSubscriptions.Object);            
            mockSubscriptions
                .Setup(r => r.LoadAsync(It.IsAny<CancellationToken>()))
                .Returns(Task.CompletedTask)
                .Verifiable();
            mockSubscriptions
                .Setup(r => r.PruneStaleAsync(It.IsAny<CancellationToken>()))
                .Returns(Task.CompletedTask)
                .Verifiable();
            mockSubscriptions
                .Setup(r => r.UpdatePermissionsAsync(It.IsAny<CancellationToken>()))
                .Returns(Task.CompletedTask)
                .Verifiable();

            await observer.RunAsync(CancellationToken.None);

            mockSubscriptions.VerifyAll();
        }
    }
}
