﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement
{
    using System;
    using System.IO;
    using System.Net;
    using Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement.Contracts.ResponseContract;

    /// <summary>
    /// The APIMExceptionFactory class
    /// </summary>
    public static class ApimExceptionFactory
    {
        /// <summary>
        /// Gets the APIM exception.
        /// </summary>
        /// <param name="ex">The ex.</param>
        /// <returns>The APIM Exception</returns>
        public static ApimException GetApimException(WebException ex)
        {
            ApimError apiError = null;
            if (ex.Response != null)
            {
                apiError = Utility.DeserializeObject<ApimError>(new StreamReader(ex.Response.GetResponseStream()).ReadToEnd());
            }

            return new ApimException(ex.Message, ex.InnerException, apiError, GetApimExceptionType(apiError));
        }

        /// <summary>
        /// Gets the type of the APIM exception.
        /// </summary>
        /// <param name="apiError">The API error.</param>
        /// <returns>The APIM Exception Type</returns>
        private static ApimExceptionType GetApimExceptionType(ApimError apiError)
        {
            ApimExceptionType result = ApimExceptionType.Unknown;
            if (apiError == null || apiError.Error == null)
            {
                return result;
            }

            switch (apiError.Error.Code.ToLowerInvariant())
            {
                case "validationerror":
                    {
                        if (string.Compare(apiError.Error.Message, "Subscriptions limit reached for same user", StringComparison.OrdinalIgnoreCase) == 0)
                        {
                            result = ApimExceptionType.SubscriptionLimitReached;
                        }

                        break;
                    }

                case "identifieralreadyinuse":
                    {
                        if (string.Compare(apiError.Error.Message, "Resource already exists.", StringComparison.OrdinalIgnoreCase) == 0)
                        {
                            result = ApimExceptionType.IdentifierAlreadyUse;
                        }

                        break;
                    }

                case "resourcenotfound":
                    {
                        result = ApimExceptionType.ResourceNotFound;
                        break;
                    }
            }

            return result;
        }
    }
}
