﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement
{
    using System;

    public class ApiManagementKey
    {
        public ApiManagementKey(string key, DateTime lastUpdated)
        {
            this.Key = key;
            this.LastUpdated = lastUpdated;
        }

        public string Key { get; private set; }

        public DateTime LastUpdated { get; private set; }
    }

    /// <summary>
    /// Class holding keys from ApiManagement.
    /// Guaranteed to have non-null keys
    /// </summary>
    public class ApiManagementKeys
    {
        public ApiManagementKeys(ApiManagementKey primaryKey, ApiManagementKey secondaryKey)
        {
            Validation.Requires.NotNull(primaryKey, nameof(primaryKey));
            Validation.Requires.NotNull(secondaryKey, nameof(secondaryKey));

            this.PrimaryKey = primaryKey;
            this.SecondaryKey = secondaryKey;
        }

        public ApiManagementKeys(string primaryKey, string secondaryKey, DateTime primaryKeyLastUpdated, DateTime secondaryKeyLastUpdated)
        {
            Validation.Requires.NotNullOrWhiteSpace(primaryKey, nameof(primaryKey));
            Validation.Requires.NotNullOrWhiteSpace(secondaryKey, nameof(secondaryKey));

            this.PrimaryKey = new ApiManagementKey(primaryKey, primaryKeyLastUpdated);
            this.SecondaryKey = new ApiManagementKey(secondaryKey, secondaryKeyLastUpdated);
        }

        public ApiManagementKey PrimaryKey { get; private set; }

        public ApiManagementKey SecondaryKey { get; private set; }
    }
}
