﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement
{
    using System;
    using System.Collections.Generic;
    using Microsoft.Azure.LocationServices.Common;
    using Validation;

    public class ApiManagementStorageFactory : IApiManagementStorageFactory
    {
        private readonly ApiManagementClient apiManagementClient;

        private readonly IReadOnlyDictionary<SkuName, ApiManagementStorageOptions> skuConfigurationMap;

        public ApiManagementStorageFactory(ApiManagementClient apiManagementClient, IReadOnlyDictionary<SkuName, ApiManagementStorageOptions> skuConfigurationMap)
        {
            Requires.NotNull(apiManagementClient, nameof(apiManagementClient));
            Requires.NotNullOrEmpty(skuConfigurationMap, nameof(skuConfigurationMap));

            this.apiManagementClient = apiManagementClient;
            this.skuConfigurationMap = skuConfigurationMap;
        }

        public IApiManagementStorage GetApiManagementStorage(SkuName sku)
        {
            if (this.skuConfigurationMap.TryGetValue(sku, out ApiManagementStorageOptions options))
            {
                return new ApiManagementStorage(options.GroupId, options.ProductId, this.apiManagementClient);
            }

            throw new ArgumentOutOfRangeException(nameof(sku), sku, "Value must be a supported sku name");
        }
    }
}
