﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement
{
    using Newtonsoft.Json;
    using Newtonsoft.Json.Converters;

    /// <summary>
    /// The HttpVerb enum
    /// </summary>
    public enum HttpVerb
    {
        /// <summary>
        /// The get
        /// </summary>
        Get,

        /// <summary>
        /// The put
        /// </summary>
        Put,

        /// <summary>
        /// The patch
        /// </summary>
        Patch,

        /// <summary>
        /// The post
        /// </summary>
        Post,

        /// <summary>
        /// The delete
        /// </summary>
        Delete
    }

    /// <summary>
    /// The SubscriptionState enum
    /// </summary>
    [JsonConverter(typeof(StringEnumConverter))]
    public enum SubscriptionState
    {
        /// <summary>
        /// The active
        /// </summary>
        Active,

        /// <summary>
        /// The suspended
        /// </summary>
        Suspended,

        /// <summary>
        /// The submitted
        /// </summary>
        Submitted,

        /// <summary>
        /// The rejected
        /// </summary>
        Rejected,

        /// <summary>
        /// The cancelled
        /// </summary>
        Cancelled,

        /// <summary>
        /// The expired
        /// </summary>
        Expired
    }

    /// <summary>
    /// The ProductState class
    /// </summary>
    [JsonConverter(typeof(StringEnumConverter))]
    public enum ProductState
    {
        /// <summary>
        /// The published
        /// </summary>
        Published,

        /// <summary>
        /// The not published
        /// </summary>
        NotPublished
    }

    /// <summary>
    /// The UserState enum
    /// </summary>
    [JsonConverter(typeof(StringEnumConverter))]
    public enum UserState
    {
        /// <summary>
        /// The active
        /// </summary>
        Active,

        /// <summary>
        /// The blocked
        /// </summary>
        Blocked
    }

    /// <summary>
    /// The Constants class
    /// </summary>
    public static class Constants
    {
        //// APIs

        /// <summary>
        /// The get APIS URL template
        /// </summary>
        public const string GetApisUrlTemplate = "/apis";

        /// <summary>
        /// The get API URL template
        /// </summary>
        public const string GetApiUrlTemplate = "/apis/{0}";

        ////operations

        /// <summary>
        /// The get operations URL template
        /// </summary>
        public const string GetOperationsUrlTemplate = "/apis/{0}/operations";

        //// Subscription

        /// <summary>
        /// The get subscriptions for user URL template
        /// </summary>
        public const string GetSubscriptionsForUserUrlTemplate = "/users/{0}/subscriptions";

        /// <summary>
        /// The get subscriptions URL template
        /// </summary>
        public const string GetSubscriptionsUrlTemplate = "/subscriptions";

        /// <summary>
        /// The get subscription URL template
        /// </summary>
        public const string GetSubscriptionUrlTemplate = "/subscriptions/{0}";

        /// <summary>
        /// The create subscription URL template
        /// </summary>
        public const string CreateSubscriptionUrlTemplate = "/subscriptions/{0}_{1}";

        /// <summary>
        /// The delete user subscription URL template
        /// </summary>
        public const string DeleteUserSubscriptionUrlTemplate = "/subscriptions/{0}_{1}";

        /// <summary>
        /// The create specific subscription URL template
        /// </summary>
        public const string CreateSpecificSubscriptionUrlTemplate = "/subscriptions/{0}";

        /// <summary>
        /// The delete specific subscription URL template
        /// </summary>
        public const string DeleteSpecificSubscriptionUrlTemplate = "/subscriptions/{0}";

        //// Users

        /// <summary>
        /// The get user URL template
        /// </summary>
        public const string GetUserUrlTemplate = "/users/{0}";

        /// <summary>
        /// The get users URL template
        /// </summary>
        public const string GetUsersUrlTemplate = "/users";

        /// <summary>
        /// The get users with group URL template
        /// </summary>
        public const string GetUserswithgroupUrlTemplate = "/users";

        /// <summary>
        /// The create user URL template
        /// </summary>
        public const string CreateUserUrlTemplate = "/users/{0}";

        /// <summary>
        /// The delete user URL template
        /// </summary>
        public const string DeleteUserUrlTemplate = "/users/{0}";

        /// <summary>
        /// The update user URL template
        /// </summary>
        public const string UpdateUserUrlTemplate = "/users/{0}";

        /// <summary>
        /// The get SSO URL URL template
        /// </summary>
        public const string GetSsoUrlUrlTemplate = "/users/{0}/generatessourl";

        /// <summary>
        /// The add user to group URL template
        /// </summary>
        public const string AddUserToGroupUrlTemplate = "/groups/{0}/users/{1}";

        /// <summary>
        /// The delete user from group URL template
        /// </summary>
        public const string DeleteUserFromGroupUrlTemplate = "/groups/{0}/users/{1}";

        //// Products

        /// <summary>
        /// The get products URL template
        /// </summary>
        public const string GetProductsUrlTemplate = "/products";

        /// <summary>
        /// The get product URL template
        /// </summary>
        public const string GetProductUrlTemplate = "/products/{0}";

        /// <summary>
        /// The get product APIS URL template
        /// </summary>
        public const string GetProductApisUrlTemplate = "/products/{0}/apis";

        /// <summary>
        /// The create product URL template
        /// </summary>
        public const string CreateProductUrlTemplate = "/products/{0}";

        /// <summary>
        /// The update product URL template
        /// </summary>
        public const string UpdateProductUrlTemplate = "/products/{0}";

        /// <summary>
        /// The create product API URL template
        /// </summary>
        public const string CreateProductApiUrlTemplate = "/products/{0}/apis/{1}";

        /// <summary>
        /// The delete product API URL template
        /// </summary>
        public const string DeleteProductApiUrlTemplate = "/products/{0}/apis/{1}";

        ////Report

        /// <summary>
        /// The get report by operation URL template
        /// </summary>
        public const string GetReportByOperationUrlTemplate = "/reports/byOperation";

        /// <summary>
        /// The get report by user URL template
        /// </summary>
        public const string GetReportByUserUrlTemplate = "/reports/byUser";

        /// <summary>
        /// The get report by time URL template
        /// </summary>
        public const string GetReportByTimeUrlTemplate = "/reports/byTime";

        /// <summary>
        /// The get report by subscription URI template
        /// </summary>
        public const string GetReportBySubscriptionUriTemplate = "/reports/bysubscription";

        /// <summary>
        /// The get policy URL template
        /// </summary>
        public const string GetPolicyUrlTemplate = "/products/{0}/policy";

        //// Miscellaneous

        /// <summary>
        /// The APIM user format
        /// </summary>
        public const string ApimUserFormat = "/users/{0}";

        /// <summary>
        /// The APIM product format
        /// </summary>
        public const string ApimProductFormat = "/products/{0}";

        /// <summary>
        /// The suspended primary key
        /// </summary>
        public const string SuspendedPrimaryKey = "X";

        /// <summary>
        /// The suspended secondary key
        /// </summary>
        public const string SuspendedSecondaryKey = "X";
    }
}
