﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement.Contracts.ResponseContract
{
    using System;
    using System.Runtime.Serialization;
    using Newtonsoft.Json;

    /// <summary>
    ///     The APIMException class
    /// </summary>
    /// <seealso cref="System.Exception" />
    /// <seealso cref="System.Runtime.Serialization.ISerializable" />
    [SerializableAttribute]
    public class ApimException : Exception, ISerializable
    {
        /// <summary>
        ///     Initializes a new instance of the <see cref="ApimException" /> class.
        /// </summary>
        public ApimException()
        {
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="ApimException" /> class.
        /// </summary>
        /// <param name="message">The message that describes the error.</param>
        public ApimException(string message)
            : base(message)
        {
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="ApimException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="inner">The inner.</param>
        public ApimException(string message, Exception inner)
            : base(message, inner)
        {
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="ApimException" /> class.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <param name="inner">The inner.</param>
        /// <param name="err">The error.</param>
        /// <param name="type">The type.</param>
        public ApimException(string message, Exception inner, ApimError err, ApimExceptionType type)
            : base(message, inner)
        {
            ApiError = err;
            ExceptionType = type;
            base.Data.Add("ApimExceptionDetails", JsonConvert.SerializeObject(err));
        }

        /// <summary>
        ///     Initializes a new instance of the <see cref="ApimException" /> class.
        /// </summary>
        /// <param name="info">
        ///     The <see cref="T:System.Runtime.Serialization.SerializationInfo" /> that holds the serialized object
        ///     data about the exception being thrown.
        /// </param>
        /// <param name="context">
        ///     The <see cref="T:System.Runtime.Serialization.StreamingContext" /> that contains contextual
        ///     information about the source or destination.
        /// </param>
        protected ApimException(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        /// <summary>
        ///     Gets or sets the API error.
        /// </summary>
        /// <value>
        ///     The API error.
        /// </value>
        public ApimError ApiError { get; set; }

        /// <summary>
        ///     Gets or sets the type of the exception.
        /// </summary>
        /// <value>
        ///     The type of the exception.
        /// </value>
        public ApimExceptionType ExceptionType { get; set; }

        /// <summary>
        ///     When overridden in a derived class, sets the <see cref="T:System.Runtime.Serialization.SerializationInfo" /> with
        ///     information about the exception.
        /// </summary>
        /// <param name="info">
        ///     The <see cref="T:System.Runtime.Serialization.SerializationInfo" /> that holds the serialized object
        ///     data about the exception being thrown.
        /// </param>
        /// <param name="context">
        ///     The <see cref="T:System.Runtime.Serialization.StreamingContext" /> that contains contextual
        ///     information about the source or destination.
        /// </param>
        /// <PermissionSet>
        ///     <IPermission
        ///         class="System.Security.Permissions.FileIOPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"
        ///         version="1" Read="*AllFiles*" PathDiscovery="*AllFiles*" />
        ///     <IPermission
        ///         class="System.Security.Permissions.SecurityPermission, mscorlib, Version=2.0.3600.0, Culture=neutral, PublicKeyToken=b77a5c561934e089"
        ///         version="1" Flags="SerializationFormatter" />
        /// </PermissionSet>
        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            base.GetObjectData(info, context);
        }
    }
}