﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement
{    
    public interface IApiManagementStorage
    {
        ApiManagementIdentity AddNewServiceKeys(AccountIdentifier accountIdentifier, out ApiManagementKeys keys);

        ApiManagementIdentity WriteServiceKeys(AccountIdentifier accountIdentifier, string apiManagementSubscriptionId, out ApiManagementKeys keys, ApiManagementKey primaryKey, ApiManagementKey secondaryKey);

        /// <summary>
        /// Writes current accountIdentifier information to api management.
        /// </summary>
        /// <param name="apimIdentity"></param>
        /// <param name="accountIdentifier"></param>
        void PatchUser(ApiManagementIdentity apimIdentity, AccountIdentifier accountIdentifier);

        /// <summary>
        /// noop if record doesn't exist
        /// </summary>
        /// <param name="id"></param>
        void DeleteUser(ApiManagementIdentity id);

        /// <summary>
        /// returns null if key doesn't exist
        /// </summary>
        /// <remarks>No longer the preferred approach to retrieve keys. Preserved to handle case where keys aren't already in storage.</remarks>
        /// <param name="id"></param>
        /// <returns></returns>
        ApiManagementKeys GetServiceKeys(ApiManagementIdentity id);

        /// <summary>
        /// TODO: failure behavior?
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        ApiManagementKey RegeneratePrimaryKey(ApiManagementIdentity id);

        /// <summary>
        /// TODO: failure behavior?
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        ApiManagementKey RegenerateSecondaryKey(ApiManagementIdentity id);

        bool UserExists(ApiManagementIdentity id);        
    }
}