﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement
{
    using Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement.Contracts.ResponseContract;
    using Newtonsoft.Json;
    using Newtonsoft.Json.Serialization;

    /// <summary>
    ///     The Utility class
    /// </summary>
    internal static class Utility
    {
        /// <summary>
        ///     Serializes the object.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns>The string value</returns>
        public static string SerializeObject(object value)
        {
            return JsonConvert.SerializeObject(
                value,
                new JsonSerializerSettings
                {
                    NullValueHandling = NullValueHandling.Ignore,
                    ContractResolver = new CamelCasePropertyNamesContractResolver()
                });
        }

        /// <summary>
        ///     Deserializes the object.
        /// </summary>
        /// <typeparam name="T">The Type 'T'</typeparam>
        /// <param name="json">The JSON.</param>
        /// <returns>
        ///     The value of 'T'
        /// </returns>
        /// <exception cref="Microsoft.CognitiveServices.APIMWrapper.Contracts.ResponseContract.ApimException">The APIM Exception</exception>
        /// <exception cref="ApimException">The ApimException</exception>
        public static T DeserializeObject<T>(string json)
        {
            try
            {
                return (T)JsonConvert.DeserializeObject(json, typeof(T));
            }
            catch (JsonException ex)
            {
                var errorMessage =
                    string.Format(
                        "Failed to deserialize response from APIM to type {0}, error = '{1}', response body = '{2}'",
                        typeof(T).Name,
                        ex.Message,
                        json);

                throw new ApimException(errorMessage, ex, null, ApimExceptionType.BadResponse);
            }
        }
    }
}