﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI
{
    using System.Net.Http;
    using System.Web.Http.Controllers;
    using System.Web.Http.Filters;

    internal class CheckClientCertificateFilter : AuthorizationFilterAttribute
    {
        private readonly IClientCertificateResolverFactory clientCertificateResolverFactory;

        public CheckClientCertificateFilter(IClientCertificateResolverFactory clientCertificateResolverFactory)
        {
            this.clientCertificateResolverFactory = clientCertificateResolverFactory;
        }

        public override void OnAuthorization(HttpActionContext actionContext)
        {
            base.OnAuthorization(actionContext);

            // Client auth doesn't apply to Heartbeat controller. Client auth only applies to our ARM APIs, which derive ResourceProviderControllerBase.
            if (actionContext.ControllerContext.ControllerDescriptor.ControllerType == typeof(Controllers.HeartbeatController))
            {
                return;
            }

            if (this.clientCertificateResolverFactory.GetClientCertificateResolver().ValidateClientCertificate(actionContext.Request.GetClientCertificate()))
            {
                var rpController = actionContext.ControllerContext.Controller as ResourceProviderControllerBase;
                Validation.Requires.NotNull(rpController, "rpController");

                rpController.SetAuthorized();

                // important:  don't set the Response here (as tempting as that may be).  If done, it will bypass all other controller code and filters
                // actionContext.Response = actionContext.Request.CreateResponse(HttpStatusCode.Forbidden);
            }
        }
    }
}
