﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Controllers.Deprecated
{
    using System.Net;
    using System.Net.Http;
    using System.Web.Http;

    [Route(@"subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{providerName:regex(^Microsoft\.Location(Based)?Services$)}/accounts/{resourceName}")]
    public class DeprecatedAccountController : ResourceProviderControllerBase
    {
        [HttpGet]
        public HttpResponseMessage Get([FromUri] string subscriptionId,
                                       [FromUri] string resourceGroupName,
                                       [FromUri] string resourceName)
        {
            HttpResponseMessage response;
            try
            {
                this.Operation.LogCommonParameters(subscriptionId, resourceGroupName, resourceName);

                // This will throw a ResourceProviderException on failure
                this.ValidateAuthorized();

                // ARM mandates that "all the GET APIs should be modified to return 404." in order to invalidate the cache.
                // http://sharepoint/sites/AzureUX/Sparta/SpartaWiki/Resource%20Provider%20Un-Registration.aspx
                this.Operation.SetResultClientError("notfound", "Client called " + this.GetType().Name);
                response = this.Request.CreateResponse(HttpStatusCode.NotFound, Operations.LBS_Deprecated_Message);
            }
            catch (ResourceProviderException ex)
            {
                // these are errors which get converted to nice return codes and messages
                // let unknown exceptions fall through - they will be handled by global hook
                response = this.BuildErrorResponseAndLog(ex);
            }

            return response;
        }

        [HttpPatch]
        [HttpDelete]
        [HttpPut]
        public HttpResponseMessage BadRequests([FromUri] string subscriptionId,
                                         [FromUri] string resourceGroupName,
                                         [FromUri] string resourceName)
        {
            HttpResponseMessage response;
            try
            {
                this.Operation.LogCommonParameters(subscriptionId, resourceGroupName, resourceName);

                // This will throw a ResourceProviderException on failure
                this.ValidateAuthorized();

                this.Operation.SetResultClientError("badrequest", "Client called " + this.GetType().Name);
                response = this.Request.CreateResponse(HttpStatusCode.BadRequest, Operations.LBS_Deprecated_Message);
            }
            catch (ResourceProviderException ex)
            {
                // these are errors which get converted to nice return codes and messages
                // let unknown exceptions fall through - they will be handled by global hook
                response = this.BuildErrorResponseAndLog(ex);
            }

            return response;
        }
    }
}