﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Controllers.Deprecated
{
    using System.Collections.Generic;
    using System.Net;
    using System.Net.Http;
    using System.Web.Http;

    [Route(@"subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{providerName:regex(^Microsoft\.Location(Based)?Services$)}/accounts")]
    public class DeprecatedAccountsByResourceGroupController : ResourceProviderControllerBase
    {
        [HttpGet]
        public HttpResponseMessage Get([FromUri] string subscriptionId,
                                       [FromUri] string resourceGroupName)
        {
            HttpResponseMessage response;
            try
            {
                this.Operation.LogCommonParameters(subscriptionId, resourceGroupName);

                // This will throw a ResourceProviderException on failure
                this.ValidateAuthorized();

                var accounts = new ExternalContracts.MultipleAccounts
                {
                    Value = new List<ExternalContracts.Account>()
                };

                // ARM mandates that "A collection GET from the RP must return an empty array with 200 response." in order to invalidate the cache.
                // http://sharepoint/sites/AzureUX/Sparta/SpartaWiki/Resource%20Provider%20Un-Registration.aspx
                response = this.Request.CreateResponse(HttpStatusCode.OK, accounts);
                this.Operation.SetResultSuccess();
            }
            catch (ResourceProviderException ex)
            {
                // these are errors which get converted to nice return codes and messages
                // let unknown exceptions fall through - they will be handled by global hook
                response = this.BuildErrorResponseAndLog(ex);
            }

            return response;
        }
    }
}
