﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ExternalContracts
{
    using System.Collections.Generic;
    using System.Dynamic;
    using Microsoft.Azure.LocationServices.Common;
    using Newtonsoft.Json;

    public class SkuType
    {
        [JsonProperty(PropertyName = "name", DefaultValueHandling = DefaultValueHandling.Include)]
        public SkuName Name { get; set; }
        [JsonProperty(PropertyName = "tier")]
        public string Tier { get; set; }
    }

    /// <summary>
    /// Representation of an Account in the web api input and output
    /// </summary>
    public class Account
    {
        /// <summary>
        /// Default ctor needed for json deserialization
        /// </summary>
        public Account()
        {
        }

        internal Account(ResourceProviderWebAPI.Account account, string providerName)
        {
            Validation.Requires.NotNull(account, nameof(account));

            this.Id = BuildExternalId(account.Id, providerName);
            this.Name = account.Id.ResourceName;
            this.Properties = account.ClientId == null ? null : new Dictionary<string, string>() { { "clientId",  account.ClientId } };
            this.Type = $"{providerName}/{ResourceProviderWebApiService.ResourceTypeName}";

            this.Location = account.Location.LocationString;
            this.Sku = new ExternalContracts.SkuType()
                       {
                           Name = account.Sku.Name,
                           Tier = account.Sku.Tier.ToString()
                       };

            this.Tags = account.Tags;
        }

        [JsonProperty(PropertyName = "id")]
        public string Id { get; set; }
        [JsonProperty(PropertyName = "name")]
        public string Name { get; set; }
        [JsonProperty(PropertyName = "type")]
        public string Type { get; set; }
        [JsonProperty(PropertyName = "location")]
        public string Location { get; set; }
        [JsonProperty(PropertyName = "tags")]
        public ExpandoObject Tags { get; set; }
        [JsonProperty(PropertyName = "sku")]
        public SkuType Sku { get; set; }

        [JsonProperty(PropertyName = "properties", NullValueHandling = NullValueHandling.Ignore)]        
        public Dictionary<string, string> Properties { get; set; }  

        internal static string BuildExternalId(AccountIdentifier id, string providerName = ResourceProviderWebApiService.ResourceProviderName)
        {
            return $"/subscriptions/{id.SubscriptionId}/resourceGroups/{id.ResourceGroupName}/providers/{providerName}/{ResourceProviderWebApiService.ResourceTypeName}/{id.ResourceName}";
        }
    }
}
