﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ExternalContracts
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Controllers;
    using Newtonsoft.Json;

    public class ErrorDetail
    {
        [JsonProperty(PropertyName = "code")]
        public string Code { get; set; }

        [JsonProperty(PropertyName = "message")]
        public string Message { get; set; }

        [JsonProperty(PropertyName = "target")]
        public string Target { get; set; }
    }

    public class Error
    {
        internal Error(ResourceProviderException ex)
        {
            if (ex.Type == ResourceProviderException.ExceptionType.ClientParams || ex.Type == ResourceProviderException.ExceptionType.JscriptParsing)
            {
                const string code = "InvalidParameter";
                Code = code;
                Message = ((ResourceProviderException)ex).Message;

                // if there is an innerexception, create Details from it
                // (can do either an AggregateException to create many Details, or just a single
                if (ex.InnerException != null)
                {
                    if (IsTypeOrSubclass(ex.InnerException, typeof(AggregateException)))
                    {
                        Details =
                            ((AggregateException)ex.InnerException).InnerExceptions.Select(
                                e => new ErrorDetail() { Code = code, Message = e.Message }).ToList();
                    }
                    else
                    {
                        Details = new List<ErrorDetail>()
                                  {
                                      new ErrorDetail()
                                      {
                                          Code = code,
                                          Message = ex.Message
                                      }
                                  };
                    }
                }
            }
            else
            {
                // really shouldn't get here.   other kinds of ResourceProviderExceptions don't get the full Error object treatment and should
                // have been handled elsewhere
                Code = "unknown";
            }
        }

        /// <summary>
        /// ctor.  Uses code "unknown"
        /// </summary>
        /// <param name="message"></param>
        internal Error(string message)
        {
            Code = "unknown";
            Message = message;
        }

        [JsonProperty(PropertyName = "code")]
        public string Code { get; set; }

        [JsonProperty(PropertyName = "message")]
        public string Message { get; set; }

        [JsonProperty(PropertyName = "target")]
        public string Target { get; set; }

        [JsonProperty(PropertyName = "details")]
        public List<ErrorDetail> Details { get; set; }

        private bool IsTypeOrSubclass(object objectToTest, Type type)
        {
            var t = objectToTest.GetType();
            return t.IsSubclassOf(type) || (t == type);
        }
    }

    public class ErrorResponse
    {
        public ErrorResponse(ResourceProviderException ex)
        {
            Error = new Error(ex);
        }

        /// <summary>
        /// This displays a message appropropriate to a client
        /// </summary>
        /// <param name="message"></param>
        public ErrorResponse(string message)
        {
            Error = new Error(message);
        }

        [JsonProperty(PropertyName = "error")]
        public Error Error { get; private set; }
    }
}