// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI
{
    /// <summary>
    ///     <para>
    ///     Used for 'expected' exceptions, such as bad params, and conditions that can be communicated back to the user.
    ///     It is assumed that Message is safe for user consumption.
    ///     </para>
    ///     This exception is typically created only at the highest (ie controller) layer.  Other layers will throw things like
    ///     ArgumentExceptions, which will be transformed into this.
    /// </summary>
    [System.Serializable]
    public sealed class ResourceProviderException : System.Exception
    {
        public ResourceProviderException(ExceptionType type, string userMessage)
            : base(userMessage)
        {
            this.Type = type;
        }

        /// <summary>
        ///     ctor if wrapping an exception.   Note that innerException.Message will be returned to the user
        /// </summary>
        /// <param name="type"></param>
        /// <param name="userMessage"></param>
        /// <param name="innerException"></param>
        public ResourceProviderException(ExceptionType type, string userMessage, System.Exception innerException)
            : base(userMessage, innerException)
        {
            this.Type = type;
        }

        /// <summary>
        /// A special purpose ctor used from Controllers that need to wrap an ArgumentException thrown from a lower level that is actionable by the client
        /// </summary>
        /// <param name="ex"></param>
        public ResourceProviderException(System.ArgumentException ex)
            : this(ResourceProviderException.ExceptionType.ClientParams, ex.ParamName)
        {
        }

        public enum ExceptionType
        {
            Unauthorized,
            ClientParams,
            JscriptParsing,
            Other
        }

        public ExceptionType Type { get; set; }

        public override void GetObjectData(System.Runtime.Serialization.SerializationInfo info, System.Runtime.Serialization.StreamingContext context)
        {
            if (info == null)
            {
                throw new System.ArgumentNullException("info");
            }

            info.AddValue("Type", this.Type);
            base.GetObjectData(info, context);
        }
    }
}