﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI
{
    using System;
    using System.Collections.Generic;
    using System.Fabric;
    using System.Threading;
    using System.Threading.Tasks;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.ServiceHosting;
    using Microsoft.ServiceFabric.Services.Communication.Runtime;
    using Microsoft.ServiceFabric.Services.Runtime;

    /// <summary>
    /// The FabricRuntime creates an instance of this class for each service type instance. 
    /// </summary>
    internal sealed class ResourceProviderWebApiService : StatelessService
    {
        public const string ResourceProviderName = "Microsoft.Maps";
        public const string ResourceTypeName = "accounts";

        public ResourceProviderWebApiService(StatelessServiceContext context)
            : base(context)
        {
        }

        /// <summary>
        /// Optional override to create listeners (like tcp, http) for this service instance.
        /// </summary>
        /// <returns>The collection of listeners.</returns>
        protected override IEnumerable<ServiceInstanceListener> CreateServiceInstanceListeners()
        {              
            return new ServiceInstanceListener[]
                   {
                       new ServiceInstanceListener(
                           serviceContext =>
                               new OwinCommunicationListener(Startup.ConfigureApp,
                                                             serviceContext,
                                                             "ServiceEndpoint"))
                   };
        }

        protected override async Task RunAsync(CancellationToken cancellationToken)
        {
            try
            {
                Utility.ConfigurationReader.Instance.Initialize(Context);
                await Startup.RegisterDependenciesAsync();
                IClientCertificateResolver clientCertificateResolver = Startup.ResolveClientCertificateResolver();
                await clientCertificateResolver.FetchClientAuthCertificatesAsync(cancellationToken);
            }            
            catch (TaskCanceledException)
            {
                // TaskCanceledException is expected when the service fabric is terminating this service instance.
                Agent.Instance.LogEvent(EventType.Information, "Stopping the ResourceProviderWebApiService on request from SF.");
            }
            catch (Exception ex)
            {
                Agent.Instance.LogException(ex, "resourceProvider", "initialization");
                throw;
            }
        }
    }
}
