﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Storage
{
    using System.Collections.Generic;
    using System.Threading.Tasks;

    public interface IAccountRecordStorage
    {
        /// <summary>
        ///     Marks record as deleted (soft delete).
        ///     Noop if record does not exist
        /// </summary>
        /// <param name="id"></param>
        Task DeleteAccountRecord(AccountIdentifier id);

        /// <summary>
        ///     Returns the record, or null if it doesn't exist
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        Task<AccountRecord> GetAccountRecord(AccountIdentifier id);

        IEnumerable<AccountRecord> GetAccountRecordsByResourceGroup(string subscriptionId,
                                                                    string resourceGroupName,
                                                                    bool includeDeleted = false);        

        IEnumerable<AccountRecord> GetAccountRecordsBySubscription(string subscriptionId, bool includeDeleted = false);

        Task PutAccountRecord(AccountRecord account);
    }
}