﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Storage
{
    using System;
    using Microsoft.WindowsAzure.Storage.Table;
    using Newtonsoft.Json;

    public class SubscriptionRecord : TableEntity
    {
        /// <summary>
        /// ctor required by storage deserialization
        /// </summary>
        public SubscriptionRecord()
        {
        }

        public SubscriptionRecord(Subscription subscription)
        {
            Validation.Requires.NotNull(subscription, nameof(subscription));
            Validation.Requires.NotNull(subscription.Id, nameof(subscription.Id));

            this.PartitionKey = SubscriptionRecordStorage.PartitionKey(subscription.Id);
            this.RowKey = SubscriptionRecordStorage.RowKey(subscription.Id);

            this.SubscriptionState = (int)subscription.State;
            this.Properties = JsonConvert.SerializeObject(subscription.Properties);
            this.Created = DateTime.UtcNow;
        }

        public string Properties { get; set; }

        public int SubscriptionState { get; set; }

        public DateTime Created { get; set; }
    }
}
