﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Utility
{
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement;

    /// <summary>
    /// The staging environment definition.
    /// </summary>
    public class ResourceProviderHostStagingSettings : ResourceProviderHostSettings
    {        
        private readonly IReadOnlyDictionary<SkuName, ApiManagementStorageOptions> skuApimMap = new Dictionary<SkuName, ApiManagementStorageOptions>
        {
            { SkuName.S0, new ApiManagementStorageOptions("5925c98a5996f60c3cb1083e", "5925c92f5996f60c3cb1083d") }
        };

        /// <inheritdoc />
        public override InitParams MdmParameters => new InitParams("AzLocStgIfx", "AzLocStg");

        /// <inheritdoc />
        public override IReadOnlyDictionary<SkuName, ApiManagementStorageOptions> SkuApimMap => this.skuApimMap;

        /// <inheritdoc />
        public override Collection<string> CosmosDbPreferredLocations
        {
            get
            {
                Collection<string> locations = null;
                switch (CurrentRegion)
                {
                    case AzureRegions.EastUS:
                        locations = CosmosDbConstants.Staging_PreferredEastUsLocations;
                        break;
                    case AzureRegions.WestUS2:
                        locations = CosmosDbConstants.Staging_PreferredWestUs2Locations;
                        break;
                    case AzureRegions.WestEurope:
                        locations = CosmosDbConstants.Staging_PreferredWestEuropeLocations;
                        break;
                    default:
                        locations = CosmosDbConstants.Staging_PreferredEastUsLocations;
                        break;
                }

                return locations;
            }
        }

        /// <inheritdoc />
        public override bool EnableAccountReplication => false;

        /// <inheritdoc />
        protected override string GlobalKeyVaultName => "s-global";

        /// <inheritdoc />
        protected override string MapsFrontEndCosmosDbAccountName => CosmosDbConstants.Staging_CosmosDbAccountName;
    }
}
