﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI.Utility
{
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.Azure.LocationServices.Instrumentation;
    using Microsoft.Azure.LocationServices.ResourceProviderWebAPI.ApiManagement;

    /// <summary>
    /// The Integration/Test environment definition.
    /// </summary>
    public class ResourceProviderHostTestSettings : ResourceProviderHostSettings
    {        
        private readonly IReadOnlyDictionary<SkuName, ApiManagementStorageOptions> skuApimMap = new Dictionary<SkuName, ApiManagementStorageOptions>
        {
            { SkuName.S0, new ApiManagementStorageOptions("5925c98a5996f60c3cb1083e", "5925c92f5996f60c3cb1083d") }
        };

        /// <inheritdoc />
        public override InitParams MdmParameters
        {
            get
            {
                InitParams p = new InitParams("DevTestIfx", "AzLocTest");
                return p;
            }
        }

        /// <inheritdoc />
        public override IReadOnlyDictionary<SkuName, ApiManagementStorageOptions> SkuApimMap => this.skuApimMap;

        /// <inheritdoc />
        public override Collection<string> CosmosDbPreferredLocations
        {
            get
            {
                return CosmosDbConstants.Test_PreferredEastUsLocations;
            }
        }

        /// <inheritdoc />
        public override bool EnableAccountReplication => false;

        /// <inheritdoc />
        protected override string GlobalKeyVaultName => "t-global";

        /// <inheritdoc />
        protected override string MapsFrontEndCosmosDbAccountName => CosmosDbConstants.Test_CosmosDbAccountName;
    }
}
