﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ResourceProviderWebAPI
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Web.Http.Controllers;
    using System.Web.Http.Filters;

    public class ValidModelsOnlyFilter : ActionFilterAttribute
    {
        /// <summary>
        /// Inspect the result of the model parsing.  If there are exceptions, squirrel them away in the controller where they will
        /// be found later.  See <see cref="ResourceProviderControllerBase.ValidateModelParse"/>
        /// </summary>
        /// <param name="actionContext"></param>
        public override void OnActionExecuting(HttpActionContext actionContext)
        {
            if (actionContext.ModelState.IsValid)
            {
                base.OnActionExecuting(actionContext);
            }
            else
            {
                var rpController = actionContext.ControllerContext.Controller as ResourceProviderControllerBase;
                Validation.Requires.NotNull(rpController, "rpController");
                Validation.Requires.NotNull(rpController.Operation, "rpController.Operation");

                var exceptions = new List<Exception>();

                foreach (var state in actionContext.ModelState)
                {
                    if (state.Value.Errors.Count != 0)
                    {
                        exceptions.AddRange(state.Value.Errors.Select(error => error.Exception));
                    }
                }

                if (exceptions.Count == 1)
                {
                    rpController.SetModelParsingException(exceptions[0]);
                }
                else if (exceptions.Count > 1)
                {
                    rpController.SetModelParsingException(new AggregateException(exceptions));
                }
            }
        }
    }
}