import lbs.implementation.AzureLocationBasedServicesImpl;
import lbs.models.*;

import java.util.ArrayList;
import java.util.LinkedHashMap;

public class TestLbs {
    public static void main(String args[]) {
        if (args.length != 1 || args[0].length() == 0) {
            System.out.println("ERROR: provide subscription key in command line args");
        }
        String subscriptionKey = args[0];
        AzureLocationBasedServicesImpl albs = new AzureLocationBasedServicesImpl();

        String coordinates = null;

        albs.setQuery("Seattle");
        System.out.println("----- SearchFuzzy: " + albs.query() + " -----");
        try {
            Searchfuzzy200 r = (Searchfuzzy200)albs.searchs().getSearchFuzzy("json", "1.0", subscriptionKey);
            if (r.summary().numResults() == 0) {
                System.out.println("no results for " + albs.query());
            }
            else {
                int i = 0;
                for (Searchfuzzy200ResultsItem item : r.results()) {
                    if (i == 0) {
                        coordinates = item.position().lat() + "," + item.position().lon();
                    }
                    System.out.println(++i + " - type: " + item.entityType() + ",  item: " + item.address().freeformAddress() + ",  position: " + item.position().lat() + ", " + item.position().lon());
                }
            }
        }
        catch(Exception ex) {
            System.out.println(ex.toString());
        }

        System.out.println("----- TimeZones Version -----");
        try {
            Object o = albs.timezones().getTimezoneIANAVersion("json", "1.0", subscriptionKey);
            System.out.println(((Timezoneianaversion200)o).version());
        }
        catch(Exception ex) {
            System.out.println(ex.toString());
        }

        System.out.println("----- Enumerate (non-alias) IANA TimeZones -----");
        try {
            ArrayList r = (ArrayList)albs.timezones().getTimezoneEnumIANA("json", "1.0", subscriptionKey);
            if (r.isEmpty()) {
                System.out.println("no results for enumerate IANA zones");
            }
            else {
                int i = 0;
                for (Object items : r) {
                    LinkedHashMap<Object, Object> item = (LinkedHashMap<Object, Object>) items;
                    if(((Boolean)item.get("IsAlias")) == false) {
                        System.out.println(++i + " - " + item.get("Id"));
                    }
                }
            }
        }
        catch(Exception ex) {
            System.out.println(ex.toString());
        }

        albs.setQuery("Europe/Berlin");
        System.out.println("----- TimeZones by ID: " + albs.query() + " -----");
        try {
            Timezonebyid200 r = (Timezonebyid200)albs.timezones().getTimezoneByID("json", "1.0", subscriptionKey);
            if (r.timeZones() == null || r.timeZones().isEmpty()) {
                System.out.println("no results for " + albs.query());
            }
            else {
                int i = 0;
                for (Timezonebyid200TimeZonesItem item : r.timeZones()) {

                    System.out.println(++i + " - time zone: " + item.id() + ",  wall time: " + item.referenceTime().wallTime());
                }
            }
        }
        catch(Exception ex) {
            System.out.println(ex.toString());
        }

        albs.setQuery(coordinates);
        System.out.println("----- TimeZones by coordinates: " + albs.query() + " -----");
        try {
            Timezonebycoordinates200 r = (Timezonebycoordinates200)albs.timezones().getTimezoneByCoordinates("json", "1.0", subscriptionKey);
            if (r.timeZones() == null || r.timeZones().isEmpty()) {
                System.out.println("no results for " + albs.query());
            }
            else {
                int i = 0;
                for (Timezonebycoordinates200TimeZonesItem item : r.timeZones()) {

                    System.out.println(++i + " - time zone: " + item.id() + ",  wall time: " + item.referenceTime().wallTime());
                }
            }
        }
        catch(Exception ex) {
            System.out.println(ex.toString());
        }
    }
}
