Param(
    [Parameter(Mandatory = $True)] $variablesJsonPath,
    [switch] $isMajorUpdate
)

$variablesJson = Get-Content -Path $variablesJsonPath | ConvertFrom-Json
$packageRootDir = Invoke-Expression "npm prefix"

$env_staticAssetsDomain = $variablesJson.env.staticAssetsDomain;
$major_version = $variablesJson.majorVersion;
$minor_version = $variablesJson.minorVersion;

$distribution_dir_path = "$packageRootDir/dist"
$full_version_sdk_dir_path = "$distribution_dir_path/sdk/$major_version.$minor_version"

# Clean up previous builds
If (test-path $distribution_dir_path) {
    Remove-Item -Path $distribution_dir_path -Recurse -Force
}

# Create sdk distribution folder
If (!(test-path $full_version_sdk_dir_path)) {
    New-Item -ItemType Directory -Force -Path $full_version_sdk_dir_path
}

# Copy third party notice to sdk distribution folder
Copy-Item "$packageRootDir/thirdpartynotices.txt" "$full_version_sdk_dir_path" -Force

# Copy license to sdk distribution folder
Copy-Item "$packageRootDir/license.txt" "$full_version_sdk_dir_path" -Force

# Minify and output minified css to css distribution folder
$css_dir_path = "$full_version_sdk_dir_path/css"
If (!(test-path $css_dir_path)) {
    New-Item -ItemType Directory -Force -Path $css_dir_path
}

Invoke-Expression "npm run lessc -- --clean-css=`"--compatibility=ie8 --advanced`" --global-var=`"domain='$env_staticAssetsDomain'`" ./src/css/atlas.less $css_dir_path/atlas.min.css"

# Minify and output minified js to js distribution folder
$js_dir_path = "$full_version_sdk_dir_path/js"
If (!(test-path $js_dir_path)) {
    New-Item -ItemType Directory -Force -Path $js_dir_path
}

$tmp_atlas_sdk_file_path = "$js_dir_path/atlas.min.js.tmp"
$atlas_sdk_file_path = "$js_dir_path/atlas.min.js"

Add-Content -Path $atlas_sdk_file_path -Value "/*`n"
Add-Content -Path $atlas_sdk_file_path -Value (Get-Content "$packageRootDir/license.txt")
Add-Content -Path $atlas_sdk_file_path -Value "`n"
Add-Content -Path $atlas_sdk_file_path -Value (Get-Content "$packageRootDir/thirdpartynotices.txt")
Add-Content -Path $atlas_sdk_file_path -Value "`n*/`n"

Invoke-Expression "npm run browserify -- ./src/index.ts -g deamdify -p tsify -g uglifyify --s atlas -o $tmp_atlas_sdk_file_path"
Add-Content -Path $atlas_sdk_file_path -Value (Get-Content $tmp_atlas_sdk_file_path)

Remove-Item $tmp_atlas_sdk_file_path

# Copy mapbox-gl-rtl-text.js plugin to js distribution folder
Copy-Item "$packageRootDir/node_modules/@mapbox/mapbox-gl-rtl-text/mapbox-gl-rtl-text.js" $js_dir_path -Force

# Copy examples to examples distribution folder
Copy-Item "$packageRootDir/examples" "$full_version_sdk_dir_path" -Recurse -Force

If ($isMajorUpdate) {
    # Copy the MAJOR.MINOR SDK to the MAJOR SDK folder
    $major_version_sdk_dir_path = "$distribution_dir_path/sdk/$major_version"
    Copy-Item $full_version_sdk_dir_path $major_version_sdk_dir_path -Recurse -Force

    # Copy examples to examples MAJOR SDK folder
    Copy-Item "$packageRootDir/examples" "$major_version_sdk_dir_path" -Recurse -Force
}
