import { Map } from "../map/Map";
import { ControlOptions } from "../map/options/ControlOptions";

/**
 * An interface for defining a control of the map.
 * @module atlas.controls
 */
export interface Control {

    /**
     * Initialization method for the control which is called when added to the map.
     * @param map The map that the control will be added to.
     * @param options The ControlOptions for this control.
     * @return An HTMLElement to be placed on the map for the control.
     */
    onAdd(map: Map, options?: ControlOptions): HTMLElement;

    /**
     * Method that is called when the control is removed from the map. Should perform any necessary cleanup for the
     * control.
     */
    onRemove(): void;

}
