import { Position } from "./Position";

/**
 * A GeoJSON BoundingBox object - an array that defines a shape whose edges follow lines of constant longitude,
 * latitude, and elevation. All axes of the most southwesterly point are followed by all axes of the more northeasterly
 * point. The axes order of the BoundingBox follows the axes order of geometries. The full description is detailed in
 * [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-5}.
 * @module atlas.data
 */
export class BoundingBox extends Array<number> {

    /**
     * Constructs a BoundingBox.
     * @param southwestPosition The southwestern most position of the bounding box.
     * @param northeastPosition The northeastern most position of the bounding box.
     */
    constructor(southwestPosition: Position, northeastPosition: Position) {
        if (southwestPosition[2] && northeastPosition[2]) {
            super(southwestPosition[0], southwestPosition[1], southwestPosition[2] || 0, northeastPosition[0],
                northeastPosition[1], northeastPosition[2] || 0);
        } else {
            super(southwestPosition[0], southwestPosition[1], northeastPosition[0], northeastPosition[1]);
        }
    }

}
