import { Geometry } from "./Geometry";

/**
 * A GeoJSON Feature object - a JSON object representing a spatially bounded entity. The full description is detailed in
 * [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.2}.
 * @module atlas.data
 */
export class Feature<G extends Geometry, P extends any> {

    /**
     * A static GeoJSON type descriptor for the Feature class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "Feature";

    /**
     * A GeoJSON type descriptor with value "Feature".
     */
    public type: string = "Feature";

    /**
     * The geometry of the feature.
     */
    public geometry: G;

    /**
     * The properties of the feature.
     */
    public properties?: any;

    /**
     * The id of the feature.
     */
    public id?: string;

    /**
     * Constructs a Feature.
     * @param geometry The geometry of the feature.
     * @param properties The properties of the feature.
     * @param id The id of the feature.
     */
    constructor(geometry: G, properties?: P, id?: string) {
        this.type = Feature.TYPE;
        this.properties = properties || {};
        this.id = id;
        this.geometry = geometry;
    }

}
