import { Feature } from "./Feature";
import { Geometry } from "./Geometry";

/**
 * A GeoJSON FeatureCollection object - a JSON object that contains a collection of GeoJSON features. The full
 * description is detailed in [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.3}.
 * @module atlas.data
 */
export class FeatureCollection {

    /**
     * A static GeoJSON type descriptor for the FeatureCollection class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "FeatureCollection";

    /**
     * A GeoJSON type descriptor with value "FeatureCollection".
     */
    public type: string = "FeatureCollection";

    /**
     * The collection of GeoJSON features contained in the feature collection.
     */
    public features: Array<Feature<Geometry, any>>;

    /**
     * Constructs a FeatureCollection.
     * @param features The collection of features that make up the feature collection.
     */
    constructor(features: Array<Feature<Geometry, any>>) {
        this.type = FeatureCollection.TYPE;
        this.features = features;
    }

}
