import { Geometry } from "./Geometry";

/**
 * A GeoJSON GeometryCollection object - a JSON object that contains a collection of a GeoJSON Geometry objects. The
 * full description is detailed in [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.8}.
 * @module atlas.data
 */
export class GeometryCollection {

    /**
     * A static GeoJSON type descriptor for the GeometryCollection class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "GeometryCollection";

    /**
     * A GeoJSON type descriptor with value "GeometryCollection".
     */
    public type: string = "GeometryCollection";

    /**
     * The collection of GeoJSON geometries contained in the geometry collection.
     */
    public geometries: Geometry[];

    /**
     * Constructs a GeometryCollection.
     * @param geometries The collection of geometries that make up the geometry collection.
     */
    constructor(geometries: Geometry[]) {
        this.type = GeometryCollection.TYPE;
        this.geometries = geometries;
    }

}
