﻿import { BoundingBox } from "./BoundingBox";
import { Position } from "./Position";

/**
 * A GeoJSON LineString object - a JSON object that represents a geographic curve. The full description is detailed
 * in [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.4}.
 * @module atlas.data
 */
export class LineString {

    /**
     * A static GeoJSON type descriptor for the LineString class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "LineString";

    /**
     * A GeoJSON type descriptor with value "LineString".
     */
    public readonly type: string = "LineString";

    /**
     * The ordered list of positions defining the linestring.
     */
    public readonly coordinates: Position[];

    /**
     * The bounding box of the linestring.
     */
    public readonly bbox?: BoundingBox;

    /**
     * Constructs a LineString.
     * @param coordinates The ordered list of positions defining the linestring.
     * @param bbox The bounding box of the linestring.
     */
    constructor(coordinates: Position[], bbox?: BoundingBox) {
        if (coordinates.length < 2) {
            throw new Error("Linestring must be composed of at least two points.");
        }

        this.type = LineString.TYPE;
        this.coordinates = coordinates;
        this.bbox = bbox;
    }

}
