import { BoundingBox } from "./BoundingBox";
import { Position } from "./Position";

/**
 * A GeoJSON MultiLineString object - a JSON object that represents multiple geographic curves. The full description
 * is detailed in [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.5}.
 * @module atlas.data
 */
export class MultiLineString {

    /**
     * A static GeoJSON type descriptor for the MultiLineString class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "MultiLineString";

    /**
     * A GeoJSON type descriptor with value "MutliLineString".
     */
    public type: string = "MultiLineString";

    /**
     * The array of LineString coordinate arrays defining the multilinestring.
     */
    public coordinates: Position[][];

    /**
     * The bounding box of the multilinestring.
     */
    public readonly bbox?: BoundingBox;

    /**
     * Constructs a MultiLineString.
     * @param coordinates The array of LineString coordinate arrays defining the multilinestring.
     * @param bbox The bounding box of the multilinestring.
     */
    constructor(coordinates: Position[][], bbox?: BoundingBox) {
        this.type = MultiLineString.TYPE;
        this.coordinates = coordinates;
        this.bbox = bbox;
    }
}
