import { BoundingBox } from "./BoundingBox";
import { Position } from "./Position";

/**
 * A GeoJSON MultiPoint object - a JSON object that represents multiple geographic positions. The full description
 * is detailed in [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.3}.
 * @module atlas.data
 */
export class MultiPoint {

    /**
     * A static GeoJSON type descriptor for the MultiPoint class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "MultiPoint";

    /**
     * A GeoJSON type descriptor with value "MutliPoint".
     */
    public type: string = "MultiPoint";

    /**
     * The array of multiple positions defining the multipoint.
     */
    public coordinates: Position[];

    /**
     * The bounding box of the multipoint.
     */
    public readonly bbox?: BoundingBox;

    /**
     * Constructs a MultiPoint.
     * @param coordinates The array of multiple positions defining the multipoint.
     * @param bbox The bounding box of the multipoint.
     */
    constructor(coordinates: Position[], bbox?: BoundingBox) {
        this.type = MultiPoint.TYPE;
        this.coordinates = coordinates;
        this.bbox = bbox;
    }
}
