import { BoundingBox } from "./BoundingBox";
import { Position } from "./Position";

/**
 * A GeoJSON MultiPolygon object - a JSON object that represents multiple geographic polygons. The full description is
 * detailed in [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.7}.
 * @module atlas.data
 */
export class MultiPolygon {

    /**
     * A static GeoJSON type descriptor for the MultiPolygon class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "MultiPolygon";

    /**
     * A GeoJSON type descriptor with value "MutliPolygon".
     */
    public type: string = "MultiPolygon";

    /**
     * The array of polygon coordinate arrays defining the multipolygon.
     */
    public coordinates: Position[][][];

    /**
     * The bounding box of the multipolygon.
     */
    public readonly bbox?: BoundingBox;

    /**
     * Constructs a MultiPolygon.
     * @param coordinates The array of polygon coordinate arrays defining the multipolygon.
     * @param bbox The bounding box of the multipolygon.
     */
    constructor(coordinates: Position[][][], bbox?: BoundingBox) {
        this.type = MultiPolygon.TYPE;
        this.coordinates = coordinates;
        this.bbox = bbox;
    }
}
