import { Position } from "./Position";

/**
 * A GeoJSON Point object - a JSON object that represents a geographic position. The full description is detailed in
 * [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.2}.
 * @module atlas.data
 */
export class Point {

    /**
     * A static GeoJSON type descriptor for the Point class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "Point";

    /**
     * A GeoJSON type descriptor with value "Point".
     */
    public readonly type: string = "Point";

    /**
     * The position defining the point.
     */
    public readonly coordinates: Position;

    /**
     * Constructs a Point.
     * @param coordinates The position defining the point.
     */
    constructor(coordinates: Position) {
        this.type = Point.TYPE;
        this.coordinates = coordinates;
    }

}
