﻿import { BoundingBox } from "./BoundingBox";
import { Position } from "./Position";

/**
 * A GeoJSON Polygon object - a JSON object that represents a geographic polygon. The full description is detailed in
 * [RFC 7946]{@link https://tools.ietf.org/html/rfc7946#section-3.1.6}.
 * @module atlas.data
 */
export class Polygon {

    /**
     * A static GeoJSON type descriptor for the Polygon class to be used in runtime comparisons.
     */
    public static readonly TYPE: string = "Polygon";

    /**
     * A GeoJSON type descriptor with value "Polygon".
     */
    public type: string = "Polygon";

    /**
     * The array of linear ring coordinate arrays defining the polygon.
     */
    public coordinates: Position[][];

    /**
     * The bounding box of the polygon.
     */
    public readonly bbox?: BoundingBox;

    /**
     * Constructs a Polygon.
     * @param coordinates The array of linear ring coordinate arrays defining the polygon.
     * @param bbox The bounding box of the polygon.
     */
    constructor(coordinates: Position[][], bbox?: BoundingBox) {
        this.type = Polygon.TYPE;
        this.coordinates = coordinates;
        this.bbox = bbox;
    }

}
