﻿import { Options } from "./Options";

/**
 * @private
 */
export class Url {
    public readonly options: UrlOptions;

    constructor(options: UrlOptions) {
        this.options = new UrlOptions().merge(options);
    }

    public get(): Promise<JSON> {
        return fetch(this.toString(), {
            method: "GET",
            mode: "cors",
            headers: new Headers(this.options.headers)
        }).then((response) => {
            if (response.ok) {
                return response.json();
            } else {
                throw new Error(`HTTP ${response.status}: ${response.statusText}`);
            }
        });
    }

    public toString(): string {
        let queryParamsString = "";
        for (const queryParam in this.options.queryParams) {
            if (this.options.queryParams.hasOwnProperty(queryParam)) {
                queryParamsString += `&${queryParam}=${this.options.queryParams[queryParam]}`;
            }
        }

        let url: string = `${this.options.protocol}://`;
        url += (this.options.subdomain) ? `${this.options.subdomain}.` : "";
        url += this.options.domain;
        url += (this.options.path) ? `/${this.options.path}` : "";
        url += (queryParamsString) ? `?${queryParamsString}` : "";

        return url;
    }

}

/**
 * @private
 */
export class UrlOptions extends Options {
    public domain: string = undefined;
    public path: string = "";
    public queryParams: any = {};
    public protocol: "http" | "https" = "https";
    public subdomain: string = "";
    public headers: { [key: string]: string } = {};
}
