import { AppInsights } from "applicationinsights-js";
import { env } from "../../variables.json";
import { ServiceOptions } from "../services/options/ServiceOptions";

/**
 * @private
 */
export class Insights {
    private options: ServiceOptions;

    constructor(options: ServiceOptions) {
        this.options = options;

        if (!this.options["disable-telemetry"]) {
            AppInsights.downloadAndSetup({
                instrumentationKey: env.appInsightsKey,
                // Don't globally track exceptions unless we call trackException()
                disableExceptionTracking: true,
                // Don't track every AJAX call
                disableAjaxTracking: true,
                // Play nice with other's AI instances by not using cookies
                isCookieUseDisabled: true,
                // Tighten the batch size window
                maxBatchInterval: 5000,
                // Disable telemetry completly (default: false)
                disableTelemetry: this.options["disable-telemetry"]
            });
        }
    }

    public trackEvent(...args: any[]) {
        if (!this.options["disable-telemetry"]) {
            AppInsights.trackEvent(...args);
        }
    }

    public setAuthenticatedUserContext(...args: any[]) {
        if (!this.options["disable-telemetry"]) {
            AppInsights.setAuthenticatedUserContext(...args);
        }
    }

    public trackException(...args: any[]) {
        if (!this.options["disable-telemetry"]) {
            AppInsights.trackException(...args);
        }
    }

    public flush(...args: any[]) {
        if (!this.options["disable-telemetry"]) {
            AppInsights.flush(...args);
        }
    }

}
