import * as _ from "lodash";
import { env } from "../../../../variables.json";
import { Url, UrlOptions } from "../../../helpers/Url";
import { ServiceOptions } from "../../../services/options/ServiceOptions";
import { Map } from "../../Map";
import { RasterLayerOptions } from "../../options/RasterLayerOptions";
import { RasterLayer } from "../../style/layers/basic/RasterLayer";
import { MapDelegate } from "../MapDelegate";

/**
 * @private
 */
export class FlowServiceDelegate implements MapDelegate {
    private readonly layerName: string = "flow";
    private map: Map;

    constructor(map: Map) {
        this.map = map;
    }

    public addToMap = () => {
        const transitLayerIndex: number = _.findIndex(this.map.getLayers(), (layerName) => layerName === "transit");
        const layerAfterTransitName: string = this.map.getLayers()[transitLayerIndex + 1];

        const flowTileUrl = new Url(new UrlOptions().merge({
            domain: env.domain,
            path: "traffic/flow/tile/png",
            queryParams: {
                "subscription-key": this.map.getServiceOptions()["subscription-key"],
                "api-version": "1.0",
                "style": this.map.getTraffic().flow,
                "zoom": "{z}",
                "x": "{x}",
                "y": "{y}",
                "thickness": 5
            }
        }));

        const rasterLayerOptions: RasterLayerOptions = {
            name: this.layerName,
            before: layerAfterTransitName,
            opacity: 1,
            minZoom: 8
        };

        this.map.addRaster([flowTileUrl.toString()], rasterLayerOptions);
    }

    public removeFromMap = () => {
        this.map.removeLayers([this.layerName]);
    }

}
