import { Feature, Point, Position } from "../../../data";
import { PinProperties } from "../../data_properties/PinProperties";
import { Popup } from "../../Popup";

/**
 * @private
 */
export class Incident extends Feature<Point, any> {

    private static getIncidentTypeName(incidentCategory: number): string {
        switch (incidentCategory) {
            case 0:
                return "Unknown";
            case 1:
                return "Accident";
            case 2:
                return "Fog";
            case 3:
                return "Danger";
            case 4:
                return "Rain";
            case 5:
                return "Ice";
            case 6:
                return "Jam";
            case 7:
                return "Lane Closed";
            case 8:
                return "Road Closed";
            case 9:
                return "Road Works";
            case 10:
                return "Wind";
            case 11:
                return "Flood";
            case 12:
                return "Detour";
            default:
                return "Unknown";
        }
    }

    private static getIncidentTypeIconName(incidentCategory: number): string {
        switch (incidentCategory) {
            case 0:
                return "unknown";
            case 1:
                return "accident";
            case 2:
                return "fog";
            case 3:
                return "danger";
            case 4:
                return "rain";
            case 5:
                return "ice";
            case 6:
                return "jam";
            case 7:
                return "lane-closed";
            case 8:
                return "road-closed";
            case 9:
                return "road-work";
            case 10:
                return "wind";
            case 11:
                return "flood";
            case 12:
                return "detour";
            default:
                return "unknown";
        }
    }

    private static getSeverityColorName(severity: number): string {
        switch (severity) {
            case 0:
                return "grey";
            case 1:
                return "yellow";
            case 2:
                return "orange";
            case 3:
                return "red";
            case 4:
                return "grey";
            default:
                return "grey";
        }
    }

    private static getSeverityColorHexCode(severity: number) {
        switch (severity) {
            case 0:
                return "#B5B6B7";
            case 1:
                return "#F4C951";
            case 2:
                return "#F4873F";
            case 3:
                return "#F04F4E";
            case 4:
                return "#B5B6B7";
            default:
                return "#B5B6B7";
        }
    }

    constructor(data: any) {
        const point: Point = new Point(new Position(data["op"]["x"], data["op"]["y"]));

        const incidentPinProperties: PinProperties = {
            icon: `${Incident.getSeverityColorName(data.ty)}-${Incident.getIncidentTypeIconName(data.ic)}`,
            incidentType: Incident.getIncidentTypeName(data.ic),
            description: data.d,
            length: data.l,
            delay: data.dl,
            from: data.f,
            color: Incident.getSeverityColorHexCode(data.ty),
            to: data.t,
            id: data.id
        };

        super(point, incidentPinProperties);
    }

}
