import { Feature, Position } from "../../data";
import { MapEventData } from "../options/MapEventData";
import { CallbackGroup } from "./CallbackGroup";

/**
 * @private
 */
export class MapCallbackHandler {
    private callbacks: { [callbackIdentifier: string]: CallbackGroup[] } = {};

    public addCallback(eventType: string, layer: string, callback: (data: any) => void) {
        const modifiedCallback = (data: any) => {
            const mapEventData: MapEventData = {
                type: data.type,
                originalEvent: data.originalEvent,
                position: (data.lngLat) ? new Position(data.lngLat.lng, data.lngLat.lat) : undefined,
                coordinate: (data.point) ? [data.point.x, data.point.y] : undefined,
                features: (data.features) ? data.features.map((f) => new Feature(f.geometry, f.properties, f.id)) : []
            };
            callback(mapEventData);
        };

        const callbackGroup: CallbackGroup = new CallbackGroup(callback, modifiedCallback);
        const callbackIdentifier: string = `${eventType}-${layer}`;
        this.callbacks[callbackIdentifier] = this.callbacks[callbackIdentifier] || [];
        this.callbacks[callbackIdentifier].push(callbackGroup);
    }

    public getModifiedCallback(eventType: string, layer: string, callback: (data: any) => void): any {
        const callbackGroups = this.callbacks[`${eventType}-${layer}`] || [];

        for (const callbackGroup of callbackGroups) {
            if (callbackGroup.userCallback === callback) {
                return callbackGroup.modifiedCallback;
            }
        }

        const voidCallback = (data: any) => { const a = 0; };
        return voidCallback;
    }

}
