import { Options } from "../../helpers/Options";

/**
 * The options for a layer of the map.
 * @module Object Definitions
 */
export class LayerOptions extends Options {
    /**
     * The name of the layer.
     */
    public name?: string = undefined;

    /**
     * The name of the layer to place this layer before.  If not specified, the layer will be placed at the top of the
     * layer list.
     */
    public before?: string = undefined;

    /**
     * The minimum zoom at which the layer will display inclusive.
     */
    public minZoom?: number = 1;

    /**
     * The maximum zoom at which the layer will display inclusive.
     */
    public maxZoom?: number = 20;

    /**
     * Whether to overwrite previous data for the layer.
     */
    public overwrite?: boolean = false;

    /**
     * Whether to defer the layer's updates to the map until a subsequent layer's update or an update to the map's
     * style options.
     */
    public defer?: boolean = false;

    /**
     * The opacity of the layer.
     */
    public opacity?: number = 1;
}
