import { LinestringProperties } from "../data_properties/LinestringProperties";
import { LayerOptions } from "./LayerOptions";

/**
 * The options for a linestring layer.
 * @module Object Definitions
 */
export class LinestringLayerOptions extends LayerOptions implements LinestringProperties {
    /**
     * The type of cap to use for the ends of the linestrings:
     * <p>"butt" is a squared ending that is drawn to the exact endpoint of the line.</p>
     * <p>"round" is a circular ending that is drawn past the exact endpoint with a radius of half the linestring's
     * width.</p>
     * <p>"square" is a squared ending that is drawn past the exact endpoint by half the linestring's width.
     */
    public cap?: "butt" | "round" | "square" = "butt";

    /**
     * The type of join to use for the linestrings:
     * <p>"bevel" is a squared join that cuts a corner flat at half the linestring's width from the vertex.</p>
     * <p>"round" is a circular join that rounds a corner with a radius of half the linestring's width from the
     * vertex.</p>
     * <p>"miter" is a sharp join that extends the segments of the linestring past the vertex until they intersect.</p>
     */
    public join?: "bevel" | "round" | "miter" = "miter";

    /**
     * The name of the layer.
     */
    public name?: string = "default-linestrings";

    /**
     * The color of the lines for the layer. Is used as the default if a specific color is not specified for a line.
     */
    public color?: string = "#000";

    /**
     * The width of the lines for the layer. Is used as the default if a specific width is not specified for a line.
     */
    public width?: number = 1;
}
