import { Feature, Geometry, Position } from "../../data";

/**
 * A MapEventData object is passed to the callback function when a map event is fired.
 * @module Object Definitions
 */
export interface MapEventData {

    /**
     * The event type.
     */
    type: string;

    /**
     * The original event that was fired.
     */
    originalEvent: Event;

    /**
     * The geographic position on the map where the event occurred.
     */
    position?: Position;

    /**
     * The pixel coordinate where the event occurred as an array of [x, y].
     */
    coordinate?: number[];

    /**
     * The features associated with the event.
     */
    features?: Array<Feature<Geometry, any>>;
}
