import { PinProperties } from "../data_properties/PinProperties";
import { LayerOptions } from "./LayerOptions";

/**
 * The options for a pin layer.
 * @module Object Definitions
 */
export class PinLayerOptions extends LayerOptions implements PinProperties {
    /**
     * The factor by which the icon should be scaled. A value of 1 is the original size; a value of 2 will double the
     * size of the icon.
     */
    public iconSize?: number = 1;

    /**
     * The size of the title text.
     */
    public fontSize?: number = 14;

    /**
     * The color of the title text.
     */
    public fontColor?: string = "#000";

    /**
     * The font for the title of the pin.
     */
    public textFont?: "SegoeUi-Bold" | "SegoeUi-Regular" | "StandardFont-Bold" | "StandardFont-Regular"
        = "SegoeUi-Regular";

    /**
     * Whether the icons should cluster when they collide on the map, or stay separate.
     */
    public cluster?: boolean = true;

    /**
     * The icon to use to represent a cluster.
     */
    public clusterIcon?: string = undefined;

    /**
     * An array of [pixelsRight, pixelsDown] for how many pixels to the right and down the title text should be
     * offset. Negative numbers can be used to offset the title left and up.
     */
    public textOffset?: number[] = [0, 0];

    /**
     * The name of the layer.
     */
    public name?: string = "default-pins";

    /**
     * The icon of the pins for the layer. Is used as the default if an icon is not specified for a pin.
     */
    public icon?: "pin-darkblue" | "pin-blue" | "pin-red" | "pin-round-darkblue" | "pin-round-blue" | "pin-round-red"
        | "none" = "pin-darkblue";

    /**
     * The title of the pins for the layer. Is used as the default if a title is not specified for a pin.
     */
    public title?: string = "";

}
