import { Options } from "../../helpers/Options";

/**
 * The options for enabling/disabling user interaction with the map.
 * @module Object Definitions
 */
export class UserInteractionOptions extends Options {
    /**
     * Whether the map is interactive or static. If false, all user interaction is disabled.  If true, only selected
     * user interactions will enabled.
     */
    public interactive?: boolean = true;

    /**
     * Whether the map should zoom on scroll input.
     */
    public scrollZoomInteraction?: boolean = true;

    /**
     * Whether the Shift + left click and drag will draw a zoom box.
     */
    public boxZoomInteraction?: boolean = true;

    /**
     * Whether right click and drag will rotate and pitch the map.
     */
    public dragRotateInteraction?: boolean = true;

    /**
     * Whether left click and drag will pan the map.
     */
    public dragPanInteraction?: boolean = true;

    /**
     * Whether the keyboard interactions are enabled.
     * <style> .k-key { border: 1px solid grey; border-radius: 6px; background-color: #ccc; line-height: 14px;
     * font-size: 14px; padding: 2px; } </style>
     * <p><span class="k-key">+=</span>: Increase zoom level by 1.</p>
     * <p><span class="k-key">Shift</span> + <span class="k-key">+=</span>: Increase the zoom level by 2.</p>
     * <p><span class="k-key">-</span>: Decrease zoom level by 1.</p>
     * <p><span class="k-key">Shift</span> + <span class="k-key">-</span>: Decrease zoom level by 2.</p>
     * <p><span class="k-key">⇢</span>: Pan right 100 pixels.</p>
     * <p><span class="k-key">⇠</span>: Pan left 100 pixels.</p>
     * <p><span class="k-key">⇡</span>: Pan up 100 pixels.</p>
     * <p><span class="k-key">⇣</span>: Pan down 100 pixels.</p>
     * <p><span class="k-key">Shift</span> + <span class="k-key">⇢</span>: Rotate 15 degrees clockwise.</p>
     * <p><span class="k-key">Shift</span> + <span class="k-key">⇠</span>: Rotate 15 degrees counter-clockwise.</p>
     * <p><span class="k-key">Shift</span> + <span class="k-key">⇡</span>: Increase pitch by 10 degrees.</p>
     * <p><span class="k-key">Shift</span> + <span class="k-key">⇣</span>: Decrease pitch by 10 degrees.</p>
     */
    public keyboardInteraction?: boolean = true;

    /**
     * Whether double left click will zoom the map inwards.
     */
    public dblClickZoomInteraction?: boolean = true;

    /**
     * Whether touch interactions are enabled for touch devices.
     */
    public touchInteraction?: boolean = true;
}
