import * as _ from "lodash";
import { Layer } from "./layers/Layer";
import { Style } from "./Style";

/**
 * @private
 */
export class StyleBuilder {
    private layers: Layer[] = [
        {
            name: "base",
            sources: {},
            layers: [],
            push: () => { return; },
            clear: () => { return; }
        },
        {
            name: "transit",
            sources: {},
            layers: [],
            push: () => { return; },
            clear: () => { return; }
        },
        {
            name: "labels",
            sources: {},
            layers: [],
            push: () => { return; },
            clear: () => { return; }
        }
    ];
    private sprite = "";
    private glyphs = "{fontstack}/{range}";

    public upsert(layer: Layer, before?: string): StyleBuilder {
        if (before && before !== layer.name) {
            this.remove(layer.name);
            const beforeIndex: number = _.findIndex(this.layers, (o) => o.name === before);

            if (beforeIndex !== -1) {
                this.layers.splice(beforeIndex, 0, layer);
            } else {
                this.layers.push(layer);
            }
        } else {
            const index: number = _.findIndex(this.layers, (o) => o.name === layer.name);

            if (index !== -1) {
                this.layers.splice(index, 1, layer);
            } else {
                this.layers.push(layer);
            }
        }

        return this;
    }

    public upsertAll(layers: Layer[]) {
        for (const layerSection of layers) {
            this.upsert(layerSection);
        }
    }

    public remove(layerName: string): StyleBuilder {
        _.remove(this.layers, (layer) => layer.name === layerName);
        return this;
    }

    public get(layerName?: string): Layer | undefined {
        return _.find(this.layers, (layer) => layer.name === layerName);
    }

    public getLayers(): Layer[] {
        return this.layers;
    }

    public build(): mapboxgl.Style {
        const style: Style = new Style();
        if (this.glyphs != null) {
            style.glyphs = this.glyphs;
        }
        if (this.sprite != null) {
            style.sprite = this.sprite;
        }

        for (const layerSection of this.layers) {
            _.merge(style.sources, layerSection.sources);
            style.layers.push(...layerSection.layers);
        }

        return style;
    }

    public setGlyphs(glyphs: string) {
        this.glyphs = glyphs;
    }

    public setSprite(sprite: string) {
        this.sprite = sprite;
    }

}
