import { Map } from "../..";
import { env } from "../../../variables.json";
import { Url, UrlOptions } from "../../helpers/Url";
import { ServiceOptions } from "../../services/options/ServiceOptions";
import { StyleOptions } from "../options/StyleOptions";
import { MapboxLayer } from "./layers/azure/MapboxLayer";

/**
 * @private
 */
export class StyleDefinitions {
    private definitions: any;

    public initialize(): Promise<void> {
        return this.request(env.staticAssetsDomain, env.styleDefinitionsPath).then((response) => {
            this.definitions = response;
        });
    }

    public async getLayers(options: StyleOptions & ServiceOptions): Promise<any[]> {
        const layerPromises = [];
        const style = this.lookUp(options);
        const layersGroups = style["layerGroups"];

        for (let i = 0; i < layersGroups.length; i++) {
            const layerName = layersGroups[i]["name"];
            const path = layersGroups[i]["layerPath"];
            layerPromises.push(this.request(this.definitions.domain, path).then((response) => {
                return new MapboxLayer(options, layerName, response);
            }));
        }

        return Promise.all(layerPromises);
    }

    public getCopyrightCaption(options: StyleOptions): string {
        const style = this.lookUp(options);
        const copyrightsArray = style["copyright"];
        let copyrights: string = "";
        copyrightsArray.forEach((copyright) => {
            copyrights += (copyright + " ");
        });
        return copyrights;
    }

    public getSprite(options: StyleOptions): string {
        const style = this.lookUp(options);
        const spriteUrl = new Url(new UrlOptions().merge({
            domain: this.definitions.domain,
            path: style["spritePath"]
        }));
        return spriteUrl.toString();
    }

    public getGlyphs(options: StyleOptions): string {
        const style = this.lookUp(options);
        const glyphsUrl = new Url(new UrlOptions().merge({
            domain: this.definitions.domain,
            path: style["glyphsPath"]
        }));
        return glyphsUrl.toString();
    }

    public getBillingHeaderAndCleanUpURL(url: string): any {
        let billingHeader: any = {};
        let tileType: string = "";
        const billingKey = "&billing=";
        const index = url.indexOf(billingKey);
        if (index !== -1 && index < url.length) {
            tileType = url.substring(index + billingKey.length);
            billingHeader = {
                "billing": tileType
            };
            url = url.replace(billingKey + tileType, "");
        }

        const result: any = {
            "url": url,
            "billing": billingHeader
        };
        return result;
    }

    private lookUp(options: StyleOptions): any {
        let style: any;
        const styles = this.definitions["styles"];
        const styleName: string = options.style || this.definitions["defaultStyle"];
        for (let i = 0; i < styles.length; i++) {
            if (styles[i]["name"] === styleName) {
                style = styles[i];
                break;
            }
        }

        return style;
    }

    private async request(domain: string, path: string): Promise<JSON> {
        const request = new Url(new UrlOptions().merge({
            domain: domain,
            path: path
        }));

        return request.get();
    }

}
