import { env } from "../../../../../variables.json";
import { Url, UrlOptions } from "../../../../helpers/Url";
import { ServiceOptions } from "../../../../services/options/ServiceOptions";
import { StyleOptions } from "../../../options/StyleOptions";
import { Layer } from "../Layer";

/**
 * @private
 */
export class MapboxLayer implements Layer {
    public name: string;
    public sources: any;
    public layers: any;

    constructor(options: StyleOptions & ServiceOptions, layerName: string, stylesheet: JSON) {
        this.name = layerName;

        let sources: any[] = stylesheet["sources"];
        let sourcesJSON: any = {};
        for (let i = 0; i < sources.length; i++) {
            let sourceName = sources[i]["name"];
            sourcesJSON[sourceName] = this.constructSource(sources[i], options);
        }

        this.sources = sourcesJSON;
        this.layers = stylesheet["layers"];
    }

    private constructSource(data: any, options: StyleOptions & ServiceOptions): any {
        let tiles: any[] = data["tiles"];
        let tilesArray: string[] = [];
        for (let i = 0; i < tiles.length; i++) {
            const tileUrl = new Url(new UrlOptions().merge({
                domain: env.domain,
                path: tiles[i]["path"],
                queryParams: {
                    "subscription-key": options["subscription-key"],
                    "view": options.view,
                    "language": options.language,
                    ...tiles[i]["queryParams"]
                }
            }));
            tilesArray.push(tileUrl.toString());
        }

        data["tiles"] = tilesArray;

        return data;
    }

    public push(data: any): void {
        throw new Error("Method not implemented.");
    }

    public clear(): void {
        throw new Error("Method not implemented.");
    }

}
