import { Feature, Point } from "../../../../data";
import { CircleProperties } from "../../../data_properties/CircleProperties";
import { CircleLayerOptions } from "../../../options/CircleLayerOptions";
import { LayerOptions } from "../../../options/LayerOptions";
import { Layer } from "../Layer";

/**
 * @private
 */
export class CircleLayer implements Layer {
    public name: string;
    public sources: any;
    public layers: any;
    public sourceName: string;

    constructor(circleLayerOptions: CircleLayerOptions & CircleProperties) {
        this.name = circleLayerOptions.name;
        this.sourceName = `${this.name}-source`;

        this.sources = {};
        this.sources[this.sourceName] = {
            type: "geojson",
            data: {
                type: "FeatureCollection",
                features: []
            }
        };

        this.layers = [
            {
                id: this.name,
                type: "circle",
                source: this.sourceName,
                layout: {},
                paint: {
                    "circle-opacity": circleLayerOptions.opacity,
                    "circle-radius": {
                        type: "identity",
                        property: "radius",
                        default: circleLayerOptions.radius
                    },
                    "circle-color": {
                        type: "identity",
                        property: "color",
                        default: circleLayerOptions.color
                    },
                    "circle-stroke-color": {
                        type: "identity",
                        property: "outlineColor",
                        default: circleLayerOptions.outlineColor
                    },
                    "circle-stroke-width": {
                        type: "identity",
                        property: "outlineWidth",
                        default: circleLayerOptions.outlineWidth
                    }
                },
                minzoom: circleLayerOptions.minZoom,
                maxzoom: circleLayerOptions.maxZoom
            }
        ];

    }

    public push(circles: Array<Feature<Point, CircleProperties>>) {
        this.sources[this.sourceName].data.features.push(...circles);
    }

    public clear() {
        this.sources[this.sourceName].data.features = [];
    }

}
