import { Feature, LineString, MultiLineString } from "../../../../data";
import { LinestringProperties } from "../../../data_properties/LinestringProperties";
import { LayerOptions } from "../../../options/LayerOptions";
import { LinestringLayerOptions } from "../../../options/LinestringLayerOptions";
import { Layer } from "../Layer";

/**
 * @private
 */
export class LinestringLayer implements Layer {
    public name: string;
    public sources: any;
    public layers: any;
    public sourceName: string;

    constructor(linestringLayerOptions: LinestringLayerOptions & LinestringProperties) {
        this.name = linestringLayerOptions.name;
        this.sourceName = `${this.name}-source`;

        this.sources = {};
        this.sources[this.sourceName] = {
            type: "geojson",
            data: {
                type: "FeatureCollection",
                features: []
            }
        };

        this.layers = [
            {
                id: this.name,
                type: "line",
                source: this.sourceName,
                layout: {
                    "line-cap": linestringLayerOptions.cap,
                    "line-join": linestringLayerOptions.join
                },
                paint: {
                    "line-opacity": linestringLayerOptions.opacity,
                    "line-color": {
                        type: "identity",
                        property: "color",
                        default: linestringLayerOptions.color
                    },
                    "line-width": {
                        type: "identity",
                        property: "width",
                        default: linestringLayerOptions.width
                    }
                },
                minzoom: linestringLayerOptions.minZoom,
                maxzoom: linestringLayerOptions.maxZoom
            }
        ];

    }

    public push(linestrings: Array<Feature<LineString | MultiLineString, LinestringProperties>>) {
        for (const linestring of linestrings) {
            this.sources[this.sourceName].data.features.push(linestring);
        }
    }

    public clear() {
        this.sources[this.sourceName].data.features = [];
    }

}
