import { Feature, MultiPolygon, Polygon } from "../../../../data";
import { PolygonProperties } from "../../../data_properties/PolygonProperties";
import { LayerOptions } from "../../../options/LayerOptions";
import { PolygonLayerOptions } from "../../../options/PolygonLayerOptions";
import { Layer } from "../Layer";

/**
 * @private
 */
export class PolygonLayer implements Layer {
    public name: string;
    public sources: any;
    public layers: any;
    public sourceName: string;

    constructor(polygonLayerOptions: PolygonLayerOptions & PolygonProperties) {
        this.name = polygonLayerOptions.name;
        this.sourceName = `${this.name}-source`;

        this.sources = {};
        this.sources[this.sourceName] = {
            type: "geojson",
            data: {
                type: "FeatureCollection",
                features: []
            }
        };

        this.layers = [
            {
                id: this.name,
                type: "fill",
                source: this.sourceName,
                layout: {},
                paint: {
                    "fill-opacity": polygonLayerOptions.opacity,
                    "fill-color": {
                        type: "identity",
                        property: "color",
                        default: polygonLayerOptions.color
                    },
                    "fill-outline-color": {
                        type: "identity",
                        property: "outlineColor",
                        default: polygonLayerOptions.outlineColor
                    }
                },
                minzoom: polygonLayerOptions.minZoom,
                maxzoom: polygonLayerOptions.maxZoom
            }
        ];

    }

    public push(polygons: Array<Feature<Polygon | MultiPolygon, PolygonProperties>>) {
        for (const polygon of polygons) {
            this.sources[this.sourceName].data.features.push(polygon);
        }
    }

    public clear() {
        this.sources[this.sourceName].data.features = [];
    }

}
