import { LayerOptions } from "../../../options/LayerOptions";
import { RasterLayerOptions } from "../../../options/RasterLayerOptions";
import { Layer } from "../Layer";

/**
 * @private
 */
export class RasterLayer implements Layer {
    public name: string;
    public sources: any;
    public layers: any;
    public sourceName: string;

    constructor(rasterLayerOptions: RasterLayerOptions) {
        this.name = rasterLayerOptions.name;
        this.sourceName = `${this.name}-source`;

        this.sources = {};
        this.sources[this.sourceName] = {
            tiles: [],
            tileSize: 256,
            type: "raster"
        };

        this.layers = [
            {
                id: this.name,
                type: "raster",
                source: this.sourceName,
                paint: {
                    "raster-opacity": rasterLayerOptions.opacity
                },
                minzoom: rasterLayerOptions.minZoom,
                maxzoom: rasterLayerOptions.maxZoom
            }
        ];
    }

    public push(tileSources: string[]) {
        this.sources[this.sourceName].tiles.push(...tileSources);
    }

    public clear() {
        this.sources[this.sourceName].tiles = [];
    }

}
