import { env } from "../../variables.json";
import { Url, UrlOptions } from "../helpers/Url";
import { CopyrightOptions } from "./options/CopyrightOptions";
import { RequestOptions } from "./options/RequestOptions";
import { ServiceOptions } from "./options/ServiceOptions";
import { Service } from "./Service";

/**
 * @private
 */
export class CopyrightService implements Service {
    public readonly serviceOptions: ServiceOptions;

    constructor(options: ServiceOptions) {
        this.serviceOptions = new ServiceOptions().merge(options);
    }

    public request(options?: RequestOptions): Promise<JSON> {
        const copyrightOptions: CopyrightOptions = new CopyrightOptions().merge(options);
        return new Url(new UrlOptions().merge({
            domain: env.domain,
            path: "map/copyright/caption/json",
            queryParams: {
                "subscription-key": this.serviceOptions["subscription-key"],
                "api-version": copyrightOptions.version
            },
            headers: {
                "Session-Id": this.serviceOptions["session-id"],
                "Ms-Am-Request-Origin": "MapControl"
            }
        })).get();
    }

}
