import { env } from "../../variables.json";
import { Position } from "../data";
import { Url, UrlOptions } from "../helpers/Url";
import { IncidentOptions } from "./options/IncidentOptions";
import { ServiceOptions } from "./options/ServiceOptions";
import { Service } from "./Service";

/**
 * @private
 */
export class IncidentService implements Service {
    public readonly serviceOptions: ServiceOptions;

    constructor(options: ServiceOptions) {
        this.serviceOptions = new ServiceOptions().merge(options);
    }

    public request(options: IncidentOptions): Promise<JSON> {
        const incidentOptions: IncidentOptions = new IncidentOptions().merge(options);
        const trafficModelId: number = -1;
        const bboxString: string = `${incidentOptions.bbox[1]},${incidentOptions.bbox[0]},` +
            `${incidentOptions.bbox[3]},${incidentOptions.bbox[2]}`;
        return new Url(new UrlOptions().merge({
            domain: env.domain,
            path: "traffic/incident/detail/json",
            queryParams: {
                "subscription-key": this.serviceOptions["subscription-key"],
                "api-version": incidentOptions.version,
                "projection": "EPSG4326",
                style: "s3",
                boundingbox: bboxString,
                boundingzoom: incidentOptions.zoom,
                trafficmodelid: trafficModelId,
                ...incidentOptions.params
            },
            headers: {
                "Session-Id": this.serviceOptions["session-id"],
                "Ms-Am-Request-Origin": "MapControl"
            }
        })).get();
    }

}
