import { env } from "../../variables.json";
import { Position } from "../data";
import { Url, UrlOptions } from "../helpers/Url";
import { ServiceOptions } from "./options/ServiceOptions";
import { Service } from "./Service";

/**
 * @private
 */
export class NearbyGeographySearchService implements Service {
    public readonly serviceOptions: ServiceOptions;

    constructor(options: ServiceOptions) {
        this.serviceOptions = new ServiceOptions().merge(options);
    }

    public request(position: Position): Promise<JSON> {
        return new Url(new UrlOptions().merge({
            domain: env.domain,
            path: "search/fuzzy/json",
            queryParams: {
                "subscription-key": this.serviceOptions["subscription-key"],
                "api-version": "1.0",
                "query": `${position[1].toFixed(6)}, ${position[0].toFixed(6)}`,
                "idxSet": "Geo",
                "limit": 1,
                "lat": position[1].toFixed(6),
                "lon": position[0].toFixed(6),
                "radius": 1000
            },
            headers: {
                "Session-Id": this.serviceOptions["session-id"],
                "Ms-Am-Request-Origin": "MapControl"
            }
        })).get();
    }

}
