import "mocha";

import * as assert from "assert";
import * as path from "path";
import * as puppeteer from "puppeteer";

import { env } from "../variables.json";
import { request } from "http";

describe("Map Load", function () {
    let browser: puppeteer.Browser;
    let page: puppeteer.Page;
    let examplePath: string;
    this.timeout(20000);

    before(async function () {
        examplePath = `file:///${path.resolve("examples/map.html")}`;
        browser = await puppeteer.launch();
    });

    after(async function () {
        await page.close();
        await browser.close();
    });

    beforeEach(async function () {
        page = await browser.newPage();
    });

    it("Map copyright caption should be called", async function () {
        var requestSent = false;
        page.on('request', (request: puppeteer.Request) => {
            if (request.url().toLowerCase().includes("map/copyright/caption")) {
                requestSent = true;
            }
        });
        await page.goto(examplePath, { waitUntil: "networkidle0" });
        assert.strictEqual(true, requestSent);
    });

    it("There shouldn't be any exceptions", async function () {
        var errorFound: boolean = false;
        var errorMessage: string;
        page.on('pageerror', err => {
            errorFound = true;
            errorMessage = err;
        });
        await page.goto(examplePath, { waitUntil: "networkidle0" });
        assert.strictEqual(false, errorFound, "Error found: " + errorMessage);
    });
});