/*
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import * as msRest from "ms-rest-js";
import * as Models from "../models";
import * as Mappers from "../models/routeMappers";
import { MapsClientContext } from "../mapsClientContext";

const WebResource = msRest.WebResource;

/** Class representing a Route. */
export class Route {
  private readonly client: MapsClientContext;
  private readonly serializer = new msRest.Serializer(Mappers);
  /**
   * Create a Route.
   * @param {MapsClientContext} client Reference to the service client.
   */
  constructor(client: MapsClientContext) {
    this.client = client;
  }

  /**
   * Returns  a route between an origin and a destination, passing through waypoints if they are
   * specified. The route will take into account factors such as current traffic and the typical road
   * speeds on the requested day of the week and time of day.
   *
   * Information returned includes the distance, estimated travel time, and a representation of the
   * route geometry. Additional routing information such as optimized waypoint order or turn by turn
   * instructions is also available, depending on the options selected.
   *
   * Routing service provides a set of parameters for a detailed description of vehicle-specific
   * Consumption Model. Please check [Consumption
   * Model](https://docs.microsoft.com/azure/azure-maps/consumption-model) for detailed explanation
   * of the concepts and parameters involved.
   *
   * @param {string} query The Coordinates through which the route is calculated, delimited by a
   * colon.  A minimum of two coordinates is required.  The first one is the origin and the last is
   * the destination of the route. Optional coordinates in-between act as WayPoints in the route.
   * You can pass up to 50 WayPoints unless computeBestOrder is set to true, in which case the limit
   * is reduced to 20.
   *
   * @param {RouteGetRouteDirectionsOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getRouteDirectionsWithHttpOperationResponse(query: string, options?: Models.RouteGetRouteDirectionsOptionalParams): Promise<msRest.HttpOperationResponse<Models.RouteDirectionsResponse>> {
    let maxAlternatives = (options && options.maxAlternatives !== undefined) ? options.maxAlternatives : 0;
    let alternativeType = (options && options.alternativeType !== undefined) ? options.alternativeType : 'anyRoute';
    let minDeviationDistance = (options && options.minDeviationDistance !== undefined) ? options.minDeviationDistance : 0;
    let arriveAt = (options && options.arriveAt !== undefined) ? options.arriveAt : undefined;
    let departAt = (options && options.departAt !== undefined) ? options.departAt : undefined;
    let minDeviationTime = (options && options.minDeviationTime !== undefined) ? options.minDeviationTime : 0;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let computeBestOrder = (options && options.computeBestOrder !== undefined) ? options.computeBestOrder : false;
    let routeRepresentation = (options && options.routeRepresentation !== undefined) ? options.routeRepresentation : 'polyline';
    let computeTravelTimeFor = (options && options.computeTravelTimeFor !== undefined) ? options.computeTravelTimeFor : 'none';
    let vehicleHeading = (options && options.vehicleHeading !== undefined) ? options.vehicleHeading : undefined;
    let report = (options && options.report !== undefined) ? options.report : undefined;
    let sectionType = (options && options.sectionType !== undefined) ? options.sectionType : 'travelMode';
    let vehicleAxleWeight = (options && options.vehicleAxleWeight !== undefined) ? options.vehicleAxleWeight : 0;
    let vehicleWidth = (options && options.vehicleWidth !== undefined) ? options.vehicleWidth : 0;
    let vehicleHeight = (options && options.vehicleHeight !== undefined) ? options.vehicleHeight : 0;
    let vehicleLength = (options && options.vehicleLength !== undefined) ? options.vehicleLength : 0;
    let vehicleMaxSpeed = (options && options.vehicleMaxSpeed !== undefined) ? options.vehicleMaxSpeed : 0;
    let vehicleWeight = (options && options.vehicleWeight !== undefined) ? options.vehicleWeight : 0;
    let windingness = (options && options.windingness !== undefined) ? options.windingness : 'normal';
    let hilliness = (options && options.hilliness !== undefined) ? options.hilliness : 'normal';
    let travelMode = (options && options.travelMode !== undefined) ? options.travelMode : 'car';
    let avoid = (options && options.avoid !== undefined) ? options.avoid : undefined;
    let traffic = (options && options.traffic !== undefined) ? options.traffic : true;
    let routeType = (options && options.routeType !== undefined) ? options.routeType : 'fastest';
    let vehicleLoadType = (options && options.vehicleLoadType !== undefined) ? options.vehicleLoadType : undefined;
    let vehicleEngineType = (options && options.vehicleEngineType !== undefined) ? options.vehicleEngineType : 'combustion';
    let constantSpeedConsumptionInLitersPerHundredkm = (options && options.constantSpeedConsumptionInLitersPerHundredkm !== undefined) ? options.constantSpeedConsumptionInLitersPerHundredkm : undefined;
    let currentFuelInLiters = (options && options.currentFuelInLiters !== undefined) ? options.currentFuelInLiters : undefined;
    let auxiliaryPowerInLitersPerHour = (options && options.auxiliaryPowerInLitersPerHour !== undefined) ? options.auxiliaryPowerInLitersPerHour : undefined;
    let fuelEnergyDensityInMJoulesPerLiter = (options && options.fuelEnergyDensityInMJoulesPerLiter !== undefined) ? options.fuelEnergyDensityInMJoulesPerLiter : undefined;
    let accelerationEfficiency = (options && options.accelerationEfficiency !== undefined) ? options.accelerationEfficiency : undefined;
    let decelerationEfficiency = (options && options.decelerationEfficiency !== undefined) ? options.decelerationEfficiency : undefined;
    let uphillEfficiency = (options && options.uphillEfficiency !== undefined) ? options.uphillEfficiency : undefined;
    let downhillEfficiency = (options && options.downhillEfficiency !== undefined) ? options.downhillEfficiency : undefined;
    let constantSpeedConsumptionInkWhPerHundredkm = (options && options.constantSpeedConsumptionInkWhPerHundredkm !== undefined) ? options.constantSpeedConsumptionInkWhPerHundredkm : undefined;
    let currentChargeInkWh = (options && options.currentChargeInkWh !== undefined) ? options.currentChargeInkWh : undefined;
    let maxChargeInkWh = (options && options.maxChargeInkWh !== undefined) ? options.maxChargeInkWh : undefined;
    let auxiliaryPowerInkW = (options && options.auxiliaryPowerInkW !== undefined) ? options.auxiliaryPowerInkW : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          maxAlternatives,
          alternativeType,
          minDeviationDistance,
          arriveAt,
          departAt,
          minDeviationTime,
          "this.client.instructionsType": this.client.instructionsType,
          language,
          computeBestOrder,
          routeRepresentation,
          computeTravelTimeFor,
          vehicleHeading,
          report,
          sectionType,
          vehicleAxleWeight,
          vehicleWidth,
          vehicleHeight,
          vehicleLength,
          vehicleMaxSpeed,
          vehicleWeight,
          windingness,
          hilliness,
          travelMode,
          avoid,
          traffic,
          routeType,
          vehicleLoadType,
          vehicleEngineType,
          constantSpeedConsumptionInLitersPerHundredkm,
          currentFuelInLiters,
          auxiliaryPowerInLitersPerHour,
          fuelEnergyDensityInMJoulesPerLiter,
          accelerationEfficiency,
          decelerationEfficiency,
          uphillEfficiency,
          downhillEfficiency,
          constantSpeedConsumptionInkWhPerHundredkm,
          currentChargeInkWh,
          maxChargeInkWh,
          auxiliaryPowerInkW
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "route/directions/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "maxAlternatives",
              mapper: {
                serializedName: "maxAlternatives",
                defaultValue: 0,
                constraints: {
                  InclusiveMaximum: 5,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "alternativeType",
              mapper: {
                serializedName: "alternativeType",
                defaultValue: 'anyRoute',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "anyRoute",
                    "betterRoute"
                  ]
                }
              }
            },
            {
              parameterPath: "minDeviationDistance",
              mapper: {
                serializedName: "minDeviationDistance",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "arriveAt",
              mapper: {
                serializedName: "arriveAt",
                type: {
                  name: "DateTime"
                }
              }
            },
            {
              parameterPath: "departAt",
              mapper: {
                serializedName: "departAt",
                type: {
                  name: "DateTime"
                }
              }
            },
            {
              parameterPath: "minDeviationTime",
              mapper: {
                serializedName: "minDeviationTime",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "this.client.instructionsType",
              mapper: {
                serializedName: "instructionsType",
                type: {
                  name: "Enum",
                  allowedValues: [
                    "coded",
                    "text",
                    "tagged"
                  ]
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "computeBestOrder",
              mapper: {
                serializedName: "computeBestOrder",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "routeRepresentation",
              mapper: {
                serializedName: "routeRepresentation",
                defaultValue: 'polyline',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "polyline",
                    "none"
                  ]
                }
              }
            },
            {
              parameterPath: "computeTravelTimeFor",
              mapper: {
                serializedName: "computeTravelTimeFor",
                defaultValue: 'none',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "none",
                    "all"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleHeading",
              mapper: {
                serializedName: "vehicleHeading",
                constraints: {
                  InclusiveMaximum: 359,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "report",
              mapper: {
                serializedName: "report",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "sectionType",
              mapper: {
                serializedName: "sectionType",
                defaultValue: 'travelMode',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "carTrain",
                    "country",
                    "ferry",
                    "motorway",
                    "pedestrian",
                    "tollRoad",
                    "tollVignette",
                    "traffic",
                    "travelMode",
                    "tunnel"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleAxleWeight",
              mapper: {
                serializedName: "vehicleAxleWeight",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleWidth",
              mapper: {
                serializedName: "vehicleWidth",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleHeight",
              mapper: {
                serializedName: "vehicleHeight",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleLength",
              mapper: {
                serializedName: "vehicleLength",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleMaxSpeed",
              mapper: {
                serializedName: "vehicleMaxSpeed",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleWeight",
              mapper: {
                serializedName: "vehicleWeight",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "windingness",
              mapper: {
                serializedName: "windingness",
                defaultValue: 'normal',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "low",
                    "normal",
                    "high"
                  ]
                }
              }
            },
            {
              parameterPath: "hilliness",
              mapper: {
                serializedName: "hilliness",
                defaultValue: 'normal',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "low",
                    "normal",
                    "high"
                  ]
                }
              }
            },
            {
              parameterPath: "travelMode",
              mapper: {
                serializedName: "travelMode",
                defaultValue: 'car',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "car",
                    "truck",
                    "taxi",
                    "bus",
                    "van",
                    "motorcycle",
                    "bicycle",
                    "pedestrian"
                  ]
                }
              }
            },
            {
              parameterPath: "avoid",
              mapper: {
                serializedName: "avoid",
                type: {
                  name: "Enum",
                  allowedValues: [
                    "tollRoads",
                    "motorways",
                    "ferries",
                    "unpavedRoads",
                    "carpools",
                    "alreadyUsedRoads"
                  ]
                }
              }
            },
            {
              parameterPath: "traffic",
              mapper: {
                serializedName: "traffic",
                defaultValue: true,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "routeType",
              mapper: {
                serializedName: "routeType",
                defaultValue: 'fastest',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "fastest",
                    "shortest",
                    "eco",
                    "thrilling"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleLoadType",
              mapper: {
                serializedName: "vehicleLoadType",
                type: {
                  name: "Enum",
                  allowedValues: [
                    "USHazmatClass1",
                    "USHazmatClass2",
                    "USHazmatClass3",
                    "USHazmatClass4",
                    "USHazmatClass5",
                    "USHazmatClass6",
                    "USHazmatClass7",
                    "USHazmatClass8",
                    "USHazmatClass9",
                    "otherHazmatExplosive",
                    "otherHazmatGeneral",
                    "otherHazmatHarmfulToWater"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleEngineType",
              mapper: {
                serializedName: "vehicleEngineType",
                defaultValue: 'combustion',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "combustion",
                    "electric"
                  ]
                }
              }
            },
            {
              parameterPath: "constantSpeedConsumptionInLitersPerHundredkm",
              mapper: {
                serializedName: "constantSpeedConsumptionInLitersPerHundredkm",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "currentFuelInLiters",
              mapper: {
                serializedName: "currentFuelInLiters",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "auxiliaryPowerInLitersPerHour",
              mapper: {
                serializedName: "auxiliaryPowerInLitersPerHour",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "fuelEnergyDensityInMJoulesPerLiter",
              mapper: {
                serializedName: "fuelEnergyDensityInMJoulesPerLiter",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "accelerationEfficiency",
              mapper: {
                serializedName: "accelerationEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "decelerationEfficiency",
              mapper: {
                serializedName: "decelerationEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "uphillEfficiency",
              mapper: {
                serializedName: "uphillEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "downhillEfficiency",
              mapper: {
                serializedName: "downhillEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "constantSpeedConsumptionInkWhPerHundredkm",
              mapper: {
                serializedName: "constantSpeedConsumptionInkWhPerHundredkm",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "currentChargeInkWh",
              mapper: {
                serializedName: "currentChargeInkWh",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "maxChargeInkWh",
              mapper: {
                serializedName: "maxChargeInkWh",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "auxiliaryPowerInkW",
              mapper: {
                serializedName: "auxiliaryPowerInkW",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.RouteDirectionsResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.RouteDirectionsResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * Returns  a route between an origin and a destination, passing through waypoints if they are
   * specified. The route will take into account factors such as current traffic and the typical road
   * speeds on the requested day of the week and time of day.
   *
   * Information returned includes the distance, estimated travel time, and a representation of the
   * route geometry. Additional routing information such as optimized waypoint order or turn by turn
   * instructions is also available, depending on the options selected.
   *
   * Routing service provides a set of parameters for a detailed description of vehicle-specific
   * Consumption Model. Please check [Consumption
   * Model](https://docs.microsoft.com/en-us/rest/api/maps/consumption-model) for detailed
   * explanation of the concepts and parameters involved.
   *
   * @param {string} query The Coordinates through which the route is calculated. Needs two
   * coordinates at least. Delimited by colon. First one is the origin and the last is the
   * destination of the route. Optional coordinates in between act as WayPoints in the route.
   *
   * @param {RoutePostRouteDirectionsOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postRouteDirectionsWithHttpOperationResponse(query: string, options?: Models.RoutePostRouteDirectionsOptionalParams): Promise<msRest.HttpOperationResponse<Models.RouteDirectionsResponse>> {
    let maxAlternatives = (options && options.maxAlternatives !== undefined) ? options.maxAlternatives : 0;
    let alternativeType = (options && options.alternativeType !== undefined) ? options.alternativeType : 'anyRoute';
    let minDeviationDistance = (options && options.minDeviationDistance !== undefined) ? options.minDeviationDistance : 0;
    let minDeviationTime = (options && options.minDeviationTime !== undefined) ? options.minDeviationTime : 0;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let computeBestOrder = (options && options.computeBestOrder !== undefined) ? options.computeBestOrder : false;
    let routeRepresentation = (options && options.routeRepresentation !== undefined) ? options.routeRepresentation : 'polyline';
    let computeTravelTimeFor = (options && options.computeTravelTimeFor !== undefined) ? options.computeTravelTimeFor : 'none';
    let vehicleHeading = (options && options.vehicleHeading !== undefined) ? options.vehicleHeading : undefined;
    let report = (options && options.report !== undefined) ? options.report : undefined;
    let sectionType = (options && options.sectionType !== undefined) ? options.sectionType : 'travelMode';
    let arriveAt = (options && options.arriveAt !== undefined) ? options.arriveAt : undefined;
    let departAt = (options && options.departAt !== undefined) ? options.departAt : undefined;
    let vehicleAxleWeight = (options && options.vehicleAxleWeight !== undefined) ? options.vehicleAxleWeight : 0;
    let vehicleLength = (options && options.vehicleLength !== undefined) ? options.vehicleLength : 0;
    let vehicleHeight = (options && options.vehicleHeight !== undefined) ? options.vehicleHeight : 0;
    let vehicleWidth = (options && options.vehicleWidth !== undefined) ? options.vehicleWidth : 0;
    let vehicleMaxSpeed = (options && options.vehicleMaxSpeed !== undefined) ? options.vehicleMaxSpeed : 0;
    let vehicleWeight = (options && options.vehicleWeight !== undefined) ? options.vehicleWeight : 0;
    let windingness = (options && options.windingness !== undefined) ? options.windingness : 'normal';
    let hilliness = (options && options.hilliness !== undefined) ? options.hilliness : 'normal';
    let travelMode = (options && options.travelMode !== undefined) ? options.travelMode : 'car';
    let avoid = (options && options.avoid !== undefined) ? options.avoid : undefined;
    let traffic = (options && options.traffic !== undefined) ? options.traffic : true;
    let routeType = (options && options.routeType !== undefined) ? options.routeType : 'fastest';
    let vehicleLoadType = (options && options.vehicleLoadType !== undefined) ? options.vehicleLoadType : undefined;
    let vehicleEngineType = (options && options.vehicleEngineType !== undefined) ? options.vehicleEngineType : 'combustion';
    let constantSpeedConsumptionInLitersPerHundredkm = (options && options.constantSpeedConsumptionInLitersPerHundredkm !== undefined) ? options.constantSpeedConsumptionInLitersPerHundredkm : undefined;
    let currentFuelInLiters = (options && options.currentFuelInLiters !== undefined) ? options.currentFuelInLiters : undefined;
    let auxiliaryPowerInLitersPerHour = (options && options.auxiliaryPowerInLitersPerHour !== undefined) ? options.auxiliaryPowerInLitersPerHour : undefined;
    let fuelEnergyDensityInMJoulesPerLiter = (options && options.fuelEnergyDensityInMJoulesPerLiter !== undefined) ? options.fuelEnergyDensityInMJoulesPerLiter : undefined;
    let accelerationEfficiency = (options && options.accelerationEfficiency !== undefined) ? options.accelerationEfficiency : undefined;
    let decelerationEfficiency = (options && options.decelerationEfficiency !== undefined) ? options.decelerationEfficiency : undefined;
    let uphillEfficiency = (options && options.uphillEfficiency !== undefined) ? options.uphillEfficiency : undefined;
    let downhillEfficiency = (options && options.downhillEfficiency !== undefined) ? options.downhillEfficiency : undefined;
    let constantSpeedConsumptionInkWhPerHundredkm = (options && options.constantSpeedConsumptionInkWhPerHundredkm !== undefined) ? options.constantSpeedConsumptionInkWhPerHundredkm : undefined;
    let currentChargeInkWh = (options && options.currentChargeInkWh !== undefined) ? options.currentChargeInkWh : undefined;
    let maxChargeInkWh = (options && options.maxChargeInkWh !== undefined) ? options.maxChargeInkWh : undefined;
    let auxiliaryPowerInkW = (options && options.auxiliaryPowerInkW !== undefined) ? options.auxiliaryPowerInkW : undefined;
    let coordinateList = (options && options.coordinateList !== undefined) ? options.coordinateList : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          maxAlternatives,
          alternativeType,
          minDeviationDistance,
          minDeviationTime,
          "this.client.instructionsType": this.client.instructionsType,
          language,
          computeBestOrder,
          routeRepresentation,
          computeTravelTimeFor,
          vehicleHeading,
          report,
          sectionType,
          arriveAt,
          departAt,
          vehicleAxleWeight,
          vehicleLength,
          vehicleHeight,
          vehicleWidth,
          vehicleMaxSpeed,
          vehicleWeight,
          windingness,
          hilliness,
          travelMode,
          avoid,
          traffic,
          routeType,
          vehicleLoadType,
          vehicleEngineType,
          constantSpeedConsumptionInLitersPerHundredkm,
          currentFuelInLiters,
          auxiliaryPowerInLitersPerHour,
          fuelEnergyDensityInMJoulesPerLiter,
          accelerationEfficiency,
          decelerationEfficiency,
          uphillEfficiency,
          downhillEfficiency,
          constantSpeedConsumptionInkWhPerHundredkm,
          currentChargeInkWh,
          maxChargeInkWh,
          auxiliaryPowerInkW,
          coordinateList
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "route/directions/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "maxAlternatives",
              mapper: {
                serializedName: "maxAlternatives",
                defaultValue: 0,
                constraints: {
                  InclusiveMaximum: 5,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "alternativeType",
              mapper: {
                serializedName: "alternativeType",
                defaultValue: 'anyRoute',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "anyRoute",
                    "betterRoute"
                  ]
                }
              }
            },
            {
              parameterPath: "minDeviationDistance",
              mapper: {
                serializedName: "minDeviationDistance",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "minDeviationTime",
              mapper: {
                serializedName: "minDeviationTime",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "this.client.instructionsType",
              mapper: {
                serializedName: "instructionsType",
                type: {
                  name: "Enum",
                  allowedValues: [
                    "coded",
                    "text",
                    "tagged"
                  ]
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "computeBestOrder",
              mapper: {
                serializedName: "computeBestOrder",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "routeRepresentation",
              mapper: {
                serializedName: "routeRepresentation",
                defaultValue: 'polyline',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "polyline",
                    "none"
                  ]
                }
              }
            },
            {
              parameterPath: "computeTravelTimeFor",
              mapper: {
                serializedName: "computeTravelTimeFor",
                defaultValue: 'none',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "none",
                    "all"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleHeading",
              mapper: {
                serializedName: "vehicleHeading",
                constraints: {
                  InclusiveMaximum: 359,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "report",
              mapper: {
                serializedName: "report",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "sectionType",
              mapper: {
                serializedName: "sectionType",
                defaultValue: 'travelMode',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "carTrain",
                    "country",
                    "ferry",
                    "motorway",
                    "pedestrian",
                    "tollRoad",
                    "tollVignette",
                    "traffic",
                    "travelMode",
                    "tunnel"
                  ]
                }
              }
            },
            {
              parameterPath: "arriveAt",
              mapper: {
                serializedName: "arriveAt",
                type: {
                  name: "DateTime"
                }
              }
            },
            {
              parameterPath: "departAt",
              mapper: {
                serializedName: "departAt",
                type: {
                  name: "DateTime"
                }
              }
            },
            {
              parameterPath: "vehicleAxleWeight",
              mapper: {
                serializedName: "vehicleAxleWeight",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleLength",
              mapper: {
                serializedName: "vehicleLength",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleHeight",
              mapper: {
                serializedName: "vehicleHeight",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleWidth",
              mapper: {
                serializedName: "vehicleWidth",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleMaxSpeed",
              mapper: {
                serializedName: "vehicleMaxSpeed",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "vehicleWeight",
              mapper: {
                serializedName: "vehicleWeight",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "windingness",
              mapper: {
                serializedName: "windingness",
                defaultValue: 'normal',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "low",
                    "normal",
                    "high"
                  ]
                }
              }
            },
            {
              parameterPath: "hilliness",
              mapper: {
                serializedName: "hilliness",
                defaultValue: 'normal',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "low",
                    "normal",
                    "high"
                  ]
                }
              }
            },
            {
              parameterPath: "travelMode",
              mapper: {
                serializedName: "travelMode",
                defaultValue: 'car',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "car",
                    "truck",
                    "taxi",
                    "bus",
                    "van",
                    "motorcycle",
                    "bicycle",
                    "pedestrian"
                  ]
                }
              }
            },
            {
              parameterPath: "avoid",
              mapper: {
                serializedName: "avoid",
                type: {
                  name: "Enum",
                  allowedValues: [
                    "tollRoads",
                    "motorways",
                    "ferries",
                    "unpavedRoads",
                    "carpools",
                    "alreadyUsedRoads"
                  ]
                }
              }
            },
            {
              parameterPath: "traffic",
              mapper: {
                serializedName: "traffic",
                defaultValue: true,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "routeType",
              mapper: {
                serializedName: "routeType",
                defaultValue: 'fastest',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "fastest",
                    "shortest",
                    "eco",
                    "thrilling"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleLoadType",
              mapper: {
                serializedName: "vehicleLoadType",
                type: {
                  name: "Enum",
                  allowedValues: [
                    "USHazmatClass1",
                    "USHazmatClass2",
                    "USHazmatClass3",
                    "USHazmatClass4",
                    "USHazmatClass5",
                    "USHazmatClass6",
                    "USHazmatClass7",
                    "USHazmatClass8",
                    "USHazmatClass9",
                    "otherHazmatExplosive",
                    "otherHazmatGeneral",
                    "otherHazmatHarmfulToWater"
                  ]
                }
              }
            },
            {
              parameterPath: "vehicleEngineType",
              mapper: {
                serializedName: "vehicleEngineType",
                defaultValue: 'combustion',
                type: {
                  name: "Enum",
                  allowedValues: [
                    "combustion",
                    "electric"
                  ]
                }
              }
            },
            {
              parameterPath: "constantSpeedConsumptionInLitersPerHundredkm",
              mapper: {
                serializedName: "constantSpeedConsumptionInLitersPerHundredkm",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "currentFuelInLiters",
              mapper: {
                serializedName: "currentFuelInLiters",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "auxiliaryPowerInLitersPerHour",
              mapper: {
                serializedName: "auxiliaryPowerInLitersPerHour",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "fuelEnergyDensityInMJoulesPerLiter",
              mapper: {
                serializedName: "fuelEnergyDensityInMJoulesPerLiter",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "accelerationEfficiency",
              mapper: {
                serializedName: "accelerationEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "decelerationEfficiency",
              mapper: {
                serializedName: "decelerationEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "uphillEfficiency",
              mapper: {
                serializedName: "uphillEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "downhillEfficiency",
              mapper: {
                serializedName: "downhillEfficiency",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "constantSpeedConsumptionInkWhPerHundredkm",
              mapper: {
                serializedName: "constantSpeedConsumptionInkWhPerHundredkm",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "currentChargeInkWh",
              mapper: {
                serializedName: "currentChargeInkWh",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "maxChargeInkWh",
              mapper: {
                serializedName: "maxChargeInkWh",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "auxiliaryPowerInkW",
              mapper: {
                serializedName: "auxiliaryPowerInkW",
                type: {
                  name: "String"
                }
              }
            }
          ],
          requestBody: {
            parameterPath: "coordinateList",
            mapper: Mappers.GeoJSONFeatureCollection
          },
          contentType: "application/json; charset=utf-8",
          responses: {
            200: {
              bodyMapper: Mappers.RouteDirectionsResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.RouteDirectionsResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Route Directions Batch API**<br><br> The Route Directions Batch API allows the caller to batch
   * up to 1000 [Route Directions
   * API](https://docs.microsoft.com/en-us/rest/api/maps/routeservice/getroutedirections)
   * queries/requests using just a single api call. To send the _route directions_ queries you will
   * use a `POST` request where the request body will contain the `queries` array in `JSON` format
   * and the `Content-Type` header will be set to `application/json`.<br><br>An _route directions_
   * query in a batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Route Directions Batch API call
   * itself. The string values in the _route directions_ query must be properly escaped (e.g. "
   * character should be escaped with \\) and it should also be properly URL-encoded.<br><br> for
   * example:
   * "?_query_=47.639987,-122.128384:47.621252,-122.184408:47.596437,-122.332000&_routeType_=fastest&_travelMode_=car"<br><br>
   * The maximum number of queries that can be specified in the batch is 1000 and the batch should
   * contain at least 1 query.<br><br>Please note that the Route Directions Batch API is a
   * long-running request. When the client `POST`'s the initial batch request, the server responds
   * immediately with a `202 Accepted` response with the `Location` header containing the URL to
   * download the results of the batch request later. To download the batch results you will issue a
   * `GET` request to this download endpoint. If the batch processing has finished, this call returns
   * a `200 OK` response with all the batch results, else it continues to return a `202 Accepted`
   * response. Please, be aware that batch results are available for download for 14 days, after
   * which the request for results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postRouteDirectionsBatchWithHttpOperationResponse(options?: msRest.RequestOptionsBase): Promise<msRest.HttpOperationResponse<Models.PostRouteDirectionsBatchAcceptedResponse>> {
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "route/directions/batch/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            202: {
              bodyMapper: Mappers.PostRouteDirectionsBatchAcceptedResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 202) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.PostRouteDirectionsBatchAcceptedResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * Returns  a route between an origin and a destination, passing through waypoints if they are
   * specified. The route will take into account factors such as current traffic and the typical road
   * speeds on the requested day of the week and time of day.
   *
   * Information returned includes the distance, estimated travel time, and a representation of the
   * route geometry. Additional routing information such as optimized waypoint order or turn by turn
   * instructions is also available, depending on the options selected.
   *
   * Routing service provides a set of parameters for a detailed description of vehicle-specific
   * Consumption Model. Please check [Consumption
   * Model](https://docs.microsoft.com/azure/azure-maps/consumption-model) for detailed explanation
   * of the concepts and parameters involved.
   *
   * @param {string} query The Coordinates through which the route is calculated, delimited by a
   * colon.  A minimum of two coordinates is required.  The first one is the origin and the last is
   * the destination of the route. Optional coordinates in-between act as WayPoints in the route.
   * You can pass up to 50 WayPoints unless computeBestOrder is set to true, in which case the limit
   * is reduced to 20.
   *
   * @param {RouteGetRouteDirectionsOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.RouteDirectionsResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.RouteDirectionsResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getRouteDirections(query: string): Promise<Models.RouteDirectionsResponse>;
  getRouteDirections(query: string, options: Models.RouteGetRouteDirectionsOptionalParams): Promise<Models.RouteDirectionsResponse>;
  getRouteDirections(query: string, callback: msRest.ServiceCallback<Models.RouteDirectionsResponse>): void;
  getRouteDirections(query: string, options: Models.RouteGetRouteDirectionsOptionalParams, callback: msRest.ServiceCallback<Models.RouteDirectionsResponse>): void;
  getRouteDirections(query: string, options?: Models.RouteGetRouteDirectionsOptionalParams, callback?: msRest.ServiceCallback<Models.RouteDirectionsResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.RouteDirectionsResponse>;
    if (!callback) {
      return this.getRouteDirectionsWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.RouteDirectionsResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getRouteDirectionsWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.RouteDirectionsResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * Returns  a route between an origin and a destination, passing through waypoints if they are
   * specified. The route will take into account factors such as current traffic and the typical road
   * speeds on the requested day of the week and time of day.
   *
   * Information returned includes the distance, estimated travel time, and a representation of the
   * route geometry. Additional routing information such as optimized waypoint order or turn by turn
   * instructions is also available, depending on the options selected.
   *
   * Routing service provides a set of parameters for a detailed description of vehicle-specific
   * Consumption Model. Please check [Consumption
   * Model](https://docs.microsoft.com/en-us/rest/api/maps/consumption-model) for detailed
   * explanation of the concepts and parameters involved.
   *
   * @param {string} query The Coordinates through which the route is calculated. Needs two
   * coordinates at least. Delimited by colon. First one is the origin and the last is the
   * destination of the route. Optional coordinates in between act as WayPoints in the route.
   *
   * @param {RoutePostRouteDirectionsOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.RouteDirectionsResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.RouteDirectionsResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postRouteDirections(query: string): Promise<Models.RouteDirectionsResponse>;
  postRouteDirections(query: string, options: Models.RoutePostRouteDirectionsOptionalParams): Promise<Models.RouteDirectionsResponse>;
  postRouteDirections(query: string, callback: msRest.ServiceCallback<Models.RouteDirectionsResponse>): void;
  postRouteDirections(query: string, options: Models.RoutePostRouteDirectionsOptionalParams, callback: msRest.ServiceCallback<Models.RouteDirectionsResponse>): void;
  postRouteDirections(query: string, options?: Models.RoutePostRouteDirectionsOptionalParams, callback?: msRest.ServiceCallback<Models.RouteDirectionsResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.RouteDirectionsResponse>;
    if (!callback) {
      return this.postRouteDirectionsWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.RouteDirectionsResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postRouteDirectionsWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.RouteDirectionsResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Route Directions Batch API**<br><br> The Route Directions Batch API allows the caller to batch
   * up to 1000 [Route Directions
   * API](https://docs.microsoft.com/en-us/rest/api/maps/routeservice/getroutedirections)
   * queries/requests using just a single api call. To send the _route directions_ queries you will
   * use a `POST` request where the request body will contain the `queries` array in `JSON` format
   * and the `Content-Type` header will be set to `application/json`.<br><br>An _route directions_
   * query in a batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Route Directions Batch API call
   * itself. The string values in the _route directions_ query must be properly escaped (e.g. "
   * character should be escaped with \\) and it should also be properly URL-encoded.<br><br> for
   * example:
   * "?_query_=47.639987,-122.128384:47.621252,-122.184408:47.596437,-122.332000&_routeType_=fastest&_travelMode_=car"<br><br>
   * The maximum number of queries that can be specified in the batch is 1000 and the batch should
   * contain at least 1 query.<br><br>Please note that the Route Directions Batch API is a
   * long-running request. When the client `POST`'s the initial batch request, the server responds
   * immediately with a `202 Accepted` response with the `Location` header containing the URL to
   * download the results of the batch request later. To download the batch results you will issue a
   * `GET` request to this download endpoint. If the batch processing has finished, this call returns
   * a `200 OK` response with all the batch results, else it continues to return a `202 Accepted`
   * response. Please, be aware that batch results are available for download for 14 days, after
   * which the request for results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.PostRouteDirectionsBatchAcceptedResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.PostRouteDirectionsBatchAcceptedResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postRouteDirectionsBatch(): Promise<Models.PostRouteDirectionsBatchAcceptedResponse>;
  postRouteDirectionsBatch(options: msRest.RequestOptionsBase): Promise<Models.PostRouteDirectionsBatchAcceptedResponse>;
  postRouteDirectionsBatch(callback: msRest.ServiceCallback<Models.PostRouteDirectionsBatchAcceptedResponse>): void;
  postRouteDirectionsBatch(options: msRest.RequestOptionsBase, callback: msRest.ServiceCallback<Models.PostRouteDirectionsBatchAcceptedResponse>): void;
  postRouteDirectionsBatch(options?: msRest.RequestOptionsBase, callback?: msRest.ServiceCallback<Models.PostRouteDirectionsBatchAcceptedResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.PostRouteDirectionsBatchAcceptedResponse>;
    if (!callback) {
      return this.postRouteDirectionsBatchWithHttpOperationResponse(options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.PostRouteDirectionsBatchAcceptedResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postRouteDirectionsBatchWithHttpOperationResponse(options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.PostRouteDirectionsBatchAcceptedResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

}
