/*
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

import * as msRest from "ms-rest-js";
import * as Models from "../models";
import * as Mappers from "../models/searchMappers";
import { MapsClientContext } from "../mapsClientContext";

const WebResource = msRest.WebResource;

/** Class representing a Search. */
export class Search {
  private readonly client: MapsClientContext;
  private readonly serializer = new msRest.Serializer(Mappers);
  /**
   * Create a Search.
   * @param {MapsClientContext} client Reference to the service client.
   */
  constructor(client: MapsClientContext) {
    this.client = client;
  }

  /**
   * **Free Form Search**<br><br>The basic default API is Free Form Search which handles the most
   * fuzzy of inputs handling any combination of address or POI tokens. This search API is the
   * canonical 'single line search'. The Free Form Search API is a seamless combination of POI search
   * and geocoding. The API can also be weighted with a contextual position (lat./lon. pair), or
   * fully constrained by a coordinate and radius, or it can be executed more generally without any
   * geo biasing anchor point.<br><br>We strongly advise you to use the 'countrySet' parameter to
   * specify only the countries for which your application needs coverage, as the default behavior
   * will be to search the entire world, potentially returning unnecessary results.<br><br> E.g.:
   * `countrySet`=US,FR <br><br>Please see [Search
   * Coverage](https://docs.microsoft.com/en-us/azure/location-based-services/geocoding-coverage) for
   * a complete list of all the supported countries.<br><br>Most Search queries default to
   * `maxFuzzyLevel`=2 to gain performance and also reduce unusual results. This new default can be
   * overridden as needed per request by passing in the query param `maxFuzzyLevel`=3 or 4.
   *
   * @param {string} query The applicable query string (e.g., "seattle", "pizza"). Can _also_ be
   * specified as a comma separated string composed by latitude followed by longitude (e.g.,
   * "47.641268, -122.125679"). Must be properly URL encoded.
   *
   * @param {SearchGetSearchFuzzyOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchFuzzyWithHttpOperationResponse(query: string, options?: Models.SearchGetSearchFuzzyOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchFuzzyResponse>> {
    let typeahead = (options && options.typeahead !== undefined) ? options.typeahead : false;
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let ofs = (options && options.ofs !== undefined) ? options.ofs : 0;
    let countrySet = (options && options.countrySet !== undefined) ? options.countrySet : undefined;
    let lat = (options && options.lat !== undefined) ? options.lat : undefined;
    let lon = (options && options.lon !== undefined) ? options.lon : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let topLeft = (options && options.topLeft !== undefined) ? options.topLeft : undefined;
    let btmRight = (options && options.btmRight !== undefined) ? options.btmRight : undefined;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let minFuzzyLevel = (options && options.minFuzzyLevel !== undefined) ? options.minFuzzyLevel : 1;
    let maxFuzzyLevel = (options && options.maxFuzzyLevel !== undefined) ? options.maxFuzzyLevel : 2;
    let idxSet = (options && options.idxSet !== undefined) ? options.idxSet : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          typeahead,
          limit,
          ofs,
          countrySet,
          lat,
          lon,
          radius,
          topLeft,
          btmRight,
          language,
          extendedPostalCodesFor,
          minFuzzyLevel,
          maxFuzzyLevel,
          idxSet
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/fuzzy/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "typeahead",
              mapper: {
                serializedName: "typeahead",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 100,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "ofs",
              mapper: {
                serializedName: "ofs",
                defaultValue: 0,
                constraints: {
                  InclusiveMaximum: 1900,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "countrySet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "countrySet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "stringElementType",
                    type: {
                      name: "String"
                    }
                  }
                }
              }
            },
            {
              parameterPath: "lat",
              mapper: {
                serializedName: "lat",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "lon",
              mapper: {
                serializedName: "lon",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "topLeft",
              mapper: {
                serializedName: "topLeft",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "btmRight",
              mapper: {
                serializedName: "btmRight",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "minFuzzyLevel",
              mapper: {
                serializedName: "minFuzzyLevel",
                defaultValue: 1,
                constraints: {
                  InclusiveMaximum: 4,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "maxFuzzyLevel",
              mapper: {
                serializedName: "maxFuzzyLevel",
                defaultValue: 2,
                constraints: {
                  InclusiveMaximum: 4,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "idxSet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "idxSet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "SearchIndexSetElementType",
                    type: {
                      name: "Enum",
                      allowedValues: [
                        "Addr",
                        "Geo",
                        "PAD",
                        "POI",
                        "Str",
                        "Xstr"
                      ]
                    }
                  }
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchFuzzyResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchFuzzyResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Get POI by Name**<br><br>If your search use case only requires POI results, you may use the
   * POI endpoint for searching. This endpoint will only return POI results.
   *
   * @param {string} query The POI name to search for (e.g., "statue of liberty", "starbucks"), must
   * be properly URL encoded.
   *
   * @param {SearchGetSearchPOIOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchPOIWithHttpOperationResponse(query: string, options?: Models.SearchGetSearchPOIOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchPoiResponse>> {
    let typeahead = (options && options.typeahead !== undefined) ? options.typeahead : false;
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let ofs = (options && options.ofs !== undefined) ? options.ofs : 0;
    let countrySet = (options && options.countrySet !== undefined) ? options.countrySet : undefined;
    let lat = (options && options.lat !== undefined) ? options.lat : undefined;
    let lon = (options && options.lon !== undefined) ? options.lon : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let topLeft = (options && options.topLeft !== undefined) ? options.topLeft : undefined;
    let btmRight = (options && options.btmRight !== undefined) ? options.btmRight : undefined;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          typeahead,
          limit,
          ofs,
          countrySet,
          lat,
          lon,
          radius,
          topLeft,
          btmRight,
          language,
          extendedPostalCodesFor
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/poi/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "typeahead",
              mapper: {
                serializedName: "typeahead",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 100,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "ofs",
              mapper: {
                serializedName: "ofs",
                defaultValue: 0,
                constraints: {
                  InclusiveMaximum: 1900,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "countrySet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "countrySet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "stringElementType",
                    type: {
                      name: "String"
                    }
                  }
                }
              }
            },
            {
              parameterPath: "lat",
              mapper: {
                serializedName: "lat",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "lon",
              mapper: {
                serializedName: "lon",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "topLeft",
              mapper: {
                serializedName: "topLeft",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "btmRight",
              mapper: {
                serializedName: "btmRight",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchPoiResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchPoiResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Nearby Search**<br><br>If you have a use case for only retrieving POI results around a
   * specific location, the nearby search method may be the right choice. This endpoint will only
   * return POI results, and does not take in a search query parameter.
   *
   * @param {number} lat Latitude where results should be biased. E.g. 37.337.
   *
   * @param {number} lon Longitude where results should be biased. E.g. -121.89.
   *
   * @param {SearchGetSearchNearbyOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchNearbyWithHttpOperationResponse(lat: number, lon: number, options?: Models.SearchGetSearchNearbyOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchNearbyResponse>> {
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let ofs = (options && options.ofs !== undefined) ? options.ofs : 0;
    let countrySet = (options && options.countrySet !== undefined) ? options.countrySet : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          lat,
          lon,
          limit,
          ofs,
          countrySet,
          radius,
          language,
          extendedPostalCodesFor
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/nearby/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "lat",
              mapper: {
                required: true,
                serializedName: "lat",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "lon",
              mapper: {
                required: true,
                serializedName: "lon",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 100,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "ofs",
              mapper: {
                serializedName: "ofs",
                defaultValue: 0,
                constraints: {
                  InclusiveMaximum: 1900,
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "countrySet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "countrySet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "stringElementType",
                    type: {
                      name: "String"
                    }
                  }
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchNearbyResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchNearbyResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Get POI by Category**<br><br>If your search use case only requires POI results filtered by
   * category, you may use the category endpoint. This endpoint will only return POI results which
   * are categorized as specified.<br><br>List of available categories can be found
   * [here](https://docs.microsoft.com/en-us/azure/azure-maps/search-categories).
   *
   * @param {string} query The POI category to search for (e.g., "AIRPORT", "BEACH"), must be
   * properly URL encoded.
   *
   * @param {SearchGetSearchPOICategoryOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchPOICategoryWithHttpOperationResponse(query: string, options?: Models.SearchGetSearchPOICategoryOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchPoiCategoryResponse>> {
    let typeahead = (options && options.typeahead !== undefined) ? options.typeahead : false;
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let multiplier = (options && options.multiplier !== undefined) ? options.multiplier : 2;
    let routingTimeout = (options && options.routingTimeout !== undefined) ? options.routingTimeout : 4000;
    let countrySet = (options && options.countrySet !== undefined) ? options.countrySet : undefined;
    let lat = (options && options.lat !== undefined) ? options.lat : undefined;
    let lon = (options && options.lon !== undefined) ? options.lon : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let topLeft = (options && options.topLeft !== undefined) ? options.topLeft : undefined;
    let btmRight = (options && options.btmRight !== undefined) ? options.btmRight : undefined;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          typeahead,
          limit,
          multiplier,
          routingTimeout,
          countrySet,
          lat,
          lon,
          radius,
          topLeft,
          btmRight,
          language,
          extendedPostalCodesFor
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/poi/category/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "typeahead",
              mapper: {
                serializedName: "typeahead",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 100,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "multiplier",
              mapper: {
                serializedName: "multiplier",
                defaultValue: 2,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "routingTimeout",
              mapper: {
                serializedName: "routingTimeout",
                defaultValue: 4000,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "countrySet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "countrySet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "stringElementType",
                    type: {
                      name: "String"
                    }
                  }
                }
              }
            },
            {
              parameterPath: "lat",
              mapper: {
                serializedName: "lat",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "lon",
              mapper: {
                serializedName: "lon",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "topLeft",
              mapper: {
                serializedName: "topLeft",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "btmRight",
              mapper: {
                serializedName: "btmRight",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchPoiCategoryResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchPoiCategoryResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Address Geocoding**<br><br>In many cases, the complete search service might be too much, for
   * instance if you are only interested in traditional geocoding. Search can also be accessed for
   * address look up exclusively. The geocoding is performed by hitting the geocode endpoint with
   * just the address or partial address in question. The geocoding search index will be queried for
   * everything above the street level data. No POIs will be returned. Note that the geocoder is very
   * tolerant of typos and incomplete addresses. It will also handle everything from exact street
   * addresses or street or intersections as well as higher level geographies such as city centers,
   * counties, states etc.
   *
   * @param {string} query The address to search for (e.g., "1 Microsoft way, Redmond, WA"), must be
   * properly URL encoded.
   *
   * @param {SearchGetSearchAddressOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchAddressWithHttpOperationResponse(query: string, options?: Models.SearchGetSearchAddressOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchAddressResponse>> {
    let typeahead = (options && options.typeahead !== undefined) ? options.typeahead : false;
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let ofs = (options && options.ofs !== undefined) ? options.ofs : 0;
    let countrySet = (options && options.countrySet !== undefined) ? options.countrySet : undefined;
    let lat = (options && options.lat !== undefined) ? options.lat : undefined;
    let lon = (options && options.lon !== undefined) ? options.lon : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let topLeft = (options && options.topLeft !== undefined) ? options.topLeft : undefined;
    let btmRight = (options && options.btmRight !== undefined) ? options.btmRight : undefined;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          typeahead,
          limit,
          ofs,
          countrySet,
          lat,
          lon,
          radius,
          topLeft,
          btmRight,
          language,
          extendedPostalCodesFor
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/address/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "typeahead",
              mapper: {
                serializedName: "typeahead",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 100,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "ofs",
              mapper: {
                serializedName: "ofs",
                defaultValue: 0,
                constraints: {
                  InclusiveMinimum: 0
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "countrySet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "countrySet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "stringElementType",
                    type: {
                      name: "String"
                    }
                  }
                }
              }
            },
            {
              parameterPath: "lat",
              mapper: {
                serializedName: "lat",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "lon",
              mapper: {
                serializedName: "lon",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "topLeft",
              mapper: {
                serializedName: "topLeft",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "btmRight",
              mapper: {
                serializedName: "btmRight",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchAddressResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchAddressResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Reverse Geocode to an Address**<br><br>There may be times when you need to translate a
   * coordinate (example: 37.786505, -122.3862) into a human understandable street address. Most
   * often this is needed in tracking applications where you receive a GPS feed from the device or
   * asset and wish to know what address where the coordinate is located. This endpoint will return
   * address information for a given coordinate.
   *
   * @param {string} query The applicable query specified as a comma separated string composed by
   * latitude followed by longitude e.g. "47.641268, -122.125679".
   *
   * @param {SearchGetSearchAddressReverseOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchAddressReverseWithHttpOperationResponse(query: string, options?: Models.SearchGetSearchAddressReverseOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchAddressReverseResponse>> {
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let returnSpeedLimit = (options && options.returnSpeedLimit !== undefined) ? options.returnSpeedLimit : false;
    let heading = (options && options.heading !== undefined) ? options.heading : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let number = (options && options.number !== undefined) ? options.number : undefined;
    let returnRoadUse = (options && options.returnRoadUse !== undefined) ? options.returnRoadUse : false;
    let roadUse = (options && options.roadUse !== undefined) ? options.roadUse : undefined;
    let allowFreeformNewline = (options && options.allowFreeformNewline !== undefined) ? options.allowFreeformNewline : undefined;
    let dateTime = (options && options.dateTime !== undefined) ? options.dateTime : undefined;
    let returnMatchType = (options && options.returnMatchType !== undefined) ? options.returnMatchType : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          language,
          returnSpeedLimit,
          heading,
          radius,
          number,
          returnRoadUse,
          roadUse,
          allowFreeformNewline,
          dateTime,
          returnMatchType
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/address/reverse/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "returnSpeedLimit",
              mapper: {
                serializedName: "returnSpeedLimit",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "heading",
              mapper: {
                serializedName: "heading",
                constraints: {
                  InclusiveMaximum: 360,
                  InclusiveMinimum: -360
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "number",
              mapper: {
                serializedName: "number",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "returnRoadUse",
              mapper: {
                serializedName: "returnRoadUse",
                defaultValue: false,
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "roadUse",
              mapper: {
                serializedName: "roadUse",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "allowFreeformNewline",
              mapper: {
                serializedName: "allowFreeformNewline",
                type: {
                  name: "Boolean"
                }
              }
            },
            {
              parameterPath: "dateTime",
              mapper: {
                serializedName: "dateTime",
                type: {
                  name: "DateTime"
                }
              }
            },
            {
              parameterPath: "returnMatchType",
              mapper: {
                serializedName: "returnMatchType",
                type: {
                  name: "Boolean"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchAddressReverseResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchAddressReverseResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Reverse Geocode to a Cross Street**<br><br>There may be times when you need to translate a
   * coordinate (example: 37.786505, -122.3862) into a human understandable cross street. Most often
   * this is needed in tracking applications where you receive a GPS feed from the device or asset
   * and wish to know what address where the coordinate is located.<br><br>This endpoint will return
   * cross street information for a given coordinate.
   *
   * @param {string} query The applicable query specified as a comma separated string composed by
   * latitude followed by longitude e.g. "47.641268, -122.125679".
   *
   * @param {SearchGetSearchAddressReverseCrossStreetOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchAddressReverseCrossStreetWithHttpOperationResponse(query: string, options?: Models.SearchGetSearchAddressReverseCrossStreetOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchAddressReverseCrossStreetResponse>> {
    let heading = (options && options.heading !== undefined) ? options.heading : undefined;
    let radius = (options && options.radius !== undefined) ? options.radius : undefined;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          query,
          heading,
          radius,
          language
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/address/reverse/crossStreet/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "heading",
              mapper: {
                serializedName: "heading",
                constraints: {
                  InclusiveMaximum: 360,
                  InclusiveMinimum: -360
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "radius",
              mapper: {
                serializedName: "radius",
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchAddressReverseCrossStreetResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchAddressReverseCrossStreetResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Structured Address Geocoding**<br><br>Azure Address Geocoding can also be accessed for
   * structured address look up exclusively. The geocoding search index will be queried for
   * everything above the street level data. No POIs will be returned. Note that the geocoder is very
   * tolerant of typos and incomplete addresses. It will also handle everything from exact street
   * addresses or street or intersections as well as higher level geographies such as city centers,
   * counties, states etc.
   *
   * @param {string} countryCode The 2 or 3 letter country code portion of an address. E.g. US
   *
   * @param {SearchGetSearchAddressStructuredOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async getSearchAddressStructuredWithHttpOperationResponse(countryCode: string, options?: Models.SearchGetSearchAddressStructuredOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchAddressStructuredResponse>> {
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let ofs = (options && options.ofs !== undefined) ? options.ofs : 0;
    let streetNumber = (options && options.streetNumber !== undefined) ? options.streetNumber : undefined;
    let streetName = (options && options.streetName !== undefined) ? options.streetName : undefined;
    let crossStreet = (options && options.crossStreet !== undefined) ? options.crossStreet : undefined;
    let municipality = (options && options.municipality !== undefined) ? options.municipality : undefined;
    let municipalitySubdivision = (options && options.municipalitySubdivision !== undefined) ? options.municipalitySubdivision : undefined;
    let countryTertiarySubdivision = (options && options.countryTertiarySubdivision !== undefined) ? options.countryTertiarySubdivision : undefined;
    let countrySecondarySubdivision = (options && options.countrySecondarySubdivision !== undefined) ? options.countrySecondarySubdivision : undefined;
    let countrySubdivision = (options && options.countrySubdivision !== undefined) ? options.countrySubdivision : undefined;
    let postalCode = (options && options.postalCode !== undefined) ? options.postalCode : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format,
          language,
          countryCode,
          limit,
          ofs,
          streetNumber,
          streetName,
          crossStreet,
          municipality,
          municipalitySubdivision,
          countryTertiarySubdivision,
          countrySecondarySubdivision,
          countrySubdivision,
          postalCode,
          extendedPostalCodesFor
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "GET",
          baseUrl: this.client.baseUri,
          path: "search/address/structured/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "countryCode",
              mapper: {
                required: true,
                serializedName: "countryCode",
                defaultValue: 'US',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "ofs",
              mapper: {
                serializedName: "ofs",
                defaultValue: 0,
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "streetNumber",
              mapper: {
                serializedName: "streetNumber",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "streetName",
              mapper: {
                serializedName: "streetName",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "crossStreet",
              mapper: {
                serializedName: "crossStreet",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "municipality",
              mapper: {
                serializedName: "municipality",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "municipalitySubdivision",
              mapper: {
                serializedName: "municipalitySubdivision",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "countryTertiarySubdivision",
              mapper: {
                serializedName: "countryTertiarySubdivision",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "countrySecondarySubdivision",
              mapper: {
                serializedName: "countrySecondarySubdivision",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "countrySubdivision",
              mapper: {
                serializedName: "countrySubdivision",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "postalCode",
              mapper: {
                serializedName: "postalCode",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            200: {
              bodyMapper: Mappers.SearchAddressStructuredResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchAddressStructuredResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * The Search Geometry endpoint allows you to perform a free form search inside a single geometry
   * or many of them. The search results that fall inside the geometry/geometries will be
   * returned.<br><br>To send the geometry you will use a `POST` request where the request body will
   * contain the `geometry` object represented as a `GeoJSON` type and the `Content-Type` header will
   * be set to `application/json`. The geographical features to be searched can be modeled as Polygon
   * and/or Circle geometries represented using any one of the following `GeoJSON`
   * types:<ul><li>**GeoJSON FeatureCollection** <br>The `geometry` can be represented as a `GeoJSON
   * FeatureCollection` object. This is the recommended option if the geometry contains both Polygons
   * and Circles. The `FeatureCollection` can contain a max of 50 `GeoJSON Feature` objects. Each
   * `Feature` object should represent either a Polygon or a Circle with the following conditions:<ul
   * style="list-style-type:none"><li>A `Feature` object for the Polygon geometry can have a max of
   * 50 coordinates and it's properties must be empty.</li><li>A `Feature` object for the Circle
   * geometry is composed of a _center_ represented using a `GeoJSON Point` type and a _radius_ value
   * (in meters) which must be specified in the object's properties along with the _subType_ property
   * whose value should be 'Circle'.</li></ul><br> Please see the Examples section below for a sample
   * `FeatureCollection` representation.<br><br></li><li>**GeoJSON GeometryCollection**<br>The
   * `geometry` can be represented as a `GeoJSON GeometryCollection` object. This is the recommended
   * option if the geometry contains a list of Polygons only. The `GeometryCollection` can contain a
   * max of 50 `GeoJSON Polygon` objects. Each `Polygon` object can have a max of 50 coordinates.
   * Please see the Examples section below for a sample `GeometryCollection`
   * representation.<br><br></li><li>**GeoJSON Polygon**<br>The `geometry` can be represented as a
   * `GeoJSON Polygon` object. This is the recommended option if the geometry contains a single
   * Polygon. The `Polygon` object can have a max of 50 coordinates. Please see the Examples section
   * below for a sample `Polygon` representation.<br><br></li></ul>.<br><br>
   *
   * @param {string} query The applicable query string (e.g., "burger joints", "pizza"). Can _also_
   * be specified as a comma separated string composed by latitude followed by longitude (e.g.,
   * "47.641268, -122.125679"). Must be properly URL encoded.
   *
   * @param {SearchInsideGeometryRequestBody} searchInsideGeometryRequestBody This represents the
   * geometry for one or more geographical features (parks, state boundary etc.) to search in and
   * should be a GeoJSON compliant type. Please refer to [RFC
   * 7946](https://tools.ietf.org/html/rfc7946) for details.
   *
   * @param {SearchPostSearchInsideGeometryOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postSearchInsideGeometryWithHttpOperationResponse(query: string, searchInsideGeometryRequestBody: Models.SearchInsideGeometryRequestBody, options?: Models.SearchPostSearchInsideGeometryOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchGeometryResponse>> {
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let language = (options && options.language !== undefined) ? options.language : undefined;
    let extendedPostalCodesFor = (options && options.extendedPostalCodesFor !== undefined) ? options.extendedPostalCodesFor : undefined;
    let idxSet = (options && options.idxSet !== undefined) ? options.idxSet : undefined;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          query,
          format,
          limit,
          language,
          extendedPostalCodesFor,
          idxSet,
          searchInsideGeometryRequestBody
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "search/geometry/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 100,
                  InclusiveMinimum: 1
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "language",
              mapper: {
                serializedName: "language",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "extendedPostalCodesFor",
              mapper: {
                serializedName: "extendedPostalCodesFor",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "idxSet",
              collectionFormat: msRest.QueryCollectionFormat.Csv,
              mapper: {
                serializedName: "idxSet",
                type: {
                  name: "Sequence",
                  element: {
                    serializedName: "SearchIndexSetElementType",
                    type: {
                      name: "Enum",
                      allowedValues: [
                        "Addr",
                        "Geo",
                        "PAD",
                        "POI",
                        "Str",
                        "Xstr"
                      ]
                    }
                  }
                }
              }
            }
          ],
          requestBody: {
            parameterPath: "searchInsideGeometryRequestBody",
            mapper: {
              ...Mappers.SearchInsideGeometryRequestBody,
              required: true
            }
          },
          contentType: "application/json; charset=utf-8",
          responses: {
            200: {
              bodyMapper: Mappers.SearchGeometryResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchGeometryResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * The Search Along Route endpoint allows you to perform a fuzzy search for POIs along a specified
   * route. This search is constrained by specifying the `maxDetourTime` limiting measure.<br><br>To
   * send the route-points you will use a `POST` request where the request body will contain the
   * `route` object represented as a `GeoJSON LineString` type and the `Content-Type` header will be
   * set to `application/json`. Each route-point in `route` is represented as a `GeoJSON Position`
   * type i.e. an array where the _longitude_ value is followed by the _latitude_ value and the
   * _altitude_ value is ignored. The `route` should contain at least 2 route-points.<br><br>It is
   * possible that original route will be altered, some of it's points may be skipped. If the route
   * that passes through the found point is faster than the original one, the `detourTime` value in
   * the response is negative.
   *
   * @param {string} query The applicable query string (e.g., "burger joints", "pizza"). Can _also_
   * be specified as a comma separated string composed by latitude followed by longitude (e.g.,
   * "47.641268, -122.125679"). Must be properly URL encoded.
   *
   * @param {number} maxDetourTime Maximum detour time of the point of interest in seconds. Max value
   * is 3600 seconds
   *
   * @param {SearchAlongRouteRequestBody} searchAlongRouteRequestBody This represents the route to
   * search along and should be a valid `GeoJSON LineString` type. Please refer to [RFC
   * 7946](https://tools.ietf.org/html/rfc7946#section-3.1.4) for details.
   *
   * @param {SearchPostSearchAlongRouteOptionalParams} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postSearchAlongRouteWithHttpOperationResponse(query: string, maxDetourTime: number, searchAlongRouteRequestBody: Models.SearchAlongRouteRequestBody, options?: Models.SearchPostSearchAlongRouteOptionalParams): Promise<msRest.HttpOperationResponse<Models.SearchAlongRouteResponse>> {
    let limit = (options && options.limit !== undefined) ? options.limit : 10;
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          query,
          format,
          maxDetourTime,
          limit,
          searchAlongRouteRequestBody
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "search/alongRoute/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "query",
              mapper: {
                required: true,
                serializedName: "query",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "maxDetourTime",
              mapper: {
                required: true,
                serializedName: "maxDetourTime",
                constraints: {
                  InclusiveMaximum: 3600
                },
                type: {
                  name: "Number"
                }
              }
            },
            {
              parameterPath: "limit",
              mapper: {
                serializedName: "limit",
                defaultValue: 10,
                constraints: {
                  InclusiveMaximum: 20
                },
                type: {
                  name: "Number"
                }
              }
            }
          ],
          requestBody: {
            parameterPath: "searchAlongRouteRequestBody",
            mapper: {
              ...Mappers.SearchAlongRouteRequestBody,
              required: true
            }
          },
          contentType: "application/json; charset=utf-8",
          responses: {
            200: {
              bodyMapper: Mappers.SearchAlongRouteResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 200) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.SearchAlongRouteResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Search Fuzzy Batch API**<br><br> The Search Fuzzy Batch API allows the caller to batch up to
   * 10000 [Search Fuzzy
   * API](https://docs.microsoft.com/en-us/rest/api/maps/searchservice/getsearchfuzzy)
   * queries/requests using just a single api call. To send the _search fuzzy_ queries you will use a
   * `POST` request where the request body will contain the `queries` array in `JSON` format and the
   * `Content-Type` header will be set to `application/json`.<br><br>A _search fuzzy_ query in a
   * batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Search Fuzzy Batch API call itself.
   * The string values in the _search fuzzy_ query must be properly escaped (e.g. " character should
   * be escaped with \\) and it should also be properly URL-encoded.<br><br> for example:
   * "?_query_=atm&_lat_=47.639769&_lon_=-122.128362&_radius_=5000&_limit_=5"<br><br> The maximum
   * number of queries that can be specified in the batch is 10000  and the batch should contain at
   * least 1 query.<br><br>Please note that the Search Fuzzy Batch API is a long-running request.
   * When the client `POST`'s the initial batch request, the server responds immediately with a `202
   * Accepted` response with the `Location` header containing the URL to download the results of the
   * batch request later. To download the batch results you will issue a `GET` request to this
   * download endpoint. If the batch processing has finished, this call returns a `200 OK` response
   * with all the batch results, else it continues to return a `202 Accepted` response. Please, be
   * aware that batch results are available for download for 14 days, after which the request for
   * results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postSearchFuzzyBatchWithHttpOperationResponse(options?: msRest.RequestOptionsBase): Promise<msRest.HttpOperationResponse<Models.PostSearchFuzzyBatchAcceptedResponse>> {
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "search/fuzzy/batch/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            202: {
              bodyMapper: Mappers.PostSearchFuzzyBatchAcceptedResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 202) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.PostSearchFuzzyBatchAcceptedResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Address Geocoding Batch API**<br><br> The Address Geocoding Batch API allows the caller to
   * batch up to 10000 [Address Geocoding
   * API](https://docs.microsoft.com/en-us/rest/api/maps/searchservice/getsearchaddress)
   * queries/requests using just a single api call. To send the _address geocoding_ queries you will
   * use a `POST` request where the request body will contain the `queries` array in `JSON` format
   * and the `Content-Type` header will be set to `application/json`.<br><br>An _address geocoding_
   * query in a batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Address Geocoding Batch API call
   * itself. The string values in the _address geocoding_ query must be properly escaped (e.g. "
   * character should be escaped with \\) and it should also be properly URL-encoded.<br><br> for
   * example: "?_query_=Pike Pl, Seattle, WA
   * 98101&_lat_=47.610970&_lon_=-122.342469&_radius_=1000&_limit_=3"<br><br> The maximum number of
   * queries that can be specified in the batch is 10000  and the batch should contain at least 1
   * query.<br><br>Please note that the Address Geocoding Batch API is a long-running request. When
   * the client `POST`'s the initial batch request, the server responds immediately with a `202
   * Accepted` response with the `Location` header containing the URL to download the results of the
   * batch request later. To download the batch results you will issue a `GET` request to this
   * download endpoint. If the batch processing has finished, this call returns a `200 OK` response
   * with all the batch results, else it continues to return a `202 Accepted` response. Please, be
   * aware that batch results are available for download for 14 days, after which the request for
   * results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postSearchAddressBatchWithHttpOperationResponse(options?: msRest.RequestOptionsBase): Promise<msRest.HttpOperationResponse<Models.PostSearchAddressBatchAcceptedResponse>> {
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "search/address/batch/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            202: {
              bodyMapper: Mappers.PostSearchAddressBatchAcceptedResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 202) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.PostSearchAddressBatchAcceptedResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Reverse Geocoding Batch API**<br><br> The Reverse Geocoding Batch API allows the caller to
   * batch up to 10000 [Reverse Geocoding
   * API](https://docs.microsoft.com/en-us/rest/api/maps/searchservice/getsearchaddressreverse)
   * queries/requests using just a single api call. To send the _reverse geocoding_ queries you will
   * use a `POST` request where the request body will contain the `queries` array in `JSON` format
   * and the `Content-Type` header will be set to `application/json`.<br><br>An _reverse geocoding_
   * query in a batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Reverse Geocoding Batch API call
   * itself. The string values in the _reverse geocoding_ query must be properly escaped (e.g. "
   * character should be escaped with \\) and it should also be properly URL-encoded.<br><br> for
   * example: "?_query_=47.639765,-122.127896&_radius_=5000&_limit_=2"<br><br> The maximum number of
   * queries that can be specified in the batch is 10000  and the batch should contain at least 1
   * query.<br><br>Please note that the Reverse Geocoding Batch API is a long-running request. When
   * the client `POST`'s the initial batch request, the server responds immediately with a `202
   * Accepted` response with the `Location` header containing the URL to download the results of the
   * batch request later. To download the batch results you will issue a `GET` request to this
   * download endpoint. If the batch processing has finished, this call returns a `200 OK` response
   * with all the batch results, else it continues to return a `202 Accepted` response. Please, be
   * aware that batch results are available for download for 14 days, after which the request for
   * results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse} The deserialized result object.
   *
   * @reject {Error|ServiceError} The error object.
   */
  async postSearchAddressReverseBatchWithHttpOperationResponse(options?: msRest.RequestOptionsBase): Promise<msRest.HttpOperationResponse<Models.PostSearchAddressReverseBatchAcceptedResponse>> {
    let format = 'json';

    // Create HTTP transport objects
    const httpRequest = new WebResource();
    let operationRes: msRest.HttpOperationResponse;
    try {
      const operationArguments: msRest.OperationArguments = msRest.createOperationArguments(
        {
          "this.client.subscriptionKey": this.client.subscriptionKey,
          "this.client.apiVersion": this.client.apiVersion,
          format
        },
        options);
      operationRes = await this.client.sendOperationRequest(
        httpRequest,
        operationArguments,
        {
          httpMethod: "POST",
          baseUrl: this.client.baseUri,
          path: "search/address/reverse/batch/{format}",
          urlParameters: [
            {
              parameterPath: "format",
              mapper: {
                required: true,
                isConstant: true,
                serializedName: "format",
                defaultValue: 'json',
                type: {
                  name: "String"
                }
              }
            }
          ],
          queryParameters: [
            {
              parameterPath: "this.client.subscriptionKey",
              mapper: {
                required: true,
                serializedName: "subscription-key",
                type: {
                  name: "String"
                }
              }
            },
            {
              parameterPath: "this.client.apiVersion",
              mapper: {
                required: true,
                serializedName: "api-version",
                defaultValue: '1.0',
                type: {
                  name: "String"
                }
              }
            }
          ],
          responses: {
            202: {
              bodyMapper: Mappers.PostSearchAddressReverseBatchAcceptedResponse
            },
            default: {}
          },
          serializer: this.serializer
        });
      // Deserialize Response
      let statusCode = operationRes.status;
      if (statusCode === 202) {
        let parsedResponse = operationRes.parsedBody as { [key: string]: any };
        try {
          if (parsedResponse != undefined) {
            const resultMapper = Mappers.PostSearchAddressReverseBatchAcceptedResponse;
            operationRes.parsedBody = this.serializer.deserialize(resultMapper, parsedResponse, 'operationRes.parsedBody');
          }
        } catch (error) {
          let deserializationError = new msRest.RestError(`Error ${error} occurred in deserializing the responseBody - ${operationRes.bodyAsText}`);
          deserializationError.request = msRest.stripRequest(httpRequest);
          deserializationError.response = msRest.stripResponse(operationRes);
          return Promise.reject(deserializationError);
        }
      }
    } catch (err) {
      return Promise.reject(err);
    }
    return Promise.resolve(operationRes);
  }

  /**
   * **Free Form Search**<br><br>The basic default API is Free Form Search which handles the most
   * fuzzy of inputs handling any combination of address or POI tokens. This search API is the
   * canonical 'single line search'. The Free Form Search API is a seamless combination of POI search
   * and geocoding. The API can also be weighted with a contextual position (lat./lon. pair), or
   * fully constrained by a coordinate and radius, or it can be executed more generally without any
   * geo biasing anchor point.<br><br>We strongly advise you to use the 'countrySet' parameter to
   * specify only the countries for which your application needs coverage, as the default behavior
   * will be to search the entire world, potentially returning unnecessary results.<br><br> E.g.:
   * `countrySet`=US,FR <br><br>Please see [Search
   * Coverage](https://docs.microsoft.com/en-us/azure/location-based-services/geocoding-coverage) for
   * a complete list of all the supported countries.<br><br>Most Search queries default to
   * `maxFuzzyLevel`=2 to gain performance and also reduce unusual results. This new default can be
   * overridden as needed per request by passing in the query param `maxFuzzyLevel`=3 or 4.
   *
   * @param {string} query The applicable query string (e.g., "seattle", "pizza"). Can _also_ be
   * specified as a comma separated string composed by latitude followed by longitude (e.g.,
   * "47.641268, -122.125679"). Must be properly URL encoded.
   *
   * @param {SearchGetSearchFuzzyOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchFuzzyResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchFuzzyResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchFuzzy(query: string): Promise<Models.SearchFuzzyResponse>;
  getSearchFuzzy(query: string, options: Models.SearchGetSearchFuzzyOptionalParams): Promise<Models.SearchFuzzyResponse>;
  getSearchFuzzy(query: string, callback: msRest.ServiceCallback<Models.SearchFuzzyResponse>): void;
  getSearchFuzzy(query: string, options: Models.SearchGetSearchFuzzyOptionalParams, callback: msRest.ServiceCallback<Models.SearchFuzzyResponse>): void;
  getSearchFuzzy(query: string, options?: Models.SearchGetSearchFuzzyOptionalParams, callback?: msRest.ServiceCallback<Models.SearchFuzzyResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchFuzzyResponse>;
    if (!callback) {
      return this.getSearchFuzzyWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchFuzzyResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchFuzzyWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchFuzzyResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Get POI by Name**<br><br>If your search use case only requires POI results, you may use the
   * POI endpoint for searching. This endpoint will only return POI results.
   *
   * @param {string} query The POI name to search for (e.g., "statue of liberty", "starbucks"), must
   * be properly URL encoded.
   *
   * @param {SearchGetSearchPOIOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchPoiResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchPoiResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchPOI(query: string): Promise<Models.SearchPoiResponse>;
  getSearchPOI(query: string, options: Models.SearchGetSearchPOIOptionalParams): Promise<Models.SearchPoiResponse>;
  getSearchPOI(query: string, callback: msRest.ServiceCallback<Models.SearchPoiResponse>): void;
  getSearchPOI(query: string, options: Models.SearchGetSearchPOIOptionalParams, callback: msRest.ServiceCallback<Models.SearchPoiResponse>): void;
  getSearchPOI(query: string, options?: Models.SearchGetSearchPOIOptionalParams, callback?: msRest.ServiceCallback<Models.SearchPoiResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchPoiResponse>;
    if (!callback) {
      return this.getSearchPOIWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchPoiResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchPOIWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchPoiResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Nearby Search**<br><br>If you have a use case for only retrieving POI results around a
   * specific location, the nearby search method may be the right choice. This endpoint will only
   * return POI results, and does not take in a search query parameter.
   *
   * @param {number} lat Latitude where results should be biased. E.g. 37.337.
   *
   * @param {number} lon Longitude where results should be biased. E.g. -121.89.
   *
   * @param {SearchGetSearchNearbyOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchNearbyResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchNearbyResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchNearby(lat: number, lon: number): Promise<Models.SearchNearbyResponse>;
  getSearchNearby(lat: number, lon: number, options: Models.SearchGetSearchNearbyOptionalParams): Promise<Models.SearchNearbyResponse>;
  getSearchNearby(lat: number, lon: number, callback: msRest.ServiceCallback<Models.SearchNearbyResponse>): void;
  getSearchNearby(lat: number, lon: number, options: Models.SearchGetSearchNearbyOptionalParams, callback: msRest.ServiceCallback<Models.SearchNearbyResponse>): void;
  getSearchNearby(lat: number, lon: number, options?: Models.SearchGetSearchNearbyOptionalParams, callback?: msRest.ServiceCallback<Models.SearchNearbyResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchNearbyResponse>;
    if (!callback) {
      return this.getSearchNearbyWithHttpOperationResponse(lat, lon, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchNearbyResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchNearbyWithHttpOperationResponse(lat, lon, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchNearbyResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Get POI by Category**<br><br>If your search use case only requires POI results filtered by
   * category, you may use the category endpoint. This endpoint will only return POI results which
   * are categorized as specified.<br><br>List of available categories can be found
   * [here](https://docs.microsoft.com/en-us/azure/azure-maps/search-categories).
   *
   * @param {string} query The POI category to search for (e.g., "AIRPORT", "BEACH"), must be
   * properly URL encoded.
   *
   * @param {SearchGetSearchPOICategoryOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchPoiCategoryResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchPoiCategoryResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchPOICategory(query: string): Promise<Models.SearchPoiCategoryResponse>;
  getSearchPOICategory(query: string, options: Models.SearchGetSearchPOICategoryOptionalParams): Promise<Models.SearchPoiCategoryResponse>;
  getSearchPOICategory(query: string, callback: msRest.ServiceCallback<Models.SearchPoiCategoryResponse>): void;
  getSearchPOICategory(query: string, options: Models.SearchGetSearchPOICategoryOptionalParams, callback: msRest.ServiceCallback<Models.SearchPoiCategoryResponse>): void;
  getSearchPOICategory(query: string, options?: Models.SearchGetSearchPOICategoryOptionalParams, callback?: msRest.ServiceCallback<Models.SearchPoiCategoryResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchPoiCategoryResponse>;
    if (!callback) {
      return this.getSearchPOICategoryWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchPoiCategoryResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchPOICategoryWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchPoiCategoryResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Address Geocoding**<br><br>In many cases, the complete search service might be too much, for
   * instance if you are only interested in traditional geocoding. Search can also be accessed for
   * address look up exclusively. The geocoding is performed by hitting the geocode endpoint with
   * just the address or partial address in question. The geocoding search index will be queried for
   * everything above the street level data. No POIs will be returned. Note that the geocoder is very
   * tolerant of typos and incomplete addresses. It will also handle everything from exact street
   * addresses or street or intersections as well as higher level geographies such as city centers,
   * counties, states etc.
   *
   * @param {string} query The address to search for (e.g., "1 Microsoft way, Redmond, WA"), must be
   * properly URL encoded.
   *
   * @param {SearchGetSearchAddressOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchAddressResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchAddressResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchAddress(query: string): Promise<Models.SearchAddressResponse>;
  getSearchAddress(query: string, options: Models.SearchGetSearchAddressOptionalParams): Promise<Models.SearchAddressResponse>;
  getSearchAddress(query: string, callback: msRest.ServiceCallback<Models.SearchAddressResponse>): void;
  getSearchAddress(query: string, options: Models.SearchGetSearchAddressOptionalParams, callback: msRest.ServiceCallback<Models.SearchAddressResponse>): void;
  getSearchAddress(query: string, options?: Models.SearchGetSearchAddressOptionalParams, callback?: msRest.ServiceCallback<Models.SearchAddressResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchAddressResponse>;
    if (!callback) {
      return this.getSearchAddressWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchAddressResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchAddressWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchAddressResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Reverse Geocode to an Address**<br><br>There may be times when you need to translate a
   * coordinate (example: 37.786505, -122.3862) into a human understandable street address. Most
   * often this is needed in tracking applications where you receive a GPS feed from the device or
   * asset and wish to know what address where the coordinate is located. This endpoint will return
   * address information for a given coordinate.
   *
   * @param {string} query The applicable query specified as a comma separated string composed by
   * latitude followed by longitude e.g. "47.641268, -122.125679".
   *
   * @param {SearchGetSearchAddressReverseOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchAddressReverseResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchAddressReverseResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchAddressReverse(query: string): Promise<Models.SearchAddressReverseResponse>;
  getSearchAddressReverse(query: string, options: Models.SearchGetSearchAddressReverseOptionalParams): Promise<Models.SearchAddressReverseResponse>;
  getSearchAddressReverse(query: string, callback: msRest.ServiceCallback<Models.SearchAddressReverseResponse>): void;
  getSearchAddressReverse(query: string, options: Models.SearchGetSearchAddressReverseOptionalParams, callback: msRest.ServiceCallback<Models.SearchAddressReverseResponse>): void;
  getSearchAddressReverse(query: string, options?: Models.SearchGetSearchAddressReverseOptionalParams, callback?: msRest.ServiceCallback<Models.SearchAddressReverseResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchAddressReverseResponse>;
    if (!callback) {
      return this.getSearchAddressReverseWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchAddressReverseResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchAddressReverseWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchAddressReverseResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Reverse Geocode to a Cross Street**<br><br>There may be times when you need to translate a
   * coordinate (example: 37.786505, -122.3862) into a human understandable cross street. Most often
   * this is needed in tracking applications where you receive a GPS feed from the device or asset
   * and wish to know what address where the coordinate is located.<br><br>This endpoint will return
   * cross street information for a given coordinate.
   *
   * @param {string} query The applicable query specified as a comma separated string composed by
   * latitude followed by longitude e.g. "47.641268, -122.125679".
   *
   * @param {SearchGetSearchAddressReverseCrossStreetOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchAddressReverseCrossStreetResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchAddressReverseCrossStreetResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchAddressReverseCrossStreet(query: string): Promise<Models.SearchAddressReverseCrossStreetResponse>;
  getSearchAddressReverseCrossStreet(query: string, options: Models.SearchGetSearchAddressReverseCrossStreetOptionalParams): Promise<Models.SearchAddressReverseCrossStreetResponse>;
  getSearchAddressReverseCrossStreet(query: string, callback: msRest.ServiceCallback<Models.SearchAddressReverseCrossStreetResponse>): void;
  getSearchAddressReverseCrossStreet(query: string, options: Models.SearchGetSearchAddressReverseCrossStreetOptionalParams, callback: msRest.ServiceCallback<Models.SearchAddressReverseCrossStreetResponse>): void;
  getSearchAddressReverseCrossStreet(query: string, options?: Models.SearchGetSearchAddressReverseCrossStreetOptionalParams, callback?: msRest.ServiceCallback<Models.SearchAddressReverseCrossStreetResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchAddressReverseCrossStreetResponse>;
    if (!callback) {
      return this.getSearchAddressReverseCrossStreetWithHttpOperationResponse(query, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchAddressReverseCrossStreetResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchAddressReverseCrossStreetWithHttpOperationResponse(query, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchAddressReverseCrossStreetResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Structured Address Geocoding**<br><br>Azure Address Geocoding can also be accessed for
   * structured address look up exclusively. The geocoding search index will be queried for
   * everything above the street level data. No POIs will be returned. Note that the geocoder is very
   * tolerant of typos and incomplete addresses. It will also handle everything from exact street
   * addresses or street or intersections as well as higher level geographies such as city centers,
   * counties, states etc.
   *
   * @param {string} countryCode The 2 or 3 letter country code portion of an address. E.g. US
   *
   * @param {SearchGetSearchAddressStructuredOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchAddressStructuredResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchAddressStructuredResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  getSearchAddressStructured(countryCode: string): Promise<Models.SearchAddressStructuredResponse>;
  getSearchAddressStructured(countryCode: string, options: Models.SearchGetSearchAddressStructuredOptionalParams): Promise<Models.SearchAddressStructuredResponse>;
  getSearchAddressStructured(countryCode: string, callback: msRest.ServiceCallback<Models.SearchAddressStructuredResponse>): void;
  getSearchAddressStructured(countryCode: string, options: Models.SearchGetSearchAddressStructuredOptionalParams, callback: msRest.ServiceCallback<Models.SearchAddressStructuredResponse>): void;
  getSearchAddressStructured(countryCode: string, options?: Models.SearchGetSearchAddressStructuredOptionalParams, callback?: msRest.ServiceCallback<Models.SearchAddressStructuredResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchAddressStructuredResponse>;
    if (!callback) {
      return this.getSearchAddressStructuredWithHttpOperationResponse(countryCode, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchAddressStructuredResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.getSearchAddressStructuredWithHttpOperationResponse(countryCode, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchAddressStructuredResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * The Search Geometry endpoint allows you to perform a free form search inside a single geometry
   * or many of them. The search results that fall inside the geometry/geometries will be
   * returned.<br><br>To send the geometry you will use a `POST` request where the request body will
   * contain the `geometry` object represented as a `GeoJSON` type and the `Content-Type` header will
   * be set to `application/json`. The geographical features to be searched can be modeled as Polygon
   * and/or Circle geometries represented using any one of the following `GeoJSON`
   * types:<ul><li>**GeoJSON FeatureCollection** <br>The `geometry` can be represented as a `GeoJSON
   * FeatureCollection` object. This is the recommended option if the geometry contains both Polygons
   * and Circles. The `FeatureCollection` can contain a max of 50 `GeoJSON Feature` objects. Each
   * `Feature` object should represent either a Polygon or a Circle with the following conditions:<ul
   * style="list-style-type:none"><li>A `Feature` object for the Polygon geometry can have a max of
   * 50 coordinates and it's properties must be empty.</li><li>A `Feature` object for the Circle
   * geometry is composed of a _center_ represented using a `GeoJSON Point` type and a _radius_ value
   * (in meters) which must be specified in the object's properties along with the _subType_ property
   * whose value should be 'Circle'.</li></ul><br> Please see the Examples section below for a sample
   * `FeatureCollection` representation.<br><br></li><li>**GeoJSON GeometryCollection**<br>The
   * `geometry` can be represented as a `GeoJSON GeometryCollection` object. This is the recommended
   * option if the geometry contains a list of Polygons only. The `GeometryCollection` can contain a
   * max of 50 `GeoJSON Polygon` objects. Each `Polygon` object can have a max of 50 coordinates.
   * Please see the Examples section below for a sample `GeometryCollection`
   * representation.<br><br></li><li>**GeoJSON Polygon**<br>The `geometry` can be represented as a
   * `GeoJSON Polygon` object. This is the recommended option if the geometry contains a single
   * Polygon. The `Polygon` object can have a max of 50 coordinates. Please see the Examples section
   * below for a sample `Polygon` representation.<br><br></li></ul>.<br><br>
   *
   * @param {string} query The applicable query string (e.g., "burger joints", "pizza"). Can _also_
   * be specified as a comma separated string composed by latitude followed by longitude (e.g.,
   * "47.641268, -122.125679"). Must be properly URL encoded.
   *
   * @param {SearchInsideGeometryRequestBody} searchInsideGeometryRequestBody This represents the
   * geometry for one or more geographical features (parks, state boundary etc.) to search in and
   * should be a GeoJSON compliant type. Please refer to [RFC
   * 7946](https://tools.ietf.org/html/rfc7946) for details.
   *
   * @param {SearchPostSearchInsideGeometryOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchGeometryResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchGeometryResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postSearchInsideGeometry(query: string, searchInsideGeometryRequestBody: Models.SearchInsideGeometryRequestBody): Promise<Models.SearchGeometryResponse>;
  postSearchInsideGeometry(query: string, searchInsideGeometryRequestBody: Models.SearchInsideGeometryRequestBody, options: Models.SearchPostSearchInsideGeometryOptionalParams): Promise<Models.SearchGeometryResponse>;
  postSearchInsideGeometry(query: string, searchInsideGeometryRequestBody: Models.SearchInsideGeometryRequestBody, callback: msRest.ServiceCallback<Models.SearchGeometryResponse>): void;
  postSearchInsideGeometry(query: string, searchInsideGeometryRequestBody: Models.SearchInsideGeometryRequestBody, options: Models.SearchPostSearchInsideGeometryOptionalParams, callback: msRest.ServiceCallback<Models.SearchGeometryResponse>): void;
  postSearchInsideGeometry(query: string, searchInsideGeometryRequestBody: Models.SearchInsideGeometryRequestBody, options?: Models.SearchPostSearchInsideGeometryOptionalParams, callback?: msRest.ServiceCallback<Models.SearchGeometryResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchGeometryResponse>;
    if (!callback) {
      return this.postSearchInsideGeometryWithHttpOperationResponse(query, searchInsideGeometryRequestBody, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchGeometryResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postSearchInsideGeometryWithHttpOperationResponse(query, searchInsideGeometryRequestBody, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchGeometryResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * The Search Along Route endpoint allows you to perform a fuzzy search for POIs along a specified
   * route. This search is constrained by specifying the `maxDetourTime` limiting measure.<br><br>To
   * send the route-points you will use a `POST` request where the request body will contain the
   * `route` object represented as a `GeoJSON LineString` type and the `Content-Type` header will be
   * set to `application/json`. Each route-point in `route` is represented as a `GeoJSON Position`
   * type i.e. an array where the _longitude_ value is followed by the _latitude_ value and the
   * _altitude_ value is ignored. The `route` should contain at least 2 route-points.<br><br>It is
   * possible that original route will be altered, some of it's points may be skipped. If the route
   * that passes through the found point is faster than the original one, the `detourTime` value in
   * the response is negative.
   *
   * @param {string} query The applicable query string (e.g., "burger joints", "pizza"). Can _also_
   * be specified as a comma separated string composed by latitude followed by longitude (e.g.,
   * "47.641268, -122.125679"). Must be properly URL encoded.
   *
   * @param {number} maxDetourTime Maximum detour time of the point of interest in seconds. Max value
   * is 3600 seconds
   *
   * @param {SearchAlongRouteRequestBody} searchAlongRouteRequestBody This represents the route to
   * search along and should be a valid `GeoJSON LineString` type. Please refer to [RFC
   * 7946](https://tools.ietf.org/html/rfc7946#section-3.1.4) for details.
   *
   * @param {SearchPostSearchAlongRouteOptionalParams} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.SearchAlongRouteResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.SearchAlongRouteResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postSearchAlongRoute(query: string, maxDetourTime: number, searchAlongRouteRequestBody: Models.SearchAlongRouteRequestBody): Promise<Models.SearchAlongRouteResponse>;
  postSearchAlongRoute(query: string, maxDetourTime: number, searchAlongRouteRequestBody: Models.SearchAlongRouteRequestBody, options: Models.SearchPostSearchAlongRouteOptionalParams): Promise<Models.SearchAlongRouteResponse>;
  postSearchAlongRoute(query: string, maxDetourTime: number, searchAlongRouteRequestBody: Models.SearchAlongRouteRequestBody, callback: msRest.ServiceCallback<Models.SearchAlongRouteResponse>): void;
  postSearchAlongRoute(query: string, maxDetourTime: number, searchAlongRouteRequestBody: Models.SearchAlongRouteRequestBody, options: Models.SearchPostSearchAlongRouteOptionalParams, callback: msRest.ServiceCallback<Models.SearchAlongRouteResponse>): void;
  postSearchAlongRoute(query: string, maxDetourTime: number, searchAlongRouteRequestBody: Models.SearchAlongRouteRequestBody, options?: Models.SearchPostSearchAlongRouteOptionalParams, callback?: msRest.ServiceCallback<Models.SearchAlongRouteResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.SearchAlongRouteResponse>;
    if (!callback) {
      return this.postSearchAlongRouteWithHttpOperationResponse(query, maxDetourTime, searchAlongRouteRequestBody, options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.SearchAlongRouteResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postSearchAlongRouteWithHttpOperationResponse(query, maxDetourTime, searchAlongRouteRequestBody, options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.SearchAlongRouteResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Search Fuzzy Batch API**<br><br> The Search Fuzzy Batch API allows the caller to batch up to
   * 10000 [Search Fuzzy
   * API](https://docs.microsoft.com/en-us/rest/api/maps/searchservice/getsearchfuzzy)
   * queries/requests using just a single api call. To send the _search fuzzy_ queries you will use a
   * `POST` request where the request body will contain the `queries` array in `JSON` format and the
   * `Content-Type` header will be set to `application/json`.<br><br>A _search fuzzy_ query in a
   * batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Search Fuzzy Batch API call itself.
   * The string values in the _search fuzzy_ query must be properly escaped (e.g. " character should
   * be escaped with \\) and it should also be properly URL-encoded.<br><br> for example:
   * "?_query_=atm&_lat_=47.639769&_lon_=-122.128362&_radius_=5000&_limit_=5"<br><br> The maximum
   * number of queries that can be specified in the batch is 10000  and the batch should contain at
   * least 1 query.<br><br>Please note that the Search Fuzzy Batch API is a long-running request.
   * When the client `POST`'s the initial batch request, the server responds immediately with a `202
   * Accepted` response with the `Location` header containing the URL to download the results of the
   * batch request later. To download the batch results you will issue a `GET` request to this
   * download endpoint. If the batch processing has finished, this call returns a `200 OK` response
   * with all the batch results, else it continues to return a `202 Accepted` response. Please, be
   * aware that batch results are available for download for 14 days, after which the request for
   * results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.PostSearchFuzzyBatchAcceptedResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.PostSearchFuzzyBatchAcceptedResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postSearchFuzzyBatch(): Promise<Models.PostSearchFuzzyBatchAcceptedResponse>;
  postSearchFuzzyBatch(options: msRest.RequestOptionsBase): Promise<Models.PostSearchFuzzyBatchAcceptedResponse>;
  postSearchFuzzyBatch(callback: msRest.ServiceCallback<Models.PostSearchFuzzyBatchAcceptedResponse>): void;
  postSearchFuzzyBatch(options: msRest.RequestOptionsBase, callback: msRest.ServiceCallback<Models.PostSearchFuzzyBatchAcceptedResponse>): void;
  postSearchFuzzyBatch(options?: msRest.RequestOptionsBase, callback?: msRest.ServiceCallback<Models.PostSearchFuzzyBatchAcceptedResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.PostSearchFuzzyBatchAcceptedResponse>;
    if (!callback) {
      return this.postSearchFuzzyBatchWithHttpOperationResponse(options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.PostSearchFuzzyBatchAcceptedResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postSearchFuzzyBatchWithHttpOperationResponse(options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.PostSearchFuzzyBatchAcceptedResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Address Geocoding Batch API**<br><br> The Address Geocoding Batch API allows the caller to
   * batch up to 10000 [Address Geocoding
   * API](https://docs.microsoft.com/en-us/rest/api/maps/searchservice/getsearchaddress)
   * queries/requests using just a single api call. To send the _address geocoding_ queries you will
   * use a `POST` request where the request body will contain the `queries` array in `JSON` format
   * and the `Content-Type` header will be set to `application/json`.<br><br>An _address geocoding_
   * query in a batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Address Geocoding Batch API call
   * itself. The string values in the _address geocoding_ query must be properly escaped (e.g. "
   * character should be escaped with \\) and it should also be properly URL-encoded.<br><br> for
   * example: "?_query_=Pike Pl, Seattle, WA
   * 98101&_lat_=47.610970&_lon_=-122.342469&_radius_=1000&_limit_=3"<br><br> The maximum number of
   * queries that can be specified in the batch is 10000  and the batch should contain at least 1
   * query.<br><br>Please note that the Address Geocoding Batch API is a long-running request. When
   * the client `POST`'s the initial batch request, the server responds immediately with a `202
   * Accepted` response with the `Location` header containing the URL to download the results of the
   * batch request later. To download the batch results you will issue a `GET` request to this
   * download endpoint. If the batch processing has finished, this call returns a `200 OK` response
   * with all the batch results, else it continues to return a `202 Accepted` response. Please, be
   * aware that batch results are available for download for 14 days, after which the request for
   * results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.PostSearchAddressBatchAcceptedResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.PostSearchAddressBatchAcceptedResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postSearchAddressBatch(): Promise<Models.PostSearchAddressBatchAcceptedResponse>;
  postSearchAddressBatch(options: msRest.RequestOptionsBase): Promise<Models.PostSearchAddressBatchAcceptedResponse>;
  postSearchAddressBatch(callback: msRest.ServiceCallback<Models.PostSearchAddressBatchAcceptedResponse>): void;
  postSearchAddressBatch(options: msRest.RequestOptionsBase, callback: msRest.ServiceCallback<Models.PostSearchAddressBatchAcceptedResponse>): void;
  postSearchAddressBatch(options?: msRest.RequestOptionsBase, callback?: msRest.ServiceCallback<Models.PostSearchAddressBatchAcceptedResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.PostSearchAddressBatchAcceptedResponse>;
    if (!callback) {
      return this.postSearchAddressBatchWithHttpOperationResponse(options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.PostSearchAddressBatchAcceptedResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postSearchAddressBatchWithHttpOperationResponse(options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.PostSearchAddressBatchAcceptedResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

  /**
   * **Reverse Geocoding Batch API**<br><br> The Reverse Geocoding Batch API allows the caller to
   * batch up to 10000 [Reverse Geocoding
   * API](https://docs.microsoft.com/en-us/rest/api/maps/searchservice/getsearchaddressreverse)
   * queries/requests using just a single api call. To send the _reverse geocoding_ queries you will
   * use a `POST` request where the request body will contain the `queries` array in `JSON` format
   * and the `Content-Type` header will be set to `application/json`.<br><br>An _reverse geocoding_
   * query in a batch is just a partial URL without the protocol, base URL, path, api-version and
   * subscription-key. All these values will be reused from the Reverse Geocoding Batch API call
   * itself. The string values in the _reverse geocoding_ query must be properly escaped (e.g. "
   * character should be escaped with \\) and it should also be properly URL-encoded.<br><br> for
   * example: "?_query_=47.639765,-122.127896&_radius_=5000&_limit_=2"<br><br> The maximum number of
   * queries that can be specified in the batch is 10000  and the batch should contain at least 1
   * query.<br><br>Please note that the Reverse Geocoding Batch API is a long-running request. When
   * the client `POST`'s the initial batch request, the server responds immediately with a `202
   * Accepted` response with the `Location` header containing the URL to download the results of the
   * batch request later. To download the batch results you will issue a `GET` request to this
   * download endpoint. If the batch processing has finished, this call returns a `200 OK` response
   * with all the batch results, else it continues to return a `202 Accepted` response. Please, be
   * aware that batch results are available for download for 14 days, after which the request for
   * results download will return `400 Bad Request` response.
   *
   * @param {RequestOptionsBase} [options] Optional Parameters.
   *
   * @param {ServiceCallback} callback The callback.
   *
   * @returns {ServiceCallback} callback(err, result, request, operationRes)
   *                      {Error|ServiceError}  err        - The Error object if an error occurred, null otherwise.
   *                      {Models.PostSearchAddressReverseBatchAcceptedResponse} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link Models.PostSearchAddressReverseBatchAcceptedResponse} for more information.
   *                      {WebResource} [request]  - The HTTP Request object if an error did not occur.
   *                      {HttpOperationResponse} [response] - The HTTP Response stream if an error did not occur.
   */
  postSearchAddressReverseBatch(): Promise<Models.PostSearchAddressReverseBatchAcceptedResponse>;
  postSearchAddressReverseBatch(options: msRest.RequestOptionsBase): Promise<Models.PostSearchAddressReverseBatchAcceptedResponse>;
  postSearchAddressReverseBatch(callback: msRest.ServiceCallback<Models.PostSearchAddressReverseBatchAcceptedResponse>): void;
  postSearchAddressReverseBatch(options: msRest.RequestOptionsBase, callback: msRest.ServiceCallback<Models.PostSearchAddressReverseBatchAcceptedResponse>): void;
  postSearchAddressReverseBatch(options?: msRest.RequestOptionsBase, callback?: msRest.ServiceCallback<Models.PostSearchAddressReverseBatchAcceptedResponse>): any {
    if (!callback && typeof options === 'function') {
      callback = options;
      options = undefined;
    }
    let cb = callback as msRest.ServiceCallback<Models.PostSearchAddressReverseBatchAcceptedResponse>;
    if (!callback) {
      return this.postSearchAddressReverseBatchWithHttpOperationResponse(options).then((operationRes: msRest.HttpOperationResponse) => {
        return Promise.resolve(operationRes.parsedBody as Models.PostSearchAddressReverseBatchAcceptedResponse);
      }).catch((err: Error) => {
        return Promise.reject(err);
      });
    } else {
      msRest.promiseToCallback(this.postSearchAddressReverseBatchWithHttpOperationResponse(options))((err: Error, data: msRest.HttpOperationResponse) => {
        if (err) {
          return cb(err);
        }
        let result = data.parsedBody as Models.PostSearchAddressReverseBatchAcceptedResponse;
        return cb(err, result, data.request, data);
      });
    }
  }

}
