import { MapsModels } from "../../lib/autorest/src/mapsClient";

/**
 * 
 */
export class GeoJsonRouteDirectionsResponse {
    private _response: MapsModels.RouteDirectionsResponse;
    private _geoJsonRoutes: GeoJSON.Feature[] = [];

    constructor(response: MapsModels.RouteDirectionsResponse) {
        this._response = response;
    }

    /**
     * 
     */
    public getGeoJsonRoutes(): GeoJSON.Feature[] {
        if (this._geoJsonRoutes && this._geoJsonRoutes.length > 0) {
            return this._geoJsonRoutes;
        }

        try {
            this._geoJsonRoutes = this._extractGeoJsonRouteFeatures();
            return this._geoJsonRoutes;
        } catch (err) {
            throw Error("Couldn't extract GeoJSON data from response.");
        }
    }

    /**
     * @private
     */
    private _extractGeoJsonRouteFeatures(): GeoJSON.Feature[] {
        const routes: MapsModels.RouteDirectionsResult[] = this._response.routes;

        return routes.map((route: MapsModels.RouteDirectionsResult) => {
            const multiLinestringCoords = route.legs.map((leg: MapsModels.RouteResultLeg) => {
                const linestringCoords = leg.points.map((coord: MapsModels.Coordinate) => {
                    return [coord.longitude, coord.latitude];
                });

                return linestringCoords;
            });

            return {
                type: "Feature",
                geometry: {
                    type: "MultiLineString",
                    coordinates: multiLinestringCoords
                },
                properties: {
                    summary: route.summary,
                    legSummaries: route.legs.map(l => l.summary),
                    sections: route.sections,
                    guidance: route.guidance
                }
            } as GeoJSON.Feature;
        });
    }

}

