import { MapsModels } from "../../lib/autorest/src/mapsClient";

/**
 * 
 */
export class GeoJsonRouteRangeResponse {
    private _response: MapsModels.RouteRangeResponse;
    private _geoJsonRanges: GeoJSON.Feature[] = [];

    constructor(response: MapsModels.RouteRangeResponse) {
        this._response = response;
    }

    /**
     * 
     */
    public getGeoJsonRanges(): GeoJSON.Feature[] {
        if (this._geoJsonRanges && this._geoJsonRanges.length > 0) {
            return this._geoJsonRanges;
        }

        try {
            this._geoJsonRanges = this._extractGeoJsonFeatures();
            return this._geoJsonRanges;
        } catch (err) {
            throw Error("Couldn't extract GeoJSON data from response.");
        }
    }

    /**
     * @private
     */
    private _extractGeoJsonFeatures(): GeoJSON.Feature[] {
        const range: MapsModels.RouteRangeResponseReachableRange = this._response.reachableRange;

        const outsideLinearRing = range.boundary.map((coord: MapsModels.Coordinate) => {
            return [coord.longitude, coord.latitude]
        });

        if (outsideLinearRing[0] !== outsideLinearRing[outsideLinearRing.length - 1]) {
            outsideLinearRing.push(outsideLinearRing[0]);
        }

        return [
            {
                type: "Feature",
                geometry: {
                    type: "Polygon",
                    coordinates: [outsideLinearRing]
                },
                properties: {
                    center: range.center
                }
            } as GeoJSON.Feature
        ];
    }

}

