import { MapsModels } from "../../lib/autorest/src/mapsClient";

type SearchResponse = MapsModels.SearchAddressResponse | MapsModels.SearchAddressStructuredResponse |
    MapsModels.SearchAlongRouteResponse | MapsModels.SearchFuzzyResponse | MapsModels.SearchGeometryResponse |
    MapsModels.SearchNearbyResponse | MapsModels.SearchPoiCategoryResponse | MapsModels.SearchPoiResponse;

type SearchResult = MapsModels.SearchAddressResult | MapsModels.SearchAddressStructuredResult |
    MapsModels.SearchAlongRouteResult | MapsModels.SearchFuzzyResult | MapsModels.SearchGeometryResult |
    MapsModels.SearchNearbyResult | MapsModels.SearchPoiCategoryResult | MapsModels.SearchPoiResult;

/**
 * 
 */
export class GeoJsonSearchResponse {
    private _response: SearchResponse;
    private _geoJsonResults: GeoJSON.Feature[] = [];

    constructor(response: SearchResponse) {
        this._response = response;
    }

    /**
     * 
     */
    public getGeoJsonResults(): GeoJSON.Feature[] {
        if (this._geoJsonResults && this._geoJsonResults.length > 0) {
            return this._geoJsonResults;
        }

        try {
            this._geoJsonResults = this._extractGeoJsonFeatures();
            return this._geoJsonResults;
        } catch (err) {
            throw Error("Couldn't extract GeoJSON data from response.");
        }
    }

    /**
     * @private
     */
    private _extractGeoJsonFeatures(): GeoJSON.Feature[] {
        const results: SearchResult[] = this._response.results;

        return results.map((result: SearchResult) => {
            return {
                ...(result.id && { id: result.id }),
                ...(result.viewport && {
                    bbox: [
                        result.viewport.topLeftPoint.lon,
                        result.viewport.btmRightPoint.lat,
                        result.viewport.btmRightPoint.lon,
                        result.viewport.topLeftPoint.lat
                    ]
                }),
                type: "Feature",
                geometry: {
                    type: "Point",
                    coordinates: [result.position.lon, result.position.lat]
                },
                properties: result
            } as GeoJSON.Feature;
        });
    }

}

