import { MapsModels } from "../../lib/autorest/src/mapsClient";

type SearchReverseResponse = MapsModels.SearchAddressReverseCrossStreetResponse |
    MapsModels.SearchAddressReverseResponse;

type SearchReverseResult = MapsModels.SearchAddressReverseCrossStreetResult |
    MapsModels.SearchAddressReverseResult;

/**
 * 
 */
export class GeoJsonSearchReverseResponse {
    private _response: SearchReverseResponse;
    private _geoJsonAddresses: GeoJSON.Feature[] = [];

    constructor(response: SearchReverseResponse) {
        this._response = response;
    }

    /**
     * 
     */
    public getGeoJsonAddresses(): GeoJSON.Feature[] {
        if (this._geoJsonAddresses && this._geoJsonAddresses.length > 0) {
            return this._geoJsonAddresses;
        }

        try {
            this._geoJsonAddresses = this._extractGeoJsonFeatures();
            return this._geoJsonAddresses;
        } catch (err) {
            throw Error("Couldn't extract GeoJSON data from response.");
        }
    }

    /**
     * @private
     */
    private _extractGeoJsonFeatures(): GeoJSON.Feature[] {
        const addresses: SearchReverseResult[] = this._response.addresses;

        return addresses.map((result: SearchReverseResult) => {

            const [resultLat, resultLon] = result.position.split(",").map(coord => {
                return Number.parseFloat(coord)
            });

            return {
                type: "Feature",
                geometry: {
                    type: "Point",
                    coordinates: [resultLon, resultLat]
                },
                properties: result
            } as GeoJSON.Feature;
        });
    }

}

