﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Common
{
    using System.IO;

    /// <summary>
    /// The block info.
    /// </summary>
    public class BlockInfo
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="BlockInfo"/> class.
        /// </summary>
        /// <param name="blockId">
        /// The block id.
        /// </param>
        /// <param name="startOffset">
        /// The start offset.
        /// </param>
        /// <param name="length">
        /// The length.
        /// </param>
        public BlockInfo(int blockId, long startOffset, int length)
        {
            this.StartOffset = startOffset;
            this.Length = length;
            this.BlockId =
                blockId; // Convert.ToBase64String(ASCIIEncoding.ASCII.GetBytes($"BlockId:{startOffset.ToString("d7")}"));
        }

        /// <summary>
        /// Gets the block id.
        /// </summary>
        public long BlockId { get; private set; }

        /// <summary>
        /// Gets the start offset.
        /// </summary>
        public long StartOffset { get; private set; }

        /// <summary>
        /// Gets the length.
        /// </summary>
        public int Length { get; private set; }

        /// <summary>
        /// Gets or sets the block bytes.
        /// </summary>
        public byte[] BlockBuffer { get; set; }
    }
}
