﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Common
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Threading.Tasks;

    /// <summary>
    /// Supports random block reading from the source.
    /// </summary>
    public interface IRandomBlockReader
    {
        /// <summary>
        /// Gets the max length of block allowed.
        /// </summary>
        int MaxLengthOfBlockAllowed { get; }

        /// <summary>
        /// Gets the preferred length of block.
        /// </summary>
        int? PreferredLengthOfBlock { get; }

        /// <summary>
        /// Gets the length of the stream.
        /// </summary>
        long LengthOfTheStream { get; }

        /// <summary>
        /// Gets the last modified timestamp of the stream.
        /// </summary>
        DateTimeOffset LastModifieDateTime { get; }

        /// <summary>
        /// Gets the md5 hash of source.
        /// </summary>
        string Md5HashOfSource { get; }

        /// <summary>
        /// Gets the metadata map.
        /// </summary>
        Dictionary<string, string> MetadataMap { get; }

        /// <summary>
        /// Gets the max degree of parallelism allowed.
        /// </summary>
        int MaxDegreeOfParallelismAllowed { get; }

        /// <summary>
        /// The get bytes from offset.
        /// </summary>
        /// <param name="offset">
        /// The offset.
        /// </param>
        /// <param name="lengthOfStream">
        /// The number of bytes.
        /// </param>
        /// <returns>
        /// The <see cref="Task"/>.
        /// </returns>
        Task<byte[]> GetBufferFromOffset(long offset, int lengthOfStream);

        /// <summary>
        /// Initializes the required objects.
        /// </summary>
        /// <returns></returns>
        Task InitializeAsync();
    }
}