﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Common
{
    using System;
    using System.Diagnostics.Tracing;

    public enum LogType
    {
        /// <summary>Log an error</summary>
        Error = 0,

        /// <summary>Log information</summary>
        Info,
        
        /// <summary>Show a message in the Debug output window</summary>
        Verbose,
        
        /// <summary>Log a warning</summary>
        Warning
    }

    [EventData]
    public class LogEventArgs : EventArgs
    {
        public LogEventArgs(LogType t, string src, string txt)
        {
            if (string.IsNullOrEmpty(src))
            {
                throw new InvalidOperationException("LogEvent Source property cannot be null or empty");
            }

            if (string.IsNullOrEmpty(txt))
            {
                throw new InvalidOperationException("LogEvent Text property cannot be null or empty");
            }

            UtcStamp = DateTime.UtcNow;
            LogTypeId = t;
            Source = src;
            Message = txt;
        }

        public DateTime UtcStamp { get; private set; }

        public LogType LogTypeId { get; private set; }

        public string LogTypeDesc => LogTypeId.ToString();

        public string Source { get; private set; }

        public string Message { get; private set; }

        public override string ToString()
        {
            return $"{UtcStamp.TimeOfDay}\t{LogTypeDesc}\t{Source}\t{Message}";
        }
    }
}
