﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Common
{
    using System;
    using System.Diagnostics;

    public class Logger : ILogger
    {
        public readonly string Source;

        public Logger(string source)
        {
            Source = source;
        }

        public event EventHandler<LogEventArgs> LogEvent;

        public void LogIf(bool condition, LogType t, string msgFmt, params object[] args)
        {
            if (condition)
            {
                Log(t, msgFmt, args);
            }
        }

        public void Log(LogType t, string msgFmt, params object[] args)
        {
            var txt = string.Format(msgFmt, args);
            Debug.WriteLine($"{GetTypedStamp(t)} - {txt}");

            switch (t)
            {
                case LogType.Verbose:
                    break;
                case LogType.Info:
                case LogType.Warning:
                case LogType.Error:
                    LogEvent?.Invoke(this, new Common.LogEventArgs(t, Source, txt));
                    break;
                default:
                    throw new InvalidOperationException($"Logger internal error: unhandled LogType={t} in switch statement");
            }
        }

        private static string GetTypedStamp(LogType t)
        {
            DateTime dt = DateTime.Now;
            var ts = $"{dt.Hour.ToString("00")}:{dt.Minute.ToString("00")}:{dt.Second.ToString("00")}.{dt.Millisecond.ToString("000")}";
            string tc = null;
            switch (t)
            {
                case LogType.Error: tc = "E"; break;
                case LogType.Info: tc = "I"; break;
                case LogType.Verbose: tc = "V"; break;
                case LogType.Warning: tc = "W"; break;
                default: throw new ArgumentException($"Unhandled LogType={t}");
            }

            return $"{ts} [{tc}]";
        }
    }
}
