﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.Common.UnitTests
{
    using System;
    using System.Diagnostics.CodeAnalysis;
    using System.Threading.Tasks;
    using Microsoft.VisualStudio.TestTools.UnitTesting;
    
    [SuppressMessage("Microsoft.Design", "CA1001:TypesThatOwnDisposableFieldsShouldBeDisposable", Justification = "Using UnitTest framework for Dispose")]
    [TestClass]
    public class MemoryConfigurationCacheUnitTests
    {
        IConfigurationCache _cache;

        [TestInitialize]
        public void Initialize()
        {
            _cache = new MemoryConfigurationCache("UnitTest");
        }

        [TestCleanup]
        public void Cleanup()
        {
            _cache.Dispose();
        }

        [TestMethod, TestCategory("L1")]
        public async Task ResolveSettingAsync_RetrieveValue()
        {
            string cacheValueExpected = "mycachevalue";
            string cacheValueKey = "myKey";
            var settingFactory = new Lazy<Task<string>>(
                () => Task.FromResult(cacheValueExpected));

            string actualResult = await _cache.ResolveSettingAsync(cacheValueKey, settingFactory, null);

            Assert.AreEqual(cacheValueExpected, actualResult);
        }

        [TestMethod, TestCategory("L1")]
        public async Task ResolveSettingAsync_SetExpiration_RetrieveValue()
        {
            string cacheValueExpected = "mycachevalue";
            string cacheValueKey = "myKey";
            var settingFactory = new Lazy<Task<string>>(
                () => Task.FromResult(cacheValueExpected));

            string actualResult = await _cache.ResolveSettingAsync(cacheValueKey, settingFactory, TimeSpan.FromTicks(1));        
            Assert.AreEqual(cacheValueExpected, actualResult);            
        }

        [TestMethod, TestCategory("L1"), ExpectedException(typeof(InvalidOperationException))]
        public async Task ResolveSettingAsync_FactoryException_Thrown()
        {            
            string cacheValueKey = "myKey";
            var settingFactory = new Lazy<Task<string>>(
                () => throw new InvalidOperationException("Unit Testing"));

            await _cache.ResolveSettingAsync(cacheValueKey, settingFactory, null);
        }
    }
}
