﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.Utility
{
    using System.Net.Http;
    using System.Web;

    using Microsoft.Owin;

    /// <summary>
    /// Implements various extensions on top of HttpRequestMessage
    /// </summary>
    public static class HttpRequestExtensions
    {
        /// <summary>
        /// Obtains client IP address from request context.
        /// </summary>
        public static string GetClientIpAddress(this HttpRequestMessage request)
        {
            // Self-hosting using Owin
            if (request.GetOwinContext() is IOwinContext owinContext)
            {
                return owinContext.Request.RemoteIpAddress;
            }

            const string HttpContextPropertyName = "MS_HttpContext";

            // Web-hosting
            if (request.Properties.ContainsKey(HttpContextPropertyName))
            {
                if (request.Properties[HttpContextPropertyName] is HttpContextWrapper httpContext)
                {
                    return httpContext.Request.UserHostAddress;
                }
            }

            return null;
        }
    }
}