﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ServiceFabricHosting
{
    using System;
    using System.Diagnostics;
    using System.Fabric;
    using System.Threading;

    using Microsoft.ServiceFabric.Services.Runtime;

    public abstract class StatelessServiceRuntimeBase<TService> where TService : StatelessService
    {
        /// <summary>
        /// This is the entry point of the service host process.
        /// </summary>
        protected void Run(string serviceTypeName)
        {
            try
            {
                // The ServiceManifest.XML file defines one or more service type names.
                // Registering a service maps a service type name to a .NET type.
                // When Service Fabric creates an instance of this service type,
                // an instance of the class is created in this host process.
                ServiceRuntime.RegisterServiceAsync(serviceTypeName, this.CreateStatelessSerivceInstance)
                              .GetAwaiter()
                              .GetResult();

                Instrumentation.Agent.Instance.LogServiceTypeRegistered(Process.GetCurrentProcess().Id, typeof(TService).Name);

                // Prevents this host process from terminating so services keeps running. 
                Thread.Sleep(Timeout.Infinite);
            }
            catch (Exception ex)
            {
                Instrumentation.Agent.Instance.LogException(ex, "common", "initialization");
                throw;
            }
        }

        protected abstract TService CreateStatelessSerivceInstance(StatelessServiceContext context);

        /// <summary>
        /// Initializes the monitoring agent.
        /// </summary>
        protected void InitializeMonitoringAgent(string sessionName, string accountName, bool enableLocalOutput)
        {
            var ifxParams = new Instrumentation.InitParams(sessionName, accountName);

            if (enableLocalOutput)
            {
                ifxParams.SetLocalTracing(@"c:\ifxTrace");
                ifxParams.SetEtwProvider("DevTest", "DB25DB83-F4C7-4D76-9DBA-5D08A7DEC472");
            }

            Instrumentation.Agent.CreateInstance(ifxParams);
        }        
    }
}