﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// 

namespace Microsoft.Azure.LocationServices.ServiceFabricHosting.UnitTests
{
    using System;
    using Microsoft.Azure.LocationServices.ServiceFabricHosting;
    using Microsoft.VisualStudio.TestTools.UnitTesting;    

    internal enum TestEnum
    {
        Value1,
        Value2
    }

    [TestClass]
    public class ConfigurationReaderTests
    {
        [TestMethod]
        public void CanParsePrimitives()
        {
            Instrumentation.Agent.CreateInstance(new Instrumentation.InitParams("test", "test"));

            Assert.AreEqual("abc", ConfigurationReader.Parse("abc", typeof(string)));
            Assert.AreEqual(1, ConfigurationReader.Parse("1", typeof(int)));
            Assert.IsNull(ConfigurationReader.Parse("1a", typeof(int)));

            Assert.AreEqual(1, ConfigurationReader.Parse("1", typeof(int?)));
            Assert.IsNull(ConfigurationReader.Parse("1a", typeof(int?)));
            Assert.IsNull((int?)ConfigurationReader.Parse("", typeof(int?)));

            Assert.IsTrue((bool)ConfigurationReader.Parse("true", typeof(bool)));
            Assert.IsFalse((bool)ConfigurationReader.Parse("false", typeof(bool)));
            Assert.IsNull(ConfigurationReader.Parse("1", typeof(bool)));

            Assert.AreEqual(1.2, ConfigurationReader.Parse("1.2", typeof(double)));
            Assert.IsNull(ConfigurationReader.Parse("a.b", typeof(double)));
            Assert.AreEqual(new Uri("http://tempuri.org"), ConfigurationReader.Parse("http://tempuri.org", typeof(Uri)));

            Assert.AreEqual(TestEnum.Value2, ConfigurationReader.Parse("Value2", typeof(TestEnum)));
            Assert.IsNull(ConfigurationReader.Parse("Value3", typeof(TestEnum)));
        }        
    }
}
