# Working with the Specification Files

## Prerequisites
1. Install [node.js](https://nodejs.org/en/).  
 
   **Note:** If already installed, update your node and npm versions by running:

   ```
   npm install -g npm
   ```

   ```
   npm install -g node
   ```

2. Run npm install on package before building a swagger file.  From *src/Specifications* run:

   ```
   npm install
   ```

3. Join the necessary organizations on Github through this [site](https://repos.opensource.microsoft.com/):
    1. [Microsoft](https://repos.opensource.microsoft.com/Microsoft)
    2. [Azure](https://repos.opensource.microsoft.com/Azure)
    3. [MicrosoftDocs](https://repos.opensource.microsoft.com/MicrosoftDocs)
    4. [Azure-Samples](https://repos.opensource.microsoft.com/Azure-Samples)

4. Request to be added to the [Azure/Azure-Maps-Admins](https://github.com/orgs/Azure/teams/azure-maps-admins) team.
   This grants you read/write permissions for the [Azure-REST-API-Specs-PR](https://github.com/Azure/azure-rest-api-specs-pr) repo.

5. Request to be added to the [MicrosoftDocs/Azure-Maps](https://github.com/orgs/MicrosoftDocs/teams/azure-maps) team.

## Best Practices When Working with the Specifications


`"Leave this world [the specification files] a little better than you found it [them]." - Robert Baden-Powell`

1. [Don't Repeat Yourself](https://en.wikipedia.org/wiki/Don%27t_repeat_yourself) - When adding to a specification
   try to reduce duplication.  Instead of writing the same parameter multiple times in different operations, add
   it once to the top parameters section and reference it throughout the operations.  If the parameter can be used 
   across multiple service specification files, add it to the common.yml file.  This is true for definitions,
   responses and other parts of the specifications as well.

2. Take advantage of the fact that we can easily use Markdown in YML.

   Instead of this:

   ```
   <strong>Get POI by Category</strong><br><br>If your search use case only requires POI results filtered by
   category, you may use the category endpoint. This endpoint will only return POI results which are categorized 
   as specified.<br><br>List of available categories can be found [here](https://docs.microsoft.com/en-us/azure/azure-maps/search-categories).
   ```

   Try this:

   ```
   **Get POI by Category**
   
   If your search use case only requires POI results filtered by category, you may use the category endpoint. This
   endpoint will only return POI results which are categorized as specified.
   
   List of available categories can be found [here](https://docs.microsoft.com/en-us/azure/azure-maps/search-categories).
   ```

   **[Markdown Cheat Sheet](https://guides.github.com/features/mastering-markdown/)**

3. Links to other Azure Maps APIs should be done with relative links. [Example](https://opsdocs.azurewebsites.net/en-us/opsdocs/reference/cheatsheet-reference-content?branch=master#linking)

4. Naming and Organizational Rules:
    * Prepend service specific Definitions with the service moniker.  For example, instead of _Poi_ use _SearchPoi_.
      If the Definition is exclusively used within the 'results' property of responses, consider _SearchResultPoi_ as
      a better name. This will help to avoid naming collisions across our services.
    * Name the Definitions returned by an operation with the pattern __"*Response."__ For example, 
      _SearchFuzzyResponse_ is returned by a successfull call, not _SearchFuzzyResult_.
    * __Keep Definitions 1 level deep.__ Every schema of type "object" should have its own definition, or in other words
      all properties with type "object" should be referenced through an appropriately named definition.
    * Avoid naming Definitions like _SearchFuzzyResultArray_.  Usually such Definitions becomes documented as 
      _SearchFuzzyResultArray_[] where calling it _SearchFuzzyResult_ would have been cleaner.
    * If the Definition is for a body of a request, please following the naming convention __"*RequestBody."__

## Special Annotations in the Specifications

| Annotation | Type    | Description                                                             |
| :--------- | :------ | :---------------------------------------------------------------------- |
| x-publish  | boolean | Whether to publish this API when running the build with the **-p** flag |

## How to Validate a Specification

The best way to validate locally is to run the specification build.  Run the help command for instructions:

```
node build.js -h
```

**Note:** Use the **-p** flag when you want to publish the specification.  This removes all operations that are not
currently marked as published from the built specification.

## How to Validate A Doc Change

1. Clone the Azure-REST-API-Specs-PR repo locally

   ```
   git clone https://github.com/Azure/azure-rest-api-specs-pr.git
   ```

   **Note:** If you experience authentication issues make sure you are currently signed into github and are
   a member of the Microsoft and Azure organizations.  Additionally, request to be added to the [Azure-Maps-Admins](https://github.com/orgs/Azure/teams/azure-maps-admins) team.

2. Checkout the [Azure-Maps-DEV](https://github.com/Azure/azure-rest-api-specs-pr/tree/Azure-Maps-DEV) branch.

3. In the [specification](https://github.com/Azure/azure-rest-api-specs-pr/tree/Azure-Maps-DEV/specification)
   folder create your own dev folder with name maps_{user} where **{user}** is your custom identifier.

4. Copy the folder structure and files from what is currently in the specification/maps_master folder into your 
   specification/maps_{user} folder.

5. Build the API specification that you want to validate doc changes for and overwrite the one in your maps_{user}
   folder.

6. Commit and Push your changes to the Azure-REST-API-Specs-PR repo on the **Azure-Maps-DEV** branch.

7. Clone the Staging-Docs-REST-APIs repo locally:

   ```
   git clone https://github.com/MicrosoftDocs/staging-docs-rest-apis.git
   ```

8. Checkout the [lbs](https://github.com/MicrosoftDocs/staging-docs-rest-apis/tree/lbs) branch.

9. Create your own staging documentation section by adding to the [mapping.json](https://github.com/MicrosoftDocs/staging-docs-rest-apis/blob/lbs/mapping.json)
   file of Staging-Docs-REST-APIs repo in the **lbs** branch.  Here is an example config for a user where **{user}** would
   be replaced with the identifier you chose in the previous steps:

   ```
   {
        "toc_title": "[DEV] {user}",
        "url_group": "{user}-maps",
        "swagger_files": [
            {
                "source": "azure-rest-api-specs-pr-maps/specification/maps_{user}/data-plane/Render/preview/v1/render.json"
            },
            {
                "source": "azure-rest-api-specs-pr-maps/specification/maps_{user}/data-plane/Route/preview/v1/route.json"
            },
            {
                "source": "azure-rest-api-specs-pr-maps/specification/maps_{user}/data-plane/Search/preview/v1/search.json"
            },
            {
                "source": "azure-rest-api-specs-pr-maps/specification/maps_{user}/data-plane/Timezone/preview/v1/timezone.json"
            },
            {
                "source": "azure-rest-api-specs-pr-maps/specification/maps_{user}/data-plane/Traffic/preview/v1/traffic.json"
            },
            {
                "source": "azure-rest-api-specs-pr-maps/specification/maps_{user}/data-plane/Geolocation/preview/v1/geolocation.json"
            }
        ]
    }
    ```

    Once mapping.json has been updated, commit and push your changes to the lbs branch.

10. Run the CI.  This can be done by opening up a PR from the lbs branch into the master branch.  **Do not check this PR in.**  Once the CI has successfully run, you should see your generated REST documentation [here](https://review.docs.microsoft.com/en-us/rest-staging/api/?branch=lbs).

## How to Publish REST Documentation

### Steps

1. Clone the Azure-REST-API-Specs repo locally

   ```
   git clone https://github.com/Azure/azure-rest-api-specs.git
   ```

   **Note:** If you experience authentication issues make sure you are currently signed into github and are
   a member of the Microsoft and Azure organizations.  Additionally, request to be added to the [Azure-Maps-Admins](https://github.com/orgs/Azure/teams/azure-maps-admins) team.

2. Checkout the [Azure-Maps-DEV](https://github.com/Azure/azure-rest-api-specs/tree/Azure-Maps-DEV) branch.

3. In the [maps](https://github.com/Azure/azure-rest-api-specs/tree/Azure-Maps-DEV/specification/maps) directory,
   update the specifications and associated examples with those you would like to publish docs from (the most up-to-date
   specification files).

4. Commit these changes and push to the Azure-Maps-DEV branch. Once updated, these new specifications will be read by 
   the [Azure-Docs-Rest-APIs](https://github.com/Azure/azure-docs-rest-apis) repo and used to update the documentation 
   in the following 24 hours.   

### Future Work

1. We should ideally base our documentation off of specifications in the master branch of the 
[Azure-Docs-Rest-APIs](https://github.com/Azure/azure-docs-rest-apis) repo.

## Troubleshooting

* Error: Cannot find module 'sway'
    * Make sure your node and npm installations are up to date.

* Authentication Issues for the Azure-REST-API-Specs or Azure-REST-API-Specs-PR repo.
    * Make sure you are currently signed into github and are a member of the Microsoft and Azure organizations.
    * Additionally, request to be added to the [Azure-Maps-Admins](https://github.com/orgs/Azure/teams/azure-maps-admins) 
      team

* Authentication Issues for the Staging-Docs-REST-APIs repo.
    * Request to be added to the [MicrosoftDocs/Azure-Maps](https://github.com/orgs/MicrosoftDocs/teams/azure-maps) 
      team.

* Live documentation did not update.
    * Check the [Azure-Docs-Rest-APIs](https://github.com/Azure/azure-docs-rest-apis) repo to see if a CI commit 
      occurred and the [OPS Portal](https://ops.microsoft.com/#/repos/ce92634c-462a-dc0f-0037-1cce0c1a1b03) for failed
      publishing builds.