# Azure Maps
    
> see https://aka.ms/autorest

Configuration for generating Azure Maps Proxy API SDK.

The current release is `package-1.0`.

``` yaml
title: MapsClient
openapi-type: data-plane
output-folder: ./autorest/src
```

## Command Line Tags

### Tag: package-1.0
These settings apply only when `--tag=package-1.0` is specified on the command line.

``` yaml $(tag) == 'package-1.0'
input-file: 
  - ./search/search.json
  - ./route/route.json
  - ./geolocation/geolocation.json
  - ./render/render.json
  - ./timezone/timezone.json
  - ./traffic/traffic.json
```

### Tag: sdk
These settings apply only when `--tag=sdk` is specified on the command line

``` yaml $(tag) == 'sdk'
input-file: 
  - ./search/search.json
  - ./route/route.json

typescript: true

directive:
    - from: swagger-document
      where: $..*[?(@.name === "format" && @.enum.includes('json'))]
      transform: $.enum = ['json'];
      reason: >
        Autorest historically only supported json responses. For the SDK, the over-the-wire format
        of the data isn't important.
    - from: swagger-document
      where: $.paths..responses
      transform: >
        for (var responseCode of Object.keys($)) {
          var responseCodeInt = parseInt(responseCode);
          if (responseCodeInt >= 300 || responseCodeInt < 200) { delete $[responseCode] };     
        }
        return $;
      reason: Autorest treats all responses defined for an operation as successful.
    
```

## CSharp Settings
These settings apply only when `--csharp` is specified on the command line.
``` yaml $(csharp) 
csharp: 
  license-header: MICROSOFT_MIT_NO_VERSION
  azure-arm: false
  namespace: Microsoft.Azure.Maps
  output-folder: $(csharp-sdks-folder)/Maps/Generated
  clear-output-folder: true
```

## Validation Steps
``` yaml $(validation)
azure-validator: true
semantic-validator: true
model-validator: true
```