var fs = require('fs-extra');

const argv = require("yargs")
    .usage("Usage: $0 [options]")
    .options({
        "domain": {
            alias: "d",
            describe: 'The domain to access the styles resources.',
            demandOption: true,
            string: true
        }
    })
    .help('h')
    .argv;

const publishDir = __dirname + "/publish";
const srcDir = __dirname + "/src";

// Create folder for published style resources
if (!fs.existsSync(publishDir)) {
    fs.mkdirSync(publishDir);
}

// Copy definitions to publish folder
const definitionsDirName = "definitions";
const definitionsPublishDir = publishDir + "/" + definitionsDirName;
fs.copy(srcDir + "/" + definitionsDirName, definitionsPublishDir)
    .then(() => {
        fs.readdir(definitionsPublishDir, (err, files) => {
            if (err) {
                console.error(err);
            }

            files.forEach(file => {
                fs.readJson(definitionsPublishDir + "/" + file)
                    .then(fileJson => {
                        fileJson.domain = argv.domain;
                        fs.outputJson(definitionsPublishDir + "/" + file, fileJson, {
                            spaces: 2
                        })
                            .catch(err => console.error(err));
                    })
                    .catch(err => console.error(err));
            });
        });
    })
    .catch(err => console.error(err));

// Copy glyphs to publish folder
const glyphsDirName = "glyphs";
fs.copy(srcDir + "/" + glyphsDirName, publishDir + "/" + glyphsDirName)
    .catch(err => console.error(err));

// Copy layers to publish folder
const layersDirName = "layers";
fs.copy(srcDir + "/" + layersDirName, publishDir + "/" + layersDirName)
    .catch(err => console.error(err));

// Copy thumbnails to publish folder
const thumbnailsDirName = "thumbnails";
fs.copy(srcDir + "/" + thumbnailsDirName, publishDir + "/" + thumbnailsDirName)
    .catch(err => console.error(err));

// Copy sprites to publish folder, ignore all SVGs
const spritesDirName = "sprites";
fs.copy(srcDir + "/" + spritesDirName, publishDir + "/" + spritesDirName, {
    filter: (src, dest) => {
        const svgFilePattern = /^.*\.svg$/;
        return src.match(svgFilePattern) === null;
    }
})
    .catch(err => console.error(err));