﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------
// ----------------------------------------------------------------
// Created: Feb 2017
// By:      David Stanchi (davsta@microsoft.com)
// ----------------------------------------------------------------
// Copied from GeoTimeZone under MIT License
// Copyright(c) 2014 Matt Johnson & Simon Bartlett
// https://github.com/mj1856/GeoTimeZone
// ----------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDb
{
    using GeoAPI.Geometries;
    using NetTopologySuite.Geometries;

    internal static class GeoExtensions
    {
        internal static IGeometry Simplify(this IGeometry geometry)
        {
            // Simplify the geometry.
            if (geometry.Area < 0.1)
            {
                // For very small regions, use a convex hull.
                return geometry.ConvexHull();
            }

            // Simplify the polygon.
            var tolerance = 0.05;
            while (tolerance >= 0.0)
            {
                var result = NetTopologySuite.Simplify.TopologyPreservingSimplifier.Simplify(geometry, tolerance);
                if (result is Polygon && result.IsValid && !result.IsEmpty)
                {
                    return result;
                }

                // Reduce the tolerance incrementally until we have a valid polygon.
                tolerance -= 0.005;
            }

            return geometry;
        }
    }
}
