﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDb
{
    using System;
    using System.Collections.Generic;

    public class IanaId : IEquatable<IanaId>
    {
        internal IanaId(KeyValuePair<string, string> kvp, bool zone1970)
        {
            Id = kvp.Key;
            if (!Id.Equals(kvp.Value))
            {
                AliasOf = kvp.Value;
            }

            HasZone1970Location = zone1970;
        }

        public string Id { get; set; }

        public bool IsAlias => !string.IsNullOrEmpty(AliasOf);

        public string AliasOf { get; set; }

        public bool HasZone1970Location { get; set; }

        public bool Equals(IanaId o)
        {
            if (o == null)
            {
                return false;
            }

            return Id.Equals(o.Id, StringComparison.InvariantCultureIgnoreCase);
        }

        public override string ToString()
        {
            if (!IsAlias && HasZone1970Location)
            {
                return Id;
            }

            var s = new System.Text.StringBuilder();
            s.Append("[");
            if (IsAlias)
            {
                s.Append(HasZone1970Location ? "alias" : "alias|");
            }

            if (!HasZone1970Location)
            {
                s.Append("noLoc");
            }

            s.Append($"] {Id}");
            return s.ToString();
        }
    }
}
