﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDb
{
    using System;
    using NodaTime.TimeZones;

    public class TimeTransition
    {
        internal TimeTransition(ZoneInterval zi)
        {
            Tag = zi.Name;
            StandardOffset = zi.StandardOffset.ToTimeSpan();
            DaylightSavings = zi.Savings.ToTimeSpan();

            UtcStart = zi.HasStart ? zi.Start.ToDateTimeUtc() : DateTime.MinValue.ToUniversalTime();
            UtcEnd = zi.HasEnd ? zi.End.ToDateTimeUtc() : DateTime.MaxValue.ToUniversalTime();
        }

        public string Tag { get; set; }

        public TimeSpan StandardOffset { get; set; }

        public TimeSpan DaylightSavings { get; set; }

        public DateTime UtcStart { get; set; }

        public DateTime UtcEnd { get; set; }

        public override string ToString() =>
            $"{Tag} {Utilities.To00v00(StandardOffset)} DST {Utilities.To00v00(DaylightSavings)} {Utilities.FromDateTime(UtcStart)} to {Utilities.FromDateTime(UtcEnd)}";
    }
}
