﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDb
{
    using System;
    using System.Runtime.Serialization;

    [Serializable]
    public class TimeZoneDbException : Exception
    {
        internal TimeZoneDbException(string action, Exception ex)
        {
            Action = action;
            Error = ex;
        }

        public string Action { get; private set; }

        public Exception Error { get; private set; }

        public override string ToString()
        {
            if (Error == null)
            {
                return Action;
            }
            else if (Error.InnerException == null)
            {
                return $"{Action}: {Error.Message}";
            }

            return $"{Action}: {Error.Message} [inner: {Error.InnerException.Message}]";
        }

        public override void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            if (info == null)
            {
                throw new System.ArgumentNullException("info");
            }

            info.AddValue("Action", Action);
            base.GetObjectData(info, context);
        }
    }
}
