﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDb
{
    using System;
    using System.Collections.Generic;
    using System.Runtime.Serialization;

    [DataContract]
    public class TimeZoneQueryResults
    {
        /// <summary>Constructor</summary>
        internal TimeZoneQueryResults(string version, TimeZoneQueryParams queryParams, IList<TimeZoneQueryResult> results)
        {
            ReferenceUtcTimestamp = queryParams.UtcTimestamp;
            Version = version;
            TimeZones = results;
        }

        /// <summary>Constructor</summary>
        internal TimeZoneQueryResults(string version, TimeZoneQueryParams queryParams, TimeZoneDbException ex)
        {
            ReferenceUtcTimestamp = queryParams.UtcTimestamp;
            Version = version;
            Error = ex;
        }

        /// <summary>The IANA version used to produce these results</summary>
        [DataMember]
        public string Version { get; set; }

        /// <summary>The user-provided reference UTC time for which the query was executed</summary>
        [DataMember]
        public DateTime ReferenceUtcTimestamp { get; set; }

        /// <summary>List of timezones satisfying the query criteria.  It could be null</summary>
        [DataMember]
        public IList<TimeZoneQueryResult> TimeZones { get; set; }

        /// <summary>If an error occurred, the exception is surfaced to the caller</summary>
        public TimeZoneDbException Error { get; private set; }

        /// <summary>Count</summary>
        public int Count => (TimeZones == null) ? 0 : TimeZones.Count;
    }
}
