﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDbManager
{
    using System;
    using System.IO;

    public class DbUpdateEventArgs : EventArgs, IEquatable<DbUpdateEventArgs>
    {
        private readonly string _timeZoneDbFolder;

        public DbUpdateEventArgs(string tzdbFolder, DbTableEntity e)
        {
            _timeZoneDbFolder = tzdbFolder;
            ContainerName = e.Container;
            EffectiveFrom = e.Effective;
        }

        public string ContainerName { get; private set; }

        public DateTimeOffset EffectiveFrom { get; private set; }

        public TimeSpan DueTimeFromNow => EffectiveFrom - DateTimeOffset.UtcNow;

        public bool IsSwitchoverTime => DueTimeFromNow.TotalSeconds <= 1;

        public string DbFolderPath => Path.Combine(_timeZoneDbFolder, ContainerName);

        public bool Equals(DbUpdateEventArgs o)
        {
            if (o == null)
            {
                return false;
            }

            return ContainerName.Equals(o.ContainerName) && EffectiveFrom.Equals(o.EffectiveFrom);
        }

        public override string ToString() => $"{ContainerName}";
    }
}
