﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDbManager
{
    using System;

    public class DbUpdateInfo
    {
        internal readonly bool _isBootstrap;
        internal readonly DateTimeOffset _dbEffectiveDate;
        internal readonly string _pk;
        internal readonly string _rk;
        internal readonly int _zipDbFileCount;

        internal string _dbPublisher;

        internal DateTimeOffset _celaStamp;

        internal string _cldrTypeVersion;
        internal string _cldrOtherVersion;

        internal DateTimeOffset _ianaStamp;
        internal string _ianaVersion;

        internal string[] _blobPaths = new string[(int)DbCloudStore.SourceBlobType._max];

        internal DbUpdateInfo(int zipDbFileCount, bool bootstrap, DateTimeOffset effectiveDate)
        {
            _zipDbFileCount = zipDbFileCount;
            _isBootstrap = bootstrap;
            _dbEffectiveDate = effectiveDate;
            _celaStamp = _ianaStamp = DateTimeOffset.MinValue;

            GenerateTableKeys(_dbEffectiveDate, out _pk, out _rk);
        }

        internal string ContainerName => GetContainerName(_pk, _rk);

        internal string this[DbCloudStore.SourceBlobType t]
        {
            get { return _blobPaths[(int)t]; }
            set { _blobPaths[(int)t] = value; }
        }

        internal static string GetContainerName(string pk, string rk)
        {
            return $"{pk.Replace("-", string.Empty)}-{rk.Replace(":", string.Empty)}";
        }

        internal void Validate()
        {
            if (!_isBootstrap && _dbEffectiveDate <= DateTime.UtcNow)
            {
                throw new ArgumentException($"TimeZoneDbUpdateInfo validation: now={DateTime.UtcNow} effective={_dbEffectiveDate}");
            }

            if (string.IsNullOrEmpty(_dbPublisher)
                || string.IsNullOrEmpty(_cldrTypeVersion)
                || string.IsNullOrEmpty(_cldrOtherVersion)
                || string.IsNullOrEmpty(_ianaVersion)
                || _celaStamp == DateTimeOffset.MinValue
                || _ianaStamp == DateTimeOffset.MinValue
                || _zipDbFileCount != DbCloudStore.ExpectedZipDbFileCount)
            {
                throw new ArgumentException("TimeZoneDbUpdateInfo validation");
            }

            for (int i = 0; i < (int)DbCloudStore.SourceBlobType._max; i++)
            {
                if (string.IsNullOrEmpty(_blobPaths[i]))
                {
                    throw new ArgumentException("TimeZoneDbUpdateInfo validation");
                }
            }
        }

        private static void GenerateTableKeys(DateTimeOffset dtOff, out string pk, out string rk)
        {
            var utc = dtOff;
            pk = string.Format("tz{0}-{1:00}-{2:00}", utc.Year, utc.Month, utc.Day);
            rk = string.Format("{0:00}:{1:00}:{2:00}", utc.Hour, utc.Minute, utc.Second);
        }
    }
}
