﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDbManager
{
    using System;
    using System.Linq;
    using Microsoft.Azure.LocationServices.Common;
    using Microsoft.WindowsAzure.Storage;
    using Microsoft.WindowsAzure.Storage.Table;

    internal class ServerTableEntity : TableEntity
    {
        internal static readonly string _sTableName = "tzServerRegister";
        private static readonly string _sPK = "tzserver";

        /// <summary>default ctor (required)</summary>
        public ServerTableEntity()
        {
        }

        /// <summary>ctor</summary>
        public ServerTableEntity(string machineName, ITimeZoneDbManaged iManagedDb)
            : base(_sPK, machineName)
        {
            CurrentVersion = iManagedDb.CurrentContainerName;
            AwaitingVersion = iManagedDb.AwaitingContainerName;
        }

        /// <summary>Data version currently served</summary>
        public string CurrentVersion { get; set; }

        /// <summary>Data version ready to be served</summary>
        public string AwaitingVersion { get; set; }

        internal static ServerTableEntity UpdateRegistration(CloudStorageAccount acct, ITimeZoneDbManaged iManagedDb, Logger logger)
        {
            var e = new ServerTableEntity(Environment.MachineName, iManagedDb);

            var client = acct.CreateCloudTableClient();
            var table = client.GetTableReference(_sTableName);
            try
            {
                table.CreateIfNotExists();
                var op = TableOperation.InsertOrReplace(e);
                table.Execute(op);
            }
            catch (StorageException se)
            {
                logger.Log(LogType.Error, $"TimeZoneServerEntity.RegisterAndGetDbUpdates: {se.Message}");
                return null;
            }

            return table.ExecuteQuery(FindMeQuery()).SingleOrDefault();
        }

        internal static TableQuery<ServerTableEntity> FindMeQuery()
        {
            var qry = string.Format("({0}) {1} ({2})",
                                             TableQuery.GenerateFilterCondition("PartitionKey", QueryComparisons.Equal, _sPK),
                                             TableOperators.And,
                                             TableQuery.GenerateFilterCondition("RowKey", QueryComparisons.Equal, Environment.MachineName));
            return new TableQuery<ServerTableEntity>().Where(qry);
        }
    }
}
