﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZoneDbManager
{
    using System.Collections.Generic;

    using Microsoft.Azure.LocationServices.Common;

    public interface ITimeZoneDbManaged
    {
        /// <summary>Return the logger instance</summary>
        Logger GetLogger { get; }

        /// <summary>Get the current container name.  Null if nothing loaded</summary>
        string CurrentContainerName { get; }

        /// <summary>Get the awaiting container name.  Null if the awaiting container is not loaded</summary>
        string AwaitingContainerName { get; }

        /// <summary>Notification to load a new container to a private cache</summary>
        void LoadAwaitingContainer(DbUpdateEventArgs args);

        /// <summary>Check if load has completed and subscriber is ready to swap</summary>
        bool IsReadyToSwapContainers(DbUpdateEventArgs args);

        /// <summary>Notification to serve requests from the newer (awaiting) container</summary>
        void PublishAwaitingContainer();

        /// <summary>Create a private instance of self using given input data 
        /// and run a suite of public API's to verify publish suitability</summary>
        bool TestNewDatabase(string nzdPath, string shapesPath, out string ianaVersion, out IList<string> errors);
    }
}
