﻿// ---------------------------------------------------------------------------------
// <copyright company="Microsoft">
//   Copyright (c) 2017 Microsoft Corporation. All rights reserved.
// </copyright>
// ---------------------------------------------------------------------------------

namespace Microsoft.Azure.LocationServices.TimeZonesUnitTests.TimeZoneDbManager
{
    using System;
    using System.IO;
    using Microsoft.Azure.LocationServices.TimeZoneDbManager;
    using Microsoft.VisualStudio.TestTools.UnitTesting;    

    [TestClass]
    public class ConfigValuesUnitTests
    {
        [TestMethod, TestCategory("L1")]
        public void CreateDynamic_CheckProperties()
        {
            ConfigValues cfg = ConfigValues.CreateForDynamicMode();            
            Assert.IsNotNull(cfg.CompileFolder);
            Assert.IsNotNull(cfg.ExternalSourcesFolder);
            Assert.AreEqual(ExecutionMode.Dynamic, cfg.Mode);            
        }

        [TestMethod, TestCategory("L1")]
        public void CreatePublisher_CheckProperties()
        {
            ConfigValues cfg = ConfigValues.CreateForPublisherMode();
            Assert.IsNotNull(cfg.CompileFolder);
            Assert.IsNotNull(cfg.ExternalSourcesFolder);
            Assert.AreEqual(ExecutionMode.Publisher, cfg.Mode);
        }

        [TestMethod, TestCategory("L1")]
        public void CreateStatic_CheckProperties()
        {
            ConfigValues cfg = ConfigValues.CreateForStaticMode(Directory.GetCurrentDirectory());
            Assert.IsNotNull(cfg.CompileFolder);
            Assert.IsNotNull(cfg.ExternalSourcesFolder);
            Assert.AreEqual(ExecutionMode.Static, cfg.Mode);
        }

        [TestMethod, TestCategory("L1"), ExpectedException(typeof(ArgumentException))]
        public void CreateStatic_InvalidDirectory_ExpectedException()
        {
            ConfigValues.CreateForStaticMode(null);
        }
    }
}
