/// <reference types="node" />
import { TokenInfo, TokenProvider } from "./token";
/**
 * @class SasTokenProvider
 * Defines the SasTokenProvider.
 */
export declare class SasTokenProvider implements TokenProvider {
    /**
     * @property {string} namespace - The namespace of the EventHub/ServiceBus instance.
     */
    namespace: string;
    /**
     * @property {string} keyName - The name of the EventHub/ServiceBus key.
     */
    keyName: string;
    /**
     * @property {string} key - The secret value associated with the above EventHub/ServiceBus key
     */
    key: string;
    /**
     * @property {number} tokenRenewalMarginInSeconds - The number of seconds within which it is good to renew the token. Default = 900 seconds (15 minutes).
     */
    tokenRenewalMarginInSeconds: number;
    /**
     * @property {number} tokenValidTimeInSeconds - The number of seconds for which the token is valid. Default = 3600 seconds (1 hour).
     */
    tokenValidTimeInSeconds: number;
    /**
     * Initializes a new isntance of SasTokenProvider
     * @constructor
     * @param {string} namespace - The namespace of the EventHub/ServiceBus instance.
     * @param {string} keyName - The name of the EventHub/ServiceBus key.
     * @param {string} key - The secret value associated with the above EventHub/ServiceBus key
     */
    constructor(namespace: string, keyName: string, key: string, tokenValidTimeInSeconds?: number, tokenRenewalMarginInSeconds?: number);
    /**
     * Gets the sas token for the specified audience
     * @param {string} [audience] - The audience for which the token is desired. If not
     * provided then the Endpoint from the connection string will be applied.
     */
    getToken(audience?: string): Promise<TokenInfo>;
    /**
     * @protected
     * Creates the sas token based on the provided information
     * @param {string | number} expiry - The time period in unix time after which the token will expire.
     * @param {string} [audience] - The audience for which the token is desired. If not
     * provided then the Endpoint from the connection string will be applied.
     * @param {string | Buffer} [hashInput] The input to be provided to hmac to create the hash.
     */
    protected _createToken(expiry: number, audience?: string, hashInput?: string | Buffer): TokenInfo;
    /**
     * Creates a token provider from the EventHub/ServiceBus connection string;
     * @param {string} connectionString - The EventHub/ServiceBus connection string
     */
    static fromConnectionString(connectionString: string): SasTokenProvider;
}
