import { ConnectionConfig } from "../amqp-common";
export interface IotHubConnectionConfig {
    /**
     * @property {string} endpoint - The iothub endpoint `"<iothub-namespace>.azure-devices.net"`.
     */
    hostName: string;
    /**
     * @property {string} host - The host `"<yournamespace>"`.
     */
    host: string;
    /**
     * @property {string} connectionString - The IotHub connection string.
     */
    connectionString: string;
    /**
     * @property {string} entityPath - The name/path of the entity to which the connection needs to happen.
     */
    entityPath?: string;
    /**
     * @property {string} sharedAccessKeyName - The name of the access key.
     */
    sharedAccessKeyName: string;
    /**
     * @property {string} sharedAccessKey - The secret value of the access key.
     */
    sharedAccessKey: string;
}
export declare namespace IotHubConnectionConfig {
    /**
     * Creates the connection config.
     * @param {string} connectionString - The event hub connection string
     * @param {string} [path]           - The name/path of the entity (hub name) to which the connection needs to happen
     */
    function create(connectionString: string, path?: string): IotHubConnectionConfig;
    /**
     * Validates the properties of connection config.
     * @param {ConnectionConfig} config The connection config to be validated.
     */
    function validate(config: IotHubConnectionConfig): void;
    function convertToEventHubConnectionConfig(iotHubConfig: IotHubConnectionConfig): ConnectionConfig;
}
